@@ -1,7 +1,7 @@
 /* Generic time.h */
 /* $OpenLDAP$ */
 /*
- * Copyright 1998-2017 The OpenLDAP Foundation, Redwood City, California, USA
+ * Copyright 1998-2022 The OpenLDAP Foundation, Redwood City, California, USA
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms are permitted only
@@ -13,16 +13,12 @@
 #ifndef _AC_TIME_H
 #define _AC_TIME_H
 
-#ifdef TIME_WITH_SYS_TIME
-# include <sys/time.h>
-# include <time.h>
-#elif defined HAVE_SYS_TIME_H
+#if defined HAVE_SYS_TIME_H
 # include <sys/time.h>
 # ifdef HAVE_SYS_TIMEB_H
 #  include <sys/timeb.h>
 # endif
-#else
-# include <time.h>
 #endif
+# include <time.h>
 
 #endif /* _AC_TIME_H */
