@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2007-2021 The OpenLDAP Foundation.
+ * Copyright 2007-2022 The OpenLDAP Foundation.
  * Portions Copyright 2007 Michał Szulczyński.
  * Portions Copyright 2009 Howard Chu.
  * All rights reserved.
@@ -31,7 +31,7 @@
 #include <ac/string.h>
 
 #include "slap.h"
-#include "config.h"
+#include "slap-config.h"
 #include "lutil.h"
 
 #ifndef SLAPD_MEMBEROF_ATTR
@@ -119,7 +119,7 @@ autogroup_add_member_to_group( Operation *op, BerValue *dn, BerValue *ndn, autog
 	assert( dn != NULL );
 	assert( ndn != NULL );
 	Debug(LDAP_DEBUG_TRACE, "==> autogroup_add_member_to_group adding <%s> to <%s>\n",
-		dn->bv_val, age->age_dn.bv_val, 0);
+		dn->bv_val, age->age_dn.bv_val );
 
 	vals = (BerValue *)ch_calloc( 2, sizeof( BerValue ) );
 	nvals = (BerValue *)ch_calloc( 2, sizeof( BerValue ) );
@@ -182,7 +182,7 @@ autogroup_add_member_values_to_group( Operation *op, struct berval *dn, autogrou
 	OpExtra oex;
 
 	Debug(LDAP_DEBUG_TRACE, "==> autogroup_add_member_values_to_group adding <%s> to <%s>\n",
-		dn->bv_val, age->age_dn.bv_val, 0);
+		dn->bv_val, age->age_dn.bv_val );
 
 	modlist.sml_op = LDAP_MOD_ADD;
 	modlist.sml_desc = age->age_def->agd_member_ad;
@@ -238,14 +238,14 @@ autogroup_delete_member_from_group( Operation *op, BerValue *dn, BerValue *ndn,
 
 	if ( dn == NULL || ndn == NULL ) {
 		Debug(LDAP_DEBUG_TRACE, "==> autogroup_delete_member_from_group removing all members from <%s>\n",
-			age->age_dn.bv_val, 0 ,0);
+			age->age_dn.bv_val );
 
 		modlist->sml_values = NULL;
 		modlist->sml_nvalues = NULL;
 		modlist->sml_numvals = 0;
 	} else {
 		Debug(LDAP_DEBUG_TRACE, "==> autogroup_delete_member_from_group removing <%s> from <%s>\n",
-			dn->bv_val, age->age_dn.bv_val, 0);
+			dn->bv_val, age->age_dn.bv_val );
 
 		vals = (BerValue *)ch_calloc( 2, sizeof( BerValue ) );
 		nvals = (BerValue *)ch_calloc( 2, sizeof( BerValue ) );
@@ -311,7 +311,7 @@ autogroup_delete_member_values_from_group( Operation *op, struct berval *dn, aut
 	OpExtra oex;
 
         Debug(LDAP_DEBUG_TRACE, "==> autogroup_delete_member_values_from_group removing <%s> from <%s>\n",
-			dn->bv_val, age->age_dn.bv_val, 0);
+			dn->bv_val, age->age_dn.bv_val );
 
         modlist.sml_op = LDAP_MOD_DELETE;
         modlist.sml_desc = age->age_def->agd_member_ad;
@@ -372,7 +372,7 @@ autogroup_member_search_cb( Operation *op, SlapReply *rs )
 		int			numvals;
 
 		Debug(LDAP_DEBUG_TRACE, "==> autogroup_member_search_cb <%s>\n",
-			rs->sr_entry ? rs->sr_entry->e_name.bv_val : "UNKNOWN_DN", 0, 0);
+			rs->sr_entry ? rs->sr_entry->e_name.bv_val : "UNKNOWN_DN" );
 
 		if ( agf->agf_anlist ) {
 			Attribute *attr = attrs_find( rs->sr_entry->e_attrs, agf->agf_anlist[0].an_desc );
@@ -428,7 +428,7 @@ autogroup_member_search_modify_cb( Operation *op, SlapReply *rs )
 		int			numvals;
 
 		Debug(LDAP_DEBUG_TRACE, "==> autogroup_member_search_modify_cb <%s>\n",
-			rs->sr_entry ? rs->sr_entry->e_name.bv_val : "UNKNOWN_DN", 0, 0);
+			rs->sr_entry ? rs->sr_entry->e_name.bv_val : "UNKNOWN_DN" );
 
 		if ( agf->agf_anlist ) {
 			Attribute *attr = attrs_find( rs->sr_entry->e_attrs, agf->agf_anlist[0].an_desc );
@@ -500,7 +500,7 @@ autogroup_add_members_from_filter( Operation *op, Entry *e, autogroup_entry_t *a
 	OpExtra oex;
 
 	Debug(LDAP_DEBUG_TRACE, "==> autogroup_add_members_from_filter <%s>\n",
-		age->age_dn.bv_val, 0, 0);
+		age->age_dn.bv_val );
 
 	o.ors_attrsonly = 0;
 	o.o_tag = LDAP_REQ_SEARCH;
@@ -581,7 +581,7 @@ autogroup_add_members_from_filter( Operation *op, Entry *e, autogroup_entry_t *a
 
 /* 
 ** Adds a group to the internal list from the passed entry.
-** scan specifies whether to add all maching members to the group.
+** scan specifies whether to add all matching members to the group.
 ** modify specifies whether to modify the given group entry (when modify == 0),
 **	or to modify the group entry in the database (when modify == 1 and e = NULL and ndn != NULL).
 ** agi	- pointer to the groups and the attribute definitions
@@ -603,7 +603,7 @@ autogroup_add_group( Operation *op, autogroup_info_t *agi, autogroup_def_t *agd,
 	if ( e == NULL ) {
 		if ( overlay_entry_get_ov( op, ndn, NULL, NULL, 0, &e, on ) !=
 			LDAP_SUCCESS || e == NULL ) {
-			Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: cannot get entry for <%s>\n", ndn->bv_val, 0, 0);
+			Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: cannot get entry for <%s>\n", ndn->bv_val );
 			return 1;
 		}
 
@@ -611,13 +611,13 @@ autogroup_add_group( Operation *op, autogroup_info_t *agi, autogroup_def_t *agd,
 	}
 
 	Debug(LDAP_DEBUG_TRACE, "==> autogroup_add_group <%s>\n",
-		e->e_name.bv_val, 0, 0);
+		e->e_name.bv_val );
 
 	if ( agi->agi_entry != NULL ) {
 		for ( ; *agep ; agep = &(*agep)->age_next ) {
 			dnMatch( &match, 0, NULL, NULL, &e->e_nname, &(*agep)->age_ndn );
 			if ( match == 0 ) {
-				Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: group already exists: <%s>\n", e->e_name.bv_val,0,0);
+				Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: group already exists: <%s>\n", e->e_name.bv_val );
 				return 1;
 			}
 			/* goto last */;
@@ -643,14 +643,14 @@ autogroup_add_group( Operation *op, autogroup_info_t *agi, autogroup_def_t *agd,
 	}
 
 	if( a == NULL ) {
-		Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: group has no memberURL\n", 0,0,0);
+		Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: group has no memberURL\n" );
 	} else {
 		for ( bv = a->a_nvals; !BER_BVISNULL( bv ); bv++ ) {
 
 			agf = (autogroup_filter_t*)ch_calloc( 1, sizeof( autogroup_filter_t ) );
 
 			if ( ldap_url_parse( bv->bv_val, &lud ) != LDAP_URL_SUCCESS ) {
-				Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: cannot parse url <%s>\n", bv->bv_val,0,0);
+				Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: cannot parse url <%s>\n", bv->bv_val );
 				/* FIXME: error? */
 				ch_free( agf ); 
 				continue;
@@ -666,7 +666,7 @@ autogroup_add_group( Operation *op, autogroup_info_t *agi, autogroup_def_t *agd,
 
 			rc = dnPrettyNormal( NULL, &dn, &agf->agf_dn, &agf->agf_ndn, NULL );
 			if ( rc != LDAP_SUCCESS ) {
-				Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: cannot normalize DN <%s>\n", dn.bv_val,0,0);
+				Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: cannot normalize DN <%s>\n", dn.bv_val );
 				/* FIXME: error? */
 				goto cleanup;
 			}
@@ -675,7 +675,7 @@ autogroup_add_group( Operation *op, autogroup_info_t *agi, autogroup_def_t *agd,
 				ber_str2bv( lud->lud_filter, 0, 1, &agf->agf_filterstr);
 				agf->agf_filter = str2filter( lud->lud_filter );
 			} else {
-				Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: URL filter is missing <%s>\n", bv->bv_val,0,0);
+				Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: URL filter is missing <%s>\n", bv->bv_val );
 				/* FIXME: error? */
 				goto cleanup;
 			}
@@ -689,7 +689,7 @@ autogroup_add_group( Operation *op, autogroup_info_t *agi, autogroup_def_t *agd,
 
 				if ( i > 1 ) {
 					Debug( LDAP_DEBUG_ANY, "autogroup_add_group: too many attributes specified in url <%s>\n",
-						bv->bv_val, 0, 0);
+						bv->bv_val );
 					/* FIXME: error? */
 					filter_free( agf->agf_filter );
 					ch_free( agf->agf_filterstr.bv_val );
@@ -704,7 +704,7 @@ autogroup_add_group( Operation *op, autogroup_info_t *agi, autogroup_def_t *agd,
 
 				if ( agf->agf_anlist == NULL ) {
 					Debug( LDAP_DEBUG_ANY, "autogroup_add_group: unable to find AttributeDescription \"%s\".\n",
-						lud->lud_attrs[0], 0, 0 );
+						lud->lud_attrs[0] );
 					/* FIXME: error? */
 					filter_free( agf->agf_filter );
 					ch_free( agf->agf_filterstr.bv_val );
@@ -733,7 +733,7 @@ autogroup_add_group( Operation *op, autogroup_info_t *agi, autogroup_def_t *agd,
 			}
 
 			Debug( LDAP_DEBUG_TRACE, "autogroup_add_group: added memberURL DN <%s> with filter <%s>\n",
-				agf->agf_ndn.bv_val, agf->agf_filterstr.bv_val, 0);
+				agf->agf_ndn.bv_val, agf->agf_filterstr.bv_val );
 
 			ldap_free_urldesc( lud );
 
@@ -768,7 +768,7 @@ autogroup_group_add_cb( Operation *op, SlapReply *rs )
 		autogroup_sc_t		*ags = (autogroup_sc_t *)op->o_callback->sc_private;
 
 		Debug(LDAP_DEBUG_TRACE, "==> autogroup_group_add_cb <%s>\n",
-			rs->sr_entry ? rs->sr_entry->e_name.bv_val : "UNKNOWN_DN", 0, 0);
+			rs->sr_entry ? rs->sr_entry->e_name.bv_val : "UNKNOWN_DN" );
 
 		autogroup_add_group( op, ags->ags_info, ags->ags_def, rs->sr_entry, NULL, 0, 0);
 	}
@@ -849,7 +849,7 @@ done:
 
 /*
 ** When adding a group, we first strip any existing members,
-** and add all which match the filters ourselfs.
+** and add all which match the filters ourselves.
 */
 static int
 autogroup_add_entry( Operation *op, SlapReply *rs)
@@ -861,7 +861,7 @@ autogroup_add_entry( Operation *op, SlapReply *rs)
 	ag_addinfo	*aa = NULL;
 
 	Debug( LDAP_DEBUG_TRACE, "==> autogroup_add_entry <%s>\n", 
-		op->ora_e->e_name.bv_val, 0, 0);
+		op->ora_e->e_name.bv_val );
 
 	sc = op->o_tmpcalloc( sizeof(slap_callback) + sizeof(ag_addinfo), 1, op->o_tmpmemctx );
 	sc->sc_private = (sc+1);
@@ -910,7 +910,7 @@ autogroup_delete_group( autogroup_info_t *agi, autogroup_entry_t *e )
 	int			rc = 1;
 
 	Debug( LDAP_DEBUG_TRACE, "==> autogroup_delete_group <%s>\n", 
-		age->age_dn.bv_val, 0, 0);
+		age->age_dn.bv_val );
 
 	for ( age_next = age ; age_next ; age_prev = age, age = age_next ) {
 		age_next = age->age_next;
@@ -949,7 +949,7 @@ autogroup_delete_group( autogroup_info_t *agi, autogroup_entry_t *e )
 		}
 	}
 
-	Debug( LDAP_DEBUG_TRACE, "autogroup_delete_group: group <%s> not found, should not happen\n", age->age_dn.bv_val, 0, 0);
+	Debug( LDAP_DEBUG_TRACE, "autogroup_delete_group: group <%s> not found, should not happen\n", age->age_dn.bv_val );
 
 	return rc;
 
@@ -972,13 +972,13 @@ autogroup_delete_entry( Operation *op, SlapReply *rs)
 			return SLAP_CB_CONTINUE;
 	}
 
-	Debug( LDAP_DEBUG_TRACE, "==> autogroup_delete_entry <%s>\n", op->o_req_dn.bv_val, 0, 0);
+	Debug( LDAP_DEBUG_TRACE, "==> autogroup_delete_entry <%s>\n", op->o_req_dn.bv_val );
 
 	ldap_pvt_thread_mutex_lock( &agi->agi_mutex );
 
 	if ( overlay_entry_get_ov( op, &op->o_req_ndn, NULL, NULL, 0, &e, on ) !=
 		LDAP_SUCCESS || e == NULL ) {
-		Debug( LDAP_DEBUG_TRACE, "autogroup_delete_entry: cannot get entry for <%s>\n", op->o_req_dn.bv_val, 0, 0);
+		Debug( LDAP_DEBUG_TRACE, "autogroup_delete_entry: cannot get entry for <%s>\n", op->o_req_dn.bv_val );
 		ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );			
 		return SLAP_CB_CONTINUE;
 	}
@@ -1059,7 +1059,6 @@ autogroup_response( Operation *op, SlapReply *rs )
 	autogroup_def_t		*agd = agi->agi_def;
 	autogroup_entry_t	*age;
 	autogroup_filter_t	*agf;
-	BerValue		new_dn, new_ndn, pdn;
 	Entry			*e, *group;
 	Attribute		*a, *ea, *attrs;
 	int			is_olddn, is_newdn, is_value_refresh, dn_equal;
@@ -1097,31 +1096,16 @@ autogroup_response( Operation *op, SlapReply *rs )
 	} else if ( op->o_tag == LDAP_REQ_MODRDN ) {
 		if ( rs->sr_type == REP_RESULT && rs->sr_err == LDAP_SUCCESS && !oex ) {
 
-			Debug( LDAP_DEBUG_TRACE, "==> autogroup_response MODRDN from <%s>\n", op->o_req_dn.bv_val, 0, 0);
+			Debug( LDAP_DEBUG_TRACE, "==> autogroup_response MODRDN from <%s>\n", op->o_req_dn.bv_val );
+			Debug( LDAP_DEBUG_TRACE, "autogroup_response MODRDN to <%s>\n", op->orr_newDN );
 
 			ldap_pvt_thread_mutex_lock( &agi->agi_mutex );			
 
-			if ( op->oq_modrdn.rs_newSup ) {
-				pdn = *op->oq_modrdn.rs_newSup;
-			} else {
-				dnParent( &op->o_req_dn, &pdn );
-			}
-			build_new_dn( &new_dn, &pdn, &op->orr_newrdn, op->o_tmpmemctx );
+			dnMatch( &dn_equal, 0, NULL, NULL, &op->o_req_ndn, &op->orr_nnewDN );
 
-			if ( op->oq_modrdn.rs_nnewSup ) {
-				pdn = *op->oq_modrdn.rs_nnewSup;
-			} else {
-				dnParent( &op->o_req_ndn, &pdn );
-			}
-			build_new_dn( &new_ndn, &pdn, &op->orr_nnewrdn, op->o_tmpmemctx );
-
-			Debug( LDAP_DEBUG_TRACE, "autogroup_response MODRDN to <%s>\n", new_dn.bv_val, 0, 0);
-
-			dnMatch( &dn_equal, 0, NULL, NULL, &op->o_req_ndn, &new_ndn );
-
-			if ( overlay_entry_get_ov( op, &new_ndn, NULL, NULL, 0, &e, on ) !=
+			if ( overlay_entry_get_ov( op, &op->orr_nnewDN, NULL, NULL, 0, &e, on ) !=
 				LDAP_SUCCESS || e == NULL ) {
-				Debug( LDAP_DEBUG_TRACE, "autogroup_response MODRDN cannot get entry for <%s>\n", new_dn.bv_val, 0, 0);
+				Debug( LDAP_DEBUG_TRACE, "autogroup_response MODRDN cannot get entry for <%s>\n", op->orr_newDN.bv_val );
 				ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );
 				return SLAP_CB_CONTINUE;
 			}
@@ -1130,7 +1114,7 @@ autogroup_response( Operation *op, SlapReply *rs )
 
 
 			if ( a == NULL ) {
-				Debug( LDAP_DEBUG_TRACE, "autogroup_response MODRDN entry <%s> has no objectClass\n", new_dn.bv_val, 0, 0);
+				Debug( LDAP_DEBUG_TRACE, "autogroup_response MODRDN entry <%s> has no objectClass\n", op->orr_newDN.bv_val );
 				overlay_entry_release_ov( op, e, 0, on );
 				ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );		
 				return SLAP_CB_CONTINUE;
@@ -1151,12 +1135,10 @@ autogroup_response( Operation *op, SlapReply *rs )
 
 						dnMatch( &match, 0, NULL, NULL, &age->age_ndn, &op->o_req_ndn );
 						if ( match == 0 ) {
-							Debug( LDAP_DEBUG_TRACE, "autogroup_response MODRDN updating group's DN to <%s>\n", new_dn.bv_val, 0, 0);
-							ber_dupbv( &age->age_dn, &new_dn );
-							ber_dupbv( &age->age_ndn, &new_ndn );
+							Debug( LDAP_DEBUG_TRACE, "autogroup_response MODRDN updating group's DN to <%s>\n", op->orr_newDN.bv_val );
+							ber_dupbv( &age->age_dn, &op->orr_newDN );
+							ber_dupbv( &age->age_ndn, &op->orr_nnewDN );
 
-							op->o_tmpfree( new_dn.bv_val, op->o_tmpmemctx  );
-							op->o_tmpfree( new_ndn.bv_val, op->o_tmpmemctx );
 							overlay_entry_release_ov( op, e, 0, on );
 							ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );		
 							return SLAP_CB_CONTINUE;
@@ -1167,8 +1149,8 @@ autogroup_response( Operation *op, SlapReply *rs )
 			}
 
 			/* For each group: 
-			   1. check if the orginal entry's DN is in the group.
-			   2. chceck if the any of the group filter's base DN is a suffix of the new DN 
+			   1. check if the original entry's DN is in the group.
+			   2. check if the any of the group filter's base DN is a suffix of the new DN
 
 			   If 1 and 2 are both false, we do nothing.
 			   If 1 and 2 is true, we remove the old DN from the group, and add the new DN.
@@ -1193,7 +1175,7 @@ autogroup_response( Operation *op, SlapReply *rs )
 				}
 
 				if ( age->age_modrdn_olddnmodified ) {
-					/* Resquest already marked this group to be updated */
+					/* Request already marked this group to be updated */
 					is_olddn = 1;
 					is_value_refresh = 1;
 					age->age_modrdn_olddnmodified = 0;
@@ -1201,10 +1183,7 @@ autogroup_response( Operation *op, SlapReply *rs )
 
 					if ( overlay_entry_get_ov( op, &age->age_ndn, NULL, NULL, 0, &group, on ) !=
 						LDAP_SUCCESS || group == NULL ) {
-						Debug( LDAP_DEBUG_TRACE, "autogroup_response MODRDN cannot get group entry <%s>\n", age->age_dn.bv_val, 0, 0);
-
-						op->o_tmpfree( new_dn.bv_val, op->o_tmpmemctx );
-						op->o_tmpfree( new_ndn.bv_val, op->o_tmpmemctx );
+						Debug( LDAP_DEBUG_TRACE, "autogroup_response MODRDN cannot get group entry <%s>\n", age->age_dn.bv_val );
 
 						attrs_free( attrs );
 						ldap_pvt_thread_mutex_unlock( &age->age_mutex );
@@ -1230,7 +1209,7 @@ autogroup_response( Operation *op, SlapReply *rs )
 				}
 
 				for ( agf = age->age_filter ; agf ; agf = agf->agf_next ) {
-					if ( dnIsSuffix( &new_ndn, &agf->agf_ndn ) ) {
+					if ( dnIsSuffix( &op->orr_nnewDN, &agf->agf_ndn ) ) {
 						/* TODO: should retest filter as it could imply conditions on the dn */
 						is_newdn = 1;
 						break;
@@ -1252,7 +1231,7 @@ autogroup_response( Operation *op, SlapReply *rs )
 				}
 				if ( is_olddn == 1 && is_newdn == 0 ) {
 					if ( ea )
-						autogroup_delete_member_values_from_group( op, &new_dn, age, ea );
+						autogroup_delete_member_values_from_group( op, &op->orr_newDN, age, ea );
 					else
 						autogroup_delete_member_from_group( op, &op->o_req_dn, &op->o_req_ndn, age );
 				} else
@@ -1270,9 +1249,9 @@ autogroup_response( Operation *op, SlapReply *rs )
 					for ( agf = age->age_filter; agf; agf = agf->agf_next ) {
 						if ( test_filter( op, &etmp, agf->agf_filter ) == LDAP_COMPARE_TRUE ) {
 							if ( ea ) {
-								autogroup_add_member_values_to_group( op, &new_dn, age, ea );
+								autogroup_add_member_values_to_group( op, &op->orr_newDN, age, ea );
 							} else
-								autogroup_add_member_to_group( op, &new_dn, &new_ndn, age );
+								autogroup_add_member_to_group( op, &op->orr_newDN, &op->orr_nnewDN, age );
 							break;
 						}
 					}
@@ -1290,16 +1269,13 @@ autogroup_response( Operation *op, SlapReply *rs )
 					}
 					else {
 						autogroup_delete_member_from_group( op, &op->o_req_dn, &op->o_req_ndn, age );
-						autogroup_add_member_to_group( op, &new_dn, &new_ndn, age );
+						autogroup_add_member_to_group( op, &op->orr_newDN, &op->orr_nnewDN, age );
 					}
 				}
 
 				ldap_pvt_thread_mutex_unlock( &age->age_mutex );
 			}
 
-			op->o_tmpfree( new_dn.bv_val, op->o_tmpmemctx );
-			op->o_tmpfree( new_ndn.bv_val, op->o_tmpmemctx );
-
 			attrs_free( attrs );
 
 			ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );			
@@ -1310,13 +1286,13 @@ autogroup_response( Operation *op, SlapReply *rs )
 		if ( rs->sr_type == REP_RESULT && rs->sr_err == LDAP_SUCCESS  && !oex ) {
 			Entry etmp;
 			struct berval odn, ondn;
-			Debug( LDAP_DEBUG_TRACE, "==> autogroup_response MODIFY <%s>\n", op->o_req_dn.bv_val, 0, 0);
+			Debug( LDAP_DEBUG_TRACE, "==> autogroup_response MODIFY <%s>\n", op->o_req_dn.bv_val );
 
 			ldap_pvt_thread_mutex_lock( &agi->agi_mutex );			
 
 			if ( overlay_entry_get_ov( op, &op->o_req_ndn, NULL, NULL, 0, &e, on ) !=
 				LDAP_SUCCESS || e == NULL ) {
-				Debug( LDAP_DEBUG_TRACE, "autogroup_response MODIFY cannot get entry for <%s>\n", op->o_req_dn.bv_val, 0, 0);
+				Debug( LDAP_DEBUG_TRACE, "autogroup_response MODIFY cannot get entry for <%s>\n", op->o_req_dn.bv_val );
 				ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );
 				return SLAP_CB_CONTINUE;
 			}
@@ -1325,7 +1301,7 @@ autogroup_response( Operation *op, SlapReply *rs )
 
 
 			if ( a == NULL ) {
-				Debug( LDAP_DEBUG_TRACE, "autogroup_response MODIFY entry <%s> has no objectClass\n", op->o_req_dn.bv_val, 0, 0);
+				Debug( LDAP_DEBUG_TRACE, "autogroup_response MODIFY entry <%s> has no objectClass\n", op->o_req_dn.bv_val );
 				overlay_entry_release_ov( op, e, 0, on );
 				ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );		
 				return SLAP_CB_CONTINUE;
@@ -1356,7 +1332,7 @@ autogroup_response( Operation *op, SlapReply *rs )
 								if ( m->sml_desc == age->age_def->agd_member_url_ad ) {
 									autogroup_def_t	*group_agd = age->age_def;
 									Debug( LDAP_DEBUG_TRACE, "autogroup_response MODIFY changing memberURL for group <%s>\n", 
-										op->o_req_dn.bv_val, 0, 0);
+										op->o_req_dn.bv_val );
 
 									overlay_entry_release_ov( op, e, 0, on );
 
@@ -1385,7 +1361,7 @@ autogroup_response( Operation *op, SlapReply *rs )
 
 			/* When modifying any of the attributes of an entry, we must
 			   check if the entry is in any of our groups, and if
-			   the modified entry maches any of the filters of that group.
+			   the modified entry matches any of the filters of that group.
 
 			   If the entry exists in a group, but the modified attributes do
 				not match any of the group's filters, we delete the entry from that group.
@@ -1418,7 +1394,7 @@ autogroup_response( Operation *op, SlapReply *rs )
 				if ( overlay_entry_get_ov( op, &age->age_ndn, NULL, NULL, 0, &group, on ) !=
 					LDAP_SUCCESS || group == NULL ) {
 					Debug( LDAP_DEBUG_TRACE, "autogroup_response MODIFY cannot get entry for <%s>\n", 
-						age->age_dn.bv_val, 0, 0);
+						age->age_dn.bv_val );
 
 					attrs_free( attrs );
 					ldap_pvt_thread_mutex_unlock( &age->age_mutex );
@@ -1508,7 +1484,7 @@ autogroup_memberOf_filter( Filter *f, BerValue *dn, AttributeDescription *member
 }
 
 /*
-** When modifing a group, we must deny any modifications to the member attribute,
+** When modifying a group, we must deny any modifications to the member attribute,
 ** because the group would be inconsistent.
 */
 static int
@@ -1528,12 +1504,12 @@ autogroup_modify_entry( Operation *op, SlapReply *rs)
 			return SLAP_CB_CONTINUE;
 	}
 
-	Debug( LDAP_DEBUG_TRACE, "==> autogroup_modify_entry <%s>\n", op->o_req_dn.bv_val, 0, 0);
+	Debug( LDAP_DEBUG_TRACE, "==> autogroup_modify_entry <%s>\n", op->o_req_dn.bv_val );
 	ldap_pvt_thread_mutex_lock( &agi->agi_mutex );			
 
 	if ( overlay_entry_get_ov( op, &op->o_req_ndn, NULL, NULL, 0, &e, on ) !=
 		LDAP_SUCCESS || e == NULL ) {
-		Debug( LDAP_DEBUG_TRACE, "autogroup_modify_entry cannot get entry for <%s>\n", op->o_req_dn.bv_val, 0, 0);
+		Debug( LDAP_DEBUG_TRACE, "autogroup_modify_entry cannot get entry for <%s>\n", op->o_req_dn.bv_val );
 		ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );
 		return SLAP_CB_CONTINUE;
 	}
@@ -1572,7 +1548,7 @@ autogroup_modify_entry( Operation *op, SlapReply *rs)
 	a = attrs_find( e->e_attrs, slap_schema.si_ad_objectClass );
 
 	if ( a == NULL ) {
-		Debug( LDAP_DEBUG_TRACE, "autogroup_modify_entry entry <%s> has no objectClass\n", op->o_req_dn.bv_val, 0, 0);
+		Debug( LDAP_DEBUG_TRACE, "autogroup_modify_entry entry <%s> has no objectClass\n", op->o_req_dn.bv_val );
 		ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );		
 		return SLAP_CB_CONTINUE;
 	}
@@ -1599,7 +1575,7 @@ autogroup_modify_entry( Operation *op, SlapReply *rs)
 						if ( m->sml_desc == age->age_def->agd_member_ad ) {
 							overlay_entry_release_ov( op, e, 0, on );
 							ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );
-							Debug( LDAP_DEBUG_TRACE, "autogroup_modify_entry attempted to modify group's <%s> member attribute\n", op->o_req_dn.bv_val, 0, 0);
+							Debug( LDAP_DEBUG_TRACE, "autogroup_modify_entry attempted to modify group's <%s> member attribute\n", op->o_req_dn.bv_val );
 							send_ldap_error(op, rs, LDAP_CONSTRAINT_VIOLATION, "attempt to modify dynamic group member attribute");
 							return LDAP_CONSTRAINT_VIOLATION;
 						}
@@ -1636,12 +1612,12 @@ autogroup_modrdn_entry( Operation *op, SlapReply *rs)
 			return SLAP_CB_CONTINUE;
 	}
 
-	Debug( LDAP_DEBUG_TRACE, "==> autogroup_modrdn_entry <%s>\n", op->o_req_dn.bv_val, 0, 0);
+	Debug( LDAP_DEBUG_TRACE, "==> autogroup_modrdn_entry <%s>\n", op->o_req_dn.bv_val );
 	ldap_pvt_thread_mutex_lock( &agi->agi_mutex );			
 
 	if ( overlay_entry_get_ov( op, &op->o_req_ndn, NULL, NULL, 0, &e, on ) !=
 		LDAP_SUCCESS || e == NULL ) {
-		Debug( LDAP_DEBUG_TRACE, "autogroup_modrdn_entry cannot get entry for <%s>\n", op->o_req_dn.bv_val, 0, 0);
+		Debug( LDAP_DEBUG_TRACE, "autogroup_modrdn_entry cannot get entry for <%s>\n", op->o_req_dn.bv_val );
 		ldap_pvt_thread_mutex_unlock( &agi->agi_mutex );
 		return SLAP_CB_CONTINUE;
 	}
@@ -1682,7 +1658,7 @@ autogroup_build_def_filter( autogroup_def_t *agd, Operation *op )
 {
 	char	*ptr;
 
-	Debug( LDAP_DEBUG_TRACE, "==> autogroup_build_def_filter\n", 0, 0, 0);
+	Debug( LDAP_DEBUG_TRACE, "==> autogroup_build_def_filter\n" );
 
 	op->ors_filterstr.bv_len = STRLENOF( "(=)" ) 
 			+ slap_schema.si_ad_objectClass->ad_cname.bv_len
@@ -1713,7 +1689,7 @@ static ConfigDriver	ag_cfgen;
 static ConfigTable agcfg[] = {
 	{ "autogroup-attrset", "group-oc> <URL-ad> <member-ad",
 		4, 4, 0, ARG_MAGIC|AG_ATTRSET, ag_cfgen,
-		"( OLcfgCtAt:2.1 NAME 'olcAGattrSet' "
+		"( OLcfgCtAt:2.1 NAME ( 'olcAutoGroupAttrSet' 'olcAGattrSet' ) "
 			"DESC 'Automatic groups: <group objectClass>, <URL attributeDescription>, <member attributeDescription>' "
 			"EQUALITY caseIgnoreMatch "
 			"SYNTAX OMsDirectoryString "
@@ -1722,8 +1698,9 @@ static ConfigTable agcfg[] = {
 	
 	{ "autogroup-memberof-ad", "memberOf attribute",
 		2, 2, 0, ARG_MAGIC|AG_MEMBER_OF_AD, ag_cfgen,
-		"( OLcfgCtAt:2.2 NAME 'olcAGmemberOfAd' "
+		"( OLcfgCtAt:2.2 NAME ( 'olcAutoGroupMemberOfAd' 'olcAGmemberOfAd' ) "
 			"DESC 'memberOf attribute' "
+			"EQUALITY caseIgnoreMatch "
 			"SYNTAX OMsDirectoryString SINGLE-VALUE )",
 			NULL, NULL },
 
@@ -1732,12 +1709,12 @@ static ConfigTable agcfg[] = {
 
 static ConfigOCs agocs[] = {
 	{ "( OLcfgCtOc:2.1 "
-		"NAME 'olcAutomaticGroups' "
+		"NAME ( 'olcAutoGroupConfig' 'olcAutomaticGroups' ) "
 		"DESC 'Automatic groups configuration' "
 		"SUP olcOverlayConfig "
 		"MAY ( "
-			"olcAGattrSet "
-			"$ olcAGmemberOfAd "
+			"olcAutoGroupAttrSet "
+			"$ olcAutoGroupMemberOfAd "
 		    ")"
 	  ")",
 		Cft_Overlay, agcfg, NULL, NULL },
@@ -1755,7 +1732,7 @@ ag_cfgen( ConfigArgs *c )
 
 	int rc = 0, i;
 
-	Debug( LDAP_DEBUG_TRACE, "==> autogroup_cfgen\n", 0, 0, 0);
+	Debug( LDAP_DEBUG_TRACE, "==> autogroup_cfgen\n" );
 
 	if( agi == NULL ) {
 		agi = (autogroup_info_t*)ch_calloc( 1, sizeof(autogroup_info_t) );
@@ -1916,7 +1893,7 @@ ag_cfgen( ConfigArgs *c )
 				"unable to find ObjectClass \"%s\"",
 				c->argv[ 1 ] );
 			Debug( LDAP_DEBUG_ANY, "%s: %s.\n",
-				c->log, c->cr_msg, 0 );
+				c->log, c->cr_msg );
 			return 1;
 		}
 
@@ -1928,7 +1905,7 @@ ag_cfgen( ConfigArgs *c )
 				"unable to find AttributeDescription \"%s\"",
 				c->argv[ 2 ] );
 			Debug( LDAP_DEBUG_ANY, "%s: %s.\n",
-				c->log, c->cr_msg, 0 );		
+				c->log, c->cr_msg );
 			return 1;
 		}
 
@@ -1939,7 +1916,7 @@ ag_cfgen( ConfigArgs *c )
 				"must be of a subtype \"labeledURI\"",
 				c->argv[ 2 ] );
 			Debug( LDAP_DEBUG_ANY, "%s: %s.\n",
-				c->log, c->cr_msg, 0 );
+				c->log, c->cr_msg );
 			return 1;
 		}
 
@@ -1950,7 +1927,7 @@ ag_cfgen( ConfigArgs *c )
 				"unable to find AttributeDescription \"%s\"",
 				c->argv[ 3 ] );
 			Debug( LDAP_DEBUG_ANY, "%s: %s.\n",
-				c->log, c->cr_msg, 0 );
+				c->log, c->cr_msg );
 			return 1;
 		}
 
@@ -1964,12 +1941,12 @@ ag_cfgen( ConfigArgs *c )
 					"URL attributeDescription \"%s\" already mapped",
 					member_ad->ad_cname.bv_val );
 				Debug( LDAP_DEBUG_ANY, "%s: %s.\n",
-					c->log, c->cr_msg, 0 );
+					c->log, c->cr_msg );
 /*				return 1; //warning*/
 			}
 		}
 
-		if ( c->valx > 0 ) {
+		if ( c->valx >= 0 ) {
 			int	i;
 
 			for ( i = 0, agdp = &agi->agi_def ;
@@ -1981,7 +1958,7 @@ ag_cfgen( ConfigArgs *c )
 						"invalid index {%d}",
 						c->valx );
 					Debug( LDAP_DEBUG_ANY, "%s: %s.\n",
-						c->log, c->cr_msg, 0 );
+						c->log, c->cr_msg );
 
 					return 1;
 				}
@@ -2015,7 +1992,7 @@ ag_cfgen( ConfigArgs *c )
 				"unable to find AttributeDescription \"%s\"",
 				c->argv[ 1 ] );
 			Debug( LDAP_DEBUG_ANY, "%s: %s.\n",
-				c->log, c->cr_msg, 0 );
+				c->log, c->cr_msg );
 			return 1;
 		}
 
@@ -2027,7 +2004,7 @@ ag_cfgen( ConfigArgs *c )
 				"have DN (%s) or nameUID (%s) syntax",
 				c->argv[ 1 ], SLAPD_DN_SYNTAX, SLAPD_NAMEUID_SYNTAX );
 			Debug( LDAP_DEBUG_ANY, "%s: %s.\n",
-				c->log, c->cr_msg, 0 );
+				c->log, c->cr_msg );
 			return 1;
 		}
 
@@ -2065,7 +2042,7 @@ autogroup_db_open(
 	Connection			conn = { 0 };
 	OperationBuffer 	opbuf;
 
-	Debug( LDAP_DEBUG_TRACE, "==> autogroup_db_open\n", 0, 0, 0);
+	Debug( LDAP_DEBUG_TRACE, "==> autogroup_db_open\n" );
 
 	if ( agi == NULL || !( slapMode & SLAP_SERVER_MODE )) {
 		return 0;
@@ -2137,7 +2114,7 @@ autogroup_db_close(
 {
 	slap_overinst			*on = (slap_overinst *) be->bd_info;
 
-	Debug( LDAP_DEBUG_TRACE, "==> autogroup_db_close\n", 0, 0, 0);
+	Debug( LDAP_DEBUG_TRACE, "==> autogroup_db_close\n" );
 
 	if ( on->on_bi.bi_private ) {
 		autogroup_info_t		*agi = on->on_bi.bi_private;
@@ -2179,7 +2156,7 @@ autogroup_db_destroy(
 {
 	slap_overinst			*on = (slap_overinst *) be->bd_info;
 
-	Debug( LDAP_DEBUG_TRACE, "==> autogroup_db_destroy\n", 0, 0, 0);
+	Debug( LDAP_DEBUG_TRACE, "==> autogroup_db_destroy\n" );
 
 	if ( on->on_bi.bi_private ) {
 		autogroup_info_t		*agi = on->on_bi.bi_private;
@@ -2206,6 +2183,7 @@ autogroup_initialize(void)
 	int		rc = 0;
 	autogroup.on_bi.bi_type = "autogroup";
 
+	autogroup.on_bi.bi_flags = SLAPO_BFLAG_SINGLE;
 	autogroup.on_bi.bi_db_open = autogroup_db_open;
 	autogroup.on_bi.bi_db_close = autogroup_db_close;
 	autogroup.on_bi.bi_db_destroy = autogroup_db_destroy;
