@@ -34,6 +34,7 @@ class CPDF_StructElement final : public Retainable {
 
   size_t CountKids() const;
   CPDF_StructElement* GetKidIfElement(size_t index) const;
+  int GetKidContentId(size_t index) const;
   bool UpdateKidIfElement(const CPDF_Dictionary* pDict,
                           CPDF_StructElement* pElement);
 
@@ -62,10 +63,10 @@ class CPDF_StructElement final : public Retainable {
                      RetainPtr<const CPDF_Dictionary> pDict);
   ~CPDF_StructElement() override;
 
-  void LoadKids(RetainPtr<const CPDF_Dictionary> pDict);
-  void LoadKid(uint32_t PageObjNum,
+  void LoadKids();
+  void LoadKid(uint32_t page_obj_num,
                RetainPtr<const CPDF_Object> pKidObj,
-               Kid* pKid);
+               Kid& kid);
 
   UnownedPtr<const CPDF_StructTree> const m_pTree;
   RetainPtr<const CPDF_Dictionary> const m_pDict;
