@@ -10,6 +10,7 @@
 
 #include "core/fxcodec/cfx_codec_memory.h"
 #include "core/fxcrt/data_vector.h"
+#include "core/fxcrt/span_util.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace fxcodec {
@@ -24,9 +25,9 @@ class CFX_GifContextForTest final : public CFX_GifContext {
   using CFX_GifContext::ReadLogicalScreenDescriptor;
 
   CFX_CodecMemory* InputBuffer() const { return input_buffer_.Get(); }
-  void SetTestInputBuffer(pdfium::span<uint8_t> input) {
+  void SetTestInputBuffer(pdfium::span<const uint8_t> input) {
     auto pMemory = pdfium::MakeRetain<CFX_CodecMemory>(input.size());
-    memcpy(pMemory->GetBuffer(), input.data(), input.size());
+    fxcrt::spancpy(pMemory->GetBufferSpan(), input);
     SetInputBuffer(std::move(pMemory));
   }
 };
@@ -35,15 +36,15 @@ TEST(CFX_GifContext, SetInputBuffer) {
   uint8_t buffer[] = {0x00, 0x01, 0x02};
   CFX_GifContextForTest context;
 
-  context.SetTestInputBuffer({nullptr, 0});
+  context.SetTestInputBuffer({});
   EXPECT_EQ(0u, context.InputBuffer()->GetSize());
   EXPECT_EQ(0u, context.InputBuffer()->GetPosition());
 
-  context.SetTestInputBuffer({buffer, 0});
+  context.SetTestInputBuffer({buffer, 0u});
   EXPECT_EQ(0u, context.InputBuffer()->GetSize());
   EXPECT_EQ(0u, context.InputBuffer()->GetPosition());
 
-  context.SetTestInputBuffer({buffer, 3});
+  context.SetTestInputBuffer({buffer, 3u});
   EXPECT_EQ(3u, context.InputBuffer()->GetSize());
   EXPECT_EQ(0u, context.InputBuffer()->GetPosition());
 }
@@ -54,18 +55,18 @@ TEST(CFX_GifContext, ReadAllOrNone) {
                           0x05, 0x06, 0x07, 0x08, 0x09};
   CFX_GifContextForTest context;
 
-  context.SetTestInputBuffer({nullptr, 0});
+  context.SetTestInputBuffer({});
   EXPECT_FALSE(context.ReadAllOrNone(nullptr, 0));
   EXPECT_FALSE(context.ReadAllOrNone(nullptr, 10));
 
   EXPECT_FALSE(context.ReadAllOrNone(dest_buffer.data(), 0));
   EXPECT_FALSE(context.ReadAllOrNone(dest_buffer.data(), 10));
 
-  context.SetTestInputBuffer({src_buffer, 0});
+  context.SetTestInputBuffer({src_buffer, 0u});
   dest_buffer.resize(sizeof(src_buffer));
   EXPECT_FALSE(context.ReadAllOrNone(dest_buffer.data(), sizeof(src_buffer)));
 
-  context.SetTestInputBuffer({src_buffer, 1});
+  context.SetTestInputBuffer({src_buffer, 1u});
   EXPECT_FALSE(context.ReadAllOrNone(dest_buffer.data(), sizeof(src_buffer)));
   EXPECT_EQ(0u, context.InputBuffer()->GetPosition());
   EXPECT_FALSE(context.ReadAllOrNone(nullptr, sizeof(src_buffer)));
@@ -90,7 +91,7 @@ TEST(CFX_GifContext, ReadGifSignature) {
   CFX_GifContextForTest context;
   {
     uint8_t data[1];
-    context.SetTestInputBuffer({data, 0});
+    context.SetTestInputBuffer({data, 0u});
     EXPECT_EQ(GifDecoder::Status::kUnfinished, context.ReadGifSignature());
     EXPECT_EQ(0u, context.InputBuffer()->GetPosition());
     context.SetTestInputBuffer({});
@@ -148,7 +149,7 @@ TEST(CFX_GifContext, ReadLocalScreenDescriptor) {
   CFX_GifContextForTest context;
   {
     uint8_t data[1];
-    context.SetTestInputBuffer({data, 0});
+    context.SetTestInputBuffer({data, 0u});
     EXPECT_EQ(GifDecoder::Status::kUnfinished,
               context.ReadLogicalScreenDescriptor());
     context.SetTestInputBuffer({});
