@@ -8,13 +8,31 @@
 
 #include "core/fxcodec/cfx_codec_memory.h"
 #include "core/fxcodec/gif/gif_decoder.h"
+#include "third_party/base/check.h"
 
 namespace fxcodec {
 
+namespace {
+
+GifProgressiveDecoder* g_gif_decoder = nullptr;
+
+}  // namespace
+
+// static
+void GifProgressiveDecoder::InitializeGlobals() {
+  CHECK(!g_gif_decoder);
+  g_gif_decoder = new GifProgressiveDecoder();
+}
+
+// static
+void GifProgressiveDecoder::DestroyGlobals() {
+  delete g_gif_decoder;
+  g_gif_decoder = nullptr;
+}
+
 // static
 GifProgressiveDecoder* GifProgressiveDecoder::GetInstance() {
-  static pdfium::base::NoDestructor<GifProgressiveDecoder> s;
-  return s.get();
+  return g_gif_decoder;
 }
 
 GifProgressiveDecoder::GifProgressiveDecoder() = default;
