@@ -1,4 +1,4 @@
-// Copyright 2016 PDFium Authors. All rights reserved.
+// Copyright 2016 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,46 +7,38 @@
 #ifndef CORE_FXGE_CFX_GRAPHSTATEDATA_H_
 #define CORE_FXGE_CFX_GRAPHSTATEDATA_H_
 
+#include <stdint.h>
+
 #include <vector>
 
-#include "core/fxcrt/fx_system.h"
 #include "core/fxcrt/retain_ptr.h"
 
 class CFX_GraphStateData {
  public:
-  enum LineCap : uint8_t {
-    LineCapButt = 0,
-    LineCapRound = 1,
-    LineCapSquare = 2
-  };
+  enum class LineCap : uint8_t { kButt = 0, kRound = 1, kSquare = 2 };
 
-  enum LineJoin : uint8_t {
-    LineJoinMiter = 0,
-    LineJoinRound = 1,
-    LineJoinBevel = 2
-  };
+  enum class LineJoin : uint8_t { kMiter = 0, kRound = 1, kBevel = 2 };
 
   CFX_GraphStateData();
   CFX_GraphStateData(const CFX_GraphStateData& src);
-  CFX_GraphStateData(CFX_GraphStateData&& src);
+  CFX_GraphStateData(CFX_GraphStateData&& src) noexcept;
   ~CFX_GraphStateData();
 
   CFX_GraphStateData& operator=(const CFX_GraphStateData& that);
-  CFX_GraphStateData& operator=(CFX_GraphStateData&& that);
+  CFX_GraphStateData& operator=(CFX_GraphStateData&& that) noexcept;
 
-  LineCap m_LineCap = LineCapButt;
-  LineJoin m_LineJoin = LineJoinMiter;
+  LineCap m_LineCap = LineCap::kButt;
+  LineJoin m_LineJoin = LineJoin::kMiter;
   float m_DashPhase = 0.0f;
   float m_MiterLimit = 10.0f;
   float m_LineWidth = 1.0f;
   std::vector<float> m_DashArray;
 };
 
-class CFX_RetainableGraphStateData : public Retainable,
-                                     public CFX_GraphStateData {
+class CFX_RetainableGraphStateData final : public Retainable,
+                                           public CFX_GraphStateData {
  public:
-  template <typename T, typename... Args>
-  friend RetainPtr<T> pdfium::MakeRetain(Args&&... args);
+  CONSTRUCT_VIA_MAKE_RETAIN;
 
   RetainPtr<CFX_RetainableGraphStateData> Clone() const;
 
