@@ -1,4 +1,4 @@
-// Copyright 2016 PDFium Authors. All rights reserved.
+// Copyright 2016 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -31,8 +31,8 @@ class CFX_Path {
     }
 
     CFX_PointF m_Point;
-    Type m_Type;
-    bool m_CloseFigure;
+    Type m_Type = Type::kLine;
+    bool m_CloseFigure = false;
   };
 
   CFX_Path();
