@@ -1,4 +1,4 @@
-// Copyright 2019 PDFium Authors. All rights reserved.
+// Copyright 2019 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -16,17 +16,19 @@ class TextCharPos {
   TextCharPos(const TextCharPos&);
   ~TextCharPos();
 
+  CFX_Matrix GetEffectiveMatrix(const CFX_Matrix& matrix) const;
+
   CFX_PointF m_Origin;
   uint32_t m_Unicode = 0;
   uint32_t m_GlyphIndex = 0;
   int m_FontCharWidth = 0;
-#if defined(OS_APPLE)
+#if BUILDFLAG(IS_APPLE)
   uint32_t m_ExtGID = 0;
 #endif
   int32_t m_FallbackFontPosition = 0;
   bool m_bGlyphAdjust = false;
   bool m_bFontStyle = false;
-  float m_AdjustMatrix[4];
+  float m_AdjustMatrix[4] = {};
 };
 
 #endif  // CORE_FXGE_TEXT_CHAR_POS_H_
