@@ -8,7 +8,7 @@
 
 #include "BitMatrix.h"
 #include "ODEAN13Writer.h"
-#include "TextUtfEncoding.h"
+#include "Utf.h"
 
 #include <stdexcept>
 
@@ -27,7 +27,7 @@ UPCAWriter::encode(const std::wstring& contents, int width, int height) const
 
 BitMatrix UPCAWriter::encode(const std::string& contents, int width, int height) const
 {
-	return encode(TextUtfEncoding::FromUtf8(contents), width, height);
+	return encode(FromUtf8(contents), width, height);
 }
 
 } // namespace ZXing::OneD
