@@ -6,6 +6,7 @@
  * Copyright (C) 2017-2019 Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2018 Suzuki Toshiya <mpsuzuki@hiroshima-u.ac.jp>
  * Copyright (C) 2020 Adam Reichold <adam.reichold@t-online.de>
+ * Copyright (C) 2024 Oliver Sander <oliver.sander@tu-dresden.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +27,7 @@
 
 #include "GooString.h"
 #include "Page.h"
+#include "UTF.h"
 
 #include <ctime>
 #include <iostream>
@@ -63,8 +65,8 @@ ustring detail::unicode_GooString_to_ustring(const GooString *str)
     const char *data = str->c_str();
     const int len = str->getLength();
 
-    const bool is_unicodeLE = str->hasUnicodeMarkerLE();
-    const bool is_unicode = str->hasUnicodeMarker() || is_unicodeLE;
+    const bool is_unicodeLE = hasUnicodeByteOrderMarkLE(str->toStr());
+    const bool is_unicode = hasUnicodeByteOrderMark(str->toStr()) || is_unicodeLE;
     int i = is_unicode ? 2 : 0;
     ustring::size_type ret_len = len - i;
     if (is_unicode) {
