@@ -11,20 +11,20 @@ void draw(SkCanvas* canvas) {
          canvas->drawString("GPU only!", 20, 40, font, paint);
          return;
     }
-    sk_sp<SkSurface> gpuSurface = SkSurface::MakeRenderTarget(
+    sk_sp<SkSurface> gpuSurface = SkSurfaces::RenderTarget(
             context, skgpu::Budgeted::kYes, SkImageInfo::MakeN32Premul(64, 64));
-    SkSurfaceCharacterization characterization;
+    GrSurfaceCharacterization characterization;
     if (!gpuSurface->characterize(&characterization)) {
          canvas->drawString("characterization unsupported", 20, 40, font, paint);
          return;
     }
     // start of threadable work
-    SkDeferredDisplayListRecorder recorder(characterization);
+    GrDeferredDisplayListRecorder recorder(characterization);
     SkCanvas* subCanvas = recorder.getCanvas();
     subCanvas->clear(SK_ColorGREEN);
-    sk_sp<SkDeferredDisplayList> displayList = recorder.detach();
+    sk_sp<GrDeferredDisplayList> displayList = recorder.detach();
     // end of threadable work
-    gpuSurface->draw(displayList);
+    skgpu::ganesh::DrawDDL(gpuSurface, displayList);
     sk_sp<SkImage> img = gpuSurface->makeImageSnapshot();
     canvas->drawImage(std::move(img), 0, 0);
 }
