@@ -1,4 +1,4 @@
-// Copyright 2014 PDFium Authors. All rights reserved.
+// Copyright 2014 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -17,14 +17,17 @@
 #include "core/fpdfapi/parser/cpdf_array.h"
 #include "core/fpdfapi/parser/cpdf_dictionary.h"
 #include "core/fpdfapi/parser/cpdf_name.h"
+#include "core/fpdfapi/parser/cpdf_stream.h"
 #include "core/fpdfapi/parser/cpdf_stream_acc.h"
 #include "core/fpdfapi/render/cpdf_imagerenderer.h"
 #include "core/fpdfapi/render/cpdf_rendercontext.h"
 #include "core/fpdfapi/render/cpdf_renderstatus.h"
 #include "core/fxcrt/stl_util.h"
 #include "core/fxge/cfx_defaultrenderdevice.h"
+#include "core/fxge/dib/cfx_dibitmap.h"
 #include "fpdfsdk/cpdfsdk_customaccess.h"
 #include "fpdfsdk/cpdfsdk_helpers.h"
+#include "third_party/base/notreached.h"
 
 namespace {
 
@@ -97,9 +100,10 @@ bool LoadJpegHelper(FPDF_PAGE* pages,
 
   RetainPtr<IFX_SeekableReadStream> pFile = MakeSeekableReadStream(file_access);
   if (inline_jpeg)
-    pImgObj->GetImage()->SetJpegImageInline(pFile);
+    pImgObj->GetImage()->SetJpegImageInline(std::move(pFile));
   else
-    pImgObj->GetImage()->SetJpegImage(pFile);
+    pImgObj->GetImage()->SetJpegImage(std::move(pFile));
+
   pImgObj->SetDirty(true);
   return true;
 }
@@ -195,13 +199,70 @@ FPDFImageObj_GetBitmap(FPDF_PAGEOBJECT image_object) {
   if (!pSource)
     return nullptr;
 
-  // If the source image has a representation of 1 bit per pixel, then convert
-  // it to a grayscale bitmap having 1 byte per pixel, since bitmaps have no
-  // concept of bits. Otherwise, convert the source image to a bitmap directly,
+  // If the source image has a representation of 1 bit per pixel, or if the
+  // source image has a color palette, convert it to a representation that does
+  // not have a color palette, as there is no public API to access the palette.
+  //
+  // Otherwise, convert the source image to a bitmap directly,
   // retaining its color representation.
-  RetainPtr<CFX_DIBitmap> pBitmap =
-      pSource->GetBPP() == 1 ? pSource->ConvertTo(FXDIB_Format::k8bppRgb)
-                             : pSource->Realize();
+  //
+  // Only return FPDF_BITMAPs in formats that FPDFBitmap_CreateEx() would
+  // return.
+  enum class ConversionOp {
+    kRealize,
+    kConvertTo8bppRgb,
+    kConvertToRgb,
+  };
+
+  ConversionOp op;
+  switch (pSource->GetFormat()) {
+    case FXDIB_Format::k1bppMask:
+    case FXDIB_Format::k8bppMask:
+      // Masks do not have palettes, so they can be safely converted to
+      // `FXDIB_Format::k8bppRgb`.
+      CHECK(!pSource->HasPalette());
+      op = ConversionOp::kConvertTo8bppRgb;
+      break;
+    case FXDIB_Format::k1bppRgb:
+      // If there is a palette, then convert to `FXDIB_Format::kRgb` to avoid
+      // creating a bitmap with a palette.
+      op = pSource->HasPalette() ? ConversionOp::kConvertToRgb
+                                 : ConversionOp::kConvertTo8bppRgb;
+      break;
+    case FXDIB_Format::k8bppRgb:
+      // If there is a palette, then convert to `FXDIB_Format::kRgb` to avoid
+      // creating a bitmap with a palette.
+      op = pSource->HasPalette() ? ConversionOp::kConvertToRgb
+                                 : ConversionOp::kRealize;
+      break;
+
+    case FXDIB_Format::kArgb:
+    case FXDIB_Format::kRgb:
+    case FXDIB_Format::kRgb32:
+      CHECK(!pSource->HasPalette());
+      op = ConversionOp::kRealize;
+      break;
+
+    case FXDIB_Format::kInvalid: {
+      NOTREACHED_NORETURN();
+    }
+  }
+
+  RetainPtr<CFX_DIBitmap> pBitmap;
+  switch (op) {
+    case ConversionOp::kRealize:
+      pBitmap = pSource->Realize();
+      break;
+    case ConversionOp::kConvertTo8bppRgb:
+      pBitmap = pSource->ConvertTo(FXDIB_Format::k8bppRgb);
+      break;
+    case ConversionOp::kConvertToRgb:
+      pBitmap = pSource->ConvertTo(FXDIB_Format::kRgb);
+      break;
+  }
+  if (pBitmap) {
+    CHECK(!pBitmap->HasPalette());
+  }
 
   return FPDFBitmapFromCFXDIBitmap(pBitmap.Leak());
 }
@@ -231,14 +292,14 @@ FPDFImageObj_GetRenderedBitmap(FPDF_DOCUMENT document,
     return nullptr;
 
   // Set up all the rendering code.
-  CPDF_Dictionary* page_resources =
-      optional_page ? optional_page->GetPageResources() : nullptr;
-  CPDF_RenderContext context(doc, page_resources, /*pPageCache=*/nullptr);
+  RetainPtr<CPDF_Dictionary> page_resources =
+      optional_page ? optional_page->GetMutablePageResources() : nullptr;
+  CPDF_RenderContext context(doc, std::move(page_resources),
+                             /*pPageCache=*/nullptr);
   CFX_DefaultRenderDevice device;
-  device.Attach(result_bitmap, /*bRgbByteOrder=*/false,
-                /*pBackdropBitmap=*/nullptr, /*bGroupKnockout=*/false);
+  device.Attach(result_bitmap);
   CPDF_RenderStatus status(&context, &device);
-  CPDF_ImageRenderer renderer;
+  CPDF_ImageRenderer renderer(&status);
 
   // Need to first flip the image, as expected by |renderer|.
   CFX_Matrix render_matrix(1, 0, 0, -1, 0, output_height);
@@ -247,7 +308,7 @@ FPDFImageObj_GetRenderedBitmap(FPDF_DOCUMENT document,
   render_matrix.Translate(-image_matrix.e, image_matrix.f);
 
   // Do the actual rendering.
-  bool should_continue = renderer.Start(&status, image, render_matrix,
+  bool should_continue = renderer.Start(image, render_matrix,
                                         /*bStdCS=*/false, BlendMode::kNormal);
   while (should_continue)
     should_continue = renderer.Continue(/*pPause=*/nullptr);
@@ -255,6 +316,12 @@ FPDFImageObj_GetRenderedBitmap(FPDF_DOCUMENT document,
   if (!renderer.GetResult())
     return nullptr;
 
+#if defined(_SKIA_SUPPORT_)
+  if (CFX_DefaultRenderDevice::UseSkiaRenderer()) {
+    result_bitmap->UnPreMultiply();
+  }
+#endif
+
   // Caller takes ownership.
   return FPDFBitmapFromCFXDIBitmap(result_bitmap.Leak());
 }
@@ -271,11 +338,13 @@ FPDFImageObj_GetImageDataDecoded(FPDF_PAGEOBJECT image_object,
   if (!pImg)
     return 0;
 
-  CPDF_Stream* pImgStream = pImg->GetStream();
+  RetainPtr<const CPDF_Stream> pImgStream = pImg->GetStream();
   if (!pImgStream)
     return 0;
 
-  return DecodeStreamMaybeCopyAndReturnLength(pImgStream, buffer, buflen);
+  return DecodeStreamMaybeCopyAndReturnLength(
+      std::move(pImgStream),
+      {static_cast<uint8_t*>(buffer), static_cast<size_t>(buflen)});
 }
 
 FPDF_EXPORT unsigned long FPDF_CALLCONV
@@ -290,11 +359,13 @@ FPDFImageObj_GetImageDataRaw(FPDF_PAGEOBJECT image_object,
   if (!pImg)
     return 0;
 
-  CPDF_Stream* pImgStream = pImg->GetStream();
+  RetainPtr<const CPDF_Stream> pImgStream = pImg->GetStream();
   if (!pImgStream)
     return 0;
 
-  return GetRawStreamMaybeCopyAndReturnLength(pImgStream, buffer, buflen);
+  return GetRawStreamMaybeCopyAndReturnLength(
+      std::move(pImgStream),
+      {static_cast<uint8_t*>(buffer), static_cast<size_t>(buflen)});
 }
 
 FPDF_EXPORT int FPDF_CALLCONV
@@ -307,8 +378,11 @@ FPDFImageObj_GetImageFilterCount(FPDF_PAGEOBJECT image_object) {
   if (!pImg)
     return 0;
 
-  CPDF_Dictionary* pDict = pImg->GetDict();
-  CPDF_Object* pFilter = pDict ? pDict->GetDirectObjectFor("Filter") : nullptr;
+  RetainPtr<const CPDF_Dictionary> pDict = pImg->GetDict();
+  if (!pDict)
+    return 0;
+
+  RetainPtr<const CPDF_Object> pFilter = pDict->GetDirectObjectFor("Filter");
   if (!pFilter)
     return 0;
 
@@ -330,13 +404,12 @@ FPDFImageObj_GetImageFilter(FPDF_PAGEOBJECT image_object,
     return 0;
 
   CPDF_PageObject* pObj = CPDFPageObjectFromFPDFPageObject(image_object);
-  CPDF_Object* pFilter =
-      pObj->AsImage()->GetImage()->GetDict()->GetDirectObjectFor("Filter");
-  ByteString bsFilter;
-  if (pFilter->IsName())
-    bsFilter = pFilter->AsName()->GetString();
-  else
-    bsFilter = pFilter->AsArray()->GetStringAt(index);
+  RetainPtr<const CPDF_Dictionary> pDict =
+      pObj->AsImage()->GetImage()->GetDict();
+  RetainPtr<const CPDF_Object> pFilter = pDict->GetDirectObjectFor("Filter");
+  ByteString bsFilter = pFilter->IsName()
+                            ? pFilter->AsName()->GetString()
+                            : pFilter->AsArray()->GetByteStringAt(index);
 
   return NulTerminateMaybeCopyAndReturnLength(bsFilter, buffer, buflen);
 }
@@ -376,11 +449,14 @@ FPDFImageObj_GetImageMetadata(FPDF_PAGEOBJECT image_object,
   if (!pPage || !pPage->GetDocument() || !pImg->GetStream())
     return true;
 
-  auto pSource =
-      pdfium::MakeRetain<CPDF_DIB>(pPage->GetDocument(), pImg->GetStream());
+  // A cross-document image may have come from the embedder.
+  if (pPage->GetDocument() != pImg->GetDocument())
+    return false;
+
+  RetainPtr<CPDF_DIB> pSource = pImg->CreateNewDIB();
   CPDF_DIB::LoadState ret = pSource->StartLoadDIBBase(
-      false, nullptr, pPage->GetPageResources(), false,
-      CPDF_ColorSpace::Family::kUnknown, false);
+      false, nullptr, pPage->GetPageResources().Get(), false,
+      CPDF_ColorSpace::Family::kUnknown, false, {0, 0});
   if (ret == CPDF_DIB::LoadState::kFail)
     return true;
 
@@ -391,3 +467,22 @@ FPDFImageObj_GetImageMetadata(FPDF_PAGEOBJECT image_object,
   }
   return true;
 }
+
+FPDF_EXPORT FPDF_BOOL FPDF_CALLCONV
+FPDFImageObj_GetImagePixelSize(FPDF_PAGEOBJECT image_object,
+                               unsigned int* width,
+                               unsigned int* height) {
+  CPDF_ImageObject* pImgObj = CPDFImageObjectFromFPDFPageObject(image_object);
+  if (!pImgObj || !width || !height) {
+    return false;
+  }
+
+  RetainPtr<CPDF_Image> pImg = pImgObj->GetImage();
+  if (!pImg) {
+    return false;
+  }
+
+  *width = pImg->GetPixelWidth();
+  *height = pImg->GetPixelHeight();
+  return true;
+}
