@@ -12,8 +12,9 @@ class FPDFEditPageEmbedderTest : public EmbedderTest {};
 
 TEST_F(FPDFEditPageEmbedderTest, Rotation) {
   const char* rotated_checksum = []() {
-    if (CFX_DefaultRenderDevice::SkiaIsDefaultRenderer())
+    if (CFX_DefaultRenderDevice::UseSkiaRenderer()) {
       return "eded83f75f3d0332c584c416c571c0df";
+    }
     return "d599429574ff0dcad3bc898ea8b874ca";
   }();
 
