@@ -10,6 +10,9 @@
 void fuzz_Gradients(Fuzz* f);
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
+    if (size > 4000) {
+        return 0;
+    }
     auto fuzz = Fuzz(SkData::MakeWithoutCopy(data, size));
     fuzz_Gradients(&fuzz);
     return 0;
