@@ -10,10 +10,12 @@
 void fuzz_PolyUtils(Fuzz* f);
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t *data, size_t size) {
+#if !defined(SK_ENABLE_OPTIMIZE_SIZE)
     if (size > 4000) {
         return 0;
     }
     auto fuzz = Fuzz(SkData::MakeWithoutCopy(data, size));
     fuzz_PolyUtils(&fuzz);
+#endif
     return 0;
 }
