@@ -1,4 +1,4 @@
-// Copyright 2014 PDFium Authors. All rights reserved.
+// Copyright 2014 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,7 +9,7 @@
 
 #include <memory>
 
-#include "third_party/base/optional.h"
+#include "third_party/abseil-cpp/absl/types/optional.h"
 
 class CBC_ReedSolomonGF256Poly;
 
@@ -18,22 +18,17 @@ class CBC_ReedSolomonGF256 {
   explicit CBC_ReedSolomonGF256(int32_t primitive);
   ~CBC_ReedSolomonGF256();
 
-  static void Initialize();
-  static void Finalize();
+  CBC_ReedSolomonGF256Poly* GetZero() const { return m_zero.get(); }
+  CBC_ReedSolomonGF256Poly* GetOne() const { return m_one.get(); }
 
-  CBC_ReedSolomonGF256Poly* GetZero() const;
-  CBC_ReedSolomonGF256Poly* GetOne() const;
   std::unique_ptr<CBC_ReedSolomonGF256Poly> BuildMonomial(int32_t degree,
                                                           int32_t coefficient);
   static int32_t AddOrSubtract(int32_t a, int32_t b);
   int32_t Exp(int32_t a);
-  Optional<int32_t> Inverse(int32_t a);
+  absl::optional<int32_t> Inverse(int32_t a);
   int32_t Multiply(int32_t a, int32_t b);
   void Init();
 
-  static CBC_ReedSolomonGF256* QRCodeField;
-  static CBC_ReedSolomonGF256* DataMatrixField;
-
  private:
   std::unique_ptr<CBC_ReedSolomonGF256Poly> m_zero;
   std::unique_ptr<CBC_ReedSolomonGF256Poly> m_one;
