@@ -1,4 +1,4 @@
-// Copyright 2014 PDFium Authors. All rights reserved.
+// Copyright 2014 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -22,6 +22,8 @@
 
 #include "fxbarcode/qrcode/BC_QRCoderMatrixUtil.h"
 
+#include <iterator>
+
 #include "fxbarcode/common/BC_CommonByteMatrix.h"
 #include "fxbarcode/qrcode/BC_QRCoder.h"
 #include "fxbarcode/qrcode/BC_QRCoderBitVector.h"
@@ -29,7 +31,6 @@
 #include "fxbarcode/qrcode/BC_QRCoderMaskUtil.h"
 #include "third_party/base/check.h"
 #include "third_party/base/check_op.h"
-#include "third_party/base/cxx17_backports.h"
 
 namespace {
 
@@ -106,7 +107,7 @@ bool EmbedDataBits(CBC_QRCoderBitVector* dataBits,
     if (x == 6)
       x -= 1;
 
-    while (y >= 0 && y < matrix->GetHeight()) {
+    while (y >= 0 && y < static_cast<int32_t>(matrix->GetHeight())) {
       if (y == 6) {
         y += direction;
         continue;
@@ -215,8 +216,8 @@ void MaybeEmbedVersionInfo(int32_t version, CBC_CommonByteMatrix* matrix) {
 }
 
 bool EmbedTimingPatterns(CBC_CommonByteMatrix* matrix) {
-  for (int32_t i = 8; i < matrix->GetWidth() - 8; i++) {
-    int32_t bit = (i + 1) % 2;
+  for (size_t i = 8; i + 8 < matrix->GetWidth(); i++) {
+    const uint8_t bit = static_cast<uint8_t>((i + 1) % 2);
     if (!IsValidValue(matrix->Get(i, 6)))
       return false;
 
@@ -333,7 +334,7 @@ bool MaybeEmbedPositionAdjustmentPatterns(int32_t version,
     return true;
 
   const size_t index = version - 2;
-  if (index >= pdfium::size(kPositionAdjustmentPatternCoordinates))
+  if (index >= std::size(kPositionAdjustmentPatternCoordinates))
     return false;
 
   const auto* coordinates = &kPositionAdjustmentPatternCoordinates[index][0];
@@ -378,7 +379,7 @@ bool CBC_QRCoderMatrixUtil::BuildMatrix(
   if (!dataBits || !matrix)
     return false;
 
-  matrix->clear(0xff);
+  matrix->Fill(0xff);
 
   if (!EmbedBasicPatterns(version, matrix))
     return false;
