@@ -15,22 +15,29 @@ namespace fxjs {
 // For DOM/XML-ish trees, where references outside the tree are Persistent<>.
 template <typename T>
 class GCedTreeNode : public cppgc::GarbageCollected<GCedTreeNode<T>>,
-                     public TreeNode<T, cppgc::Member<T>> {
+                     public fxcrt::TreeNodeBase<T> {
  public:
-  using TreeNode<T, cppgc::Member<T>>::RemoveChild;
-
   virtual void Trace(cppgc::Visitor* visitor) const {
-    visitor->Trace(TreeNode<T, cppgc::Member<T>>::RawParent());
-    visitor->Trace(TreeNode<T, cppgc::Member<T>>::RawFirstChild());
-    visitor->Trace(TreeNode<T, cppgc::Member<T>>::RawLastChild());
-    visitor->Trace(TreeNode<T, cppgc::Member<T>>::RawNextSibling());
-    visitor->Trace(TreeNode<T, cppgc::Member<T>>::RawPrevSibling());
+    visitor->Trace(m_pParent);
+    visitor->Trace(m_pFirstChild);
+    visitor->Trace(m_pLastChild);
+    visitor->Trace(m_pNextSibling);
+    visitor->Trace(m_pPrevSibling);
   }
 
  protected:
   GCedTreeNode() = default;
   GCedTreeNode(const GCedTreeNode& that) = delete;
   GCedTreeNode& operator=(const GCedTreeNode& that) = delete;
+
+ private:
+  friend class fxcrt::TreeNodeBase<T>;
+
+  cppgc::Member<T> m_pParent;
+  cppgc::Member<T> m_pFirstChild;
+  cppgc::Member<T> m_pLastChild;
+  cppgc::Member<T> m_pNextSibling;
+  cppgc::Member<T> m_pPrevSibling;
 };
 
 }  // namespace fxjs
