@@ -18,9 +18,14 @@
 #include "include/core/SkTypes.h"
 #include "include/gpu/GrDirectContext.h"
 #include "include/gpu/GrRecordingContext.h"
-#include "include/utils/SkRandom.h"
+#include "include/gpu/ganesh/SkSurfaceGanesh.h"
+#include "src/base/SkRandom.h"
 #include "tools/ToolUtils.h"
 
+#if defined(SK_GRAPHITE)
+#include "include/gpu/graphite/Surface.h"
+#endif
+
 namespace skiagm {
 
 /*
@@ -42,20 +47,30 @@ protected:
     }
 
     DrawResult onDraw(SkCanvas* canvas, SkString* errorMsg) override {
-        auto dContext = GrAsDirectContext(canvas->recordingContext());
-        if (!dContext || dContext->abandoned()) {
-            *errorMsg = "GM relies on having access to a live direct context.";
-            return DrawResult::kSkip;
-        }
 
         SkISize size = this->getISize();
         size.fWidth /= 10;
         size.fHeight /= 10;
         SkImageInfo info = SkImageInfo::MakeN32Premul(size);
-        sk_sp<SkSurface> surface =
-                SkSurface::MakeRenderTarget(dContext, skgpu::Budgeted::kNo, info);
-        if (nullptr == surface) {
-            *errorMsg = "Could not create render target.";
+        sk_sp<SkSurface> surface;
+
+        auto dContext = GrAsDirectContext(canvas->recordingContext());
+        if (dContext && !dContext->abandoned()) {
+            surface = SkSurfaces::RenderTarget(dContext, skgpu::Budgeted::kNo, info);
+        }
+
+#if defined(SK_GRAPHITE)
+        auto recorder = canvas->recorder();
+        if (recorder) {
+            surface = SkSurfaces::RenderTarget(recorder, info);
+        }
+#endif
+
+        if (!surface) {
+            surface = SkSurfaces::Raster(info);
+        }
+        if (!surface) {
+            *errorMsg = "Could not create surface.";
             return DrawResult::kFail;
         }
 
@@ -87,9 +102,6 @@ protected:
         surface->getCanvas()->discard();
         return DrawResult::kOk;
     }
-
-private:
-    using INHERITED = GM;
 };
 
 //////////////////////////////////////////////////////////////////////////////
