@@ -28,17 +28,26 @@
 #include "include/gpu/GrDirectContext.h"
 #include "include/gpu/GrRecordingContext.h"
 #include "include/gpu/GrTypes.h"
+#include "include/gpu/ganesh/GrExternalTextureGenerator.h"
+#include "include/gpu/ganesh/SkSurfaceGanesh.h"
+#include "include/private/gpu/ganesh/GrTextureGenerator.h"
 #include "include/private/gpu/ganesh/GrTypesPriv.h"
 #include "src/gpu/ganesh/GrRecordingContextPriv.h"
 #include "src/gpu/ganesh/GrSamplerState.h"
 #include "src/gpu/ganesh/GrTextureProxy.h"
 #include "src/gpu/ganesh/SurfaceContext.h"
+#include "src/gpu/ganesh/image/GrImageUtils.h"
+#include "src/gpu/ganesh/image/SkImage_Ganesh.h"
+#include "src/image/SkImageGeneratorPriv.h"
 #include "src/image/SkImage_Base.h"
-#include "src/image/SkImage_Gpu.h"
 
 #include <memory>
 #include <utility>
 
+#if defined(SK_GRAPHITE)
+#include "include/gpu/graphite/Surface.h"
+#endif
+
 class GrRecordingContext;
 
 static void draw_something(SkCanvas* canvas, const SkRect& bounds) {
@@ -85,13 +94,13 @@ protected:
 
         SkMatrix matrix;
         matrix.setTranslate(-100, -100);
-        fImage0 = SkImage::MakeFromPicture(fPicture, size, &matrix, nullptr,
-                                           SkImage::BitDepth::kU8, srgbColorSpace);
+        fImage0 = SkImages::DeferredFromPicture(
+                fPicture, size, &matrix, nullptr, SkImages::BitDepth::kU8, srgbColorSpace);
         matrix.postTranslate(-50, -50);
         matrix.postRotate(45);
         matrix.postTranslate(50, 50);
-        fImage1 = SkImage::MakeFromPicture(fPicture, size, &matrix, nullptr,
-                                           SkImage::BitDepth::kU8, srgbColorSpace);
+        fImage1 = SkImages::DeferredFromPicture(
+                fPicture, size, &matrix, nullptr, SkImages::BitDepth::kU8, srgbColorSpace);
     }
 
     void drawSet(SkCanvas* canvas) const {
@@ -130,9 +139,12 @@ static std::unique_ptr<SkImageGenerator> make_pic_generator(SkCanvas*,
                                                             sk_sp<SkPicture> pic) {
     SkMatrix matrix;
     matrix.setTranslate(-100, -100);
-    return SkImageGenerator::MakeFromPicture({ 100, 100 }, std::move(pic), &matrix, nullptr,
-                                            SkImage::BitDepth::kU8,
-                                            SkColorSpace::MakeSRGB());
+    return SkImageGenerators::MakeFromPicture({100, 100},
+                                              std::move(pic),
+                                              &matrix,
+                                              nullptr,
+                                              SkImages::BitDepth::kU8,
+                                              SkColorSpace::MakeSRGB());
 }
 
 class RasterGenerator : public SkImageGenerator {
@@ -161,22 +173,26 @@ static std::unique_ptr<SkImageGenerator> make_ras_generator(SkCanvas*,
     return std::make_unique<RasterGenerator>(bm);
 }
 
-class TextureGenerator : public SkImageGenerator {
+class TextureGenerator : public GrTextureGenerator {
 public:
     TextureGenerator(SkCanvas* canvas, const SkImageInfo& info, sk_sp<SkPicture> pic)
-            : SkImageGenerator(info) {
+            : GrTextureGenerator(info) {
 
         fRContext = sk_ref_sp(canvas->recordingContext());
 
         sk_sp<SkSurface> surface;
 
         if (fRContext) {
-            surface = SkSurface::MakeRenderTarget(fRContext.get(), skgpu::Budgeted::kYes, info,
-                                                  0, kTopLeft_GrSurfaceOrigin, nullptr);
+            surface = SkSurfaces::RenderTarget(fRContext.get(),
+                                               skgpu::Budgeted::kYes,
+                                               info,
+                                               0,
+                                               kTopLeft_GrSurfaceOrigin,
+                                               nullptr);
         }
-#ifdef SK_GRAPHITE_ENABLED
+#if defined(SK_GRAPHITE)
         if (skgpu::graphite::Recorder* recorder = canvas->recorder()) {
-            surface = SkSurface::MakeGraphite(recorder, info);
+            surface = SkSurfaces::RenderTarget(recorder, info);
         }
 #endif
 
@@ -195,9 +211,7 @@ protected:
         SkASSERT(rContext);
         SkASSERT(rContext->priv().matches(fRContext.get()));
 
-        GrSurfaceProxyView view;
-
-        std::tie(view, std::ignore) = as_IB(fImage)->asView(rContext, GrMipmapped::kNo);
+        auto [view, _] = skgpu::ganesh::AsView(rContext, fImage, GrMipmapped::kNo);
         if (!view) {
             return {};
         }
@@ -220,14 +234,6 @@ protected:
                 /*label=*/"SurfaceProxyView_GenerateTexture");
     }
 
-#ifdef SK_GRAPHITE_ENABLED
-    sk_sp<SkImage> onMakeTextureImage(skgpu::graphite::Recorder*,
-                                      const SkImageInfo&,
-                                      skgpu::graphite::Mipmapped) override {
-        return fImage;
-    }
-#endif
-
 private:
     sk_sp<GrRecordingContext> fRContext;
     sk_sp<SkImage>            fImage;
@@ -253,9 +259,11 @@ class ImageCacheratorGM : public skiagm::GM {
     sk_sp<SkPicture> fPicture;
     sk_sp<SkImage>   fImage;
     sk_sp<SkImage>   fImageSubset;
+    bool             fUseTexture;
 
 public:
-    ImageCacheratorGM(const char suffix[], FactoryFunc factory) : fFactory(factory) {
+    ImageCacheratorGM(const char suffix[], FactoryFunc factory, bool useTexture) :
+                    fFactory(factory), fUseTexture(useTexture) {
         fName.printf("image-cacherator-from-%s", suffix);
     }
 
@@ -283,7 +291,13 @@ protected:
             if (!gen) {
                 return false;
             }
-            fImage = SkImage::MakeFromGenerator(std::move(gen));
+            if (fUseTexture) {
+                auto textureGen = std::unique_ptr<GrTextureGenerator>(
+                        static_cast<GrTextureGenerator*>(gen.release()));
+                fImage = SkImages::DeferredFromTextureGenerator(std::move(textureGen));
+            } else {
+                fImage = SkImages::DeferredFromGenerator(std::move(gen));
+            }
             if (!fImage) {
                 return false;
             }
@@ -301,13 +315,20 @@ protected:
             }
 
             if (dContext) {
-                fImageSubset = SkImage::MakeFromGenerator(std::move(gen))->makeSubset(subset,
-                                                                                      dContext);
+                 if (fUseTexture) {
+                    auto textureGen = std::unique_ptr<GrTextureGenerator>(
+                        static_cast<GrTextureGenerator*>(gen.release()));
+                    fImageSubset = SkImages::DeferredFromTextureGenerator(std::move(textureGen))
+                            ->makeSubset(dContext, subset);
+                } else {
+                    fImageSubset = SkImages::DeferredFromGenerator(std::move(gen))
+                            ->makeSubset(dContext, subset);
+                }
             } else {
-#ifdef SK_GRAPHITE_ENABLED
+#if defined(SK_GRAPHITE)
                 auto recorder = canvas->recorder();
-                fImageSubset = SkImage::MakeFromGenerator(std::move(gen))->makeSubset(subset,
-                                                                                      recorder);
+                fImageSubset = SkImages::DeferredFromGenerator(std::move(gen))
+                                       ->makeSubset(recorder, subset, {});
 #endif
             }
             if (!fImageSubset) {
@@ -342,7 +363,8 @@ protected:
         if (as_IB(image)->isGaneshBacked()) {
             // The gpu-backed images are drawn in this manner bc the generator backed images
             // aren't considered texture-backed
-            auto [view, ct] = as_IB(image)->asView(canvas->recordingContext(), GrMipmapped::kNo);
+            auto [view, ct] =
+                    skgpu::ganesh::AsView(canvas->recordingContext(), image, GrMipmapped::kNo);
             if (!view) {
                 // show placeholder if we have no texture
                 draw_placeholder(canvas, x, y, image->width(), image->height());
@@ -352,10 +374,10 @@ protected:
                                   image->alphaType(),
                                   image->refColorSpace());
             // No API to draw a GrTexture directly, so we cheat and create a private image subclass
-            sk_sp<SkImage> texImage(new SkImage_Gpu(sk_ref_sp(canvas->recordingContext()),
-                                                    image->uniqueID(),
-                                                    std::move(view),
-                                                    std::move(colorInfo)));
+            sk_sp<SkImage> texImage(new SkImage_Ganesh(sk_ref_sp(canvas->recordingContext()),
+                                                       image->uniqueID(),
+                                                       std::move(view),
+                                                       std::move(colorInfo)));
             canvas->drawImage(texImage.get(), x, y);
         } else {
             canvas->drawImage(image, x, y);
@@ -407,6 +429,6 @@ private:
     using INHERITED = skiagm::GM;
 };
 
-DEF_GM( return new ImageCacheratorGM("picture", make_pic_generator); )
-DEF_GM( return new ImageCacheratorGM("raster", make_ras_generator); )
-DEF_GM( return new ImageCacheratorGM("texture", make_tex_generator); )
+DEF_GM( return new ImageCacheratorGM("picture", make_pic_generator, false); )
+DEF_GM( return new ImageCacheratorGM("raster", make_ras_generator, false); )
+DEF_GM( return new ImageCacheratorGM("texture", make_tex_generator, true); )
