@@ -13,6 +13,7 @@
 #include "include/core/SkRect.h"
 #include "include/core/SkTypes.h"
 #include "src/core/SkCanvasPriv.h"
+#include "src/gpu/ganesh/GrCanvas.h"
 #include "src/gpu/ganesh/GrFragmentProcessor.h"
 #include "src/gpu/ganesh/SkGr.h"
 #include "src/gpu/ganesh/SurfaceFillContext.h"
@@ -20,7 +21,7 @@
 #include "tools/Resources.h"
 
 DEF_SIMPLE_GPU_GM(swizzle, rContext, canvas, 512, 512) {
-    auto sfc = SkCanvasPriv::TopDeviceSurfaceFillContext(canvas);
+    auto sfc = skgpu::ganesh::TopDeviceSurfaceFillContext(canvas);
     if (!sfc) {
         return;
     }
