@@ -23,6 +23,7 @@
 #include "include/core/SkString.h"
 #include "include/core/SkSurface.h"
 #include "include/core/SkTileMode.h"
+#include "include/core/SkTiledImageUtils.h"
 #include "include/effects/SkGradientShader.h"
 #include "tools/ToolUtils.h"
 
@@ -37,7 +38,7 @@ static void draw(SkCanvas* canvas, int width, int height, SkColor colors[2]) {
 }
 
 static sk_sp<SkImage> make_raster_image(int width, int height, SkColor colors[2]) {
-    auto surface(SkSurface::MakeRasterN32Premul(width, height));
+    auto surface(SkSurfaces::Raster(SkImageInfo::MakeN32Premul(width, height)));
     draw(surface->getCanvas(), width, height, colors);
     return surface->makeImageSnapshot();
 }
@@ -45,16 +46,18 @@ static sk_sp<SkImage> make_raster_image(int width, int height, SkColor colors[2]
 static sk_sp<SkImage> make_picture_image(int width, int height, SkColor colors[2]) {
     SkPictureRecorder recorder;
     draw(recorder.beginRecording(SkRect::MakeIWH(width, height)), width, height, colors);
-    return SkImage::MakeFromPicture(recorder.finishRecordingAsPicture(),
-                                    {width, height}, nullptr, nullptr,
-                                    SkImage::BitDepth::kU8,
-                                    SkColorSpace::MakeSRGB());
+    return SkImages::DeferredFromPicture(recorder.finishRecordingAsPicture(),
+                                         {width, height},
+                                         nullptr,
+                                         nullptr,
+                                         SkImages::BitDepth::kU8,
+                                         SkColorSpace::MakeSRGB());
 }
 
 typedef sk_sp<SkImage> (*ImageMakerProc)(int width, int height, SkColor colors[2]);
 
 static void show_image(SkCanvas* canvas, int width, int height, SkColor colors[2],
-                       ImageMakerProc proc) {
+                       ImageMakerProc proc, bool manuallyTile) {
     sk_sp<SkImage> image = ToolUtils::MakeTextureImage(canvas, proc(width, height, colors));
     if (!image) {
         return;
@@ -68,7 +71,11 @@ static void show_image(SkCanvas* canvas, int width, int height, SkColor colors[2
 
     canvas->save();
     canvas->clipRect(dstRect);
-    canvas->drawImage(image, 0, 0);
+    if (manuallyTile) {
+        SkTiledImageUtils::DrawImage(canvas, image, 0, 0);
+    } else {
+        canvas->drawImage(image, 0, 0);
+    }
     canvas->restore();
     canvas->drawRect(dstRect, borderPaint);
 
@@ -76,24 +83,45 @@ static void show_image(SkCanvas* canvas, int width, int height, SkColor colors[2
     int hw = width / 2;
     int hh = height / 2;
     SkRect subset = SkRect::MakeLTRB(hw - 64, hh - 32, hw + 64, hh + 32);
-    canvas->drawImageRect(image, subset, dstRect, SkSamplingOptions(), nullptr,
-                          SkCanvas::kStrict_SrcRectConstraint);
+    if (manuallyTile) {
+        SkTiledImageUtils::DrawImageRect(canvas, image, subset, dstRect, {}, nullptr,
+                                         SkCanvas::kStrict_SrcRectConstraint);
+    } else {
+        canvas->drawImageRect(image, subset, dstRect, {}, nullptr,
+                              SkCanvas::kStrict_SrcRectConstraint);
+    }
     canvas->drawRect(dstRect, borderPaint);
 
     dstRect.offset(SkIntToScalar(150), 0);
-    canvas->drawImageRect(image, dstRect, SkSamplingOptions(), nullptr);
+    if (manuallyTile) {
+        SkTiledImageUtils::DrawImageRect(canvas, image, dstRect);
+    } else {
+        canvas->drawImageRect(image, dstRect, {});
+    }
     canvas->drawRect(dstRect, borderPaint);
 }
 
 class VeryLargeBitmapGM : public skiagm::GM {
     ImageMakerProc  fProc;
-    const char*     fName;
+    const char*     fBaseName;
+    bool            fManuallyTile;
 
 public:
-    VeryLargeBitmapGM(ImageMakerProc proc, const char name[]) : fProc(proc), fName(name) {}
+    VeryLargeBitmapGM(ImageMakerProc proc, const char baseName[], bool manuallyTile)
+            : fProc(proc)
+            , fBaseName(baseName)
+            , fManuallyTile(manuallyTile) {}
 
 private:
-    SkString onShortName() override { return SkString(fName); }
+    SkString onShortName() override {
+        SkString name(fBaseName);
+
+        if (fManuallyTile) {
+            name.append("_manual");
+        }
+
+        return name;
+    }
 
     SkISize onISize() override { return {500, 600}; }
 
@@ -109,25 +137,31 @@ private:
         canvas->translate(SkIntToScalar(10), SkIntToScalar(10));
         colors[0] = SK_ColorRED;
         colors[1] = SK_ColorGREEN;
-        show_image(canvas, small, small, colors, fProc);
+        show_image(canvas, small, small, colors, fProc, fManuallyTile);
         canvas->translate(0, SkIntToScalar(150));
 
         colors[0] = SK_ColorBLUE;
         colors[1] = SK_ColorMAGENTA;
-        show_image(canvas, big, small, colors, fProc);
+        show_image(canvas, big, small, colors, fProc, fManuallyTile);
         canvas->translate(0, SkIntToScalar(150));
 
         colors[0] = SK_ColorMAGENTA;
         colors[1] = SK_ColorYELLOW;
-        show_image(canvas, medium, medium, colors, fProc);
+        show_image(canvas, medium, medium, colors, fProc, fManuallyTile);
         canvas->translate(0, SkIntToScalar(150));
 
         colors[0] = SK_ColorGREEN;
         colors[1] = SK_ColorYELLOW;
         // This used to be big enough that we didn't draw on CPU, but now we do.
-        show_image(canvas, veryBig, small, colors, fProc);
+        show_image(canvas, veryBig, small, colors, fProc, fManuallyTile);
     }
 };
 
-DEF_GM( return new VeryLargeBitmapGM(make_raster_image, "verylargebitmap"); )
-DEF_GM( return new VeryLargeBitmapGM(make_picture_image, "verylarge_picture_image"); )
+DEF_GM( return new VeryLargeBitmapGM(make_raster_image, "verylargebitmap",
+                                     /* manuallyTile= */ false); )
+DEF_GM( return new VeryLargeBitmapGM(make_raster_image, "verylargebitmap",
+                                     /* manuallyTile= */ true); )
+DEF_GM( return new VeryLargeBitmapGM(make_picture_image, "verylarge_picture_image",
+                                     /* manuallyTile= */ false); )
+DEF_GM( return new VeryLargeBitmapGM(make_picture_image, "verylarge_picture_image",
+                                     /* manuallyTile= */ true); )
