@@ -8,25 +8,28 @@
 #ifndef GrMockOptions_DEFINED
 #define GrMockOptions_DEFINED
 
-#include "include/gpu/GrTypes.h"
-#include "include/private/GrTypesPriv.h"
+#include "include/core/SkTextureCompressionType.h"
+#include "include/gpu/GpuTypes.h"
+#include "include/private/gpu/ganesh/GrTypesPriv.h"
 
 class GrBackendFormat;
 
 struct GrMockTextureInfo {
     GrMockTextureInfo()
         : fColorType(GrColorType::kUnknown)
-        , fCompressionType(SkImage::CompressionType::kNone)
+        , fCompressionType(SkTextureCompressionType::kNone)
         , fID(0) {}
 
     GrMockTextureInfo(GrColorType colorType,
-                      SkImage::CompressionType compressionType,
-                      int id)
+                      SkTextureCompressionType compressionType,
+                      int id,
+                      skgpu::Protected isProtected = skgpu::Protected::kNo)
             : fColorType(colorType)
             , fCompressionType(compressionType)
-            , fID(id) {
+            , fID(id)
+            , fProtected(isProtected) {
         SkASSERT(fID);
-        if (fCompressionType != SkImage::CompressionType::kNone) {
+        if (fCompressionType != SkTextureCompressionType::kNone) {
             SkASSERT(colorType == GrColorType::kUnknown);
         }
     }
@@ -34,24 +37,29 @@ struct GrMockTextureInfo {
     bool operator==(const GrMockTextureInfo& that) const {
         return fColorType == that.fColorType &&
                fCompressionType == that.fCompressionType &&
-               fID == that.fID;
+               fID == that.fID &&
+               fProtected == that.fProtected;
     }
 
     GrBackendFormat getBackendFormat() const;
 
-    SkImage::CompressionType compressionType() const { return fCompressionType; }
+    SkTextureCompressionType compressionType() const { return fCompressionType; }
 
     GrColorType colorType() const {
-        SkASSERT(fCompressionType == SkImage::CompressionType::kNone);
+        SkASSERT(fCompressionType == SkTextureCompressionType::kNone);
         return fColorType;
     }
 
     int id() const { return fID; }
 
+    skgpu::Protected getProtected() const { return fProtected; }
+    bool isProtected() const { return fProtected == skgpu::Protected::kYes; }
+
 private:
     GrColorType              fColorType;
-    SkImage::CompressionType fCompressionType;
+    SkTextureCompressionType fCompressionType;
     int                      fID;
+    skgpu::Protected         fProtected = skgpu::Protected::kNo;
 };
 
 struct GrMockRenderTargetInfo {
@@ -59,35 +67,46 @@ struct GrMockRenderTargetInfo {
             : fColorType(GrColorType::kUnknown)
             , fID(0) {}
 
-    GrMockRenderTargetInfo(GrColorType colorType, int id)
+    GrMockRenderTargetInfo(GrColorType colorType,
+                           int id,
+                           skgpu::Protected isProtected = skgpu::Protected::kNo)
             : fColorType(colorType)
-            , fID(id) {
+            , fID(id)
+            , fProtected(isProtected) {
         SkASSERT(fID);
     }
 
     bool operator==(const GrMockRenderTargetInfo& that) const {
         return fColorType == that.fColorType &&
-               fID == that.fID;
+               fID == that.fID &&
+               fProtected == that.fProtected;
     }
 
     GrBackendFormat getBackendFormat() const;
 
     GrColorType colorType() const { return fColorType; }
 
+    skgpu::Protected getProtected() const { return fProtected; }
+    bool isProtected() const { return fProtected == skgpu::Protected::kYes; }
+
 private:
-    GrColorType   fColorType;
-    int           fID;
+    GrColorType      fColorType;
+    int              fID;
+    skgpu::Protected fProtected = skgpu::Protected::kNo;
 };
 
 struct GrMockSurfaceInfo {
     uint32_t fSampleCount = 1;
     uint32_t fLevelCount = 0;
-    GrProtected fProtected = GrProtected::kNo;
+    skgpu::Protected fProtected = skgpu::Protected::kNo;
 
     GrColorType fColorType = GrColorType::kUnknown;
-    SkImage::CompressionType fCompressionType = SkImage::CompressionType::kNone;
+    SkTextureCompressionType fCompressionType = SkTextureCompressionType::kNone;
 };
 
+static constexpr int kSkTextureCompressionTypeCount =
+        static_cast<int>(SkTextureCompressionType::kLast) + 1;
+
 /**
  * A pointer to this type is used as the GrBackendContext when creating a Mock GrContext. It can be
  * used to specify capability options for the mock context. If nullptr is used a default constructed
@@ -105,9 +124,9 @@ struct GrMockOptions {
 
         fConfigOptions[(int)GrColorType::kBGRA_8888] = fConfigOptions[(int)GrColorType::kRGBA_8888];
 
-        fCompressedOptions[(int)SkImage::CompressionType::kETC2_RGB8_UNORM].fTexturable = true;
-        fCompressedOptions[(int)SkImage::CompressionType::kBC1_RGB8_UNORM].fTexturable = true;
-        fCompressedOptions[(int)SkImage::CompressionType::kBC1_RGBA8_UNORM].fTexturable = true;
+        fCompressedOptions[(int)SkTextureCompressionType::kETC2_RGB8_UNORM].fTexturable = true;
+        fCompressedOptions[(int)SkTextureCompressionType::kBC1_RGB8_UNORM].fTexturable = true;
+        fCompressedOptions[(int)SkTextureCompressionType::kBC1_RGBA8_UNORM].fTexturable = true;
     }
 
     struct ConfigOptions {
@@ -125,9 +144,8 @@ struct GrMockOptions {
     int fMaxRenderTargetSize = 2048;
     int fMaxWindowRectangles = 0;
     int fMaxVertexAttributes = 16;
-    int fMaxTessellationSegments = 0;
     ConfigOptions fConfigOptions[kGrColorTypeCnt];
-    ConfigOptions fCompressedOptions[SkImage::kCompressionTypeCount];
+    ConfigOptions fCompressedOptions[kSkTextureCompressionTypeCount];
 
     // GrShaderCaps options.
     bool fIntegerSupport = false;
