@@ -23,7 +23,7 @@ extern "C" {
 
 /***********************************************************************
  *
- * <dsig:Signature> node
+ * &lt;dsig:Signature/&gt; node
  *
  **********************************************************************/
 XMLSEC_EXPORT xmlNodePtr xmlSecTmplSignatureCreate              (xmlDocPtr doc,
@@ -64,7 +64,7 @@ XMLSEC_EXPORT xmlNodePtr xmlSecTmplManifestAddReference         (xmlNodePtr mani
 
 /***********************************************************************
  *
- * <enc:EncryptedData> node
+ * &lt;enc:EncryptedData/&gt; node
  *
  **********************************************************************/
 XMLSEC_EXPORT xmlNodePtr xmlSecTmplEncDataCreate                (xmlDocPtr doc,
@@ -89,7 +89,7 @@ XMLSEC_EXPORT xmlNodePtr xmlSecTmplCipherReferenceAddTransform  (xmlNodePtr ciph
 
 /***********************************************************************
  *
- * <enc:EncryptedKey> node
+ * &lt;enc:EncryptedKey/&gt; node
  *
  **********************************************************************/
 XMLSEC_EXPORT xmlNodePtr xmlSecTmplReferenceListAddDataReference(xmlNodePtr encNode,
@@ -99,7 +99,7 @@ XMLSEC_EXPORT xmlNodePtr xmlSecTmplReferenceListAddKeyReference (xmlNodePtr encN
 
 /***********************************************************************
  *
- * <dsig:KeyInfo> node
+ * &lt;dsig:KeyInfo/&gt; node
  *
  **********************************************************************/
 XMLSEC_EXPORT xmlNodePtr xmlSecTmplKeyInfoAddKeyName            (xmlNodePtr keyInfoNode,
@@ -119,7 +119,7 @@ XMLSEC_EXPORT xmlNodePtr xmlSecTmplKeyInfoAddEncryptedKey       (xmlNodePtr keyI
 
 /***********************************************************************
  *
- * <dsig:X509Data> node
+ * &lt;dsig:X509Data/&gt; node
  *
  **********************************************************************/
 XMLSEC_EXPORT xmlNodePtr xmlSecTmplX509DataAddIssuerSerial      (xmlNodePtr x509DataNode);
@@ -127,12 +127,13 @@ XMLSEC_EXPORT xmlNodePtr xmlSecTmplX509IssuerSerialAddIssuerName(xmlNodePtr x509
 XMLSEC_EXPORT xmlNodePtr xmlSecTmplX509IssuerSerialAddSerialNumber(xmlNodePtr x509IssuerSerialNode, const xmlChar* serial);
 XMLSEC_EXPORT xmlNodePtr xmlSecTmplX509DataAddSubjectName       (xmlNodePtr x509DataNode);
 XMLSEC_EXPORT xmlNodePtr xmlSecTmplX509DataAddSKI               (xmlNodePtr x509DataNode);
+XMLSEC_EXPORT xmlNodePtr xmlSecTmplX509DataAddDigest            (xmlNodePtr x509DataNode, const xmlChar* digestAlgorithm);
 XMLSEC_EXPORT xmlNodePtr xmlSecTmplX509DataAddCertificate       (xmlNodePtr x509DataNode);
 XMLSEC_EXPORT xmlNodePtr xmlSecTmplX509DataAddCRL               (xmlNodePtr x509DataNode);
 
 /***********************************************************************
  *
- * <dsig:Transform> node
+ * &lt;dsig:Transform/&gt; node
  *
  **********************************************************************/
 XMLSEC_EXPORT int       xmlSecTmplTransformAddHmacOutputLength  (xmlNodePtr transformNode,
@@ -160,4 +161,3 @@ XMLSEC_EXPORT int       xmlSecTmplTransformAddXPointer          (xmlNodePtr tran
 #endif /* __cplusplus */
 
 #endif /* __XMLSEC_TEMPLATES_H__ */
-
