@@ -14,9 +14,9 @@
 #include "include/core/SkPaint.h"
 #include "include/core/SkPoint.h"
 #include "include/core/SkStream.h"
-#include "include/private/SkTArray.h"
-#include "include/private/SkTPin.h"
-#include "include/private/SkTo.h"
+#include "include/private/base/SkTArray.h"
+#include "include/private/base/SkTPin.h"
+#include "include/private/base/SkTo.h"
 #include "modules/skottie/include/ExternalLayer.h"
 #include "modules/skottie/include/SkottieProperty.h"
 #include "modules/skottie/src/Composition.h"
@@ -30,20 +30,31 @@
 #include "modules/sksg/include/SkSGPaint.h"
 #include "modules/sksg/include/SkSGPath.h"
 #include "modules/sksg/include/SkSGRenderEffect.h"
-#include "modules/sksg/include/SkSGScene.h"
+#include "modules/sksg/include/SkSGRenderNode.h"
 #include "modules/sksg/include/SkSGTransform.h"
 #include "src/core/SkTraceEvent.h"
 
 #include <chrono>
 #include <cmath>
 #include <memory>
+#include <ratio>
 
-#include "stdlib.h"
+#include <stdlib.h>
 
 namespace skottie {
 
 namespace internal {
 
+void SceneGraphRevalidator::setRoot(sk_sp<sksg::RenderNode> root) {
+    fRoot = std::move(root);
+}
+
+void SceneGraphRevalidator::revalidate() {
+    if (fRoot) {
+        fRoot->revalidate(nullptr, SkMatrix::I());
+    }
+}
+
 void AnimationBuilder::log(Logger::Level lvl, const skjson::Value* json,
                            const char fmt[], ...) const {
     if (!fLogger) {
@@ -71,8 +82,6 @@ void AnimationBuilder::log(Logger::Level lvl, const skjson::Value* json,
     fLogger->log(lvl, buff, jsonstr.c_str());
 }
 
-namespace  {
-
 class OpacityAdapter final : public DiscardableAdapterBase<OpacityAdapter, sksg::OpacityEffect> {
 public:
     OpacityAdapter(const skjson::ObjectValue& jobject,
@@ -83,6 +92,7 @@ public:
     }
 
 private:
+    friend class skottie::internal::AnimationBuilder;
     void onSync() override {
         this->node()->setOpacity(fOpacity * 0.01f);
     }
@@ -92,39 +102,6 @@ private:
     using INHERITED = DiscardableAdapterBase<OpacityAdapter, sksg::OpacityEffect>;
 };
 
-static SkBlendMode GetBlendMode(const skjson::ObjectValue& jobject,
-                                const AnimationBuilder* abuilder) {
-    static constexpr SkBlendMode kBlendModeMap[] = {
-        SkBlendMode::kSrcOver,    // 0:'normal'
-        SkBlendMode::kMultiply,   // 1:'multiply'
-        SkBlendMode::kScreen,     // 2:'screen'
-        SkBlendMode::kOverlay,    // 3:'overlay
-        SkBlendMode::kDarken,     // 4:'darken'
-        SkBlendMode::kLighten,    // 5:'lighten'
-        SkBlendMode::kColorDodge, // 6:'color-dodge'
-        SkBlendMode::kColorBurn,  // 7:'color-burn'
-        SkBlendMode::kHardLight,  // 8:'hard-light'
-        SkBlendMode::kSoftLight,  // 9:'soft-light'
-        SkBlendMode::kDifference, // 10:'difference'
-        SkBlendMode::kExclusion,  // 11:'exclusion'
-        SkBlendMode::kHue,        // 12:'hue'
-        SkBlendMode::kSaturation, // 13:'saturation'
-        SkBlendMode::kColor,      // 14:'color'
-        SkBlendMode::kLuminosity, // 15:'luminosity'
-        SkBlendMode::kPlus,       // 16:'add'
-    };
-
-    const auto bm_index = ParseDefault<size_t>(jobject["bm"], 0);
-    if (bm_index >= SK_ARRAY_COUNT(kBlendModeMap)) {
-            abuilder->log(Logger::Level::kWarning, &jobject,
-                          "Unsupported blend mode %lu\n", bm_index);
-            return SkBlendMode::kSrcOver;
-    }
-
-    return kBlendModeMap[bm_index];
-}
-
-} // namespace
 
 sk_sp<sksg::RenderNode> AnimationBuilder::attachOpacity(const skjson::ObjectValue& jobject,
                                                         sk_sp<sksg::RenderNode> child_node) const {
@@ -132,10 +109,11 @@ sk_sp<sksg::RenderNode> AnimationBuilder::attachOpacity(const skjson::ObjectValu
         return nullptr;
 
     auto adapter = OpacityAdapter::Make(jobject, child_node, *this);
-    const auto dispatched = this->dispatchOpacityProperty(adapter->node());
-
     if (adapter->isStatic()) {
         adapter->seek(0);
+    }
+    auto dispatched = this->dispatchOpacityProperty(adapter->node(), jobject["o"], adapter);
+    if (adapter->isStatic()) {
         if (!dispatched && adapter->node()->getOpacity() >= 1) {
             // No obeservable effects - we can discard.
             return child_node;
@@ -147,17 +125,6 @@ sk_sp<sksg::RenderNode> AnimationBuilder::attachOpacity(const skjson::ObjectValu
     return adapter->node();
 }
 
-sk_sp<sksg::RenderNode> AnimationBuilder::attachBlendMode(const skjson::ObjectValue& jobject,
-                                                          sk_sp<sksg::RenderNode> child) const {
-    const auto bm = GetBlendMode(jobject, this);
-    if (bm != SkBlendMode::kSrcOver) {
-        fHasNontrivialBlending = true;
-        child = sksg::BlendModeEffect::Make(std::move(child), bm);
-    }
-
-    return child;
-}
-
 AnimationBuilder::AnimationBuilder(sk_sp<ResourceProvider> rp, sk_sp<SkFontMgr> fontmgr,
                                    sk_sp<PropertyObserver> pobserver, sk_sp<Logger> logger,
                                    sk_sp<MarkerObserver> mobserver, sk_sp<PrecompInterceptor> pi,
@@ -172,6 +139,8 @@ AnimationBuilder::AnimationBuilder(sk_sp<ResourceProvider> rp, sk_sp<SkFontMgr>
     , fMarkerObserver(std::move(mobserver))
     , fPrecompInterceptor(std::move(pi))
     , fExpressionManager(std::move(expressionmgr))
+    , fRevalidator(sk_make_sp<SceneGraphRevalidator>())
+    , fSlotManager(sk_make_sp<SlotManager>(fRevalidator))
     , fStats(stats)
     , fCompSize(comp_size)
     , fDuration(duration)
@@ -182,17 +151,23 @@ AnimationBuilder::AnimationBuilder(sk_sp<ResourceProvider> rp, sk_sp<SkFontMgr>
 AnimationBuilder::AnimationInfo AnimationBuilder::parse(const skjson::ObjectValue& jroot) {
     this->dispatchMarkers(jroot["markers"]);
 
-    this->parseAssets(jroot["assets"]);
-    this->parseFonts(jroot["fonts"], jroot["chars"]);
-
     AutoScope ascope(this);
     AutoPropertyTracker apt(this, jroot, PropertyObserver::NodeType::COMPOSITION);
+
+    this->parseAssets(jroot["assets"]);
+    this->parseFonts(jroot["fonts"], jroot["chars"]);
+    fSlotsRoot = jroot["slots"];
+
     auto root = CompositionBuilder(*this, fCompSize, jroot).build(*this);
 
     auto animators = ascope.release();
     fStats->fAnimatorCount = animators.size();
 
-    return { sksg::Scene::Make(std::move(root)), std::move(animators) };
+    // Point the revalidator to our final root, and perform initial revalidation.
+    fRevalidator->setRoot(root);
+    fRevalidator->revalidate();
+
+    return { std::move(root), std::move(animators) };
 }
 
 void AnimationBuilder::parseAssets(const skjson::ArrayValue* jassets) {
@@ -236,42 +211,68 @@ void AnimationBuilder::dispatchMarkers(const skjson::ArrayValue* jmarkers) const
     }
 }
 
-bool AnimationBuilder::dispatchColorProperty(const sk_sp<sksg::Color>& c) const {
+bool AnimationBuilder::dispatchColorProperty(const sk_sp<sksg::Color>& c,
+                                             const skjson::ObjectValue* jcolor) const {
     bool dispatched = false;
 
+    if (jcolor) {
+        if (const skjson::StringValue* slotID = (*jcolor)["sid"]) {
+            fSlotManager->trackColorValue(SkString(slotID->begin()), &(c->fColor), c);
+            dispatched = true;
+        }
+    }
+
     if (fPropertyObserver) {
-        fPropertyObserver->onColorProperty(fPropertyObserverContext,
+        const char * node_name = fPropertyObserverContext;
+        fPropertyObserver->onColorProperty(node_name,
             [&]() {
                 dispatched = true;
-                return std::make_unique<ColorPropertyHandle>(c);
+                return std::make_unique<ColorPropertyHandle>(c, fRevalidator);
             });
     }
 
     return dispatched;
 }
 
-bool AnimationBuilder::dispatchOpacityProperty(const sk_sp<sksg::OpacityEffect>& o) const {
+bool AnimationBuilder::dispatchOpacityProperty(const sk_sp<sksg::OpacityEffect>& o,
+                                               const skjson::ObjectValue* jopacity,
+                                               const sk_sp<OpacityAdapter> adapter) const {
     bool dispatched = false;
 
+    if (jopacity) {
+        if (const skjson::StringValue* slotID = (*jopacity)["sid"]) {
+            fSlotManager->trackScalarValue(SkString(slotID->begin()), &(adapter->fOpacity),
+                                            adapter);
+            dispatched = true;
+        }
+    }
+
     if (fPropertyObserver) {
         fPropertyObserver->onOpacityProperty(fPropertyObserverContext,
             [&]() {
                 dispatched = true;
-                return std::make_unique<OpacityPropertyHandle>(o);
+                return std::make_unique<OpacityPropertyHandle>(o, fRevalidator);
             });
     }
 
     return dispatched;
 }
 
-bool AnimationBuilder::dispatchTextProperty(const sk_sp<TextAdapter>& t) const {
+bool AnimationBuilder::dispatchTextProperty(const sk_sp<TextAdapter>& t,
+                                            const skjson::ObjectValue* jtext) const {
     bool dispatched = false;
 
     if (fPropertyObserver) {
-        fPropertyObserver->onTextProperty(fPropertyObserverContext,
+        const char * node_name = fPropertyObserverContext;
+        if (jtext) {
+            if (const skjson::StringValue* slotID = (*jtext)["sid"]) {
+                node_name = slotID->begin();
+            }
+        }
+        fPropertyObserver->onTextProperty(node_name,
             [&]() {
                 dispatched = true;
-                return std::make_unique<TextPropertyHandle>(t);
+                return std::make_unique<TextPropertyHandle>(t, fRevalidator);
             });
     }
 
@@ -285,7 +286,7 @@ bool AnimationBuilder::dispatchTransformProperty(const sk_sp<TransformAdapter2D>
         fPropertyObserver->onTransformProperty(fPropertyObserverContext,
             [&]() {
                 dispatched = true;
-                return std::make_unique<TransformPropertyHandle>(t);
+                return std::make_unique<TransformPropertyHandle>(t, fRevalidator);
             });
     }
 
@@ -298,10 +299,8 @@ sk_sp<ExpressionManager> AnimationBuilder::expression_manager() const {
 
 void AnimationBuilder::AutoPropertyTracker::updateContext(PropertyObserver* observer,
                                                           const skjson::ObjectValue& obj) {
-
     const skjson::StringValue* name = obj["nm"];
-
-    fBuilder->fPropertyObserverContext = name ? name->begin() : nullptr;
+    fBuilder->fPropertyObserverContext = name ? name->begin() : fPrevContext;
 }
 
 } // namespace internal
@@ -428,7 +427,7 @@ sk_sp<Animation> Animation::Builder::make(const char* data, size_t data_len) {
     fStats.fSceneParseTimeMS = std::chrono::duration<float, std::milli>{t2-t1}.count();
     fStats.fTotalLoadTimeMS  = std::chrono::duration<float, std::milli>{t2-t0}.count();
 
-    if (!ainfo.fScene && fLogger) {
+    if (!ainfo.fSceneRoot && fLogger) {
         fLogger->log(Logger::Level::kError, "Could not parse animation.\n");
     }
 
@@ -437,7 +436,7 @@ sk_sp<Animation> Animation::Builder::make(const char* data, size_t data_len) {
         flags |= Animation::Flags::kRequiresTopLevelIsolation;
     }
 
-    return sk_sp<Animation>(new Animation(std::move(ainfo.fScene),
+    return sk_sp<Animation>(new Animation(std::move(ainfo.fSceneRoot),
                                           std::move(ainfo.fAnimators),
                                           std::move(version),
                                           size,
@@ -455,11 +454,11 @@ sk_sp<Animation> Animation::Builder::makeFromFile(const char path[]) {
                 : nullptr;
 }
 
-Animation::Animation(std::unique_ptr<sksg::Scene> scene,
+Animation::Animation(sk_sp<sksg::RenderNode> scene_root,
                      std::vector<sk_sp<internal::Animator>>&& animators,
                      SkString version, const SkSize& size,
                      double inPoint, double outPoint, double duration, double fps, uint32_t flags)
-    : fScene(std::move(scene))
+    : fSceneRoot(std::move(scene_root))
     , fAnimators(std::move(animators))
     , fVersion(std::move(version))
     , fSize(size)
@@ -478,7 +477,7 @@ void Animation::render(SkCanvas* canvas, const SkRect* dstR) const {
 void Animation::render(SkCanvas* canvas, const SkRect* dstR, RenderFlags renderFlags) const {
     TRACE_EVENT0("skottie", TRACE_FUNC);
 
-    if (!fScene)
+    if (!fSceneRoot)
         return;
 
     SkAutoCanvasRestore restore(canvas, true);
@@ -499,13 +498,13 @@ void Animation::render(SkCanvas* canvas, const SkRect* dstR, RenderFlags renderF
         canvas->saveLayer(srcR, nullptr);
     }
 
-    fScene->render(canvas);
+    fSceneRoot->render(canvas);
 }
 
 void Animation::seekFrame(double t, sksg::InvalidationController* ic) {
     TRACE_EVENT0("skottie", TRACE_FUNC);
 
-    if (!fScene)
+    if (!fSceneRoot)
         return;
 
     // Per AE/Lottie semantics out_point is exclusive.
@@ -516,7 +515,7 @@ void Animation::seekFrame(double t, sksg::InvalidationController* ic) {
         anim->seek(comp_time);
     }
 
-    fScene->revalidate(ic);
+    fSceneRoot->revalidate(ic, SkMatrix::I());
 }
 
 void Animation::seekFrameTime(double t, sksg::InvalidationController* ic) {
