@@ -2,10 +2,14 @@
 
 #include "modules/skparagraph/src/Iterators.h"
 #include "modules/skparagraph/src/OneLineShaper.h"
-#include "src/utils/SkUTF.h"
+#include "src/base/SkUTF.h"
+
 #include <algorithm>
+#include <cstdint>
 #include <unordered_set>
 
+using namespace skia_private;
+
 static inline SkUnichar nextUtf8Unit(const char** ptr, const char* end) {
     SkUnichar val = SkUTF::NextUTF8(ptr, end);
     return val < 0 ? 0xFFFD : val;
@@ -156,6 +160,7 @@ void OneLineShaper::finish(const Block& block, SkScalar height, SkScalar& advanc
         }
         fResolvedBlocks.emplace_back(unresolved);
         fUnresolvedGlyphs += unresolved.fGlyphs.width();
+        fParagraph->addUnresolvedCodepoints(unresolved.fText);
     }
 
     // Sort all pieces by text
@@ -211,22 +216,22 @@ void OneLineShaper::finish(const Block& block, SkScalar height, SkScalar& advanc
                     height,
                     block.fStyle.getHalfLeading(),
                     block.fStyle.getBaselineShift(),
-                    this->fParagraph->fRuns.count(),
+                    this->fParagraph->fRuns.size(),
                     advanceX
                 );
         auto piece = &this->fParagraph->fRuns.back();
 
         // TODO: Optimize copying
-        auto zero = run->fPositions[glyphs.start];
+        SkPoint zero = {run->fPositions[glyphs.start].fX, 0};
         for (size_t i = glyphs.start; i <= glyphs.end; ++i) {
 
             auto index = i - glyphs.start;
             if (i < glyphs.end) {
                 piece->fGlyphs[index] = run->fGlyphs[i];
-                piece->fBounds[index] = run->fBounds[i];
             }
             piece->fClusterIndexes[index] = run->fClusterIndexes[i];
             piece->fPositions[index] = run->fPositions[i] - zero;
+            piece->fOffsets[index] = run->fOffsets[i];
             piece->addX(index, advanceX);
         }
 
@@ -267,7 +272,8 @@ void OneLineShaper::addFullyResolved() {
 }
 
 void OneLineShaper::addUnresolvedWithRun(GlyphRange glyphRange) {
-    RunBlock unresolved(fCurrentRun, clusteredText(glyphRange), glyphRange, 0);
+    auto extendedText = this->clusteredText(glyphRange); // It also modifies glyphRange if needed
+    RunBlock unresolved(fCurrentRun, extendedText, glyphRange, 0);
     if (unresolved.fGlyphs.width() == fCurrentRun->size()) {
         SkASSERT(unresolved.fText.width() == fCurrentRun->fTextRange.width());
     } else if (fUnresolvedBlocks.size() > 0) {
@@ -290,7 +296,7 @@ void OneLineShaper::addUnresolvedWithRun(GlyphRange glyphRange) {
                 // Few pieces of the same unresolved text block can ignore the second one
                 lastUnresolved.fGlyphs.start = std::min(lastUnresolved.fGlyphs.start, glyphRange.start);
                 lastUnresolved.fGlyphs.end = std::max(lastUnresolved.fGlyphs.end, glyphRange.end);
-                lastUnresolved.fText = clusteredText(lastUnresolved.fGlyphs);
+                lastUnresolved.fText = this->clusteredText(lastUnresolved.fGlyphs);
                 return;
             }
         }
@@ -300,12 +306,8 @@ void OneLineShaper::addUnresolvedWithRun(GlyphRange glyphRange) {
 
 // Glue whitespaces to the next/prev unresolved blocks
 // (so we don't have chinese text with english whitespaces broken into millions of tiny runs)
-#ifndef SK_PARAGRAPH_GRAPHEME_EDGES
 void OneLineShaper::sortOutGlyphs(std::function<void(GlyphRange)>&& sortOutUnresolvedBLock) {
 
-    auto text = fCurrentRun->fOwner->text();
-    size_t unresolvedGlyphs = 0;
-
     GlyphRange block = EMPTY_RANGE;
     bool graphemeResolved = false;
     TextIndex graphemeStart = EMPTY_INDEX;
@@ -321,9 +323,8 @@ void OneLineShaper::sortOutGlyphs(std::function<void(GlyphRange)>&& sortOutUnres
         if ((fCurrentRun->leftToRight() ? gi > graphemeStart : gi < graphemeStart) || graphemeStart == EMPTY_INDEX) {
             // This is the Flutter change
             // Do not count control codepoints as unresolved
-            const char* cluster = text.begin() + ci;
-            SkUnichar codepoint = nextUtf8Unit(&cluster, text.end());
-            bool isControl8 = fParagraph->getUnicode()->isControl(codepoint);
+            bool isControl8 = fParagraph->codeUnitHasProperty(ci,
+                                                              SkUnicode::CodeUnitFlags::kControl);
             // We only count glyph resolved if all the glyphs in its grapheme are resolved
             graphemeResolved = glyph != 0 || isControl8;
             graphemeStart = gi;
@@ -333,7 +334,6 @@ void OneLineShaper::sortOutGlyphs(std::function<void(GlyphRange)>&& sortOutUnres
         }
 
         if (!graphemeResolved) { // Unresolved glyph and not control codepoint
-            ++unresolvedGlyphs;
             if (block.start == EMPTY_INDEX) {
                 // Start new unresolved block
                 block.start = i;
@@ -359,76 +359,12 @@ void OneLineShaper::sortOutGlyphs(std::function<void(GlyphRange)>&& sortOutUnres
         sortOutUnresolvedBLock(block);
     }
 }
-#else
-void OneLineShaper::sortOutGlyphs(std::function<void(GlyphRange)>&& sortOutUnresolvedBLock) {
-
-    auto text = fCurrentRun->fOwner->text();
-    size_t unresolvedGlyphs = 0;
-
-    TextIndex whitespacesStart = EMPTY_INDEX;
-    GlyphRange block = EMPTY_RANGE;
-    for (size_t i = 0; i < fCurrentRun->size(); ++i) {
-
-        const char* cluster = text.begin() + clusterIndex(i);
-        SkUnichar codepoint = nextUtf8Unit(&cluster, text.end());
-        bool isControl8 = fParagraph->getUnicode()->isControl(codepoint);
-        // TODO: This is a temp change to match space handiling in LibTxt
-        // (all spaces are resolved with the main font)
-#ifdef SK_PARAGRAPH_LIBTXT_SPACES_RESOLUTION
-        bool isWhitespace8 = false; // fParagraph->getUnicode()->isWhitespace(codepoint);
-#else
-        bool isWhitespace8 = fParagraph->getUnicode()->isWhitespace(codepoint);
-#endif
-        // Inspect the glyph
-        auto glyph = fCurrentRun->fGlyphs[i];
-        if (glyph == 0 && !isControl8) { // Unresolved glyph and not control codepoint
-            ++unresolvedGlyphs;
-            if (block.start == EMPTY_INDEX) {
-                // Start new unresolved block
-                // (all leading whitespaces glued to the resolved part if it's not empty)
-                block.start = whitespacesStart == 0 ? 0 : i;
-                block.end = EMPTY_INDEX;
-            } else {
-                // Keep skipping unresolved block
-            }
-        } else { // Resolved glyph or control codepoint
-            if (block.start == EMPTY_INDEX) {
-                // Keep skipping resolved code points
-            } else if (isWhitespace8) {
-                // Glue whitespaces after to the unresolved block
-                ++unresolvedGlyphs;
-            } else {
-                // This is the end of unresolved block (all trailing whitespaces glued to the resolved part)
-                block.end = whitespacesStart == EMPTY_INDEX ? i : whitespacesStart;
-                sortOutUnresolvedBLock(block);
-                block = EMPTY_RANGE;
-                whitespacesStart = EMPTY_INDEX;
-            }
-        }
-
-        // Keep updated the start of the latest whitespaces patch
-        if (isWhitespace8) {
-            if (whitespacesStart == EMPTY_INDEX) {
-                whitespacesStart = i;
-            }
-        } else {
-            whitespacesStart = EMPTY_INDEX;
-        }
-    }
-
-    // One last block could have been left
-    if (block.start != EMPTY_INDEX) {
-        block.end = fCurrentRun->size();
-        sortOutUnresolvedBLock(block);
-    }
-}
-#endif
 
 void OneLineShaper::iterateThroughFontStyles(TextRange textRange,
                                              SkSpan<Block> styleSpan,
                                              const ShapeSingleFontVisitor& visitor) {
     Block combinedBlock;
-    SkTArray<SkShaper::Feature> features;
+    TArray<SkShaper::Feature> features;
 
     auto addFeatures = [&features](const Block& block) {
         for (auto& ff : block.fStyle.getFontFeatures()) {
@@ -444,6 +380,12 @@ void OneLineShaper::iterateThroughFontStyles(TextRange textRange,
             };
             features.emplace_back(feature);
         }
+        // Disable ligatures if letter spacing is enabled.
+        if (block.fStyle.getLetterSpacing() > 0) {
+            features.emplace_back(SkShaper::Feature{
+                SkSetFourByteTag('l', 'i', 'g', 'a'), 0, block.fRange.start, block.fRange.end
+            });
+        }
     };
 
     for (auto& block : styleSpan) {
@@ -465,7 +407,7 @@ void OneLineShaper::iterateThroughFontStyles(TextRange textRange,
 
         combinedBlock.fRange = blockRange;
         combinedBlock.fStyle = block.fStyle;
-        features.reset();
+        features.clear();
         addFeatures(block);
     }
 
@@ -477,7 +419,7 @@ void OneLineShaper::iterateThroughFontStyles(TextRange textRange,
 
 void OneLineShaper::matchResolvedFonts(const TextStyle& textStyle,
                                        const TypefaceVisitor& visitor) {
-    std::vector<sk_sp<SkTypeface>> typefaces = fParagraph->fFontCollection->findTypefaces(textStyle.getFontFamilies(), textStyle.getFontStyle());
+    std::vector<sk_sp<SkTypeface>> typefaces = fParagraph->fFontCollection->findTypefaces(textStyle.getFontFamilies(), textStyle.getFontStyle(), textStyle.getFontArguments());
 
     for (const auto& typeface : typefaces) {
         if (visitor(typeface) == Resolved::Everything) {
@@ -496,13 +438,30 @@ void OneLineShaper::matchResolvedFonts(const TextStyle& textStyle,
             const char* ch = unresolvedText.begin();
             // We have the global cache for all already found typefaces for SkUnichar
             // but we still need to keep track of all SkUnichars used in this unresolved block
-            SkTHashSet<SkUnichar> alreadyTried;
-            SkUnichar unicode = nextUtf8Unit(&ch, unresolvedText.end());
+            THashSet<SkUnichar> alreadyTriedCodepoints;
+            THashSet<SkTypefaceID> alreadyTriedTypefaces;
             while (true) {
 
-                sk_sp<SkTypeface> typeface;
+                if (ch == unresolvedText.end()) {
+                    // Not a single codepoint could be resolved but we finished the block
+                    hopelessBlocks.push_back(fUnresolvedBlocks.front());
+                    fUnresolvedBlocks.pop_front();
+                    break;
+                }
+
+                // See if we can switch to the next DIFFERENT codepoint
+                SkUnichar unicode = -1;
+                while (ch != unresolvedText.end()) {
+                    unicode = nextUtf8Unit(&ch, unresolvedText.end());
+                    if (!alreadyTriedCodepoints.contains(unicode)) {
+                        alreadyTriedCodepoints.add(unicode);
+                        break;
+                    }
+                }
+                SkASSERT(unicode != -1);
 
                 // First try to find in in a cache
+                sk_sp<SkTypeface> typeface;
                 FontKey fontKey(unicode, textStyle.getFontStyle(), textStyle.getLocale());
                 auto found = fFallbackFonts.find(fontKey);
                 if (found != nullptr) {
@@ -512,40 +471,42 @@ void OneLineShaper::matchResolvedFonts(const TextStyle& textStyle,
                             unicode, textStyle.getFontStyle(), textStyle.getLocale());
 
                     if (typeface == nullptr) {
-                        return;
+                        // There is no fallback font for this character, so move on to the next character.
+                        continue;
                     }
                     fFallbackFonts.set(fontKey, typeface);
                 }
 
+                // Check if we already tried this font on this text range
+                if (!alreadyTriedTypefaces.contains(typeface->uniqueID())) {
+                    alreadyTriedTypefaces.add(typeface->uniqueID());
+                } else {
+                    continue;
+                }
+
+                auto resolvedBlocksBefore = fResolvedBlocks.size();
                 auto resolved = visitor(typeface);
                 if (resolved == Resolved::Everything) {
-                    // Resolved everything, no need to try another font
-                    return;
+                    if (hopelessBlocks.empty()) {
+                        // Resolved everything, no need to try another font
+                        return;
+                    } else if (resolvedBlocksBefore < fResolvedBlocks.size()) {
+                        // There are some resolved blocks
+                        resolved = Resolved::Something;
+                    } else {
+                        // All blocks are hopeless
+                        resolved = Resolved::Nothing;
+                    }
                 }
 
                 if (resolved == Resolved::Something) {
                     // Resolved something, no need to try another codepoint
                     break;
                 }
-
-                if (ch == unresolvedText.end()) {
-                    // Not a single codepoint could be resolved but we finished the block
-                    hopelessBlocks.push_back(fUnresolvedBlocks.front());
-                    fUnresolvedBlocks.pop_front();
-                    break;
-                }
-
-                // We can stop here or we can switch to another DIFFERENT codepoint
-                while (ch != unresolvedText.end()) {
-                    unicode = nextUtf8Unit(&ch, unresolvedText.end());
-                    if (alreadyTried.find(unicode) == nullptr) {
-                        alreadyTried.add(unicode);
-                        break;
-                    }
-                }
             }
         }
 
+        // Return hopeless blocks back
         for (auto& block : hopelessBlocks) {
             fUnresolvedBlocks.emplace_front(block);
         }
@@ -569,11 +530,13 @@ bool OneLineShaper::iterateThroughShapingRegions(const ShapeVisitor& shape) {
                 // Set up the iterators (the style iterator points to a bigger region that it could
                 TextRange textRange(start, end);
                 auto blockRange = fParagraph->findAllBlocks(textRange);
-                SkSpan<Block> styleSpan(fParagraph->blocks(blockRange));
+                if (!blockRange.empty()) {
+                    SkSpan<Block> styleSpan(fParagraph->blocks(blockRange));
 
-                // Shape the text between placeholders
-                if (!shape(textRange, styleSpan, advanceX, start, bidiRegion.level)) {
-                    return false;
+                    // Shape the text between placeholders
+                    if (!shape(textRange, styleSpan, advanceX, start, bidiRegion.level)) {
+                        return false;
+                    }
                 }
 
                 if (end == bidiRegion.end) {
@@ -591,14 +554,18 @@ bool OneLineShaper::iterateThroughShapingRegions(const ShapeVisitor& shape) {
         // Get the placeholder font
         std::vector<sk_sp<SkTypeface>> typefaces = fParagraph->fFontCollection->findTypefaces(
             placeholder.fTextStyle.getFontFamilies(),
-            placeholder.fTextStyle.getFontStyle());
+            placeholder.fTextStyle.getFontStyle(),
+            placeholder.fTextStyle.getFontArguments());
         sk_sp<SkTypeface> typeface = typefaces.size() ? typefaces.front() : nullptr;
         SkFont font(typeface, placeholder.fTextStyle.getFontSize());
 
         // "Shape" the placeholder
+        uint8_t bidiLevel = (bidiIndex < fParagraph->fBidiRegions.size())
+            ? fParagraph->fBidiRegions[bidiIndex].level
+            : 2;
         const SkShaper::RunHandler::RunInfo runInfo = {
             font,
-            (uint8_t)2,
+            bidiLevel,
             SkPoint::Make(placeholder.fStyle.fWidth, placeholder.fStyle.fHeight),
             1,
             SkShaper::RunHandler::Range(0, placeholder.fRange.width())
@@ -609,10 +576,11 @@ bool OneLineShaper::iterateThroughShapingRegions(const ShapeVisitor& shape) {
                                        0.0f,
                                        0.0f,
                                        false,
-                                       fParagraph->fRuns.count(),
+                                       fParagraph->fRuns.size(),
                                        advanceX);
 
         run.fPositions[0] = { advanceX, 0 };
+        run.fOffsets[0] = {0, 0};
         run.fClusterIndexes[0] = 0;
         run.fPlaceholderIndex = &placeholder - fParagraph->fPlaceholders.begin();
         advanceX += placeholder.fStyle.fWidth;
@@ -631,7 +599,7 @@ bool OneLineShaper::shape() {
             (TextRange textRange, SkSpan<Block> styleSpan, SkScalar& advanceX, TextIndex textStart, uint8_t defaultBidiLevel) {
 
         // Set up the shaper and shape the next
-        auto shaper = SkShaper::MakeShapeDontWrapOrReorder();
+        auto shaper = SkShaper::MakeShapeDontWrapOrReorder(fParagraph->fUnicode->copy());
         if (shaper == nullptr) {
             // For instance, loadICU does not work. We have to stop the process
             return false;
@@ -639,7 +607,7 @@ bool OneLineShaper::shape() {
 
         iterateThroughFontStyles(textRange, styleSpan,
                 [this, &shaper, defaultBidiLevel, limitlessWidth, &advanceX]
-                (Block block, SkTArray<SkShaper::Feature> features) {
+                (Block block, TArray<SkShaper::Feature> features) {
             auto blockSpan = SkSpan<Block>(&block, 1);
 
             // Start from the beginning (hoping that it's a simple case one block - one run)
@@ -650,7 +618,7 @@ bool OneLineShaper::shape() {
             fCurrentText = block.fRange;
             fUnresolvedBlocks.emplace_back(RunBlock(block.fRange));
 
-            matchResolvedFonts(block.fStyle, [&](sk_sp<SkTypeface> typeface) {
+            this->matchResolvedFonts(block.fStyle, [&](sk_sp<SkTypeface> typeface) {
 
                 // Create one more font to try
                 SkFont font(std::move(typeface), block.fStyle.getFontSize());
@@ -687,12 +655,24 @@ bool OneLineShaper::shape() {
                     LangIterator langIter(unresolvedText, blockSpan,
                                       fParagraph->paragraphStyle().getTextStyle());
                     SkShaper::TrivialBiDiRunIterator bidiIter(defaultBidiLevel, unresolvedText.size());
-                    auto scriptIter = SkShaper::MakeSkUnicodeHbScriptRunIterator
-                                     (fParagraph->getUnicode(), unresolvedText.begin(), unresolvedText.size());
+                    auto scriptIter = SkShaper::MakeSkUnicodeHbScriptRunIterator(
+                            unresolvedText.begin(), unresolvedText.size());
                     fCurrentText = unresolvedRange;
+
+                    // Map the block's features to subranges within the unresolved range.
+                    TArray<SkShaper::Feature> adjustedFeatures(features.size());
+                    for (const SkShaper::Feature& feature : features) {
+                        SkRange<size_t> featureRange(feature.start, feature.end);
+                        if (unresolvedRange.intersects(featureRange)) {
+                            SkRange<size_t> adjustedRange = unresolvedRange.intersection(featureRange);
+                            adjustedRange.Shift(-static_cast<std::make_signed_t<size_t>>(unresolvedRange.start));
+                            adjustedFeatures.push_back({feature.tag, feature.value, adjustedRange.start, adjustedRange.end});
+                        }
+                    }
+
                     shaper->shape(unresolvedText.begin(), unresolvedText.size(),
                             fontIter, bidiIter,*scriptIter, langIter,
-                            features.data(), features.size(),
+                            adjustedFeatures.data(), adjustedFeatures.size(),
                             limitlessWidth, this);
 
                     // Take off the queue the block we tried to resolved -
@@ -701,6 +681,8 @@ bool OneLineShaper::shape() {
                 }
 
                 if (fUnresolvedBlocks.empty()) {
+                    // In some cases it does not mean everything
+                    // (when we excluded some hopeless blocks from the list)
                     return Resolved::Everything;
                 } else if (resolvedCount < fResolvedBlocks.size()) {
                     return Resolved::Something;
@@ -730,7 +712,7 @@ TextRange OneLineShaper::clusteredText(GlyphRange& glyphs) {
         if (dir == Dir::right) {
             while (index < fCurrentRun->fTextRange.end) {
                 if (this->fParagraph->codeUnitHasProperty(index,
-                                                          CodeUnitFlags::kGraphemeStart)) {
+                                                      SkUnicode::CodeUnitFlags::kGraphemeStart)) {
                     return index;
                 }
                 ++index;
@@ -739,7 +721,7 @@ TextRange OneLineShaper::clusteredText(GlyphRange& glyphs) {
         } else {
             while (index > fCurrentRun->fTextRange.start) {
                 if (this->fParagraph->codeUnitHasProperty(index,
-                                                          CodeUnitFlags::kGraphemeStart)) {
+                                                      SkUnicode::CodeUnitFlags::kGraphemeStart)) {
                     return index;
                 }
                 --index;
@@ -777,11 +759,10 @@ bool OneLineShaper::FontKey::operator==(const OneLineShaper::FontKey& other) con
     return fUnicode == other.fUnicode && fFontStyle == other.fFontStyle && fLocale == other.fLocale;
 }
 
-size_t OneLineShaper::FontKey::Hasher::operator()(const OneLineShaper::FontKey& key) const {
-
+uint32_t OneLineShaper::FontKey::Hasher::operator()(const OneLineShaper::FontKey& key) const {
     return SkGoodHash()(key.fUnicode) ^
            SkGoodHash()(key.fFontStyle) ^
-           SkGoodHash()(key.fLocale.c_str());
+           SkGoodHash()(key.fLocale);
 }
 
 }  // namespace textlayout
