@@ -59,7 +59,7 @@ TextRange operator*(const TextRange& a, const TextRange& b) {
     return end > begin ? TextRange(begin, end) : EMPTY_TEXT;
 }
 
-int compareRound(SkScalar a, SkScalar b) {
+int compareRound(SkScalar a, SkScalar b, bool applyRoundingHack) {
     // There is a rounding error that gets bigger when maxWidth gets bigger
     // VERY long zalgo text (> 100000) on a VERY long line (> 10000)
     // Canvas scaling affects it
@@ -71,8 +71,13 @@ int compareRound(SkScalar a, SkScalar b) {
         return 0;
     }
 
-    auto ra = littleRound(a);
-    auto rb = littleRound(b);
+    auto ra = a;
+    auto rb = b;
+
+    if (applyRoundingHack) {
+        ra = littleRound(a);
+        rb = littleRound(b);
+    }
     if (ra < rb) {
         return -1;
     } else {
@@ -182,7 +187,7 @@ void TextLine::paint(ParagraphPainter* painter, SkScalar x, SkScalar y) {
             [painter, x, y, this]
             (const Run* run, SkScalar runOffsetInLine, TextRange textRange, SkScalar* runWidthInLine) {
                 *runWidthInLine = this->iterateThroughSingleRunByStyles(
-                run, runOffsetInLine, textRange, StyleType::kBackground,
+                TextAdjustment::GlyphCluster, run, runOffsetInLine, textRange, StyleType::kBackground,
                 [painter, x, y, this](TextRange textRange, const TextStyle& style, const ClipContext& context) {
                     this->paintBackground(painter, x, y, textRange, style, context);
                 });
@@ -195,7 +200,7 @@ void TextLine::paint(ParagraphPainter* painter, SkScalar x, SkScalar y) {
             [painter, x, y, this]
             (const Run* run, SkScalar runOffsetInLine, TextRange textRange, SkScalar* runWidthInLine) {
             *runWidthInLine = this->iterateThroughSingleRunByStyles(
-                run, runOffsetInLine, textRange, StyleType::kShadow,
+                TextAdjustment::GlyphCluster, run, runOffsetInLine, textRange, StyleType::kShadow,
                 [painter, x, y, this]
                 (TextRange textRange, const TextStyle& style, const ClipContext& context) {
                     this->paintShadow(painter, x, y, textRange, style, context);
@@ -204,7 +209,7 @@ void TextLine::paint(ParagraphPainter* painter, SkScalar x, SkScalar y) {
             });
     }
 
-    ensureTextBlobCachePopulated();
+    this->ensureTextBlobCachePopulated();
 
     for (auto& record : fTextBlobCache) {
         record.paint(painter, x, y);
@@ -215,7 +220,7 @@ void TextLine::paint(ParagraphPainter* painter, SkScalar x, SkScalar y) {
             [painter, x, y, this]
             (const Run* run, SkScalar runOffsetInLine, TextRange textRange, SkScalar* runWidthInLine) {
                 *runWidthInLine = this->iterateThroughSingleRunByStyles(
-                run, runOffsetInLine, textRange, StyleType::kDecorations,
+                TextAdjustment::GlyphCluster, run, runOffsetInLine, textRange, StyleType::kDecorations,
                 [painter, x, y, this]
                 (TextRange textRange, const TextStyle& style, const ClipContext& context) {
                     this->paintDecorations(painter, x, y, textRange, style, context);
@@ -273,6 +278,7 @@ void TextLine::ensureTextBlobCachePopulated() {
                    return true;
                }
                *runWidthInLine = this->iterateThroughSingleRunByStyles(
+                   TextAdjustment::GlyphCluster,
                    run,
                    runOffsetInLine,
                    textRange,
@@ -312,15 +318,23 @@ void TextLine::scanStyles(StyleType styleType, const RunStyleVisitor& visitor) {
         return;
     }
 
-    this->iterateThroughVisualRuns(false,
-        [this, visitor, styleType](const Run* run, SkScalar runOffset, TextRange textRange, SkScalar* width) {
-            *width = this->iterateThroughSingleRunByStyles(
-                run, runOffset, textRange, styleType,
-                [visitor](TextRange textRange, const TextStyle& style, const ClipContext& context) {
-                    visitor(textRange, style, context);
-                });
-            return true;
-        });
+    this->iterateThroughVisualRuns(
+            false,
+            [this, visitor, styleType](
+                    const Run* run, SkScalar runOffset, TextRange textRange, SkScalar* width) {
+                *width = this->iterateThroughSingleRunByStyles(
+                        TextAdjustment::GlyphCluster,
+                        run,
+                        runOffset,
+                        textRange,
+                        styleType,
+                        [visitor](TextRange textRange,
+                                  const TextStyle& style,
+                                  const ClipContext& context) {
+                            visitor(textRange, style, context);
+                        });
+                return true;
+            });
 }
 
 SkRect TextLine::extendHeight(const ClipContext& context) const {
@@ -465,7 +479,7 @@ void TextLine::justify(SkScalar maxWidth) {
 
         if (ghost) {
             if (cluster->run().leftToRight()) {
-                shiftCluster(cluster, ghostShift, ghostShift);
+                this->shiftCluster(cluster, ghostShift, ghostShift);
             }
             return true;
         }
@@ -517,86 +531,53 @@ void TextLine::createEllipsis(SkScalar maxWidth, const SkString& ellipsis, bool)
     // Go through the clusters in the reverse logical order
     // taking off cluster by cluster until the ellipsis fits
     SkScalar width = fAdvance.fX;
-    // There is one case when we need to attach the ellipsis on the left:
-    // when the first few runs together wider than the ellipsis run are RTL
-    // In all the other cases we attach the ellipsis on the right
-    RunIndex leftRun = EMPTY_RUN;
+    RunIndex lastRun = EMPTY_RUN;
     std::unique_ptr<Run> ellipsisRun;
-    iterateThroughClustersInGlyphsOrder(
-        false, false, [&](const Cluster* cluster, bool ghost) {
-            if (cluster->run().leftToRight()) {
-                return false;
+    for (auto clusterIndex = fGhostClusterRange.end; clusterIndex > fGhostClusterRange.start; --clusterIndex) {
+        auto& cluster = fOwner->cluster(clusterIndex - 1);
+        // Shape the ellipsis if the run has changed
+        if (lastRun != cluster.runIndex()) {
+            ellipsisRun = this->shapeEllipsis(ellipsis, &cluster);
+            if (ellipsisRun->advance().fX > maxWidth) {
+                // Ellipsis is bigger than the entire line; no way we can add it at all
+                // BUT! We can keep scanning in case the next run will give us better results
+                lastRun = EMPTY_RUN;
+                continue;
+            } else {
+                // We may need to continue
+                lastRun = cluster.runIndex();
             }
-            // Shape the ellipsis if the run has changed
-            if (leftRun != cluster->runIndex()) {
-                ellipsisRun = shapeEllipsis(ellipsis, cluster);
-                if (ellipsisRun->advance().fX > maxWidth) {
-                    // Ellipsis is bigger than the entire line; no way we can add it at all
-                    return false;
-                }
-                ellipsisRun->fClusterStart = cluster->textRange().start;
-                ellipsisRun->setOwner(fOwner);
-                leftRun = cluster->runIndex();
-            }
-            // See if it fits
-            if (width + ellipsisRun->advance().fX > maxWidth) {
-                width -= cluster->width();
-                // Continue if the ellipsis does not fit
-                return true;
-            }
-            fEllipsis = std::move(ellipsisRun);
-            fEllipsis->fBidiLevel = 1;
-            fClusterRange.end = cluster - fOwner->clusters().data() + 1;
-            fGhostClusterRange.end = cluster - fOwner->clusters().data() + 1;
-            fText.end = cluster->textRange().end;
-            fTextIncludingNewlines.end = cluster->textRange().end;
-            fTextExcludingSpaces.end = cluster->textRange().end;
-            fAdvance.fX = width;
-            return false;
-        });
+        }
+        // See if it fits
+        if (width + ellipsisRun->advance().fX > maxWidth) {
+            width -= cluster.width();
+            // Continue if the ellipsis does not fit
+            continue;
+        }
+        // We found enough room for the ellipsis
+        fAdvance.fX = width;
+        fEllipsis = std::move(ellipsisRun);
+        fEllipsis->setOwner(fOwner);
 
-    if (fEllipsis) return;
+        // Let's update the line
+        fClusterRange.end = clusterIndex;
+        fGhostClusterRange.end = fClusterRange.end;
+        fEllipsis->fClusterStart = cluster.textRange().start;
+        fText.end = cluster.textRange().end;
+        fTextIncludingNewlines.end = cluster.textRange().end;
+        fTextExcludingSpaces.end = cluster.textRange().end;
+        break;
+    }
 
-    RunIndex rightRun = EMPTY_RUN;
-    iterateThroughClustersInGlyphsOrder(
-        true, false, [&](const Cluster* cluster, bool ghost) {
-            // Shape the ellipsis if the run has changed
-            if (rightRun != cluster->runIndex()) {
-                // Shape the ellipsis
-                ellipsisRun = shapeEllipsis(ellipsis, cluster);
-                if (ellipsisRun->advance().fX > maxWidth) {
-                    // Ellipsis is bigger than the entire line
-                    return false;
-                }
-                ellipsisRun->fClusterStart = cluster->textRange().start;
-                ellipsisRun->setOwner(fOwner);
-                rightRun = cluster->runIndex();
-            }
-            // See if it fits
-            if (width + ellipsisRun->advance().fX > maxWidth) {
-                width -= cluster->width();
-                // Continue if the ellipsis does not fit
-                return true;
-            }
-            fEllipsis = std::move(ellipsisRun);
-            fEllipsis->fBidiLevel = 0;
-            fClusterRange.end = cluster - fOwner->clusters().data() + 1;
-            fGhostClusterRange.end = fClusterRange.end;
-            fText.end = cluster->textRange().end;
-            fTextIncludingNewlines.end = cluster->textRange().end;
-            fTextExcludingSpaces.end = cluster->textRange().end;
-            fAdvance.fX = width;
-            return false;
-        });
-    if (fEllipsis) return;
-
-    // Weird situation: ellipsis does not fit; no ellipsis then
-    fClusterRange.end = fClusterRange.start;
-    fGhostClusterRange.end = fClusterRange.start;
-    fText.end = fText.start;
-    fTextIncludingNewlines.end = fTextIncludingNewlines.start;
-    fTextExcludingSpaces.end = fTextExcludingSpaces.start;
-    fAdvance.fX = 0;
+    if (!fEllipsis) {
+        // Weird situation: ellipsis does not fit; no ellipsis then
+        fClusterRange.end = fClusterRange.start;
+        fGhostClusterRange.end = fClusterRange.start;
+        fText.end = fText.start;
+        fTextIncludingNewlines.end = fTextIncludingNewlines.start;
+        fTextExcludingSpaces.end = fTextExcludingSpaces.start;
+        fAdvance.fX = 0;
+    }
 }
 
 static inline SkUnichar nextUtf8Unit(const char** ptr, const char* end) {
@@ -714,7 +695,7 @@ TextLine::ClipContext TextLine::measureTextInsideOneRun(TextRange textRange,
                                                         SkScalar runOffsetInLine,
                                                         SkScalar textOffsetInRunInLine,
                                                         bool includeGhostSpaces,
-                                                        bool limitToGraphemes) const {
+                                                        TextAdjustment textAdjustment) const {
     ClipContext result = { run, 0, run->size(), 0, SkRect::MakeEmpty(), 0, false };
 
     if (run->fEllipsis) {
@@ -736,48 +717,31 @@ TextLine::ClipContext TextLine::measureTextInsideOneRun(TextRange textRange,
             result.clip = SkRect::MakeXYWH(runOffsetInLine, run->fFontMetrics.fAscent, run->advance().fX, 0);
         }
         return result;
+    } else if (textRange.empty()) {
+        return result;
     }
+
+    TextRange originalTextRange(textRange); // We need it for proportional measurement
     // Find [start:end] clusters for the text
-    Cluster* start = nullptr;
-    Cluster* end = nullptr;
-    do {
+    while (true) {
+        // Update textRange by cluster edges (shift start up to the edge of the cluster)
+        // TODO: remove this limitation?
+        TextRange updatedTextRange;
         bool found;
-        ClusterIndex startIndex;
-        ClusterIndex endIndex;
-        std::tie(found, startIndex, endIndex) = run->findLimitingClusters(textRange);
+        std::tie(found, updatedTextRange.start, updatedTextRange.end) =
+                                        run->findLimitingGlyphClusters(textRange);
         if (!found) {
             return result;
         }
-        start = &fOwner->cluster(startIndex);
-        end = &fOwner->cluster(endIndex);
 
-        if (!limitToGraphemes) {
+        if ((textAdjustment & TextAdjustment::Grapheme) == 0) {
+            textRange = updatedTextRange;
             break;
         }
 
-        // Update textRange by cluster edges (shift start up to the edge of the cluster)
-        TextRange updatedTextRange(textRange);
-        if (run->leftToRight()) {
-            if (textRange.start != start->textRange().start) {
-                updatedTextRange.start = start->textRange().end;
-            }
-            updatedTextRange.end = end->textRange().end;
-        } else {
-            if (textRange.start != end->textRange().start) {
-                updatedTextRange.start = end->textRange().end;
-            }
-            updatedTextRange.end = start->textRange().end;
-        }
-        SkASSERT(updatedTextRange.start >= textRange.start);
-
         // Update text range by grapheme edges (shift start up to the edge of the grapheme)
         std::tie(found, updatedTextRange.start, updatedTextRange.end) =
-                run->findLimitingGraphemes(updatedTextRange);
-        SkASSERT(updatedTextRange.start >= textRange.start);
-        SkASSERT(updatedTextRange.end >= textRange.end);
-
-        // Updated text could only shift right from the initial text
-        // so this loop is going to break soon enough
+                                    run->findLimitingGraphemes(updatedTextRange);
         if (updatedTextRange == textRange) {
             break;
         }
@@ -787,13 +751,20 @@ TextLine::ClipContext TextLine::measureTextInsideOneRun(TextRange textRange,
         textRange = updatedTextRange;
 
         // Move the start until it's on the grapheme edge (and glypheme, too)
-    } while (true);
+    }
+    Cluster* start = &fOwner->cluster(fOwner->clusterIndex(textRange.start));
+    Cluster* end = &fOwner->cluster(fOwner->clusterIndex(textRange.end - (textRange.width() == 0 ? 0 : 1)));
 
+    if (!run->leftToRight()) {
+        std::swap(start, end);
+    }
     result.pos = start->startPos();
     result.size = (end->isHardBreak() ? end->startPos() : end->endPos()) - start->startPos();
-
     auto textStartInRun = run->positionX(start->startPos());
     auto textStartInLine = runOffsetInLine + textOffsetInRunInLine;
+    if (!run->leftToRight()) {
+        std::swap(start, end);
+    }
 /*
     if (!run->fJustificationShifts.empty()) {
         SkDebugf("Justification for [%d:%d)\n", textRange.start, textRange.end);
@@ -818,20 +789,33 @@ TextLine::ClipContext TextLine::measureTextInsideOneRun(TextRange textRange,
     // TODO: This is where we get smart about selecting a part of a cluster
     //  by shaping each grapheme separately and then use the result sizes
     //  to calculate the proportions
-    auto leftCorrection = start->sizeToChar(textRange.start);
-    auto rightCorrection = end->sizeFromChar(textRange.end - 1);
-    result.clip.fLeft += leftCorrection;
-    result.clip.fRight -= rightCorrection;
+    auto leftCorrection = start->sizeToChar(originalTextRange.start);
+    auto rightCorrection = end->sizeFromChar(originalTextRange.end - 1);
+    /*
+    SkDebugf("[%d: %d) => [%d: %d), @%d, %d: [%f:%f) + [%f:%f) = ", // جَآَهُ
+             originalTextRange.start, originalTextRange.end, textRange.start, textRange.end,
+             result.pos, result.size,
+             result.clip.fLeft, result.clip.fRight, leftCorrection, rightCorrection);
+     */
     result.clippingNeeded = leftCorrection != 0 || rightCorrection != 0;
+    if (run->leftToRight()) {
+        result.clip.fLeft += leftCorrection;
+        result.clip.fRight -= rightCorrection;
+        textStartInLine -= leftCorrection;
+    } else {
+        result.clip.fRight -= leftCorrection;
+        result.clip.fLeft += rightCorrection;
+        textStartInLine -= rightCorrection;
+    }
 
-    textStartInLine -= leftCorrection;
     result.clip.offset(textStartInLine, 0);
+    //SkDebugf("@%f[%f:%f)\n", textStartInLine, result.clip.fLeft, result.clip.fRight);
 
-    if (compareRound(result.clip.fRight, fAdvance.fX) > 0 && !includeGhostSpaces) {
+    if (compareRound(result.clip.fRight, fAdvance.fX, fOwner->getApplyRoundingHack()) > 0 && !includeGhostSpaces) {
         // There are few cases when we need it.
         // The most important one: we measure the text with spaces at the end (or at the beginning in RTL)
         // and we should ignore these spaces
-        if (run->leftToRight()) {
+        if (fOwner->paragraphStyle().getTextDirection() == TextDirection::kLtr) {
             // We only use this member for LTR
             result.fExcludedTrailingSpaces = std::max(result.clip.fRight - fAdvance.fX, 0.0f);
             result.clippingNeeded = true;
@@ -880,21 +864,20 @@ void TextLine::iterateThroughClustersInGlyphsOrder(bool reversed,
     });
 }
 
-SkScalar TextLine::iterateThroughSingleRunByStyles(const Run* run,
+SkScalar TextLine::iterateThroughSingleRunByStyles(TextAdjustment textAdjustment,
+                                                   const Run* run,
                                                    SkScalar runOffset,
                                                    TextRange textRange,
                                                    StyleType styleType,
                                                    const RunStyleVisitor& visitor) const {
-
     auto correctContext = [&](TextRange textRange, SkScalar textOffsetInRun) -> ClipContext {
         auto result = this->measureTextInsideOneRun(
-                                        textRange, run, runOffset, textOffsetInRun, false, true);
+                textRange, run, runOffset, textOffsetInRun, false, textAdjustment);
         if (styleType == StyleType::kDecorations) {
             // Decorations are drawn based on the real font metrics (regardless of styles and strut)
             result.clip.fTop = this->sizes().runTop(run, LineMetricStyle::CSS);
-            result.clip.fBottom =
-                    result.clip.fTop +
-                    run->calculateHeight(LineMetricStyle::CSS, LineMetricStyle::CSS);
+            result.clip.fBottom = result.clip.fTop +
+                                  run->calculateHeight(LineMetricStyle::CSS, LineMetricStyle::CSS);
         }
         return result;
     };
@@ -997,7 +980,7 @@ void TextLine::iterateThroughVisualRuns(bool includingGhostSpaces, const RunVisi
     SkScalar totalWidth = 0;
     auto textRange = includingGhostSpaces ? this->textWithNewlines() : this->trimmedText();
 
-    if (this->ellipsis() != nullptr && !this->ellipsis()->leftToRight()) {
+    if (this->ellipsis() != nullptr && fOwner->paragraphStyle().getTextDirection() == TextDirection::kRtl) {
         runOffset = this->ellipsis()->offset().fX;
         if (visitor(ellipsis(), runOffset, ellipsis()->textRange(), &width)) {
         }
@@ -1018,7 +1001,7 @@ void TextLine::iterateThroughVisualRuns(bool includingGhostSpaces, const RunVisi
             TextRange whitespaces = intersected(
                     TextRange(fTextExcludingSpaces.end, fTextIncludingNewlines.end), run->fTextRange);
             if (whitespaces.width() > 0) {
-                auto whitespacesLen = measureTextInsideOneRun(whitespaces, run, runOffset, 0, true, false).clip.width();
+                auto whitespacesLen = measureTextInsideOneRun(whitespaces, run, runOffset, 0, true, TextAdjustment::GlyphCluster).clip.width();
                 runOffset -= whitespacesLen;
             }
         }
@@ -1032,15 +1015,15 @@ void TextLine::iterateThroughVisualRuns(bool includingGhostSpaces, const RunVisi
     runOffset += width;
     totalWidth += width;
 
-    if (this->ellipsis() != nullptr && this->ellipsis()->leftToRight()) {
+    if (this->ellipsis() != nullptr && fOwner->paragraphStyle().getTextDirection() == TextDirection::kLtr) {
         if (visitor(ellipsis(), runOffset, ellipsis()->textRange(), &width)) {
             totalWidth += width;
         }
     }
 
+    if (!includingGhostSpaces && compareRound(totalWidth, this->width(), fOwner->getApplyRoundingHack()) != 0) {
     // This is a very important assert!
     // It asserts that 2 different ways of calculation come with the same results
-    if (!includingGhostSpaces && compareRound(totalWidth, this->width()) != 0) {
         SkDebugf("ASSERT: %f != %f\n", totalWidth, this->width());
         SkASSERT(false);
     }
@@ -1063,8 +1046,12 @@ LineMetrics TextLine::getMetrics() const {
     result.fAscent = - fMaxRunMetrics.ascent();
     result.fDescent = fMaxRunMetrics.descent();
     result.fUnscaledAscent = - fMaxRunMetrics.ascent(); // TODO: implement
-    result.fHeight = littleRound(fAdvance.fY);
-    result.fWidth = littleRound(fAdvance.fX);
+    result.fHeight = fAdvance.fY;
+    result.fWidth = fAdvance.fX;
+    if (fOwner->getApplyRoundingHack()) {
+        result.fHeight = littleRound(result.fHeight);
+        result.fWidth = littleRound(result.fWidth);
+    }
     result.fLeft = this->offset().fX;
     // This is Flutter definition of a baseline
     result.fBaseline = this->offset().fY + this->height() - this->sizes().descent();
@@ -1079,7 +1066,7 @@ LineMetrics TextLine::getMetrics() const {
             return true;
         }
         *runWidthInLine = this->iterateThroughSingleRunByStyles(
-        run, runOffsetInLine, textRange, StyleType::kForeground,
+        TextAdjustment::GlyphCluster, run, runOffsetInLine, textRange, StyleType::kForeground,
         [&result, &run](TextRange textRange, const TextStyle& style, const ClipContext& context) {
             SkFontMetrics fontMetrics;
             run->fFont.getMetrics(&fontMetrics);
@@ -1111,7 +1098,7 @@ bool TextLine::endsWithHardLineBreak() const {
 void TextLine::getRectsForRange(TextRange textRange0,
                                 RectHeightStyle rectHeightStyle,
                                 RectWidthStyle rectWidthStyle,
-                                std::vector<TextBox>& boxes)
+                                std::vector<TextBox>& boxes) const
 {
     const Run* lastRun = nullptr;
     auto startBox = boxes.size();
@@ -1119,7 +1106,7 @@ void TextLine::getRectsForRange(TextRange textRange0,
         [textRange0, rectHeightStyle, rectWidthStyle, &boxes, &lastRun, startBox, this]
         (const Run* run, SkScalar runOffsetInLine, TextRange textRange, SkScalar* runWidthInLine) {
         *runWidthInLine = this->iterateThroughSingleRunByStyles(
-        run, runOffsetInLine, textRange, StyleType::kNone,
+        TextAdjustment::GraphemeGluster, run, runOffsetInLine, textRange, StyleType::kNone,
         [run, runOffsetInLine, textRange0, rectHeightStyle, rectWidthStyle, &boxes, &lastRun, startBox, this]
         (TextRange textRange, const TextStyle& style, const TextLine::ClipContext& lineContext) {
 
@@ -1131,7 +1118,8 @@ void TextLine::getRectsForRange(TextRange textRange0,
             auto paragraphStyle = fOwner->paragraphStyle();
 
             // Found a run that intersects with the text
-            auto context = this->measureTextInsideOneRun(intersect, run, runOffsetInLine, 0, true, true);
+            auto context = this->measureTextInsideOneRun(
+                    intersect, run, runOffsetInLine, 0, true, TextAdjustment::GraphemeGluster);
             SkRect clip = context.clip;
             clip.offset(lineContext.fTextShift - context.fTextShift, 0);
 
@@ -1302,11 +1290,13 @@ void TextLine::getRectsForRange(TextRange textRange0,
         });
         return true;
     });
-    for (auto& r : boxes) {
-        r.rect.fLeft = littleRound(r.rect.fLeft);
-        r.rect.fRight = littleRound(r.rect.fRight);
-        r.rect.fTop = littleRound(r.rect.fTop);
-        r.rect.fBottom = littleRound(r.rect.fBottom);
+    if (fOwner->getApplyRoundingHack()) {
+        for (auto& r : boxes) {
+            r.rect.fLeft = littleRound(r.rect.fLeft);
+            r.rect.fRight = littleRound(r.rect.fRight);
+            r.rect.fTop = littleRound(r.rect.fTop);
+            r.rect.fBottom = littleRound(r.rect.fBottom);
+        }
     }
 }
 
@@ -1325,7 +1315,7 @@ PositionWithAffinity TextLine::getGlyphPositionAtCoordinate(SkScalar dx) {
         (const Run* run, SkScalar runOffsetInLine, TextRange textRange, SkScalar* runWidthInLine) {
             bool keepLooking = true;
             *runWidthInLine = this->iterateThroughSingleRunByStyles(
-            run, runOffsetInLine, textRange, StyleType::kNone,
+            TextAdjustment::GraphemeGluster, run, runOffsetInLine, textRange, StyleType::kNone,
             [this, run, dx, &result, &keepLooking]
             (TextRange textRange, const TextStyle& style, const TextLine::ClipContext& context0) {
 
@@ -1338,19 +1328,22 @@ PositionWithAffinity TextLine::getGlyphPositionAtCoordinate(SkScalar dx) {
                 } else {
                     // Clip starts from 0; we cannot extend it to the left from that
                 }
+                // However, we need to offset the clip
+                context.clip.offset(offsetX, 0.0f);
+
                 // This patch will help us to avoid a floating point error
-                if (SkScalarNearlyEqual(context.clip.fRight, dx - offsetX, 0.01f)) {
-                    context.clip.fRight = dx - offsetX;
+                if (SkScalarNearlyEqual(context.clip.fRight, dx, 0.01f)) {
+                    context.clip.fRight = dx;
                 }
 
-                if (dx <= context.clip.fLeft + offsetX) {
+                if (dx <= context.clip.fLeft) {
                     // All the other runs are placed right of this one
                     auto utf16Index = fOwner->getUTF16Index(context.run->globalClusterIndex(context.pos));
                     if (run->leftToRight()) {
                         result = { SkToS32(utf16Index), kDownstream};
                         keepLooking = false;
                     } else {
-                        result = { SkToS32(utf16Index), kDownstream};
+                        result = { SkToS32(utf16Index + 1), kUpstream};
                         // If we haven't reached the end of the run we need to keep looking
                         keepLooking = context.pos != 0;
                     }
@@ -1358,7 +1351,7 @@ PositionWithAffinity TextLine::getGlyphPositionAtCoordinate(SkScalar dx) {
                     return !run->leftToRight();
                 }
 
-                if (dx >= context.clip.fRight + offsetX) {
+                if (dx >= context.clip.fRight) {
                     // We have to keep looking ; just in case keep the last one as the closest
                     auto utf16Index = fOwner->getUTF16Index(context.run->globalClusterIndex(context.pos + context.size));
                     if (run->leftToRight()) {
@@ -1376,7 +1369,10 @@ PositionWithAffinity TextLine::getGlyphPositionAtCoordinate(SkScalar dx) {
                 size_t found = context.pos;
                 for (size_t index = context.pos; index < context.pos + context.size; ++index) {
                     // TODO: this rounding is done to match Flutter tests. Must be removed..
-                    auto end = littleRound(context.run->positionX(index) + context.fTextShift + offsetX);
+                    auto end = context.run->positionX(index) + context.fTextShift + offsetX;
+                    if (fOwner->getApplyRoundingHack()) {
+                        end = littleRound(end);
+                    }
                     if (end > dx) {
                         break;
                     } else if (end == dx && !context.run->leftToRight()) {
@@ -1388,17 +1384,34 @@ PositionWithAffinity TextLine::getGlyphPositionAtCoordinate(SkScalar dx) {
                 }
 
                 SkScalar glyphemePosLeft = context.run->positionX(found) + context.fTextShift + offsetX;
-                SkScalar glyphemePosWidth = context.run->positionX(found + 1) - context.run->positionX(found);
+                SkScalar glyphemesWidth = context.run->positionX(found + 1) - context.run->positionX(found);
 
                 // Find the grapheme range that contains the point
                 auto clusterIndex8 = context.run->globalClusterIndex(found);
                 auto clusterEnd8 = context.run->globalClusterIndex(found + 1);
+                auto graphemes = fOwner->countSurroundingGraphemes({clusterIndex8, clusterEnd8});
 
-                SkScalar center = glyphemePosLeft + glyphemePosWidth / 2;
-                if ((dx < center) == context.run->leftToRight()) {
-                    size_t utf16Index = context.run->leftToRight()
-                                                ? fOwner->getUTF16Index(clusterIndex8)
-                                                : fOwner->getUTF16Index(clusterEnd8) + 1;
+                SkScalar center = glyphemePosLeft + glyphemesWidth / 2;
+                if (graphemes.size() > 1) {
+                    // Calculate the position proportionally based on grapheme count
+                    SkScalar averageGraphemeWidth = glyphemesWidth / graphemes.size();
+                    SkScalar delta = dx - glyphemePosLeft;
+                    int graphemeIndex = SkScalarNearlyZero(averageGraphemeWidth)
+                                         ? 0
+                                         : SkScalarFloorToInt(delta / averageGraphemeWidth);
+                    auto graphemeCenter = glyphemePosLeft + graphemeIndex * averageGraphemeWidth +
+                                          averageGraphemeWidth / 2;
+                    auto graphemeUtf8Index = graphemes[graphemeIndex];
+                    if ((dx < graphemeCenter) == context.run->leftToRight()) {
+                        size_t utf16Index = fOwner->getUTF16Index(graphemeUtf8Index);
+                        result = { SkToS32(utf16Index), kDownstream };
+                    } else {
+                        size_t utf16Index = fOwner->getUTF16Index(graphemeUtf8Index + 1);
+                        result = { SkToS32(utf16Index), kUpstream };
+                    }
+                    // Keep UTF16 index as is
+                } else if ((dx < center) == context.run->leftToRight()) {
+                    size_t utf16Index = fOwner->getUTF16Index(clusterIndex8);
                     result = { SkToS32(utf16Index), kDownstream };
                 } else {
                     size_t utf16Index = context.run->leftToRight()
@@ -1421,8 +1434,9 @@ void TextLine::getRectsForPlaceholders(std::vector<TextBox>& boxes) {
         true,
         [&boxes, this](const Run* run, SkScalar runOffset, TextRange textRange,
                         SkScalar* width) {
-            auto context = this->measureTextInsideOneRun(textRange, run, runOffset, 0, true, false);
-            *width = context.clip.width();
+                auto context = this->measureTextInsideOneRun(
+                        textRange, run, runOffset, 0, true, TextAdjustment::GraphemeGluster);
+                *width = context.clip.width();
 
             if (textRange.width() == 0) {
                 return true;
@@ -1434,10 +1448,12 @@ void TextLine::getRectsForPlaceholders(std::vector<TextBox>& boxes) {
             SkRect clip = context.clip;
             clip.offset(this->offset());
 
-            clip.fLeft = littleRound(clip.fLeft);
-            clip.fRight = littleRound(clip.fRight);
-            clip.fTop = littleRound(clip.fTop);
-            clip.fBottom = littleRound(clip.fBottom);
+            if (fOwner->getApplyRoundingHack()) {
+                clip.fLeft = littleRound(clip.fLeft);
+                clip.fRight = littleRound(clip.fRight);
+                clip.fTop = littleRound(clip.fTop);
+                clip.fBottom = littleRound(clip.fBottom);
+            }
             boxes.emplace_back(clip, run->getTextDirection());
             return true;
         });
