@@ -5,6 +5,7 @@
 #include "include/core/SkPoint.h"
 #include "include/core/SkRect.h"
 #include "include/core/SkScalar.h"
+#include "include/private/SkBitmaskEnum.h" // IWYU pragma: keep
 #include "include/private/base/SkTArray.h"
 #include "modules/skparagraph/include/DartTypes.h"
 #include "modules/skparagraph/include/Metrics.h"
@@ -37,6 +38,13 @@ public:
       bool clippingNeeded;
     };
 
+    enum TextAdjustment {
+        GlyphCluster = 0x01,    // All text producing glyphs pointing to the same ClusterIndex
+        GlyphemeCluster = 0x02, // base glyph + all attached diacritics
+        Grapheme = 0x04,        // Text adjusted to graphemes
+        GraphemeGluster = 0x05, // GlyphCluster & Grapheme
+    };
+
     TextLine() = default;
     TextLine(const TextLine&) = delete;
     TextLine& operator=(const TextLine&) = delete;
@@ -60,7 +68,7 @@ public:
     TextRange textWithNewlines() const { return fTextIncludingNewlines; }
     TextRange text() const { return fText; }
     ClusterRange clusters() const { return fClusterRange; }
-    ClusterRange clustersWithSpaces() { return fGhostClusterRange; }
+    ClusterRange clustersWithSpaces() const { return fGhostClusterRange; }
     Run* ellipsis() const { return fEllipsis.get(); }
     InternalLineMetrics sizes() const { return fSizes; }
     bool empty() const { return fTextExcludingSpaces.empty(); }
@@ -70,20 +78,29 @@ public:
     SkScalar width() const {
         return fAdvance.fX + (fEllipsis != nullptr ? fEllipsis->fAdvance.fX : 0);
     }
+    SkScalar widthWithoutEllipsis() const { return fAdvance.fX; }
     SkVector offset() const;
 
     SkScalar alphabeticBaseline() const { return fSizes.alphabeticBaseline(); }
     SkScalar ideographicBaseline() const { return fSizes.ideographicBaseline(); }
     SkScalar baseline() const { return fSizes.baseline(); }
 
-    using RunVisitor = std::function<bool(const Run* run, SkScalar runOffset, TextRange textRange, SkScalar* width)>;
+    using RunVisitor = std::function<bool(
+            const Run* run, SkScalar runOffset, TextRange textRange, SkScalar* width)>;
     void iterateThroughVisualRuns(bool includingGhostSpaces, const RunVisitor& runVisitor) const;
-    using RunStyleVisitor = std::function<void(TextRange textRange, const TextStyle& style, const ClipContext& context)>;
-    SkScalar iterateThroughSingleRunByStyles(const Run* run, SkScalar runOffset, TextRange textRange,
-                                         StyleType styleType, const RunStyleVisitor& visitor) const;
+    using RunStyleVisitor = std::function<void(
+            TextRange textRange, const TextStyle& style, const ClipContext& context)>;
+    SkScalar iterateThroughSingleRunByStyles(TextAdjustment textAdjustment,
+                                             const Run* run,
+                                             SkScalar runOffset,
+                                             TextRange textRange,
+                                             StyleType styleType,
+                                             const RunStyleVisitor& visitor) const;
 
     using ClustersVisitor = std::function<bool(const Cluster* cluster, bool ghost)>;
-    void iterateThroughClustersInGlyphsOrder(bool reverse, bool includeGhosts, const ClustersVisitor& visitor) const;
+    void iterateThroughClustersInGlyphsOrder(bool reverse,
+                                             bool includeGhosts,
+                                             const ClustersVisitor& visitor) const;
 
     void format(TextAlign align, SkScalar maxWidth);
     void paint(ParagraphPainter* painter, SkScalar x, SkScalar y);
@@ -100,7 +117,10 @@ public:
 
     bool isFirstLine() const;
     bool isLastLine() const;
-    void getRectsForRange(TextRange textRange, RectHeightStyle rectHeightStyle, RectWidthStyle rectWidthStyle, std::vector<TextBox>& boxes);
+    void getRectsForRange(TextRange textRange,
+                          RectHeightStyle rectHeightStyle,
+                          RectWidthStyle rectWidthStyle,
+                          std::vector<TextBox>& boxes) const;
     void getRectsForPlaceholders(std::vector<TextBox>& boxes);
     PositionWithAffinity getGlyphPositionAtCoordinate(SkScalar dx);
 
@@ -109,7 +129,7 @@ public:
                                         SkScalar runOffsetInLine,
                                         SkScalar textOffsetInRunInLine,
                                         bool includeGhostSpaces,
-                                        bool limitToGraphemes) const;
+                                        TextAdjustment textAdjustment) const;
 
     LineMetrics getMetrics() const;
 
@@ -156,7 +176,7 @@ private:
     ClusterRange fClusterRange;
     ClusterRange fGhostClusterRange;
     // Avoid the malloc/free in the common case of one run per line
-    SkSTArray<1, size_t, true> fRunsInVisualOrder;
+    skia_private::STArray<1, size_t, true> fRunsInVisualOrder;
     SkVector fAdvance;                  // Text size
     SkVector fOffset;                   // Text position
     SkScalar fShift;                    // Let right
@@ -192,4 +212,8 @@ public:
 }  // namespace textlayout
 }  // namespace skia
 
+namespace sknonstd {
+    template <> struct is_bitmask_enum<skia::textlayout::TextLine::TextAdjustment> : std::true_type {};
+}  // namespace sknonstd
+
 #endif  // TextLine_DEFINED
