@@ -21,16 +21,16 @@ class GeometryNode;
 class ClipEffect final : public EffectNode {
 public:
     static sk_sp<ClipEffect> Make(sk_sp<RenderNode> child, sk_sp<GeometryNode> clip,
-                                  bool aa = false) {
+                                  bool aa = false, bool force_clip = false) {
         return (child && clip)
-            ? sk_sp<ClipEffect>(new ClipEffect(std::move(child), std::move(clip), aa))
+            ? sk_sp<ClipEffect>(new ClipEffect(std::move(child), std::move(clip), aa, force_clip))
             : nullptr;
     }
 
     ~ClipEffect() override;
 
 protected:
-    ClipEffect(sk_sp<RenderNode>, sk_sp<GeometryNode>, bool aa);
+    ClipEffect(sk_sp<RenderNode>, sk_sp<GeometryNode>, bool aa, bool force_clip);
 
     void onRender(SkCanvas*, const RenderContext*) const override;
     const RenderNode* onNodeAt(const SkPoint&)     const override;
@@ -39,7 +39,8 @@ protected:
 
 private:
     const sk_sp<GeometryNode> fClipNode;
-    const bool                fAntiAlias;
+    const bool                fAntiAlias,
+                              fForceClip;
 
     bool                      fNoop = false;
 
