@@ -11,7 +11,7 @@
 #include "modules/svg/include/SkSVGHiddenContainer.h"
 #include "modules/svg/include/SkSVGTypes.h"
 
-class SkSVGMask final : public SkSVGHiddenContainer {
+class SK_API SkSVGMask final : public SkSVGHiddenContainer {
 public:
     static sk_sp<SkSVGMask> Make() {
         return sk_sp<SkSVGMask>(new SkSVGMask());
