@@ -27,7 +27,8 @@
 // Copyright 2021 Georgiy Sgibnev <georgiy@sgibnev.com>. Work sponsored by lab50.net.
 // Copyright 2021 Theofilos Intzoglou <int.teo@gmail.com>
 // Copyright 2022 Alexander Sulfrian <asulfrian@zedat.fu-berlin.de>
-// Copyright 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright 2023, 2024 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright 2024 Pratham Gandhi <ppg.1382@gmail.com>
 //
 //========================================================================
 
@@ -38,12 +39,14 @@
 #include "CharTypes.h"
 #include "Object.h"
 #include "poppler_private_export.h"
+#include "SignatureInfo.h"
 
 #include <ctime>
 
 #include <optional>
 #include <set>
 #include <vector>
+#include <functional>
 
 class GooString;
 class Array;
@@ -54,7 +57,6 @@ class Annots;
 class LinkAction;
 class GfxResources;
 class PDFDoc;
-class SignatureInfo;
 class X509CertificateInfo;
 namespace CryptoSign {
 class VerificationInterface;
@@ -140,7 +142,7 @@ public:
 
     LinkAction *getActivationAction(); // The caller should not delete the result
     std::unique_ptr<LinkAction> getAdditionalAction(Annot::FormAdditionalActionsType type);
-    bool setAdditionalAction(Annot::FormAdditionalActionsType t, const GooString &js);
+    bool setAdditionalAction(Annot::FormAdditionalActionsType t, const std::string &js);
 
     // return the unique ID corresponding to pageNum/fieldNum
     static int encodeID(unsigned pageNum, unsigned fieldNum);
@@ -255,7 +257,7 @@ public:
     const GooString *getExportVal(int i) const;
     // select the i-th choice
     void select(int i);
-
+    void setAppearanceChoiceContent(const GooString *new_content);
     // toggle selection of the i-th choice
     void toggle(int i);
 
@@ -297,7 +299,20 @@ public:
     void setSignatureType(FormSignatureType fst);
 
     // Use -1 for now as validationTime
-    SignatureInfo *validateSignature(bool doVerifyCert, bool forceRevalidation, time_t validationTime, bool ocspRevocationCheck, bool enableAIA);
+    // ocspRevocation and aiafetch might happen async in the Background
+    // doneCallback will be invoked once there is a result
+    // Note: Validation callback will likely happen from an auxillary
+    // thread and it is the caller of this method who is responsible
+    // for moving back to the main thread
+    // For synchronous code, don't provide validation callback
+    // and just call validateSignatureResult afterwards
+    // The returned SignatureInfo from this method does
+    // not have validated the certificate.
+    SignatureInfo *validateSignatureAsync(bool doVerifyCert, bool forceRevalidation, time_t validationTime, bool ocspRevocationCheck, bool enableAIA, const std::function<void()> &doneCallback);
+
+    /// Waits, if needed, on validation callback and
+    /// returns a signatureinfo with validated certificates
+    CertificateValidationStatus validateSignatureResult();
 
     // returns a list with the boundaries of the signed ranges
     // the elements of the list are of type Goffset
@@ -539,6 +554,9 @@ public:
     const GooString *getExportVal(int i) const { return choices ? choices[i].exportVal : nullptr; }
     // For multi-select choices it returns the first one
     const GooString *getSelectedChoice() const;
+    const GooString *getAppearanceSelectedChoice() const { return appearanceSelectedChoice ? appearanceSelectedChoice : getSelectedChoice(); }
+
+    void setAppearanceChoiceContentCopy(const GooString *new_content);
 
     // select the i-th choice
     void select(int i);
@@ -592,6 +610,7 @@ protected:
     ChoiceOpt *choices;
     bool *defaultChoices;
     GooString *editedChoice;
+    GooString *appearanceSelectedChoice;
     int topIdx; // TI
 };
 
@@ -605,7 +624,9 @@ public:
     FormFieldSignature(PDFDoc *docA, Object &&dict, const Ref ref, FormField *parent, std::set<int> *usedParents);
 
     // Use -1 for now as validationTime
-    SignatureInfo *validateSignature(bool doVerifyCert, bool forceRevalidation, time_t validationTime, bool ocspRevocationCheck, bool enableAIA);
+    SignatureInfo *validateSignatureAsync(bool doVerifyCert, bool forceRevalidation, time_t validationTime, bool ocspRevocationCheck, bool enableAIA, const std::function<void()> &doneCallback);
+
+    CertificateValidationStatus validateSignatureResult();
 
     // returns a list with the boundaries of the signed ranges
     // the elements of the list are of type Goffset
@@ -653,6 +674,7 @@ private:
     double customAppearanceLeftFontSize = 20;
     Ref imageResource = Ref::INVALID();
     std::unique_ptr<X509CertificateInfo> certificate_info;
+    std::unique_ptr<CryptoSign::VerificationInterface> signature_handler;
 
     void print(int indent) override;
 };
