@@ -3,6 +3,7 @@
  * Copyright (C) 2005, Brad Hards <bradh@frogmouth.net>
  * Copyright (C) 2008, 2011, Pino Toscano <pino@kde.org>
  * Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
+ * Copyright (C) 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -104,15 +105,8 @@ QByteArray EmbeddedFile::data()
     }
 
     stream->reset();
-    int dataLen = 0;
-    QByteArray fileArray;
-    int i;
-    while ((i = stream->getChar()) != EOF) {
-        fileArray[dataLen] = (char)i;
-        ++dataLen;
-    }
-    fileArray.resize(dataLen);
-    return fileArray;
+    auto data = stream->toUnsignedChars();
+    return QByteArray(reinterpret_cast<const char *>(data.data()), data.size());
 }
 
 bool EmbeddedFile::isValid() const
