@@ -16,7 +16,8 @@
  * Copyright (C) 2021 Georgiy Sgibnev <georgiy@sgibnev.com>. Work sponsored by lab50.net.
  * Copyright (C) 2021 Theofilos Intzoglou <int.teo@gmail.com>
  * Copyright (C) 2022 Alexander Sulfrian <asulfrian@zedat.fu-berlin.de>
- * Copyright (C) 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+ * Copyright (C) 2023, 2024 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+ * Copyright (C) 2024 Pratham Gandhi <ppg.1382@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -585,6 +586,14 @@ bool FormFieldChoice::canBeSpellChecked() const
     return !fwc->noSpellCheck();
 }
 
+void FormFieldChoice::setAppearanceChoiceText(const QString &text)
+{
+    FormWidgetChoice *fwc = static_cast<FormWidgetChoice *>(m_formData->fm);
+    GooString *goo = QStringToUnicodeGooString(text);
+    fwc->setAppearanceChoiceContent(goo);
+    delete goo;
+}
+
 class CertificateInfoPrivate
 {
 public:
@@ -610,6 +619,7 @@ public:
     int version;
     bool is_self_signed;
     bool is_null;
+    CertificateInfo::KeyLocation keyLocation;
 };
 
 CertificateInfo::CertificateInfo() : d_ptr(new CertificateInfoPrivate())
@@ -735,6 +745,12 @@ CertificateInfo::KeyUsageExtensions CertificateInfo::keyUsageExtensions() const
     return kuExtensions;
 }
 
+CertificateInfo::KeyLocation CertificateInfo::keyLocation() const
+{
+    Q_D(const CertificateInfo);
+    return d->keyLocation;
+}
+
 QByteArray CertificateInfo::publicKey() const
 {
     Q_D(const CertificateInfo);
@@ -968,7 +984,24 @@ FormFieldSignature::SignatureType FormFieldSignature::signatureType() const
 
 SignatureValidationInfo FormFieldSignature::validate(ValidateOptions opt) const
 {
-    return validate(opt, QDateTime());
+    auto tempResult = validateAsync(opt);
+    tempResult.first.d_ptr->certificate_status = validateResult();
+    return tempResult.first;
+}
+
+static CertificateInfo::KeyLocation fromPopplerCore(KeyLocation location)
+{
+    switch (location) {
+    case KeyLocation::Computer:
+        return CertificateInfo::KeyLocation::Computer;
+    case KeyLocation::Other:
+        return CertificateInfo::KeyLocation::Other;
+    case KeyLocation::Unknown:
+        return CertificateInfo::KeyLocation::Unknown;
+    case KeyLocation::HardwareToken:
+        return CertificateInfo::KeyLocation::HardwareToken;
+    }
+    return CertificateInfo::KeyLocation::Unknown;
 }
 
 static CertificateInfoPrivate *createCertificateInfoPrivate(const X509CertificateInfo *ci)
@@ -978,6 +1011,7 @@ static CertificateInfoPrivate *createCertificateInfoPrivate(const X509Certificat
     if (ci) {
         certPriv->version = ci->getVersion();
         certPriv->ku_extensions = ci->getKeyUsageExtensions();
+        certPriv->keyLocation = fromPopplerCore(ci->getKeyLocation());
 
         const GooString &certSerial = ci->getSerialNumber();
         certPriv->serial_number = QByteArray(certSerial.c_str(), certSerial.getLength());
@@ -1014,12 +1048,29 @@ static CertificateInfoPrivate *createCertificateInfoPrivate(const X509Certificat
     return certPriv;
 }
 
-SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &validationTime) const
+static SignatureValidationInfo::CertificateStatus fromInternal(CertificateValidationStatus status)
 {
-    FormWidgetSignature *fws = static_cast<FormWidgetSignature *>(m_formData->fm);
-    const time_t validationTimeT = validationTime.isValid() ? validationTime.toSecsSinceEpoch() : -1;
-    SignatureInfo *si = fws->validateSignature(opt & ValidateVerifyCertificate, opt & ValidateForceRevalidation, validationTimeT, !(opt & ValidateWithoutOCSPRevocationCheck), opt & ValidateUseAIACertFetch);
+    switch (status) {
+    case CERTIFICATE_TRUSTED:
+        return SignatureValidationInfo::CertificateTrusted;
+    case CERTIFICATE_UNTRUSTED_ISSUER:
+        return SignatureValidationInfo::CertificateUntrustedIssuer;
+    case CERTIFICATE_UNKNOWN_ISSUER:
+        return SignatureValidationInfo::CertificateUnknownIssuer;
+    case CERTIFICATE_REVOKED:
+        return SignatureValidationInfo::CertificateRevoked;
+    case CERTIFICATE_EXPIRED:
+        return SignatureValidationInfo::CertificateExpired;
+    default:
+    case CERTIFICATE_GENERIC_ERROR:
+        return SignatureValidationInfo::CertificateGenericError;
+    case CERTIFICATE_NOT_VERIFIED:
+        return SignatureValidationInfo::CertificateNotVerified;
+    }
+}
 
+static SignatureValidationInfo fromInternal(SignatureInfo *si, FormWidgetSignature *fws)
+{
     // get certificate info
     const X509CertificateInfo *ci = si->getCertificateInfo();
     CertificateInfoPrivate *certPriv = createCertificateInfoPrivate(ci);
@@ -1049,30 +1100,7 @@ SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &v
         priv->signature_status = SignatureValidationInfo::SignatureNotVerified;
         break;
     }
-    switch (si->getCertificateValStatus()) {
-    case CERTIFICATE_TRUSTED:
-        priv->certificate_status = SignatureValidationInfo::CertificateTrusted;
-        break;
-    case CERTIFICATE_UNTRUSTED_ISSUER:
-        priv->certificate_status = SignatureValidationInfo::CertificateUntrustedIssuer;
-        break;
-    case CERTIFICATE_UNKNOWN_ISSUER:
-        priv->certificate_status = SignatureValidationInfo::CertificateUnknownIssuer;
-        break;
-    case CERTIFICATE_REVOKED:
-        priv->certificate_status = SignatureValidationInfo::CertificateRevoked;
-        break;
-    case CERTIFICATE_EXPIRED:
-        priv->certificate_status = SignatureValidationInfo::CertificateExpired;
-        break;
-    default:
-    case CERTIFICATE_GENERIC_ERROR:
-        priv->certificate_status = SignatureValidationInfo::CertificateGenericError;
-        break;
-    case CERTIFICATE_NOT_VERIFIED:
-        priv->certificate_status = SignatureValidationInfo::CertificateNotVerified;
-        break;
-    }
+    priv->certificate_status = SignatureValidationInfo::CertificateVerificationInProgress;
     priv->signer_name = QString::fromStdString(si->getSignerName());
     priv->signer_subject_dn = QString::fromStdString(si->getSubjectDN());
     priv->hash_algorithm = si->getHashAlgorithm();
@@ -1094,6 +1122,50 @@ SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &v
     return SignatureValidationInfo(priv);
 }
 
+SignatureValidationInfo FormFieldSignature::validate(int opt, const QDateTime &validationTime) const
+{
+    auto tempResult = validateAsync(static_cast<ValidateOptions>(opt), validationTime);
+    tempResult.first.d_ptr->certificate_status = validateResult();
+    return tempResult.first;
+}
+
+class AsyncObjectPrivate
+{ /*Currently unused. Created for abi future proofing*/
+};
+
+AsyncObject::AsyncObject() : QObject(nullptr), d {} { }
+
+AsyncObject::~AsyncObject() = default;
+
+std::pair<SignatureValidationInfo, std::shared_ptr<Poppler::AsyncObject>> FormFieldSignature::validateAsync(ValidateOptions opt, const QDateTime &validationTime) const
+{
+    auto object = std::make_shared<AsyncObject>();
+    FormWidgetSignature *fws = static_cast<FormWidgetSignature *>(m_formData->fm);
+    const time_t validationTimeT = validationTime.isValid() ? validationTime.toSecsSinceEpoch() : -1;
+    SignatureInfo *si = fws->validateSignatureAsync(opt & ValidateVerifyCertificate, opt & ValidateForceRevalidation, validationTimeT, !(opt & ValidateWithoutOCSPRevocationCheck), opt & ValidateUseAIACertFetch,
+                                                    [obj = std::weak_ptr<AsyncObject>(object)]() {
+                                                        if (auto l = obj.lock()) {
+                                                            // We need to roundtrip over the eventloop
+                                                            // to ensure callers have a chance of connecting to AsyncObject::done
+                                                            QMetaObject::invokeMethod(
+                                                                    l.get(),
+                                                                    [innerObj = std::weak_ptr<AsyncObject>(l)]() {
+                                                                        if (auto innerLocked = innerObj.lock()) {
+                                                                            emit innerLocked->done();
+                                                                        }
+                                                                    },
+                                                                    Qt::QueuedConnection);
+                                                        }
+                                                    });
+
+    return { fromInternal(si, fws), object };
+}
+
+SignatureValidationInfo::CertificateStatus FormFieldSignature::validateResult() const
+{
+    return fromInternal(static_cast<FormWidgetSignature *>(m_formData->fm)->validateSignatureResult());
+}
+
 FormFieldSignature::SigningResult FormFieldSignature::sign(const QString &outputFileName, const PDFConverter::NewSignatureData &data) const
 {
     FormWidgetSignature *fws = static_cast<FormWidgetSignature *>(m_formData->fm);
