@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2001-2021 The OpenLDAP Foundation.
+ * Copyright 2001-2022 The OpenLDAP Foundation.
  * Portions Copyright 2001-2003 Pierangelo Masarati.
  * All rights reserved.
  *
@@ -26,11 +26,11 @@
 
 #include <lutil.h>
 #include "slap.h"
-#include "config.h"
+#include "slap-config.h"
 #include "lber_pvt.h"
 #include "back-monitor.h"
 
-#include "config.h"
+#include "slap-config.h"
 
 #undef INTEGRATE_CORE_SCHEMA
 
@@ -68,7 +68,8 @@ static const monitor_extra_t monitor_extra = {
 
 	monitor_back_entry_stub,
 	monitor_back_entrypriv_create,
-	monitor_back_register_subsys_late
+	monitor_back_register_subsys_late,
+	monitor_back_entry_get_unlocked
 };
 	
 
@@ -130,7 +131,7 @@ static struct monitor_subsys_t known_monitor_subsys[] = {
 		SLAPD_MONITOR_LOG_NAME,
 		BER_BVNULL, BER_BVNULL, BER_BVNULL,
 		{ BER_BVC( "This subsystem contains information about logging." ),
-		  	BER_BVC( "Set the attribute \"managedInfo\" to the desired log levels." ),
+			BER_BVC( "Set the \"monitorLogLevel\" or \"monitorDebugLevel\" attributes to the desired levels." ),
 			BER_BVNULL },
 		MONITOR_F_NONE,
 		monitor_subsys_log_init,
@@ -322,12 +323,11 @@ monitor_back_register_subsys_late(
 	if ( be_monitor == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_subsys_late: "
-			"monitor database not configured.\n",
-			0, 0, 0 );
+			"monitor database not configured.\n" );
 		return -1;
 	}
 
-	/* everyting is ready, can register already */
+	/* everything is ready, can register already */
 	if ( monitor_subsys_is_opened() ) {
 		return monitor_back_register_subsys( ms );
 	}
@@ -384,8 +384,7 @@ monitor_back_register_database_limbo(
 	if ( be_monitor == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_database_limbo: "
-			"monitor database not configured.\n",
-			0, 0, 0 );
+			"monitor database not configured.\n" );
 		return -1;
 	}
 
@@ -429,8 +428,7 @@ monitor_back_register_overlay_limbo(
 	if ( be_monitor == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_overlay_limbo: "
-			"monitor database not configured.\n",
-			0, 0, 0 );
+			"monitor database not configured.\n" );
 		return -1;
 	}
 
@@ -464,12 +462,13 @@ monitor_back_register_entry(
 	unsigned long		flags )
 {
 	monitor_info_t 	*mi;
+	int rc = 0;
 
 	if ( be_monitor == NULL ) {
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_entry(\"%s\"): "
 			"monitor database not configured.\n",
-			e->e_name.bv_val, 0, 0 );
+			e->e_name.bv_val );
 		return -1;
 	}
 
@@ -480,20 +479,17 @@ monitor_back_register_entry(
 	assert( e->e_private == NULL );
 	
 	if ( monitor_subsys_is_opened() ) {
-		Entry		*e_parent = NULL,
-				*e_new = NULL,
-				**ep = NULL;
+		Entry		*e_parent = NULL, *e_new = NULL;
 		struct berval	pdn = BER_BVNULL;
 		monitor_entry_t *mp = NULL,
 				*mp_parent = NULL;
-		int		rc = 0;
 
 		if ( monitor_cache_get( mi, &e->e_nname, &e_parent ) == 0 ) {
 			/* entry exists */
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"entry exists\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			monitor_cache_release( mi, e_parent );
 			return -1;
 		}
@@ -504,7 +500,7 @@ monitor_back_register_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"parent \"%s\" not found\n",
-				e->e_name.bv_val, pdn.bv_val, 0 );
+				e->e_name.bv_val, pdn.bv_val );
 			return -1;
 		}
 
@@ -516,7 +512,7 @@ monitor_back_register_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"parent \"%s\" is volatile\n",
-				e->e_name.bv_val, e_parent->e_name.bv_val, 0 );
+				e->e_name.bv_val, e_parent->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -526,7 +522,7 @@ monitor_back_register_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"monitor_entrypriv_create() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -536,7 +532,7 @@ monitor_back_register_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"entry_dup() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -552,18 +548,11 @@ monitor_back_register_entry(
 		}
 		mp->mp_cb = cb;
 
-		ep = &mp_parent->mp_children;
-		for ( ; *ep; ) {
-			mp_parent = ( monitor_entry_t * )(*ep)->e_private;
-			ep = &mp_parent->mp_next;
-		}
-		*ep = e_new;
-
-		if ( monitor_cache_add( mi, e_new ) ) {
+		if ( monitor_cache_add( mi, e_new, e_parent ) ) {
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"unable to add entry\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -593,7 +582,7 @@ done:;
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"entry_dup() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			return -1;
 		}
 		
@@ -617,7 +606,7 @@ done:;
 		**elpp = el;
 	}
 
-	return 0;
+	return rc;
 }
 
 int
@@ -653,15 +642,12 @@ monitor_back_register_entry_parent(
 		/* need a filter */
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_entry_parent(\"\"): "
-			"need a valid filter\n",
-			0, 0, 0 );
+			"need a valid filter\n" );
 		return -1;
 	}
 
 	if ( monitor_subsys_is_opened() ) {
-		Entry		*e_parent = NULL,
-				*e_new = NULL,
-				**ep = NULL;
+		Entry		*e_parent = NULL, *e_new = NULL;
 		struct berval	e_name = BER_BVNULL,
 				e_nname = BER_BVNULL;
 		monitor_entry_t *mp = NULL,
@@ -685,7 +671,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_parent(\"%s\"): "
 				"parent entry does not exist\n",
-				ndn.bv_val, 0, 0 );
+				ndn.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -698,7 +684,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_parent(\"%s\"): "
 				"entry is volatile\n",
-				e_parent->e_name.bv_val, 0, 0 );
+				e_parent->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -711,7 +697,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_parent(\"%s\"): "
 				"entry already exists\n",
-				e_name.bv_val, 0, 0 );
+				e_name.bv_val );
 			monitor_cache_release( mi, e_new );
 			e_new = NULL;
 			rc = -1;
@@ -723,7 +709,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_parent(\"%s\"): "
 				"monitor_entrypriv_create() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -733,7 +719,7 @@ monitor_back_register_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"entry_dup() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -753,18 +739,11 @@ monitor_back_register_entry_parent(
 		}
 		mp->mp_cb = cb;
 
-		ep = &mp_parent->mp_children;
-		for ( ; *ep; ) {
-			mp_parent = ( monitor_entry_t * )(*ep)->e_private;
-			ep = &mp_parent->mp_next;
-		}
-		*ep = e_new;
-
-		if ( monitor_cache_add( mi, e_new ) ) {
+		if ( monitor_cache_add( mi, e_new, e_parent ) ) {
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"unable to add entry\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -798,7 +777,7 @@ done:;
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry(\"%s\"): "
 				"entry_dup() failed\n",
-				e->e_name.bv_val, 0, 0 );
+				e->e_name.bv_val );
 			goto done_limbo;
 		}
 		
@@ -992,16 +971,11 @@ monitor_back_register_entry_attrs(
 	if ( filter == NULL ) filter = &empty_bv;
 
 	if ( be_monitor == NULL ) {
-		char		buf[ SLAP_TEXT_BUFLEN ];
-
-		snprintf( buf, sizeof( buf ),
-			"monitor_back_register_entry_%s(base=\"%s\" scope=%s filter=\"%s\"): "
-			"monitor database not configured.\n",
-			fname,
-			BER_BVISNULL( nbase ) ? "" : nbase->bv_val,
-			ldap_pvt_scope2str( scope ),
-			BER_BVISNULL( filter ) ? "" : filter->bv_val );
-		Debug( LDAP_DEBUG_ANY, "%s\n", buf, 0, 0 );
+		Debug(LDAP_DEBUG_ANY,
+		      "monitor_back_register_entry_%s(base=\"%s\" scope=%s filter=\"%s\"): " "monitor database not configured.\n\n",
+		      fname, BER_BVISNULL(nbase) ? "" : nbase->bv_val,
+		      ldap_pvt_scope2str(scope),
+		      BER_BVISNULL(filter) ? "" : filter->bv_val );
 
 		return -1;
 	}
@@ -1026,7 +1000,7 @@ monitor_back_register_entry_attrs(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_register_entry_%s(\"\"): "
 			"need a valid filter\n",
-			fname, 0, 0 );
+			fname );
 		return -1;
 	}
 
@@ -1040,19 +1014,12 @@ monitor_back_register_entry_attrs(
 
 		if ( BER_BVISNULL( &ndn ) ) {
 			if ( monitor_search2ndn( nbase, scope, filter, &ndn ) ) {
-				char		buf[ SLAP_TEXT_BUFLEN ];
-
-				snprintf( buf, sizeof( buf ),
-					"monitor_back_register_entry_%s(\"\"): "
-					"base=\"%s\" scope=%s filter=\"%s\": "
-					"unable to find entry\n",
-					fname,
-					nbase->bv_val ? nbase->bv_val : "\"\"",
-					ldap_pvt_scope2str( scope ),
-					filter->bv_val );
-
-				/* entry does not exist */
-				Debug( LDAP_DEBUG_ANY, "%s\n", buf, 0, 0 );
+				Debug(LDAP_DEBUG_ANY,
+				      "monitor_back_register_entry_%s(\"\"): " "base=\"%s\" scope=%s filter=\"%s\": " "unable to find entry\n\n",
+				      fname,
+				      nbase->bv_val ? nbase->bv_val : "\"\"",
+				      ldap_pvt_scope2str(scope),
+				      filter->bv_val );
 				return -1;
 			}
 
@@ -1064,7 +1031,7 @@ monitor_back_register_entry_attrs(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_%s(\"%s\"): "
 				"entry does not exist\n",
-				fname, ndn.bv_val, 0 );
+				fname, ndn.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -1077,7 +1044,7 @@ monitor_back_register_entry_attrs(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_register_entry_%s(\"%s\"): "
 				"entry is volatile\n",
-				fname, e->e_name.bv_val, 0 );
+				fname, e->e_name.bv_val );
 			rc = -1;
 			goto done;
 		}
@@ -1100,7 +1067,7 @@ monitor_back_register_entry_attrs(
 						Debug( LDAP_DEBUG_ANY,
 							"monitor_back_register_entry_%s(\"%s\"): "
 							"attr_dup() failed\n",
-							fname, e->e_name.bv_val, 0 );
+							fname, e->e_name.bv_val );
 						rc = -1;
 						goto done;
 					}
@@ -1203,7 +1170,7 @@ monitor_back_unregister_entry(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_unregister_entry(\"%s\"): "
 			"monitor database not configured.\n",
-			ndn->bv_val, 0, 0 );
+			ndn->bv_val );
 
 		return -1;
 	}
@@ -1228,7 +1195,7 @@ monitor_back_unregister_entry(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_unregister_entry(\"%s\"): "
 				"entry removal failed.\n",
-				ndn->bv_val, 0, 0 );
+				ndn->bv_val );
 			return -1;
 		}
 
@@ -1329,8 +1296,7 @@ monitor_back_unregister_entry_parent(
 		/* need a filter */
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_unregister_entry_parent(\"\"): "
-			"need a valid filter\n",
-			0, 0, 0 );
+			"need a valid filter\n" );
 		return -1;
 	}
 
@@ -1355,7 +1321,7 @@ monitor_back_unregister_entry_parent(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_unregister_entry(\"%s\"): "
 				"entry removal failed.\n",
-				ndn.bv_val, 0, 0 );
+				ndn.bv_val );
 			ber_memfree( ndn.bv_val );
 			return -1;
 		}
@@ -1448,16 +1414,11 @@ monitor_back_unregister_entry_attrs(
 	char		*fname = ( target_a == NULL ? "callback" : "attrs" );
 
 	if ( be_monitor == NULL ) {
-		char		buf[ SLAP_TEXT_BUFLEN ];
-
-		snprintf( buf, sizeof( buf ),
-			"monitor_back_unregister_entry_%s(base=\"%s\" scope=%s filter=\"%s\"): "
-			"monitor database not configured.\n",
-			fname,
-			BER_BVISNULL( nbase ) ? "" : nbase->bv_val,
-			ldap_pvt_scope2str( scope ),
-			BER_BVISNULL( filter ) ? "" : filter->bv_val );
-		Debug( LDAP_DEBUG_ANY, "%s\n", buf, 0, 0 );
+		Debug(LDAP_DEBUG_ANY,
+		      "monitor_back_unregister_entry_%s(base=\"%s\" scope=%s filter=\"%s\"): " "monitor database not configured.\n\n",
+		      fname, BER_BVISNULL(nbase) ? "" : nbase->bv_val,
+		      ldap_pvt_scope2str(scope),
+		      BER_BVISNULL(filter) ? "" : filter->bv_val );
 
 		return -1;
 	}
@@ -1488,7 +1449,7 @@ monitor_back_unregister_entry_attrs(
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_back_unregister_entry_%s(\"\"): "
 			"need a valid filter\n",
-			fname, 0, 0 );
+			fname );
 		return -1;
 	}
 
@@ -1499,18 +1460,11 @@ monitor_back_unregister_entry_attrs(
 
 		if ( BER_BVISNULL( &ndn ) ) {
 			if ( monitor_search2ndn( nbase, scope, filter, &ndn ) ) {
-				char		buf[ SLAP_TEXT_BUFLEN ];
-
-				snprintf( buf, sizeof( buf ),
-					"monitor_back_unregister_entry_%s(\"\"): "
-					"base=\"%s\" scope=%d filter=\"%s\": "
-					"unable to find entry\n",
-					fname,
-					nbase->bv_val ? nbase->bv_val : "\"\"",
-					scope, filter->bv_val );
-
-				/* entry does not exist */
-				Debug( LDAP_DEBUG_ANY, "%s\n", buf, 0, 0 );
+				Debug(LDAP_DEBUG_ANY,
+				      "monitor_back_unregister_entry_%s(\"\"): " "base=\"%s\" scope=%d filter=\"%s\": " "unable to find entry\n\n",
+				      fname,
+				      nbase->bv_val ? nbase->bv_val : "\"\"",
+				      scope, filter->bv_val );
 				return -1;
 			}
 
@@ -1522,7 +1476,7 @@ monitor_back_unregister_entry_attrs(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_unregister_entry(\"%s\"): "
 				"entry removal failed.\n",
-				ndn.bv_val, 0, 0 );
+				ndn.bv_val );
 			return -1;
 		}
 
@@ -1979,6 +1933,29 @@ monitor_back_initialize(
 			"NO-USER-MODIFICATION "
 			"USAGE dSAOperation )", SLAP_AT_FINAL|SLAP_AT_HIDE,
 			offsetof(monitor_info_t, mi_ad_monitorSuperiorDN) },
+		{ "( 1.3.6.1.4.1.4203.666.1.55.31 "
+			"NAME 'monitorConnectionOpsAsync' "
+			"DESC 'monitor number of asynchronous operations in execution within the connection' "
+			"SUP monitorCounter "
+			"NO-USER-MODIFICATION "
+			"USAGE dSAOperation )", SLAP_AT_FINAL|SLAP_AT_HIDE,
+			offsetof(monitor_info_t, mi_ad_monitorConnectionOpsAsync) },
+		{ "( 1.3.6.1.4.1.4203.666.1.55.32 "
+			"NAME 'monitorLogLevel' "
+			"DESC 'current slapd log level' "
+			"EQUALITY caseIgnoreMatch "
+			"SUBSTR caseIgnoreSubstringsMatch "
+			"SYNTAX 1.3.6.1.4.1.1466.115.121.1.15 "
+			"USAGE dSAOperation )", SLAP_AT_FINAL|SLAP_AT_HIDE,
+			offsetof(monitor_info_t, mi_ad_monitorLogLevel) },
+		{ "( 1.3.6.1.4.1.4203.666.1.55.33 "
+			"NAME 'monitorDebugLevel' "
+			"DESC 'current slapd debug level' "
+			"EQUALITY caseIgnoreMatch "
+			"SUBSTR caseIgnoreSubstringsMatch "
+			"SYNTAX 1.3.6.1.4.1.1466.115.121.1.15 "
+			"USAGE dSAOperation )", SLAP_AT_FINAL|SLAP_AT_HIDE,
+			offsetof(monitor_info_t, mi_ad_monitorDebugLevel) },
 		{ NULL, 0, -1 }
 	};
 
@@ -1990,9 +1967,11 @@ monitor_back_initialize(
 		{ "olmSubSystemAttributes",		"olmAttributes:0" },
 		{ "olmGenericAttributes",		"olmSubSystemAttributes:0" },
 		{ "olmDatabaseAttributes",		"olmSubSystemAttributes:1" },
+		{ "olmOverlayAttributes",		"olmSubSystemAttributes:2" },
+		{ "olmModuleAttributes",		"olmSubSystemAttributes:3" },
 
-		/* for example, back-bdb specific attrs
-		 * are in "olmDatabaseAttributes:1"
+		/* for example, back-mdb specific attrs
+		 * are in "olmDatabaseAttributes:12"
 		 *
 		 * NOTE: developers, please record here OID assignments
 		 * for other modules */
@@ -2001,9 +1980,11 @@ monitor_back_initialize(
 		{ "olmSubSystemObjectClasses",		"olmObjectClasses:0" },
 		{ "olmGenericObjectClasses",		"olmSubSystemObjectClasses:0" },
 		{ "olmDatabaseObjectClasses",		"olmSubSystemObjectClasses:1" },
+		{ "olmOverlayObjectClasses",		"olmSubSystemObjectClasses:2" },
+		{ "olmModuleObjectClasses",			"olmSubSystemObjectClasses:3" },
 
-		/* for example, back-bdb specific objectClasses
-		 * are in "olmDatabaseObjectClasses:1"
+		/* for example, back-mdb specific objectClasses
+		 * are in "olmDatabaseObjectClasses:12"
 		 *
 		 * NOTE: developers, please record here OID assignments
 		 * for other modules */
@@ -2029,7 +2010,7 @@ monitor_back_initialize(
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_back_initialize: unable to add "
 				"objectIdentifier \"%s=%s\"\n",
-				s_oid[ i ].name, s_oid[ i ].oid, 0 );
+				s_oid[ i ].name, s_oid[ i ].oid );
 			return 1;
 		}
 	}
@@ -2045,7 +2026,7 @@ monitor_back_initialize(
 
 		if ( code ) {
 			Debug( LDAP_DEBUG_ANY,
-				"monitor_back_db_init: register_at failed\n", 0, 0, 0 );
+				"monitor_back_db_init: register_at failed\n" );
 			return -1;
 		}
 		(*ad)->ad_type->sat_flags |= mat[ i ].flags;
@@ -2059,7 +2040,7 @@ monitor_back_initialize(
 		code = register_oc( moc[ i ].schema, Oc, 0 );
 		if ( code ) {
 			Debug( LDAP_DEBUG_ANY,
-				"monitor_back_db_init: register_oc failed\n", 0, 0, 0 );
+				"monitor_back_db_init: register_oc failed\n" );
 			return -1;
 		}
 		(*Oc)->soc_flags |= moc[ i ].flags;
@@ -2170,7 +2151,7 @@ monitor_back_db_init(
 	if( rc != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY,
 			"unable to normalize/pretty monitor DN \"%s\" (%d)\n",
-			dn.bv_val, rc, 0 );
+			dn.bv_val, rc );
 		return -1;
 	}
 
@@ -2179,7 +2160,7 @@ monitor_back_db_init(
 
 	/* NOTE: only one monitor database is allowed,
 	 * so we use static storage */
-	ldap_pvt_thread_mutex_init( &monitor_info.mi_cache_mutex );
+	ldap_pvt_thread_mutex_init( &monitor_info.mi_cache_lock );
 
 	be->be_private = &monitor_info;
 
@@ -2246,7 +2227,7 @@ monitor_back_db_open(
 {
 	monitor_info_t 		*mi = (monitor_info_t *)be->be_private;
 	struct monitor_subsys_t	**ms;
-	Entry 			*e, **ep, *root;
+	Entry 			*e, *root;
 	monitor_entry_t		*mp;
 	int			i;
 	struct berval		bv, rdn = BER_BVC(SLAPD_MONITOR_DN);
@@ -2292,7 +2273,7 @@ monitor_back_db_open(
 	if ( e == NULL) {
 		Debug( LDAP_DEBUG_ANY,
 			"unable to create \"%s\" entry\n",
-			SLAPD_MONITOR_DN, 0, 0 );
+			SLAPD_MONITOR_DN );
 		return( -1 );
 	}
 
@@ -2325,7 +2306,7 @@ monitor_back_db_open(
 					&bv, NULL ) ) {
 			Debug( LDAP_DEBUG_ANY,
 				"unable to add monitoredInfo to \"%s\" entry\n",
-				SLAPD_MONITOR_DN, 0, 0 );
+				SLAPD_MONITOR_DN );
 			return( -1 );
 		}
 	}
@@ -2335,12 +2316,11 @@ monitor_back_db_open(
 		return -1;
 	}
 	e->e_private = ( void * )mp;
-	ep = &mp->mp_children;
 
-	if ( monitor_cache_add( mi, e ) ) {
+	if ( monitor_cache_add( mi, e, NULL ) ) {
 		Debug( LDAP_DEBUG_ANY,
 			"unable to add entry \"%s\" to cache\n",
-			SLAPD_MONITOR_DN, 0, 0 );
+			SLAPD_MONITOR_DN );
 		return -1;
 	}
 	root = e;
@@ -2362,7 +2342,7 @@ monitor_back_db_open(
 		if ( rc != LDAP_SUCCESS ) {
 			Debug( LDAP_DEBUG_ANY,
 				"monitor RDN \"%s\" is invalid\n", 
-				dn.bv_val, 0, 0 );
+				dn.bv_val );
 			return( -1 );
 		}
 
@@ -2373,7 +2353,7 @@ monitor_back_db_open(
 		if ( e == NULL) {
 			Debug( LDAP_DEBUG_ANY,
 				"unable to create \"%s\" entry\n", 
-				monitor_subsys[ i ]->mss_dn.bv_val, 0, 0 );
+				monitor_subsys[ i ]->mss_dn.bv_val );
 			return( -1 );
 		}
 		monitor_subsys[i]->mss_dn = e->e_name;
@@ -2392,15 +2372,12 @@ monitor_back_db_open(
 		mp->mp_info = monitor_subsys[ i ];
 		mp->mp_flags = monitor_subsys[ i ]->mss_flags;
 
-		if ( monitor_cache_add( mi, e ) ) {
+		if ( monitor_cache_add( mi, e, root ) ) {
 			Debug( LDAP_DEBUG_ANY,
 				"unable to add entry \"%s\" to cache\n",
-				monitor_subsys[ i ]->mss_dn.bv_val, 0, 0 );
+				monitor_subsys[ i ]->mss_dn.bv_val );
 			return -1;
 		}
-
-		*ep = e;
-		ep = &mp->mp_next;
 	}
 
 	assert( be != NULL );
@@ -2560,13 +2537,13 @@ monitor_back_db_destroy(
 		int	i;
 
 		for ( i = 0; monitor_subsys[ i ] != NULL; i++ ) {
-			if ( monitor_subsys[ i ]->mss_destroy ) {
-				monitor_subsys[ i ]->mss_destroy( be, monitor_subsys[ i ] );
-			}
-
 			if ( !BER_BVISNULL( &monitor_subsys[ i ]->mss_rdn ) ) {
 				ch_free( monitor_subsys[ i ]->mss_rdn.bv_val );
 			}
+
+			if ( monitor_subsys[ i ]->mss_destroy ) {
+				monitor_subsys[ i ]->mss_destroy( be, monitor_subsys[ i ] );
+			}
 		}
 
 		ch_free( monitor_subsys );
@@ -2582,17 +2559,9 @@ monitor_back_db_destroy(
 		}
 	}
 	
-	ldap_pvt_thread_mutex_destroy( &monitor_info.mi_cache_mutex );
+	ldap_pvt_thread_mutex_destroy( &monitor_info.mi_cache_lock );
 
 	be->be_private = NULL;
 
 	return 0;
 }
-
-#if SLAPD_MONITOR == SLAPD_MOD_DYNAMIC
-
-/* conditionally define the init_module() function */
-SLAP_BACKEND_INIT_MODULE( monitor )
-
-#endif /* SLAPD_MONITOR == SLAPD_MOD_DYNAMIC */
-
