@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2017 The OpenLDAP Foundation.
+ * Copyright 1998-2022 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,7 +41,8 @@ static int	get_filter_list(
 	Operation *op,
 	BerElement *ber,
 	Filter **f,
-	const char **text );
+	const char **text,
+	int depth );
 
 static int	get_ssa(
 	Operation *op,
@@ -80,19 +81,20 @@ filter_destroy( void )
 	return;
 }
 
-int
-get_filter(
+static int
+get_filter0(
 	Operation *op,
 	BerElement *ber,
 	Filter **filt,
-	const char **text )
+	const char **text,
+	int depth )
 {
 	ber_tag_t	tag;
 	ber_len_t	len;
 	int		err;
 	Filter		f;
 
-	Debug( LDAP_DEBUG_FILTER, "begin get_filter\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_FILTER, "begin get_filter\n" );
 	/*
 	 * A filter looks like this coming in:
 	 *	Filter ::= CHOICE {
@@ -126,6 +128,11 @@ get_filter(
 	 *
 	 */
 
+	if( depth > slap_max_filter_depth ) {
+		*text = "filter nested too deeply";
+		return SLAPD_DISCONNECT;
+	}
+
 	tag = ber_peek_tag( ber, &len );
 
 	if( tag == LBER_ERROR ) {
@@ -140,7 +147,7 @@ get_filter(
 
 	switch ( f.f_choice ) {
 	case LDAP_FILTER_EQUALITY:
-		Debug( LDAP_DEBUG_FILTER, "EQUALITY\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "EQUALITY\n" );
 		err = get_ava( op, ber, &f, SLAP_MR_EQUALITY, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -150,7 +157,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_SUBSTRINGS:
-		Debug( LDAP_DEBUG_FILTER, "SUBSTRINGS\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "SUBSTRINGS\n" );
 		err = get_ssa( op, ber, &f, text );
 		if( err != LDAP_SUCCESS ) {
 			break;
@@ -159,7 +166,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_GE:
-		Debug( LDAP_DEBUG_FILTER, "GE\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "GE\n" );
 		err = get_ava( op, ber, &f, SLAP_MR_ORDERING, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -168,7 +175,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_LE:
-		Debug( LDAP_DEBUG_FILTER, "LE\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "LE\n" );
 		err = get_ava( op, ber, &f, SLAP_MR_ORDERING, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -179,7 +186,7 @@ get_filter(
 	case LDAP_FILTER_PRESENT: {
 		struct berval type;
 
-		Debug( LDAP_DEBUG_FILTER, "PRESENT\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "PRESENT\n" );
 		if ( ber_scanf( ber, "m", &type ) == LBER_ERROR ) {
 			err = SLAPD_DISCONNECT;
 			*text = "error decoding filter";
@@ -211,7 +218,7 @@ get_filter(
 		} break;
 
 	case LDAP_FILTER_APPROX:
-		Debug( LDAP_DEBUG_FILTER, "APPROX\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "APPROX\n" );
 		err = get_ava( op, ber, &f, SLAP_MR_EQUALITY_APPROX, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -220,8 +227,8 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_AND:
-		Debug( LDAP_DEBUG_FILTER, "AND\n", 0, 0, 0 );
-		err = get_filter_list( op, ber, &f.f_and, text );
+		Debug( LDAP_DEBUG_FILTER, "AND\n" );
+		err = get_filter_list( op, ber, &f.f_and, text, depth+1 );
 		if ( err != LDAP_SUCCESS ) {
 			break;
 		}
@@ -233,8 +240,8 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_OR:
-		Debug( LDAP_DEBUG_FILTER, "OR\n", 0, 0, 0 );
-		err = get_filter_list( op, ber, &f.f_or, text );
+		Debug( LDAP_DEBUG_FILTER, "OR\n" );
+		err = get_filter_list( op, ber, &f.f_or, text, depth+1 );
 		if ( err != LDAP_SUCCESS ) {
 			break;
 		}
@@ -246,9 +253,9 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_NOT:
-		Debug( LDAP_DEBUG_FILTER, "NOT\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "NOT\n" );
 		(void) ber_skip_tag( ber, &len );
-		err = get_filter( op, ber, &f.f_not, text );
+		err = get_filter0( op, ber, &f.f_not, text, depth+1 );
 		if ( err != LDAP_SUCCESS ) {
 			break;
 		}
@@ -274,7 +281,7 @@ get_filter(
 		break;
 
 	case LDAP_FILTER_EXT:
-		Debug( LDAP_DEBUG_FILTER, "EXTENSIBLE\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "EXTENSIBLE\n" );
 
 		err = get_mra( op, ber, &f, text );
 		if ( err != LDAP_SUCCESS ) {
@@ -287,7 +294,7 @@ get_filter(
 	default:
 		(void) ber_scanf( ber, "x" ); /* skip the element */
 		Debug( LDAP_DEBUG_ANY, "get_filter: unknown filter type=%lu\n",
-			f.f_choice, 0, 0 );
+			f.f_choice );
 		f.f_choice = SLAPD_FILTER_COMPUTED;
 		f.f_result = SLAPD_COMPARE_UNDEFINED;
 		break;
@@ -306,15 +313,27 @@ get_filter(
 		**filt = f;
 	}
 
-	Debug( LDAP_DEBUG_FILTER, "end get_filter %d\n", err, 0, 0 );
+	Debug( LDAP_DEBUG_FILTER, "end get_filter %d\n", err );
 
 	return( err );
 }
 
+int
+get_filter(
+	Operation *op,
+	BerElement *ber,
+	Filter **filt,
+	const char **text )
+{
+	return get_filter0( op, ber, filt, text, 0 );
+}
+
+
 static int
 get_filter_list( Operation *op, BerElement *ber,
 	Filter **f,
-	const char **text )
+	const char **text,
+	int depth )
 {
 	Filter		**new;
 	int		err;
@@ -322,20 +341,20 @@ get_filter_list( Operation *op, BerElement *ber,
 	ber_len_t	len;
 	char		*last;
 
-	Debug( LDAP_DEBUG_FILTER, "begin get_filter_list\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_FILTER, "begin get_filter_list\n" );
 	new = f;
 	for ( tag = ber_first_element( ber, &len, &last );
 		tag != LBER_DEFAULT;
 		tag = ber_next_element( ber, &len, last ) )
 	{
-		err = get_filter( op, ber, new, text );
+		err = get_filter0( op, ber, new, text, depth );
 		if ( err != LDAP_SUCCESS )
 			return( err );
 		new = &(*new)->f_next;
 	}
 	*new = NULL;
 
-	Debug( LDAP_DEBUG_FILTER, "end get_filter_list\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_FILTER, "end get_filter_list\n" );
 	return( LDAP_SUCCESS );
 }
 
@@ -355,7 +374,7 @@ get_ssa(
 
 	*text = "error decoding filter";
 
-	Debug( LDAP_DEBUG_FILTER, "begin get_ssa\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_FILTER, "begin get_ssa\n" );
 	if ( ber_scanf( ber, "{m" /*}*/, &desc ) == LBER_ERROR ) {
 		return SLAPD_DISCONNECT;
 	}
@@ -393,7 +412,7 @@ get_ssa(
 		f->f_choice |= SLAPD_FILTER_UNDEFINED;
 		Debug( LDAP_DEBUG_FILTER,
 		"get_ssa: no substring matching rule for attributeType %s\n",
-			desc.bv_val, 0, 0 );
+			desc.bv_val );
 	}
 
 	for ( tag = ber_first_element( ber, &len, &last );
@@ -444,7 +463,7 @@ get_ssa(
 		default:
 			Debug( LDAP_DEBUG_FILTER,
 				"  unknown substring choice=%ld\n",
-				(long) tag, 0, 0 );
+				(long) tag );
 
 			rc = LDAP_PROTOCOL_ERROR;
 			goto return_error;
@@ -464,17 +483,17 @@ get_ssa(
 
 		switch ( tag ) {
 		case LDAP_SUBSTRING_INITIAL:
-			Debug( LDAP_DEBUG_FILTER, "  INITIAL\n", 0, 0, 0 );
+			Debug( LDAP_DEBUG_FILTER, "  INITIAL\n" );
 			ssa.sa_initial = nvalue;
 			break;
 
 		case LDAP_SUBSTRING_ANY:
-			Debug( LDAP_DEBUG_FILTER, "  ANY\n", 0, 0, 0 );
+			Debug( LDAP_DEBUG_FILTER, "  ANY\n" );
 			ber_bvarray_add_x( &ssa.sa_any, &nvalue, op->o_tmpmemctx );
 			break;
 
 		case LDAP_SUBSTRING_FINAL:
-			Debug( LDAP_DEBUG_FILTER, "  FINAL\n", 0, 0, 0 );
+			Debug( LDAP_DEBUG_FILTER, "  FINAL\n" );
 			ssa.sa_final = nvalue;
 			break;
 
@@ -485,7 +504,7 @@ get_ssa(
 
 return_error:
 			Debug( LDAP_DEBUG_FILTER, "  error=%ld\n",
-				(long) rc, 0, 0 );
+				(long) rc );
 			slap_sl_free( ssa.sa_initial.bv_val, op->o_tmpmemctx );
 			ber_bvarray_free_x( ssa.sa_any, op->o_tmpmemctx );
 			if ( ssa.sa_desc->ad_flags & SLAP_DESC_TEMPORARY )
@@ -503,7 +522,7 @@ return_error:
 		*f->f_sub = ssa;
 	}
 
-	Debug( LDAP_DEBUG_FILTER, "end get_ssa\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_FILTER, "end get_ssa\n" );
 	return rc /* LDAP_SUCCESS */ ;
 }
 
@@ -562,7 +581,7 @@ filter_free_x( Operation *op, Filter *f, int freeme )
 
 	default:
 		Debug( LDAP_DEBUG_ANY, "filter_free: unknown filter type=%lu\n",
-			f->f_choice, 0, 0 );
+			f->f_choice );
 		break;
 	}
 
@@ -891,12 +910,7 @@ filter_dup( Filter *f, void *memctx )
 	case LDAP_FILTER_GE:
 	case LDAP_FILTER_LE:
 	case LDAP_FILTER_APPROX:
-		/* Should this be ava_dup() ? */
-		n->f_ava = mf->bmf_calloc( 1, sizeof(AttributeAssertion), memctx );
-		*n->f_ava = *f->f_ava;
-		if ( f->f_av_desc->ad_flags & SLAP_DESC_TEMPORARY )
-			n->f_av_desc = slap_bv2tmp_ad( &f->f_av_desc->ad_cname, memctx );
-		ber_dupbv_x( &n->f_av_value, &f->f_av_value, memctx );
+		n->f_ava = ava_dup( f->f_ava, memctx );
 		break;
 	case LDAP_FILTER_SUBSTRINGS:
 		n->f_sub = mf->bmf_calloc( 1, sizeof(SubstringsAssertion), memctx );
@@ -961,7 +975,7 @@ get_simple_vrFilter(
 	int		err;
 	ValuesReturnFilter vrf;
 
-	Debug( LDAP_DEBUG_FILTER, "begin get_simple_vrFilter\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_FILTER, "begin get_simple_vrFilter\n" );
 
 	tag = ber_peek_tag( ber, &len );
 
@@ -977,7 +991,7 @@ get_simple_vrFilter(
 
 	switch ( vrf.vrf_choice ) {
 	case LDAP_FILTER_EQUALITY:
-		Debug( LDAP_DEBUG_FILTER, "EQUALITY\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "EQUALITY\n" );
 		err = get_ava( op, ber, (Filter *)&vrf, SLAP_MR_EQUALITY, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -987,12 +1001,12 @@ get_simple_vrFilter(
 		break;
 
 	case LDAP_FILTER_SUBSTRINGS:
-		Debug( LDAP_DEBUG_FILTER, "SUBSTRINGS\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "SUBSTRINGS\n" );
 		err = get_ssa( op, ber, (Filter *)&vrf, text );
 		break;
 
 	case LDAP_FILTER_GE:
-		Debug( LDAP_DEBUG_FILTER, "GE\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "GE\n" );
 		err = get_ava( op, ber, (Filter *)&vrf, SLAP_MR_ORDERING, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -1000,7 +1014,7 @@ get_simple_vrFilter(
 		break;
 
 	case LDAP_FILTER_LE:
-		Debug( LDAP_DEBUG_FILTER, "LE\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "LE\n" );
 		err = get_ava( op, ber, (Filter *)&vrf, SLAP_MR_ORDERING, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -1010,7 +1024,7 @@ get_simple_vrFilter(
 	case LDAP_FILTER_PRESENT: {
 		struct berval type;
 
-		Debug( LDAP_DEBUG_FILTER, "PRESENT\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "PRESENT\n" );
 		if ( ber_scanf( ber, "m", &type ) == LBER_ERROR ) {
 			err = SLAPD_DISCONNECT;
 			*text = "error decoding filter";
@@ -1041,7 +1055,7 @@ get_simple_vrFilter(
 		} break;
 
 	case LDAP_FILTER_APPROX:
-		Debug( LDAP_DEBUG_FILTER, "APPROX\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "APPROX\n" );
 		err = get_ava( op, ber, (Filter *)&vrf, SLAP_MR_EQUALITY_APPROX, text );
 		if ( err != LDAP_SUCCESS ) {
 			break;
@@ -1049,7 +1063,7 @@ get_simple_vrFilter(
 		break;
 
 	case LDAP_FILTER_EXT:
-		Debug( LDAP_DEBUG_FILTER, "EXTENSIBLE\n", 0, 0, 0 );
+		Debug( LDAP_DEBUG_FILTER, "EXTENSIBLE\n" );
 
 		err = get_mra( op, ber, (Filter *)&vrf, text );
 		if ( err != LDAP_SUCCESS ) {
@@ -1062,7 +1076,7 @@ get_simple_vrFilter(
 	default:
 		(void) ber_scanf( ber, "x" ); /* skip the element */
 		Debug( LDAP_DEBUG_ANY, "get_simple_vrFilter: unknown filter type=%lu\n",
-			vrf.vrf_choice, 0, 0 );
+			vrf.vrf_choice );
 		vrf.vrf_choice = SLAPD_FILTER_COMPUTED;
 		vrf.vrf_result = SLAPD_COMPARE_UNDEFINED;
 		break;
@@ -1080,7 +1094,7 @@ get_simple_vrFilter(
 		**filt = vrf;
 	}
 
-	Debug( LDAP_DEBUG_FILTER, "end get_simple_vrFilter %d\n", err, 0, 0 );
+	Debug( LDAP_DEBUG_FILTER, "end get_simple_vrFilter %d\n", err );
 
 	return err;
 }
@@ -1124,7 +1138,7 @@ get_vrFilter( Operation *op, BerElement *ber,
 	ber_len_t	len;
 	char		*last;
 
-	Debug( LDAP_DEBUG_FILTER, "begin get_vrFilter\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_FILTER, "begin get_vrFilter\n" );
 
 	tag = ber_peek_tag( ber, &len );
 
@@ -1151,7 +1165,7 @@ get_vrFilter( Operation *op, BerElement *ber,
 	}
 	*n = NULL;
 
-	Debug( LDAP_DEBUG_FILTER, "end get_vrFilter\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_FILTER, "end get_vrFilter\n" );
 	return( LDAP_SUCCESS );
 }
 
@@ -1194,7 +1208,7 @@ vrFilter_free( Operation *op, ValuesReturnFilter *vrf )
 
 		default:
 			Debug( LDAP_DEBUG_ANY, "filter_free: unknown filter type=%lu\n",
-				vrf->vrf_choice, 0, 0 );
+				vrf->vrf_choice );
 			break;
 		}
 
