@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2021 The OpenLDAP Foundation.
+ * Copyright 2003-2022 The OpenLDAP Foundation.
  * Copyright 2003 by Howard Chu.
  * All rights reserved.
  *
@@ -30,7 +30,7 @@
 
 #include "lutil.h"
 #include "slap.h"
-#include "config.h"
+#include "slap-config.h"
 
 /* This overlay extends the Compare operation to detect members of a
  * dynamic group. It has no effect on any other operations. It must
@@ -90,30 +90,34 @@ static int dgroup_cf( ConfigArgs *c )
 	case SLAP_CONFIG_ADD:
 	case LDAP_MOD_ADD:
 		{
-		adpair ap = { NULL, NULL, NULL }, *a2;
+		adpair ap = { NULL, NULL, NULL }, **app, *a2;
 		const char *text;
 		if ( slap_str2ad( c->argv[1], &ap.ap_mem, &text ) ) {
 			snprintf( c->cr_msg, sizeof( c->cr_msg ), "%s attribute description unknown: \"%s\"",
 				c->argv[0], c->argv[1] );
 			Debug( LDAP_DEBUG_CONFIG|LDAP_DEBUG_NONE,
-				"%s: %s\n", c->log, c->cr_msg, 0 );
+				"%s: %s\n", c->log, c->cr_msg );
 			return ARG_BAD_CONF;
 		}
 		if ( slap_str2ad( c->argv[2], &ap.ap_uri, &text ) ) {
 			snprintf( c->cr_msg, sizeof( c->cr_msg ), "%s attribute description unknown: \"%s\"",
 				c->argv[0], c->argv[2] );
 			Debug( LDAP_DEBUG_CONFIG|LDAP_DEBUG_NONE,
-				"%s: %s\n", c->log, c->cr_msg, 0 );
+				"%s: %s\n", c->log, c->cr_msg );
 			return ARG_BAD_CONF;
 		}
 		/* The on->on_bi.bi_private pointer can be used for
 		 * anything this instance of the overlay needs.
 		 */
 		a2 = ch_malloc( sizeof(adpair) );
-		a2->ap_next = on->on_bi.bi_private;
+
+		for ( app = &on->on_bi.bi_private; *app; app = &(*app)->ap_next )
+			/* Get to the end */ ;
+
 		a2->ap_mem = ap.ap_mem;
 		a2->ap_uri = ap.ap_uri;
-		on->on_bi.bi_private = a2;
+		a2->ap_next = *app;
+		*app = a2;
 		rc = 0;
 		}
 	}
@@ -123,7 +127,8 @@ static int dgroup_cf( ConfigArgs *c )
 static ConfigTable dgroupcfg[] = {
 	{ "attrpair", "member-attribute> <URL-attribute", 3, 3, 0,
 	  ARG_MAGIC, dgroup_cf,
-	  "( OLcfgOvAt:17.1 NAME 'olcDGAttrPair' "
+	  "( OLcfgOvAt:17.1 NAME ( 'olcDynGroupAttrPair' 'olcDGAttrPair' ) "
+	  "EQUALITY caseIgnoreMatch "
 	  "DESC 'Member and MemberURL attribute pair' "
 	  "SYNTAX OMsDirectoryString )", NULL, NULL },
 	{ NULL, NULL, 0, 0, 0, ARG_IGNORED }
@@ -131,10 +136,10 @@ static ConfigTable dgroupcfg[] = {
 
 static ConfigOCs dgroupocs[] = {
 	{ "( OLcfgOvOc:17.1 "
-	  "NAME 'olcDGConfig' "
+	  "NAME ( 'olcDynGroupConfig' 'olcDGConfig' ) "
 	  "DESC 'Dynamic Group configuration' "
 	  "SUP olcOverlayConfig "
-	  "MAY olcDGAttrPair )",
+	  "MAY olcDynGroupAttrPair)",
 	  Cft_Overlay, dgroupcfg },
 	{ NULL, 0, NULL }
 };
@@ -207,6 +212,7 @@ int dyngroup_initialize() {
 	int code;
 
 	dyngroup.on_bi.bi_type = "dyngroup";
+	dyngroup.on_bi.bi_flags = SLAPO_BFLAG_SINGLE;
 	dyngroup.on_bi.bi_db_destroy = dyngroup_destroy;
 	dyngroup.on_response = dyngroup_response;
 
