@@ -5,12 +5,28 @@
  * found in the LICENSE file.
  */
 
-#include "include/core/SkShader.h"
+#include "include/core/SkColor.h"
+#include "include/core/SkColorPriv.h"
+#include "include/core/SkColorType.h"
+#include "include/core/SkPaint.h"
+#include "include/core/SkPixmap.h"
+#include "include/core/SkRect.h"
+#include "include/core/SkTypes.h"
 #include "include/private/SkColorData.h"
-#include "include/private/base/SkVx.h"
+#include "include/private/base/SkCPUTypes.h"
+#include "include/private/base/SkDebug.h"
+#include "include/private/base/SkMalloc.h"
+#include "include/private/base/SkTo.h"
+#include "src/base/SkUtils.h"
+#include "src/base/SkVx.h"
+#include "src/core/SkBlitRow.h"
 #include "src/core/SkCoreBlitters.h"
+#include "src/core/SkMask.h"
 #include "src/core/SkOpts.h"
-#include "src/core/SkXfermodePriv.h"
+#include "src/shaders/SkShaderBase.h"
+
+#include <cstddef>
+#include <cstdint>
 
 static inline int upscale_31_to_32(int value) {
     SkASSERT((unsigned)value <= 31);
@@ -682,14 +698,6 @@ SkARGB32_Blitter::SkARGB32_Blitter(const SkPixmap& device, const SkPaint& paint)
     fPMColor = SkPackARGB32(fSrcA, fSrcR, fSrcG, fSrcB);
 }
 
-const SkPixmap* SkARGB32_Blitter::justAnOpaqueColor(uint32_t* value) {
-    if (255 == fSrcA) {
-        *value = fPMColor;
-        return &fDevice;
-    }
-    return nullptr;
-}
-
 #if defined _WIN32  // disable warning : local variable used without having been initialized
 #pragma warning ( push )
 #pragma warning ( disable : 4701 )
@@ -721,7 +729,7 @@ void SkARGB32_Blitter::blitAntiH(int x, int y, const SkAlpha antialias[],
         unsigned aa = antialias[0];
         if (aa) {
             if ((opaqueMask & aa) == 255) {
-                sk_memset32(device, color, count);
+                SkOpts::memset32(device, color, count);
             } else {
                 uint32_t sc = SkAlphaMulQ(color, SkAlpha255To256(aa));
                 SkBlitRow::Color32(device, device, count, sc);
@@ -914,7 +922,7 @@ void SkARGB32_Black_Blitter::blitAntiH(int x, int y, const SkAlpha antialias[],
         unsigned aa = antialias[0];
         if (aa) {
             if (aa == 255) {
-                sk_memset32(device, black, count);
+                SkOpts::memset32(device, black, count);
             } else {
                 SkPMColor src = aa << SK_A32_SHIFT;
                 unsigned dst_scale = 256 - aa;
@@ -950,24 +958,13 @@ void SkARGB32_Black_Blitter::blitAntiV2(int x, int y, U8CPU a0, U8CPU a1) {
 
 ///////////////////////////////////////////////////////////////////////////////
 
-// Special version of SkBlitRow::Factory32 that knows we're in kSrc_Mode,
-// instead of kSrcOver_Mode
-static void blend_srcmode(SkPMColor* SK_RESTRICT device,
-                          const SkPMColor* SK_RESTRICT span,
-                          int count, U8CPU aa) {
-    int aa256 = SkAlpha255To256(aa);
-    for (int i = 0; i < count; ++i) {
-        device[i] = SkFourByteInterp256(span[i], device[i], aa256);
-    }
-}
-
 SkARGB32_Shader_Blitter::SkARGB32_Shader_Blitter(const SkPixmap& device,
         const SkPaint& paint, SkShaderBase::Context* shaderContext)
     : INHERITED(device, paint, shaderContext)
 {
     fBuffer = (SkPMColor*)sk_malloc_throw(device.width() * (sizeof(SkPMColor)));
 
-    fXfermode = SkXfermode::Peek(paint.getBlendMode_or(SkBlendMode::kSrcOver));
+    SkASSERT(paint.isSrcOver());
 
     int flags = 0;
     if (!(shaderContext->getFlags() & SkShaderBase::kOpaqueAlpha_Flag)) {
@@ -978,19 +975,8 @@ SkARGB32_Shader_Blitter::SkARGB32_Shader_Blitter(const SkPixmap& device,
     // we call this on the output from the shader + alpha from the aa buffer
     fProc32Blend = SkBlitRow::Factory32(flags | SkBlitRow::kGlobalAlpha_Flag32);
 
-    fShadeDirectlyIntoDevice = false;
-    if (fXfermode == nullptr) {
-        if (shaderContext->getFlags() & SkShaderBase::kOpaqueAlpha_Flag) {
-            fShadeDirectlyIntoDevice = true;
-        }
-    } else {
-        if (SkBlendMode::kSrc == paint.asBlendMode()) {
-            fShadeDirectlyIntoDevice = true;
-            fProc32Blend = blend_srcmode;
-        }
-    }
-
-    fConstInY = SkToBool(shaderContext->getFlags() & SkShaderBase::kConstInY32_Flag);
+    fShadeDirectlyIntoDevice =
+            SkToBool(shaderContext->getFlags() & SkShaderBase::kOpaqueAlpha_Flag);
 }
 
 SkARGB32_Shader_Blitter::~SkARGB32_Shader_Blitter() {
@@ -1007,11 +993,7 @@ void SkARGB32_Shader_Blitter::blitH(int x, int y, int width) {
     } else {
         SkPMColor*  span = fBuffer;
         fShaderContext->shadeSpan(x, y, span, width);
-        if (fXfermode) {
-            fXfermode->xfer32(device, span, width, nullptr);
-        } else {
-            fProc32(device, span, width, 255);
-        }
+        fProc32(device, span, width, 255);
     }
 }
 
@@ -1024,36 +1006,6 @@ void SkARGB32_Shader_Blitter::blitRect(int x, int y, int width, int height) {
     auto*      shaderContext = fShaderContext;
     SkPMColor* span = fBuffer;
 
-    if (fConstInY) {
-        if (fShadeDirectlyIntoDevice) {
-            // shade the first row directly into the device
-            shaderContext->shadeSpan(x, y, device, width);
-            span = device;
-            while (--height > 0) {
-                device = (uint32_t*)((char*)device + deviceRB);
-                memcpy(device, span, width << 2);
-            }
-        } else {
-            shaderContext->shadeSpan(x, y, span, width);
-            SkXfermode* xfer = fXfermode;
-            if (xfer) {
-                do {
-                    xfer->xfer32(device, span, width, nullptr);
-                    y += 1;
-                    device = (uint32_t*)((char*)device + deviceRB);
-                } while (--height > 0);
-            } else {
-                SkBlitRow::Proc32 proc = fProc32;
-                do {
-                    proc(device, span, width, 255);
-                    y += 1;
-                    device = (uint32_t*)((char*)device + deviceRB);
-                } while (--height > 0);
-            }
-        }
-        return;
-    }
-
     if (fShadeDirectlyIntoDevice) {
         do {
             shaderContext->shadeSpan(x, y, device, width);
@@ -1061,23 +1013,13 @@ void SkARGB32_Shader_Blitter::blitRect(int x, int y, int width, int height) {
             device = (uint32_t*)((char*)device + deviceRB);
         } while (--height > 0);
     } else {
-        SkXfermode* xfer = fXfermode;
-        if (xfer) {
-            do {
-                shaderContext->shadeSpan(x, y, span, width);
-                xfer->xfer32(device, span, width, nullptr);
-                y += 1;
-                device = (uint32_t*)((char*)device + deviceRB);
-            } while (--height > 0);
-        } else {
-            SkBlitRow::Proc32 proc = fProc32;
-            do {
-                shaderContext->shadeSpan(x, y, span, width);
-                proc(device, span, width, 255);
-                y += 1;
-                device = (uint32_t*)((char*)device + deviceRB);
-            } while (--height > 0);
-        }
+        SkBlitRow::Proc32 proc = fProc32;
+        do {
+            shaderContext->shadeSpan(x, y, span, width);
+            proc(device, span, width, 255);
+            y += 1;
+            device = (uint32_t*)((char*)device + deviceRB);
+        } while (--height > 0);
     }
 }
 
@@ -1087,32 +1029,7 @@ void SkARGB32_Shader_Blitter::blitAntiH(int x, int y, const SkAlpha antialias[],
     uint32_t*  device = fDevice.writable_addr32(x, y);
     auto*      shaderContext = fShaderContext;
 
-    if (fXfermode && !fShadeDirectlyIntoDevice) {
-        for (;;) {
-            SkXfermode* xfer = fXfermode;
-
-            int count = *runs;
-            if (count <= 0)
-                break;
-            int aa = *antialias;
-            if (aa) {
-                shaderContext->shadeSpan(x, y, span, count);
-                if (aa == 255) {
-                    xfer->xfer32(device, span, count, nullptr);
-                } else {
-                    // count is almost always 1
-                    for (int i = count - 1; i >= 0; --i) {
-                        xfer->xfer32(&device[i], &span[i], 1, antialias);
-                    }
-                }
-            }
-            device += count;
-            runs += count;
-            antialias += count;
-            x += count;
-        }
-    } else if (fShadeDirectlyIntoDevice ||
-               (shaderContext->getFlags() & SkShaderBase::kOpaqueAlpha_Flag)) {
+    if (fShadeDirectlyIntoDevice || (shaderContext->getFlags() & SkShaderBase::kOpaqueAlpha_Flag)) {
         for (;;) {
             int count = *runs;
             if (count <= 0) {
@@ -1165,9 +1082,9 @@ static void drive(SkPMColor* dst, const SkPMColor* src, const uint8_t* cov, int
 
     auto apply = [kernel](U32 dst, U32 src, U8 cov) -> U32 {
         U8x4 cov_splat = skvx::shuffle<0,0,0,0, 1,1,1,1, 2,2,2,2, 3,3,3,3>(cov);
-        return skvx::bit_pun<U32>(kernel(skvx::bit_pun<U8x4>(dst),
-                                         skvx::bit_pun<U8x4>(src),
-                                         cov_splat));
+        return sk_bit_cast<U32>(kernel(sk_bit_cast<U8x4>(dst),
+                                       sk_bit_cast<U8x4>(src),
+                                       cov_splat));
     };
     while (n >= 4) {
         apply(U32::Load(dst), U32::Load(src), U8::Load(cov)).store(dst);
@@ -1280,31 +1197,23 @@ static void blend_row_LCD16_opaque(SkPMColor* dst, const void* vmask, const SkPM
 }
 
 void SkARGB32_Shader_Blitter::blitMask(const SkMask& mask, const SkIRect& clip) {
-    // we only handle kA8 with an xfermode
-    if (fXfermode && (SkMask::kA8_Format != mask.fFormat)) {
-        this->INHERITED::blitMask(mask, clip);
-        return;
-    }
-
     SkASSERT(mask.fBounds.contains(clip));
 
     void (*blend_row)(SkPMColor*, const void* mask, const SkPMColor*, int) = nullptr;
 
-    if (!fXfermode) {
-        bool opaque = (fShaderContext->getFlags() & SkShaderBase::kOpaqueAlpha_Flag);
+    bool opaque = (fShaderContext->getFlags() & SkShaderBase::kOpaqueAlpha_Flag);
 
-        if (mask.fFormat == SkMask::kA8_Format && opaque) {
-            blend_row = blend_row_A8_opaque;
-        } else if (mask.fFormat == SkMask::kA8_Format) {
-            blend_row = blend_row_A8;
-        } else if (mask.fFormat == SkMask::kLCD16_Format && opaque) {
-            blend_row = blend_row_LCD16_opaque;
-        } else if (mask.fFormat == SkMask::kLCD16_Format) {
-            blend_row = blend_row_lcd16;
-        } else {
-            this->INHERITED::blitMask(mask, clip);
-            return;
-        }
+    if (mask.fFormat == SkMask::kA8_Format && opaque) {
+        blend_row = blend_row_A8_opaque;
+    } else if (mask.fFormat == SkMask::kA8_Format) {
+        blend_row = blend_row_A8;
+    } else if (mask.fFormat == SkMask::kLCD16_Format && opaque) {
+        blend_row = blend_row_LCD16_opaque;
+    } else if (mask.fFormat == SkMask::kLCD16_Format) {
+        blend_row = blend_row_lcd16;
+    } else {
+        this->INHERITED::blitMask(mask, clip);
+        return;
     }
 
     const int x = clip.fLeft;
@@ -1318,27 +1227,14 @@ void SkARGB32_Shader_Blitter::blitMask(const SkMask& mask, const SkIRect& clip)
     const size_t maskRB = mask.fRowBytes;
 
     SkPMColor* span = fBuffer;
-
-    if (fXfermode) {
-        SkASSERT(SkMask::kA8_Format == mask.fFormat);
-        SkXfermode* xfer = fXfermode;
-        do {
-            fShaderContext->shadeSpan(x, y, span, width);
-            xfer->xfer32(reinterpret_cast<SkPMColor*>(dstRow), span, width, maskRow);
-            dstRow += dstRB;
-            maskRow += maskRB;
-            y += 1;
-        } while (--height > 0);
-    } else {
-        SkASSERT(blend_row);
-        do {
-            fShaderContext->shadeSpan(x, y, span, width);
-            blend_row(reinterpret_cast<SkPMColor*>(dstRow), maskRow, span, width);
-            dstRow += dstRB;
-            maskRow += maskRB;
-            y += 1;
-        } while (--height > 0);
-    }
+    SkASSERT(blend_row);
+    do {
+        fShaderContext->shadeSpan(x, y, span, width);
+        blend_row(reinterpret_cast<SkPMColor*>(dstRow), maskRow, span, width);
+        dstRow += dstRB;
+        maskRow += maskRB;
+        y += 1;
+    } while (--height > 0);
 }
 
 void SkARGB32_Shader_Blitter::blitV(int x, int y, int height, SkAlpha alpha) {
@@ -1347,40 +1243,6 @@ void SkARGB32_Shader_Blitter::blitV(int x, int y, int height, SkAlpha alpha) {
     uint32_t* device = fDevice.writable_addr32(x, y);
     size_t    deviceRB = fDevice.rowBytes();
 
-    if (fConstInY) {
-        SkPMColor c;
-        fShaderContext->shadeSpan(x, y, &c, 1);
-
-        if (fShadeDirectlyIntoDevice) {
-            if (255 == alpha) {
-                do {
-                    *device = c;
-                    device = (uint32_t*)((char*)device + deviceRB);
-                } while (--height > 0);
-            } else {
-                do {
-                    *device = SkFourByteInterp(c, *device, alpha);
-                    device = (uint32_t*)((char*)device + deviceRB);
-                } while (--height > 0);
-            }
-        } else {
-            SkXfermode* xfer = fXfermode;
-            if (xfer) {
-                do {
-                    xfer->xfer32(device, &c, 1, &alpha);
-                    device = (uint32_t*)((char*)device + deviceRB);
-                } while (--height > 0);
-            } else {
-                SkBlitRow::Proc32 proc = (255 == alpha) ? fProc32 : fProc32Blend;
-                do {
-                    proc(device, &c, 1, alpha);
-                    device = (uint32_t*)((char*)device + deviceRB);
-                } while (--height > 0);
-            }
-        }
-        return;
-    }
-
     if (fShadeDirectlyIntoDevice) {
         if (255 == alpha) {
             do {
@@ -1399,22 +1261,12 @@ void SkARGB32_Shader_Blitter::blitV(int x, int y, int height, SkAlpha alpha) {
         }
     } else {
         SkPMColor* span = fBuffer;
-        SkXfermode* xfer = fXfermode;
-        if (xfer) {
-            do {
-                fShaderContext->shadeSpan(x, y, span, 1);
-                xfer->xfer32(device, span, 1, &alpha);
-                y += 1;
-                device = (uint32_t*)((char*)device + deviceRB);
-            } while (--height > 0);
-        } else {
-            SkBlitRow::Proc32 proc = (255 == alpha) ? fProc32 : fProc32Blend;
-            do {
-                fShaderContext->shadeSpan(x, y, span, 1);
-                proc(device, span, 1, alpha);
-                y += 1;
-                device = (uint32_t*)((char*)device + deviceRB);
-            } while (--height > 0);
-        }
+        SkBlitRow::Proc32 proc = (255 == alpha) ? fProc32 : fProc32Blend;
+        do {
+            fShaderContext->shadeSpan(x, y, span, 1);
+            proc(device, span, 1, alpha);
+            y += 1;
+            device = (uint32_t*)((char*)device + deviceRB);
+        } while (--height > 0);
     }
 }
