@@ -11,7 +11,7 @@
 #include "include/core/SkFont.h"
 #include "include/core/SkMatrix.h"
 #include "include/core/SkTypeface.h"
-#include "include/private/SkTemplates.h"
+#include "include/private/base/SkTemplates.h"
 
 class SkReadBuffer;
 class SkWriteBuffer;
@@ -64,6 +64,16 @@ public:
      */
     static SkRect GetFontBounds(const SkFont&);
 
+    /** Return the approximate largest dimension of typical text when transformed by the matrix.
+     *
+     * @param matrix  used to transform size
+     * @param textLocation  location of the text prior to matrix transformation. Used if the
+     *                      matrix has perspective.
+     * @return  typical largest dimension
+     */
+    static SkScalar ApproximateTransformedTextSize(const SkFont& font, const SkMatrix& matrix,
+                                                   const SkPoint& textLocation);
+
     static bool IsFinite(const SkFont& font) {
         return SkScalarIsFinite(font.getSize()) &&
                SkScalarIsFinite(font.getScaleX()) &&
@@ -86,7 +96,7 @@ public:
     SkAutoToGlyphs(const SkFont& font, const void* text, size_t length, SkTextEncoding encoding) {
         if (encoding == SkTextEncoding::kGlyphID || length == 0) {
             fGlyphs = reinterpret_cast<const uint16_t*>(text);
-            fCount = length >> 1;
+            fCount = SkToInt(length >> 1);
         } else {
             fCount = font.countText(text, length, encoding);
             if (fCount < 0) {
@@ -102,7 +112,7 @@ public:
     const uint16_t* glyphs() const { return fGlyphs; }
 
 private:
-    SkAutoSTArray<32, uint16_t> fStorage;
+    skia_private::AutoSTArray<32, uint16_t> fStorage;
     const uint16_t* fGlyphs;
     int             fCount;
 };
