@@ -10,16 +10,15 @@
 #ifdef SK_ENABLE_SKSL
 #include "include/core/SkColorSpace.h"
 #include "include/core/SkData.h"
-#include "include/private/SkOpts_spi.h"
-#include "include/private/SkSLProgramElement.h"
-#include "include/private/SkSLProgramKind.h"
 #include "include/private/base/SkMath.h"
 #include "src/base/SkSafeMath.h"
+#include "src/core/SkChecksum.h"
 #include "src/core/SkMeshPriv.h"
 #include "src/core/SkRuntimeEffectPriv.h"
 #include "src/sksl/SkSLAnalysis.h"
 #include "src/sksl/SkSLBuiltinTypes.h"
 #include "src/sksl/SkSLCompiler.h"
+#include "src/sksl/SkSLProgramKind.h"
 #include "src/sksl/SkSLProgramSettings.h"
 #include "src/sksl/SkSLUtil.h"
 #include "src/sksl/analysis/SkSLProgramVisitor.h"
@@ -27,6 +26,7 @@
 #include "src/sksl/ir/SkSLFunctionDeclaration.h"
 #include "src/sksl/ir/SkSLFunctionDefinition.h"
 #include "src/sksl/ir/SkSLProgram.h"
+#include "src/sksl/ir/SkSLProgramElement.h"
 #include "src/sksl/ir/SkSLReturnStatement.h"
 #include "src/sksl/ir/SkSLStructDefinition.h"
 #include "src/sksl/ir/SkSLType.h"
@@ -34,10 +34,10 @@
 #include "src/sksl/ir/SkSLVariable.h"
 #include "src/sksl/ir/SkSLVariableReference.h"
 
-#if SK_SUPPORT_GPU
+#if defined(SK_GANESH)
 #include "src/gpu/ganesh/GrGpu.h"
 #include "src/gpu/ganesh/GrStagingBufferManager.h"
-#endif  // SK_SUPPORT_GPU
+#endif  // defined(SK_GANESH)
 
 #include <locale>
 #include <string>
@@ -45,6 +45,8 @@
 #include <type_traits>
 #include <utility>
 
+using namespace skia_private;
+
 using Attribute = SkMeshSpecification::Attribute;
 using Varying   = SkMeshSpecification::Varying;
 
@@ -318,7 +320,7 @@ int check_for_passthrough_local_coords_and_dead_varyings(const SkSL::Program& fs
                 // and mark the returned field as used.
                 return false;
             }
-            const Type::Field& field = fVaryings->type().fields()[fa.fieldIndex()];
+            const Field& field = fVaryings->type().fields()[fa.fieldIndex()];
             if (!field.fType->matches(*fContext.fTypes.fFloat2)) {
                 this->passthroughFailed();
                 return ProgramVisitor::visitStatement(s);
@@ -413,7 +415,7 @@ SkMeshSpecification::Result SkMeshSpecification::Make(SkSpan<const Attribute> at
         }
     }
 
-    SkSTArray<kMaxVaryings, Varying> tempVaryings;
+    STArray<kMaxVaryings, Varying> tempVaryings;
     if (!userProvidedPositionVarying) {
         // Even though we check the # of varyings in MakeFromSourceWithStructs we check here, too,
         // to avoid overflow with + 1.
@@ -594,24 +596,24 @@ SkMeshSpecification::SkMeshSpecification(
         , fColorType(ct)
         , fColorSpace(std::move(cs))
         , fAlphaType(at) {
-    fHash = SkOpts::hash_fn(fVS->fSource->c_str(), fVS->fSource->size(), 0);
-    fHash = SkOpts::hash_fn(fFS->fSource->c_str(), fFS->fSource->size(), fHash);
+    fHash = SkChecksum::Hash32(fVS->fSource->c_str(), fVS->fSource->size(), 0);
+    fHash = SkChecksum::Hash32(fFS->fSource->c_str(), fFS->fSource->size(), fHash);
 
     // The attributes and varyings SkSL struct declarations are included in the program source.
     // However, the attribute offsets and types need to be included, the latter because the SkSL
     // struct definition has the GPU type but not the CPU data format.
     for (const auto& a : fAttributes) {
-        fHash = SkOpts::hash_fn(&a.offset, sizeof(a.offset), fHash);
-        fHash = SkOpts::hash_fn(&a.type,   sizeof(a.type),   fHash);
+        fHash = SkChecksum::Hash32(&a.offset, sizeof(a.offset), fHash);
+        fHash = SkChecksum::Hash32(&a.type,   sizeof(a.type),   fHash);
     }
 
-    fHash = SkOpts::hash_fn(&stride, sizeof(stride), fHash);
+    fHash = SkChecksum::Hash32(&stride, sizeof(stride), fHash);
 
     uint64_t csHash = fColorSpace ? fColorSpace->hash() : 0;
-    fHash = SkOpts::hash_fn(&csHash, sizeof(csHash), fHash);
+    fHash = SkChecksum::Hash32(&csHash, sizeof(csHash), fHash);
 
     auto atInt = static_cast<uint32_t>(fAlphaType);
-    fHash = SkOpts::hash_fn(&atInt, sizeof(atInt), fHash);
+    fHash = SkChecksum::Hash32(&atInt, sizeof(atInt), fHash);
 }
 
 size_t SkMeshSpecification::uniformSize() const {
@@ -655,7 +657,7 @@ sk_sp<IndexBuffer> SkMesh::MakeIndexBuffer(GrDirectContext* dc, const void* data
     if (!dc) {
         return SkMeshPriv::CpuIndexBuffer::Make(data, size);
     }
-#if SK_SUPPORT_GPU
+#if defined(SK_GANESH)
     return SkMeshPriv::GpuIndexBuffer::Make(dc, data, size);
 #else
     return nullptr;
@@ -678,7 +680,7 @@ sk_sp<VertexBuffer> SkMesh::MakeVertexBuffer(GrDirectContext* dc, const void* da
     if (!dc) {
         return SkMeshPriv::CpuVertexBuffer::Make(data, size);
     }
-#if SK_SUPPORT_GPU
+#if defined(SK_GANESH)
     return SkMeshPriv::GpuVertexBuffer::Make(dc, data, size);
 #else
     return nullptr;
@@ -868,7 +870,7 @@ bool SkMesh::VertexBuffer::update(GrDirectContext* dc,
     return check_update(data, offset, size, this->size()) && this->onUpdate(dc, data, offset, size);
 }
 
-#if SK_SUPPORT_GPU
+#if defined(SK_GANESH)
 bool SkMeshPriv::UpdateGpuBuffer(GrDirectContext* dc,
                                  sk_sp<GrGpuBuffer> buffer,
                                  const void* data,
@@ -920,6 +922,6 @@ bool SkMeshPriv::UpdateGpuBuffer(GrDirectContext* dc,
 
     return true;
 }
-#endif  // SK_SUPPORT_GPU
+#endif  // defined(SK_GANESH)
 
 #endif  // SK_ENABLE_SKSL
