@@ -13,33 +13,36 @@
 #include "src/base/SkArenaAlloc.h"
 #include "src/base/SkUtils.h"
 #include "src/core/SkBlendModePriv.h"
+#include "src/core/SkBlenderBase.h"
 #include "src/core/SkBlitter.h"
-#include "src/core/SkColorFilterBase.h"
 #include "src/core/SkColorSpacePriv.h"
 #include "src/core/SkColorSpaceXformSteps.h"
+#include "src/core/SkEffectPriv.h"
+#include "src/core/SkMask.h"
 #include "src/core/SkMatrixProvider.h"
 #include "src/core/SkOpts.h"
 #include "src/core/SkRasterPipeline.h"
+#include "src/effects/colorfilters/SkColorFilterBase.h"
 #include "src/shaders/SkShaderBase.h"
 
-#define SK_BLITTER_TRACE_IS_RASTER_PIPELINE
-#include "src/utils/SkBlitterTrace.h"
-
 class SkRasterPipelineBlitter final : public SkBlitter {
 public:
     // This is our common entrypoint for creating the blitter once we've sorted out shaders.
-    static SkBlitter* Create(const SkPixmap&, const SkPaint&, SkArenaAlloc*,
+    static SkBlitter* Create(const SkPixmap& dst,
+                             const SkPaint& paint,
+                             const SkColor4f& dstPaintColor,
+                             SkArenaAlloc* alloc,
                              const SkRasterPipeline& shaderPipeline,
-                             bool is_opaque, bool is_constant,
+                             bool is_opaque,
+                             bool is_constant,
                              sk_sp<SkShader> clipShader);
 
     SkRasterPipelineBlitter(SkPixmap dst,
-                            SkBlendMode blend,
                             SkArenaAlloc* alloc)
         : fDst(dst)
-        , fBlend(blend)
         , fAlloc(alloc)
         , fColorPipeline(alloc)
+        , fBlendPipeline(alloc)
     {}
 
     void blitH     (int x, int y, int w)                            override;
@@ -60,9 +63,11 @@ private:
     void append_clip_lerp     (SkRasterPipeline*) const;
 
     SkPixmap               fDst;
-    SkBlendMode            fBlend;
     SkArenaAlloc*          fAlloc;
     SkRasterPipeline       fColorPipeline;
+    SkRasterPipeline       fBlendPipeline;
+    // If the blender is a blend-mode, we retain that information for late-stage optimizations
+    std::optional<SkBlendMode> fBlendMode;
     // set to pipeline storage (for alpha) if we have a clipShader
     void*                  fClipShaderBuffer = nullptr; // "native" : float or U16
 
@@ -90,48 +95,46 @@ private:
     using INHERITED = SkBlitter;
 };
 
+static SkColor4f paint_color_to_dst(const SkPaint& paint, const SkPixmap& dst) {
+    SkColor4f paintColor = paint.getColor4f();
+    SkColorSpaceXformSteps(sk_srgb_singleton(), kUnpremul_SkAlphaType,
+                           dst.colorSpace(),    kUnpremul_SkAlphaType).apply(paintColor.vec());
+    return paintColor;
+}
+
 SkBlitter* SkCreateRasterPipelineBlitter(const SkPixmap& dst,
                                          const SkPaint& paint,
-                                         const SkMatrixProvider& matrixProvider,
+                                         const SkMatrix& ctm,
                                          SkArenaAlloc* alloc,
                                          sk_sp<SkShader> clipShader,
                                          const SkSurfaceProps& props) {
-    if (!paint.asBlendMode()) {
-        // The raster pipeline doesn't support SkBlender.
-        return nullptr;
-    }
-
     SkColorSpace* dstCS = dst.colorSpace();
     SkColorType dstCT = dst.colorType();
-    SkColor4f paintColor = paint.getColor4f();
-    SkColorSpaceXformSteps(sk_srgb_singleton(), kUnpremul_SkAlphaType,
-                           dstCS,               kUnpremul_SkAlphaType).apply(paintColor.vec());
+    SkColor4f dstPaintColor = paint_color_to_dst(paint, dst);
 
     auto shader = as_SB(paint.getShader());
 
     SkRasterPipeline_<256> shaderPipeline;
     if (!shader) {
-        // Having no shader makes things nice and easy... just use the paint color.
-        shaderPipeline.append_constant_color(alloc, paintColor.premul().vec());
-        bool is_opaque    = paintColor.fA == 1.0f,
+        // Having no shader makes things nice and easy... just use the paint color
+        shaderPipeline.append_constant_color(alloc, dstPaintColor.premul().vec());
+        bool is_opaque    = dstPaintColor.fA == 1.0f,
              is_constant  = true;
-        return SkRasterPipelineBlitter::Create(dst, paint, alloc,
-                                               shaderPipeline, is_opaque, is_constant,
-                                               std::move(clipShader));
+        return SkRasterPipelineBlitter::Create(dst, paint, dstPaintColor, alloc, shaderPipeline,
+                                               is_opaque, is_constant, std::move(clipShader));
     }
 
-    bool is_opaque    = shader->isOpaque() && paintColor.fA == 1.0f;
+    bool is_opaque    = shader->isOpaque() && dstPaintColor.fA == 1.0f;
     bool is_constant  = shader->isConstant();
 
-    if (shader->appendStages(
-                {&shaderPipeline, alloc, dstCT, dstCS, paint, nullptr, matrixProvider, props})) {
-        if (paintColor.fA != 1.0f) {
+    if (shader->appendRootStages({&shaderPipeline, alloc, dstCT, dstCS, dstPaintColor, props},
+                                 ctm)) {
+        if (dstPaintColor.fA != 1.0f) {
             shaderPipeline.append(SkRasterPipelineOp::scale_1_float,
-                                  alloc->make<float>(paintColor.fA));
+                                  alloc->make<float>(dstPaintColor.fA));
         }
-        return SkRasterPipelineBlitter::Create(dst, paint, alloc,
-                                               shaderPipeline, is_opaque, is_constant,
-                                               std::move(clipShader));
+        return SkRasterPipelineBlitter::Create(dst, paint, dstPaintColor, alloc, shaderPipeline,
+                                               is_opaque, is_constant, std::move(clipShader));
     }
 
     // The shader can't draw with SkRasterPipeline.
@@ -145,40 +148,39 @@ SkBlitter* SkCreateRasterPipelineBlitter(const SkPixmap& dst,
                                          SkArenaAlloc* alloc,
                                          sk_sp<SkShader> clipShader) {
     bool is_constant = false;  // If this were the case, it'd be better to just set a paint color.
-    return SkRasterPipelineBlitter::Create(dst, paint, alloc,
+    return SkRasterPipelineBlitter::Create(dst, paint, paint_color_to_dst(paint, dst), alloc,
                                            shaderPipeline, is_opaque, is_constant,
-                                           clipShader);
+                                           std::move(clipShader));
 }
 
 SkBlitter* SkRasterPipelineBlitter::Create(const SkPixmap& dst,
                                            const SkPaint& paint,
+                                           const SkColor4f& dstPaintColor,
                                            SkArenaAlloc* alloc,
                                            const SkRasterPipeline& shaderPipeline,
                                            bool is_opaque,
                                            bool is_constant,
                                            sk_sp<SkShader> clipShader) {
-    const auto bm = paint.asBlendMode();
-    if (!bm) {
-        return nullptr;
-    }
+    auto blitter = alloc->make<SkRasterPipelineBlitter>(dst, alloc);
 
-    auto blitter = alloc->make<SkRasterPipelineBlitter>(dst, bm.value(), alloc);
+    // Our job in this factory is to fill out the blitter's color and blend pipelines.
+    // The color pipeline is the common front of the full blit pipeline. The blend pipeline is just
+    // the portion that does the actual blending math (and assumes that src and dst are already
+    // loaded).
+    //
+    // The full blit pipelines are each constructed lazily on first use, and include the color
+    // pipeline, reading the dst, the blend pipeline, coverage, dithering, and writing the dst.
 
-    // Our job in this factory is to fill out the blitter's color pipeline.
-    // This is the common front of the full blit pipelines, each constructed lazily on first use.
-    // The full blit pipelines handle reading and writing the dst, blending, coverage, dithering.
+    // Start with the color pipeline
     auto colorPipeline = &blitter->fColorPipeline;
 
     if (clipShader) {
         auto clipP = colorPipeline;
-        SkPaint clipPaint;  // just need default values
         SkColorType clipCT = kRGBA_8888_SkColorType;
         SkColorSpace* clipCS = nullptr;
-        SkMatrixProvider clipMatrixProvider(SkMatrix::I());
         SkSurfaceProps props{}; // default OK; clipShader doesn't render text
-        SkStageRec rec = {clipP, alloc, clipCT, clipCS, clipPaint, nullptr, clipMatrixProvider,
-                          props};
-        if (as_SB(clipShader)->appendStages(rec)) {
+        SkStageRec rec = {clipP, alloc, clipCT, clipCS, SkColors::kBlack, props};
+        if (as_SB(clipShader)->appendRootStages(rec, SkMatrix::I())) {
             struct Storage {
                 // large enough for highp (float) or lowp(U16)
                 float   fA[SkRasterPipeline_kMaxStride];
@@ -197,10 +199,9 @@ SkBlitter* SkRasterPipelineBlitter::Create(const SkPixmap& dst,
 
     // If there's a color filter it comes next.
     if (auto colorFilter = paint.getColorFilter()) {
-        SkMatrixProvider matrixProvider(SkMatrix::I());
         SkSurfaceProps props{}; // default OK; colorFilter doesn't render text
-        SkStageRec rec = {colorPipeline, alloc, dst.colorType(), dst.colorSpace(), paint, nullptr,
-                          matrixProvider, props};
+        SkStageRec rec = {
+                colorPipeline, alloc, dst.colorType(), dst.colorSpace(), dstPaintColor, props};
         if (!as_CFB(colorFilter)->appendStages(rec, is_opaque)) {
             return nullptr;
         }
@@ -234,6 +235,7 @@ SkBlitter* SkRasterPipelineBlitter::Create(const SkPixmap& dst,
 
             case kUnknown_SkColorType:
             case kAlpha_8_SkColorType:
+            case kBGR_101010x_XR_SkColorType:
             case kRGBA_F16_SkColorType:
             case kRGBA_F16Norm_SkColorType:
             case kRGBA_F32_SkColorType:
@@ -251,9 +253,7 @@ SkBlitter* SkRasterPipelineBlitter::Create(const SkPixmap& dst,
         }
     }
 
-    // We're logically done here.  The code between here and return blitter is all optimization.
-
-    // A pipeline that's still constant here can collapse back into a constant color.
+    // Optimization: A pipeline that's still constant here can collapse back into a constant color.
     if (is_constant) {
         SkColor4f constantColor;
         SkRasterPipeline_MemoryCtx constantColorPtr = { &constantColor, 0 };
@@ -269,14 +269,23 @@ SkBlitter* SkRasterPipelineBlitter::Create(const SkPixmap& dst,
         is_opaque = constantColor.fA == 1.0f;
     }
 
+    // Now we'll build the blend pipeline
+    auto blendPipeline = &blitter->fBlendPipeline;
+
+    sk_sp<SkBlender> blender = paint.refBlender();
+    if (!blender) {
+        blender = SkBlender::Mode(SkBlendMode::kSrcOver);
+    }
+
     // We can strength-reduce SrcOver into Src when opaque.
-    if (is_opaque && blitter->fBlend == SkBlendMode::kSrcOver) {
-        blitter->fBlend = SkBlendMode::kSrc;
+    if (is_opaque && as_BB(blender)->asBlendMode() == SkBlendMode::kSrcOver) {
+        blender = SkBlender::Mode(SkBlendMode::kSrc);
     }
 
     // When we're drawing a constant color in Src mode, we can sometimes just memset.
     // (The previous two optimizations help find more opportunities for this one.)
-    if (is_constant && blitter->fBlend == SkBlendMode::kSrc) {
+    if (is_constant && as_BB(blender)->asBlendMode() == SkBlendMode::kSrc &&
+        dst.info().bytesPerPixel() <= static_cast<int>(sizeof(blitter->fMemsetColor))) {
         // Run our color pipeline all the way through to produce what we'd memset when we can.
         // Not all blits can memset, so we need to keep colorPipeline too.
         SkRasterPipeline_<256> p;
@@ -310,6 +319,16 @@ SkBlitter* SkRasterPipelineBlitter::Create(const SkPixmap& dst,
         }
     }
 
+    {
+        SkSurfaceProps props{};  // default OK; blender doesn't render text
+        SkStageRec rec = {
+                blendPipeline, alloc, dst.colorType(), dst.colorSpace(), dstPaintColor, props};
+        if (!as_BB(blender)->appendStages(rec)) {
+            return nullptr;
+        }
+        blitter->fBlendMode = as_BB(blender)->asBlendMode();
+    }
+
     blitter->fDstPtr = SkRasterPipeline_MemoryCtx{
         blitter->fDst.writable_addr(),
         blitter->fDst.rowBytesAsPixels(),
@@ -354,10 +373,6 @@ void SkRasterPipelineBlitter::blitRect(int x, int y, int w, int h) {
 
 void SkRasterPipelineBlitter::blitRectWithTrace(int x, int y, int w, int h, bool trace) {
     if (fMemset2D) {
-        SK_BLITTER_TRACE_STEP(blitRectByMemset,
-                           trace,
-                           /*scanlines=*/h,
-                           /*pixels=*/w * h);
         fMemset2D(&fDst, x,y, w,h, fMemsetColor);
         return;
     }
@@ -366,7 +381,7 @@ void SkRasterPipelineBlitter::blitRectWithTrace(int x, int y, int w, int h, bool
         SkRasterPipeline p(fAlloc);
         p.extend(fColorPipeline);
         p.append_clamp_if_normalized(fDst.info());
-        if (fBlend == SkBlendMode::kSrcOver
+        if (fBlendMode == SkBlendMode::kSrcOver
                 && (fDst.info().colorType() == kRGBA_8888_SkColorType ||
                     fDst.info().colorType() == kBGRA_8888_SkColorType)
                 && !fDst.colorSpace()
@@ -378,9 +393,9 @@ void SkRasterPipelineBlitter::blitRectWithTrace(int x, int y, int w, int h, bool
             this->append_clip_scale(&p);
             p.append(SkRasterPipelineOp::srcover_rgba_8888, &fDstPtr);
         } else {
-            if (fBlend != SkBlendMode::kSrc) {
+            if (fBlendMode != SkBlendMode::kSrc) {
                 this->append_load_dst(&p);
-                SkBlendMode_AppendStages(fBlend, &p);
+                p.extend(fBlendPipeline);
                 this->append_clip_lerp(&p);
             } else if (fClipShaderBuffer) {
                 this->append_load_dst(&p);
@@ -391,7 +406,6 @@ void SkRasterPipelineBlitter::blitRectWithTrace(int x, int y, int w, int h, bool
         fBlitRect = p.compile();
     }
 
-    SK_BLITTER_TRACE_STEP(blitRect, trace, /*scanlines=*/h, /*pixels=*/w * h);
     fBlitRect(x,y,w,h);
 }
 
@@ -400,14 +414,15 @@ void SkRasterPipelineBlitter::blitAntiH(int x, int y, const SkAlpha aa[], const
         SkRasterPipeline p(fAlloc);
         p.extend(fColorPipeline);
         p.append_clamp_if_normalized(fDst.info());
-        if (SkBlendMode_ShouldPreScaleCoverage(fBlend, /*rgb_coverage=*/false)) {
+        if (fBlendMode.has_value() &&
+            SkBlendMode_ShouldPreScaleCoverage(*fBlendMode, /*rgb_coverage=*/false)) {
             p.append(SkRasterPipelineOp::scale_1_float, &fCurrentCoverage);
             this->append_clip_scale(&p);
             this->append_load_dst(&p);
-            SkBlendMode_AppendStages(fBlend, &p);
+            p.extend(fBlendPipeline);
         } else {
             this->append_load_dst(&p);
-            SkBlendMode_AppendStages(fBlend, &p);
+            p.extend(fBlendPipeline);
             p.append(SkRasterPipelineOp::lerp_1_float, &fCurrentCoverage);
             this->append_clip_lerp(&p);
         }
@@ -416,9 +431,7 @@ void SkRasterPipelineBlitter::blitAntiH(int x, int y, const SkAlpha aa[], const
         fBlitAntiH = p.compile();
     }
 
-    SK_BLITTER_TRACE_STEP(blitAntiH, true, /*scanlines=*/1ul, /*pixels=*/0ul);
     for (int16_t run = *runs; run > 0; run = *runs) {
-        SK_BLITTER_TRACE_STEP_ACCUMULATE(blitAntiH, /*pixels=*/run);
         switch (*aa) {
             case 0x00:                                break;
             case 0xff:this->blitRectWithTrace(x,y,run, 1, false); break;
@@ -509,14 +522,15 @@ void SkRasterPipelineBlitter::blitMask(const SkMask& mask, const SkIRect& clip)
         SkRasterPipeline p(fAlloc);
         p.extend(fColorPipeline);
         p.append_clamp_if_normalized(fDst.info());
-        if (SkBlendMode_ShouldPreScaleCoverage(fBlend, /*rgb_coverage=*/false)) {
+        if (fBlendMode.has_value() &&
+            SkBlendMode_ShouldPreScaleCoverage(*fBlendMode, /*rgb_coverage=*/false)) {
             p.append(SkRasterPipelineOp::scale_u8, &fMaskPtr);
             this->append_clip_scale(&p);
             this->append_load_dst(&p);
-            SkBlendMode_AppendStages(fBlend, &p);
+            p.extend(fBlendPipeline);
         } else {
             this->append_load_dst(&p);
-            SkBlendMode_AppendStages(fBlend, &p);
+            p.extend(fBlendPipeline);
             p.append(SkRasterPipelineOp::lerp_u8, &fMaskPtr);
             this->append_clip_lerp(&p);
         }
@@ -527,15 +541,16 @@ void SkRasterPipelineBlitter::blitMask(const SkMask& mask, const SkIRect& clip)
         SkRasterPipeline p(fAlloc);
         p.extend(fColorPipeline);
         p.append_clamp_if_normalized(fDst.info());
-        if (SkBlendMode_ShouldPreScaleCoverage(fBlend, /*rgb_coverage=*/true)) {
+        if (fBlendMode.has_value() &&
+            SkBlendMode_ShouldPreScaleCoverage(*fBlendMode, /*rgb_coverage=*/true)) {
             // Somewhat unusually, scale_565 needs dst loaded first.
             this->append_load_dst(&p);
             p.append(SkRasterPipelineOp::scale_565, &fMaskPtr);
             this->append_clip_scale(&p);
-            SkBlendMode_AppendStages(fBlend, &p);
+            p.extend(fBlendPipeline);
         } else {
             this->append_load_dst(&p);
-            SkBlendMode_AppendStages(fBlend, &p);
+            p.extend(fBlendPipeline);
             p.append(SkRasterPipelineOp::lerp_565, &fMaskPtr);
             this->append_clip_lerp(&p);
         }
@@ -549,14 +564,15 @@ void SkRasterPipelineBlitter::blitMask(const SkMask& mask, const SkIRect& clip)
         p.append(SkRasterPipelineOp::emboss, &fEmbossCtx);
         // Now onward just as kA8.
         p.append_clamp_if_normalized(fDst.info());
-        if (SkBlendMode_ShouldPreScaleCoverage(fBlend, /*rgb_coverage=*/false)) {
+        if (fBlendMode.has_value() &&
+            SkBlendMode_ShouldPreScaleCoverage(*fBlendMode, /*rgb_coverage=*/false)) {
             p.append(SkRasterPipelineOp::scale_u8, &fMaskPtr);
             this->append_clip_scale(&p);
             this->append_load_dst(&p);
-            SkBlendMode_AppendStages(fBlend, &p);
+            p.extend(fBlendPipeline);
         } else {
             this->append_load_dst(&p);
-            SkBlendMode_AppendStages(fBlend, &p);
+            p.extend(fBlendPipeline);
             p.append(SkRasterPipelineOp::lerp_u8, &fMaskPtr);
             this->append_clip_lerp(&p);
         }
@@ -575,9 +591,5 @@ void SkRasterPipelineBlitter::blitMask(const SkMask& mask, const SkIRect& clip)
     }
 
     SkASSERT(blitter);
-    SK_BLITTER_TRACE_STEP(blitMask,
-                       true,
-                       /*scanlines=*/clip.height(),
-                       /*pixels=*/clip.width() * clip.height());
     (*blitter)(clip.left(),clip.top(), clip.width(),clip.height());
 }
