@@ -17,8 +17,9 @@
 #include "include/private/base/SkTo.h"
 #include "src/base/SkAutoMalloc.h"
 #include "src/core/SkAutoPixmapStorage.h"
+#include "src/core/SkBlitter_A8.h"
 #include "src/core/SkDescriptor.h"
-#include "src/core/SkDraw.h"
+#include "src/core/SkDrawBase.h"
 #include "src/core/SkFontPriv.h"
 #include "src/core/SkGlyph.h"
 #include "src/core/SkMaskGamma.h"
@@ -530,8 +531,9 @@ void SkScalerContext::GenerateImageFromPath(
     }
     sk_bzero(dst.writable_addr(), dst.computeByteSize());
 
-    SkDraw  draw;
+    SkDrawBase  draw;
     SkMatrixProvider matrixProvider(matrix);
+    draw.fBlitterChooser = SkA8Blitter_Choose;
     draw.fDst            = dst;
     draw.fRC             = &clip;
     draw.fMatrixProvider = &matrixProvider;
