@@ -14,7 +14,7 @@
 #include "include/core/SkSurfaceProps.h"
 #include "src/core/SkNextID.h"
 
-#if SK_SUPPORT_GPU
+#if defined(SK_GANESH)
 #include "include/private/gpu/ganesh/GrTypesPriv.h"
 #include "src/gpu/ganesh/GrSurfaceProxyView.h"
 #endif
@@ -98,7 +98,7 @@ public:
     static sk_sp<SkSpecialImage> CopyFromRaster(const SkIRect& subset,
                                                 const SkBitmap&,
                                                 const SkSurfaceProps&);
-#if SK_SUPPORT_GPU
+#if defined(SK_GANESH)
     static sk_sp<SkSpecialImage> MakeDeferredFromGpu(GrRecordingContext*,
                                                      const SkIRect& subset,
                                                      uint32_t uniqueID,
@@ -107,7 +107,7 @@ public:
                                                      const SkSurfaceProps&);
 #endif
 
-#if SK_GRAPHITE_ENABLED
+#if defined(SK_GRAPHITE)
     static sk_sp<SkSpecialImage> MakeGraphite(skgpu::graphite::Recorder*,
                                               const SkIRect& subset,
                                               uint32_t uniqueID,
@@ -116,26 +116,6 @@ public:
                                               const SkSurfaceProps&);
 #endif
 
-    /**
-     *  Create a new special surface with a backend that is compatible with this special image.
-     */
-    sk_sp<SkSpecialSurface> makeSurface(SkColorType,
-                                        const SkColorSpace*,
-                                        const SkISize& size,
-                                        SkAlphaType,
-                                        const SkSurfaceProps&) const;
-
-    /**
-     * Create a new surface with a backend that is compatible with this special image.
-     * TODO: switch this to makeSurface once we resolved the naming issue
-     * TODO (michaelludwig) - This is only used by SkTileImageFilter, which appears should be
-     * updated to work correctly with subsets and then makeTightSurface() can go away entirely.
-     */
-    sk_sp<SkSurface> makeTightSurface(SkColorType,
-                                      const SkColorSpace*,
-                                      const SkISize& size,
-                                      SkAlphaType = kPremul_SkAlphaType) const;
-
     /**
      * Extract a subset of this special image and return it as a special image.
      * It may or may not point to the same backing memory. The input 'subset' is relative to the
@@ -178,7 +158,7 @@ public:
      */
     GrRecordingContext* getContext() const { return this->onGetContext(); }
 
-#if SK_SUPPORT_GPU
+#if defined(SK_GANESH)
     /**
      * Regardless of how the underlying backing data is stored, returns the contents as a
      * GrSurfaceProxyView. The returned view's proxy represents the entire backing image, so texture
@@ -188,7 +168,7 @@ public:
     GrSurfaceProxyView view(GrRecordingContext* context) const { return this->onView(context); }
 #endif
 
-#if SK_GRAPHITE_ENABLED
+#if defined(SK_GRAPHITE)
     bool isGraphiteBacked() const;
 
     skgpu::graphite::TextureProxyView textureProxyView() const;
@@ -218,11 +198,11 @@ protected:
 
     virtual GrRecordingContext* onGetContext() const { return nullptr; }
 
-#if SK_SUPPORT_GPU
+#if defined(SK_GANESH)
     virtual GrSurfaceProxyView onView(GrRecordingContext*) const = 0;
 #endif
 
-#if SK_GRAPHITE_ENABLED
+#if defined(SK_GRAPHITE)
     virtual skgpu::graphite::TextureProxyView onTextureProxyView() const;
 #endif
 
@@ -230,12 +210,6 @@ protected:
     // from the content rect by the non-virtual makeSubset().
     virtual sk_sp<SkSpecialImage> onMakeSubset(const SkIRect& subset) const = 0;
 
-    virtual sk_sp<SkSpecialSurface> onMakeSurface(SkColorType colorType,
-                                                  const SkColorSpace* colorSpace,
-                                                  const SkISize& size,
-                                                  SkAlphaType at,
-                                                  const SkSurfaceProps&) const = 0;
-
     // This subset (when not null) is relative to the backing store's coordinate frame, it has
     // already been mapped from the content rect by the non-virtual asImage().
     virtual sk_sp<SkImage> onAsImage(const SkIRect* subset) const = 0;
@@ -244,11 +218,6 @@ protected:
                                        const SkSamplingOptions&,
                                        const SkMatrix&) const = 0;
 
-    virtual sk_sp<SkSurface> onMakeTightSurface(SkColorType colorType,
-                                                const SkColorSpace* colorSpace,
-                                                const SkISize& size,
-                                                SkAlphaType at) const = 0;
-
 #ifdef SK_DEBUG
     static bool RectFits(const SkIRect& rect, int width, int height);
 #endif
