@@ -18,19 +18,18 @@
 #include "include/core/SkPoint.h"
 #include "include/core/SkRect.h"
 #include "include/core/SkSize.h"
+#include "include/core/SkSurfaceProps.h"
 #include "include/effects/SkRuntimeEffect.h"
 #include "include/gpu/GrBackendSurface.h"
 #include "include/gpu/GrRecordingContext.h"
 #include "include/private/SkIDChangeListener.h"
 #include "include/private/base/SkTPin.h"
 #include "include/private/gpu/ganesh/GrTypesPriv.h"
-#include "src/core/SkBlendModePriv.h"
 #include "src/core/SkBlenderBase.h"
-#include "src/core/SkColorFilterBase.h"
-#include "src/core/SkMaskFilterBase.h"
 #include "src/core/SkMessageBus.h"
 #include "src/core/SkPaintPriv.h"
 #include "src/core/SkRuntimeEffectPriv.h"
+#include "src/gpu/DitherUtils.h"
 #include "src/gpu/ResourceKey.h"
 #include "src/gpu/Swizzle.h"
 #include "src/gpu/ganesh/GrCaps.h"
@@ -38,12 +37,14 @@
 #include "src/gpu/ganesh/GrColorSpaceXform.h"
 #include "src/gpu/ganesh/GrFPArgs.h"
 #include "src/gpu/ganesh/GrFragmentProcessor.h"
+#include "src/gpu/ganesh/GrFragmentProcessors.h"
 #include "src/gpu/ganesh/GrPaint.h"
 #include "src/gpu/ganesh/GrProxyProvider.h"
 #include "src/gpu/ganesh/GrRecordingContextPriv.h"
 #include "src/gpu/ganesh/GrSurfaceProxy.h"
 #include "src/gpu/ganesh/GrSurfaceProxyView.h"
 #include "src/gpu/ganesh/GrTextureProxy.h"
+#include "src/gpu/ganesh/GrXferProcessor.h"
 #include "src/gpu/ganesh/effects/GrSkSLFP.h"
 #include "src/gpu/ganesh/effects/GrTextureEffect.h"
 #include "src/shaders/SkShaderBase.h"
@@ -285,83 +286,8 @@ static inline bool blender_requires_shader(const SkBlender* blender) {
     return !mode.has_value() || *mode != SkBlendMode::kDst;
 }
 
+
 #ifndef SK_IGNORE_GPU_DITHER
-static inline float dither_range_for_config(GrColorType dstColorType) {
-    // We use 1 / (2^bitdepth-1) as the range since each channel can hold 2^bitdepth values
-    switch (dstColorType) {
-        // 4 bit
-        case GrColorType::kABGR_4444:
-        case GrColorType::kARGB_4444:
-        case GrColorType::kBGRA_4444:
-            return 1 / 15.f;
-        // 6 bit
-        case GrColorType::kBGR_565:
-            return 1 / 63.f;
-        // 8 bit
-        case GrColorType::kUnknown:
-        case GrColorType::kAlpha_8:
-        case GrColorType::kAlpha_8xxx:
-        case GrColorType::kGray_8:
-        case GrColorType::kGrayAlpha_88:
-        case GrColorType::kGray_8xxx:
-        case GrColorType::kR_8:
-        case GrColorType::kR_8xxx:
-        case GrColorType::kRG_88:
-        case GrColorType::kRGB_888:
-        case GrColorType::kRGB_888x:
-        case GrColorType::kRGBA_8888:
-        case GrColorType::kRGBA_8888_SRGB:
-        case GrColorType::kBGRA_8888:
-            return 1 / 255.f;
-        // 10 bit
-        case GrColorType::kRGBA_1010102:
-        case GrColorType::kBGRA_1010102:
-            return 1 / 1023.f;
-        // 16 bit
-        case GrColorType::kAlpha_16:
-        case GrColorType::kR_16:
-        case GrColorType::kRG_1616:
-        case GrColorType::kRGBA_16161616:
-            return 1 / 32767.f;
-        // Half
-        case GrColorType::kAlpha_F16:
-        case GrColorType::kGray_F16:
-        case GrColorType::kR_F16:
-        case GrColorType::kRG_F16:
-        case GrColorType::kRGBA_F16:
-        case GrColorType::kRGBA_F16_Clamped:
-        // Float
-        case GrColorType::kAlpha_F32xxx:
-        case GrColorType::kRGBA_F32:
-            return 0.f; // no dithering
-    }
-    SkUNREACHABLE;
-}
-
-static SkBitmap make_dither_lut() {
-    static constexpr struct DitherTable {
-        constexpr DitherTable() : data() {
-            for (int x = 0; x < 8; ++x) {
-                for (int y = 0; y < 8; ++y) {
-                    // The computation of 'm' and 'value' is lifted from CPU backend.
-                    unsigned int m = (y & 1) << 5 | (x & 1) << 4 |
-                                     (y & 2) << 2 | (x & 2) << 1 |
-                                     (y & 4) >> 1 | (x & 4) >> 2;
-                    float value = float(m) * 1.0 / 64.0 - 63.0 / 128.0;
-                    // Bias by 0.5 to be in 0..1, mul by 255 and round to nearest int to make byte.
-                    data[y * 8 + x] = (uint8_t)((value + 0.5) * 255.f + 0.5f);
-                }
-            }
-        }
-        uint8_t data[64];
-    } gTable;
-    SkBitmap bmp;
-    bmp.setInfo(SkImageInfo::MakeA8(8, 8));
-    bmp.setPixels(const_cast<uint8_t*>(gTable.data));
-    bmp.setImmutable();
-    return bmp;
-}
-
 static std::unique_ptr<GrFragmentProcessor> make_dither_effect(
         GrRecordingContext* rContext,
         std::unique_ptr<GrFragmentProcessor> inputFP,
@@ -394,7 +320,7 @@ static std::unique_ptr<GrFragmentProcessor> make_dither_effect(
     // TecnoSpark3Pro    PowerVRGE8320   200    299ms        820ms (2.74x)     592ms (1.98x)
     // Pixel 4           Adreno640       500    110ms        221ms (2.01x)     214ms (1.95x)
     // Galaxy S20 FE     Mali-G77 MP11   600    165ms        360ms (2.18x)     260ms (1.58x)
-    static const SkBitmap gLUT = make_dither_lut();
+    static const SkBitmap gLUT = skgpu::MakeDitherLUT();
     auto [tex, ct] = GrMakeCachedBitmapProxyView(
             rContext, gLUT, /*label=*/"MakeDitherEffect", GrMipmapped::kNo);
     if (!tex) {
@@ -428,7 +354,7 @@ static inline bool skpaint_to_grpaint_impl(
         GrRecordingContext* context,
         const GrColorInfo& dstColorInfo,
         const SkPaint& skPaint,
-        const SkMatrixProvider& matrixProvider,
+        const SkMatrix& ctm,
         std::optional<std::unique_ptr<GrFragmentProcessor>> shaderFP,
         SkBlender* primColorBlender,
         const SkSurfaceProps& surfaceProps,
@@ -436,7 +362,7 @@ static inline bool skpaint_to_grpaint_impl(
     // Convert SkPaint color to 4f format in the destination color space
     SkColor4f origColor = SkColor4fPrepForDst(skPaint.getColor4f(), dstColorInfo);
 
-    GrFPArgs fpArgs(context, matrixProvider, &dstColorInfo, surfaceProps);
+    GrFPArgs fpArgs(context, &dstColorInfo, surfaceProps);
 
     // Setup the initial color considering the shader, the SkPaint color, and the presence or not
     // of per-vertex colors.
@@ -447,7 +373,7 @@ static inline bool skpaint_to_grpaint_impl(
             paintFP = std::move(*shaderFP);
         } else {
             if (const SkShaderBase* shader = as_SB(skPaint.getShader())) {
-                paintFP = shader->asFragmentProcessor(fpArgs);
+                paintFP = GrFragmentProcessors::Make(shader, fpArgs, ctm);
                 if (paintFP == nullptr) {
                     return false;
                 }
@@ -470,9 +396,10 @@ static inline bool skpaint_to_grpaint_impl(
 
             SkPMColor4f shaderInput = origColor.makeOpaque().premul();
             paintFP = GrFragmentProcessor::OverrideInput(std::move(paintFP), shaderInput);
-            paintFP = as_BB(primColorBlender)->asFragmentProcessor(std::move(paintFP),
-                                                                   /*dstFP=*/nullptr,
-                                                                   fpArgs);
+            paintFP = GrFragmentProcessors::Make(as_BB(primColorBlender),
+                                                 /*srcFP=*/std::move(paintFP),
+                                                 /*dstFP=*/nullptr,
+                                                 fpArgs);
             if (!paintFP) {
                 return false;
             }
@@ -515,9 +442,10 @@ static inline bool skpaint_to_grpaint_impl(
             grPaint->setColor4f(SK_PMColor4fWHITE);  // won't be used.
             if (blender_requires_shader(primColorBlender)) {
                 paintFP = GrFragmentProcessor::MakeColor(origColor.makeOpaque().premul());
-                paintFP = as_BB(primColorBlender)->asFragmentProcessor(std::move(paintFP),
-                                                                       /*dstFP=*/nullptr,
-                                                                       fpArgs);
+                paintFP = GrFragmentProcessors::Make(as_BB(primColorBlender),
+                                                     /*srcFP=*/std::move(paintFP),
+                                                     /*dstFP=*/nullptr,
+                                                     fpArgs);
                 if (!paintFP) {
                     return false;
                 }
@@ -546,9 +474,8 @@ static inline bool skpaint_to_grpaint_impl(
             SkColorSpace* dstCS = dstColorInfo.colorSpace();
             grPaint->setColor4f(colorFilter->filterColor4f(origColor, dstCS, dstCS).premul());
         } else {
-            auto [success, fp] = as_CFB(colorFilter)->asFragmentProcessor(std::move(paintFP),
-                                                                          context, dstColorInfo,
-                                                                          surfaceProps);
+            auto [success, fp] = GrFragmentProcessors::Make(
+                    context, colorFilter, std::move(paintFP), dstColorInfo, surfaceProps);
             if (!success) {
                 return false;
             }
@@ -556,17 +483,17 @@ static inline bool skpaint_to_grpaint_impl(
         }
     }
 
-    SkMaskFilterBase* maskFilter = as_MFB(skPaint.getMaskFilter());
-    if (maskFilter) {
-        if (auto mfFP = maskFilter->asFragmentProcessor(fpArgs)) {
+    if (auto maskFilter = skPaint.getMaskFilter()) {
+        if (auto mfFP = GrFragmentProcessors::Make(maskFilter, fpArgs, ctm)) {
             grPaint->setCoverageFragmentProcessor(std::move(mfFP));
         }
     }
 
 #ifndef SK_IGNORE_GPU_DITHER
-    GrColorType ct = dstColorInfo.colorType();
-    if (SkPaintPriv::ShouldDither(skPaint, GrColorTypeToSkColorType(ct)) && paintFP != nullptr) {
-        float ditherRange = dither_range_for_config(ct);
+    SkColorType ct = GrColorTypeToSkColorType(dstColorInfo.colorType());
+    if (paintFP != nullptr && (
+            surfaceProps.isAlwaysDither() || SkPaintPriv::ShouldDither(skPaint, ct))) {
+        float ditherRange = skgpu::DitherRangeForConfig(ct);
         paintFP = make_dither_effect(
                 context, std::move(paintFP), ditherRange, context->priv().caps());
     }
@@ -580,19 +507,19 @@ static inline bool skpaint_to_grpaint_impl(
         // on the GrPaint to also be null (also kSrcOver).
         SkASSERT(!grPaint->getXPFactory());
         if (bm.value() != SkBlendMode::kSrcOver) {
-            grPaint->setXPFactory(SkBlendMode_AsXPFactory(bm.value()));
+            grPaint->setXPFactory(GrXPFactory::FromBlendMode(bm.value()));
         }
     } else {
         // Apply a custom blend against the surface color, and force the XP to kSrc so that the
         // computed result is applied directly to the canvas while still honoring the alpha.
-        paintFP = as_BB(skPaint.getBlender())->asFragmentProcessor(
-                std::move(paintFP),
-                GrFragmentProcessor::SurfaceColor(),
-                fpArgs);
+        paintFP = GrFragmentProcessors::Make(as_BB(skPaint.getBlender()),
+                                             std::move(paintFP),
+                                             GrFragmentProcessor::SurfaceColor(),
+                                             fpArgs);
         if (!paintFP) {
             return false;
         }
-        grPaint->setXPFactory(SkBlendMode_AsXPFactory(SkBlendMode::kSrc));
+        grPaint->setXPFactory(GrXPFactory::FromBlendMode(SkBlendMode::kSrc));
     }
 
     if (GrColorTypeClampType(dstColorInfo.colorType()) == GrClampType::kManual) {
@@ -617,13 +544,13 @@ static inline bool skpaint_to_grpaint_impl(
 bool SkPaintToGrPaint(GrRecordingContext* context,
                       const GrColorInfo& dstColorInfo,
                       const SkPaint& skPaint,
-                      const SkMatrixProvider& matrixProvider,
+                      const SkMatrix& ctm,
                       const SkSurfaceProps& surfaceProps,
                       GrPaint* grPaint) {
     return skpaint_to_grpaint_impl(context,
                                    dstColorInfo,
                                    skPaint,
-                                   matrixProvider,
+                                   ctm,
                                    /*shaderFP=*/std::nullopt,
                                    /*primColorBlender=*/nullptr,
                                    surfaceProps,
@@ -634,14 +561,14 @@ bool SkPaintToGrPaint(GrRecordingContext* context,
 bool SkPaintToGrPaintReplaceShader(GrRecordingContext* context,
                                    const GrColorInfo& dstColorInfo,
                                    const SkPaint& skPaint,
-                                   const SkMatrixProvider& matrixProvider,
+                                   const SkMatrix& ctm,
                                    std::unique_ptr<GrFragmentProcessor> shaderFP,
                                    const SkSurfaceProps& surfaceProps,
                                    GrPaint* grPaint) {
     return skpaint_to_grpaint_impl(context,
                                    dstColorInfo,
                                    skPaint,
-                                   matrixProvider,
+                                   ctm,
                                    std::move(shaderFP),
                                    /*primColorBlender=*/nullptr,
                                    surfaceProps,
@@ -653,14 +580,14 @@ bool SkPaintToGrPaintReplaceShader(GrRecordingContext* context,
 bool SkPaintToGrPaintWithBlend(GrRecordingContext* context,
                                const GrColorInfo& dstColorInfo,
                                const SkPaint& skPaint,
-                               const SkMatrixProvider& matrixProvider,
+                               const SkMatrix& ctm,
                                SkBlender* primColorBlender,
                                const SkSurfaceProps& surfaceProps,
                                GrPaint* grPaint) {
     return skpaint_to_grpaint_impl(context,
                                    dstColorInfo,
                                    skPaint,
-                                   matrixProvider,
+                                   ctm,
                                    /*shaderFP=*/std::nullopt,
                                    primColorBlender,
                                    surfaceProps,
