@@ -7,23 +7,50 @@
 
 #include "src/gpu/ganesh/effects/GrSkSLFP.h"
 
+#include "include/core/SkAlphaType.h"
+#include "include/core/SkColor.h"
+#include "include/core/SkColorSpace.h"
+#include "include/core/SkData.h"
+#include "include/core/SkString.h"
+#include "include/core/SkSurfaceProps.h"
+#include "include/effects/SkOverdrawColorFilter.h"
 #include "include/effects/SkRuntimeEffect.h"
-#include "include/private/SkSLString.h"
-#include "include/private/gpu/ganesh/GrContext_Base.h"
+#include "include/private/SkSLSampleUsage.h"
+#include "include/private/base/SkMalloc.h"
+#include "include/private/base/SkTo.h"
+#include "include/private/gpu/ganesh/GrTypesPriv.h"
+#include "src/base/SkArenaAlloc.h"
+#include "src/base/SkRandom.h"
 #include "src/core/SkColorSpacePriv.h"
+#include "src/core/SkRasterPipeline.h"
+#include "src/core/SkRasterPipelineOpContexts.h"
+#include "src/core/SkRasterPipelineOpList.h"
 #include "src/core/SkRuntimeEffectPriv.h"
 #include "src/core/SkSLTypeShared.h"
-#include "src/core/SkVM.h"
 #include "src/gpu/KeyBuilder.h"
-#include "src/gpu/ganesh/GrBaseContextPriv.h"
 #include "src/gpu/ganesh/GrColorInfo.h"
-#include "src/gpu/ganesh/GrTexture.h"
+#include "src/gpu/ganesh/GrColorSpaceXform.h"
+#include "src/gpu/ganesh/GrFragmentProcessors.h"
+#include "src/gpu/ganesh/GrShaderVar.h"
 #include "src/gpu/ganesh/glsl/GrGLSLFragmentShaderBuilder.h"
-#include "src/gpu/ganesh/glsl/GrGLSLProgramBuilder.h"
+#include "src/gpu/ganesh/glsl/GrGLSLUniformHandler.h"
+#include "src/sksl/SkSLString.h"
 #include "src/sksl/SkSLUtil.h"
 #include "src/sksl/codegen/SkSLPipelineStageCodeGenerator.h"
+#include "src/sksl/codegen/SkSLRasterPipelineBuilder.h"
 #include "src/sksl/ir/SkSLProgram.h"
+#include "src/sksl/ir/SkSLType.h"
 #include "src/sksl/ir/SkSLVarDeclarations.h"
+#include "src/sksl/ir/SkSLVariable.h"
+
+#if defined(SK_ENABLE_SKVM)
+#include "src/core/SkFilterColorProgram.h"
+#endif
+
+#include <algorithm>
+
+namespace SkSL { class Context; }
+struct GrShaderCaps;
 
 class GrSkSLFP::Impl : public ProgramImpl {
 public:
@@ -293,12 +320,17 @@ std::unique_ptr<GrSkSLFP> GrSkSLFP::MakeWithData(
     return fp;
 }
 
+GrFragmentProcessor::OptimizationFlags GrSkSLFP::DetermineOptimizationFlags(
+        OptFlags of, SkRuntimeEffect* effect) {
+    OptimizationFlags optFlags = static_cast<OptimizationFlags>(of);
+    if (SkRuntimeEffectPriv::SupportsConstantOutputForConstantInput(effect)) {
+        optFlags |= kConstantOutputForConstantInput_OptimizationFlag;
+    }
+    return optFlags;
+}
+
 GrSkSLFP::GrSkSLFP(sk_sp<SkRuntimeEffect> effect, const char* name, OptFlags optFlags)
-        : INHERITED(kGrSkSLFP_ClassID,
-                    static_cast<OptimizationFlags>(optFlags) |
-                            (effect->getFilterColorProgram()
-                                     ? kConstantOutputForConstantInput_OptimizationFlag
-                                     : kNone_OptimizationFlags))
+        : INHERITED(kGrSkSLFP_ClassID, DetermineOptimizationFlags(optFlags, effect.get()))
         , fEffect(std::move(effect))
         , fName(name)
         , fUniformSize(SkToU32(fEffect->uniformSize())) {
@@ -332,6 +364,7 @@ void GrSkSLFP::addChild(std::unique_ptr<GrFragmentProcessor> child, bool mergeOp
     if (mergeOptFlags) {
         this->mergeOptimizationFlags(ProcessorOptimizationFlags(child.get()));
     }
+    this->clearConstantOutputForConstantInputFlag();
     this->registerChild(std::move(child), fEffect->fSampleUsages[childIndex]);
 }
 
@@ -421,18 +454,61 @@ std::unique_ptr<GrFragmentProcessor> GrSkSLFP::clone() const {
 }
 
 SkPMColor4f GrSkSLFP::constantOutputForConstantInput(const SkPMColor4f& inputColor) const {
+    SkPMColor4f color = (fInputChildIndex >= 0)
+            ? ConstantOutputForConstantInput(this->childProcessor(fInputChildIndex), inputColor)
+            : inputColor;
+
+#if defined(SK_ENABLE_SKSL_IN_RASTER_PIPELINE)
+    class ConstantOutputForConstantInput_SkRPCallbacks : public SkSL::RP::Callbacks {
+    public:
+        bool appendShader(int index) override {
+           SkDEBUGFAIL("constant-output-for-constant-input unsupported when child shaders present");
+           return false;
+        }
+        bool appendColorFilter(int index) override {
+           SkDEBUGFAIL("constant-output-for-constant-input unsupported when child shaders present");
+           return false;
+        }
+        bool appendBlender(int index) override {
+           SkDEBUGFAIL("constant-output-for-constant-input unsupported when child shaders present");
+           return false;
+        }
+        void toLinearSrgb(const void* color) override { /* identity color conversion */ }
+        void fromLinearSrgb(const void* color) override { /* identity color conversion */ }
+    };
+
+    if (const SkSL::RP::Program* program = fEffect->getRPProgram(/*debugTrace=*/nullptr)) {
+        // No color conversion is happening here, so we can use untransformed uniforms.
+        SkSpan<const float> uniforms{reinterpret_cast<const float*>(this->uniformData()),
+                                     fUniformSize / sizeof(float)};
+        SkSTArenaAlloc<2048> alloc;  // sufficient for a tiny SkSL program
+        SkRasterPipeline pipeline(&alloc);
+        pipeline.append_constant_color(&alloc, color.vec());
+        ConstantOutputForConstantInput_SkRPCallbacks callbacks;
+        if (program->appendStages(&pipeline, &alloc, &callbacks, uniforms)) {
+            SkPMColor4f outputColor;
+            SkRasterPipeline_MemoryCtx outputCtx = {&outputColor, 0};
+            pipeline.append(SkRasterPipelineOp::store_f32, &outputCtx);
+            pipeline.run(0, 0, 1, 1);
+            return outputColor;
+        }
+    }
+
+    // We weren't able to run the Raster Pipeline program.
+    return color;
+#elif defined(SK_ENABLE_SKVM)
     const SkFilterColorProgram* program = fEffect->getFilterColorProgram();
     SkASSERT(program);
 
     auto evalChild = [&](int index, SkPMColor4f color) {
-        return ConstantOutputForConstantInput(this->childProcessor(index), color);
+        SkDEBUGFAIL("constant-output-for-constant-input unsupported when child shaders present");
+        return inputColor;
     };
 
-    SkPMColor4f color = (fInputChildIndex >= 0)
-                                ? ConstantOutputForConstantInput(
-                                          this->childProcessor(fInputChildIndex), inputColor)
-                                : inputColor;
     return program->eval(color, this->uniformData(), evalChild);
+#else
+    return color;
+#endif
 }
 
 /**************************************************************************************************/
@@ -441,11 +517,6 @@ GR_DEFINE_FRAGMENT_PROCESSOR_TEST(GrSkSLFP)
 
 #if GR_TEST_UTILS
 
-#include "include/effects/SkOverdrawColorFilter.h"
-#include "src/core/SkColorFilterBase.h"
-
-extern const char* SKSL_OVERDRAW_SRC;
-
 std::unique_ptr<GrFragmentProcessor> GrSkSLFP::TestCreate(GrProcessorTestData* d) {
     SkColor colors[SkOverdrawColorFilter::kNumColors];
     for (SkColor& c : colors) {
@@ -453,8 +524,8 @@ std::unique_ptr<GrFragmentProcessor> GrSkSLFP::TestCreate(GrProcessorTestData* d
     }
     auto filter = SkOverdrawColorFilter::MakeWithSkColors(colors);
     SkSurfaceProps props; // default props for testing
-    auto [success, fp] = as_CFB(filter)->asFragmentProcessor(/*inputFP=*/nullptr, d->context(),
-                                                             GrColorInfo{}, props);
+    auto [success, fp] = GrFragmentProcessors::Make(
+            d->context(), filter.get(), /*inputFP=*/nullptr, GrColorInfo{}, props);
     SkASSERT(success);
     return std::move(fp);
 }
