@@ -7,11 +7,15 @@
 
 
 #include "include/core/SkMatrix.h"
+#include "include/core/SkString.h"
 #include "include/private/gpu/ganesh/GrTypesPriv.h"
+#include "src/core/SkStringUtils.h"
 #include "src/gpu/ganesh/GrDataUtils.h"
 #include "src/gpu/ganesh/gl/GrGLUtil.h"
 #include <stdio.h>
 
+using namespace skia_private;
+
 ///////////////////////////////////////////////////////////////////////////////
 
 #if GR_GL_LOG_CALLS
@@ -174,6 +178,9 @@ static GrGLRenderer get_renderer(const char* rendererString, const GrGLExtension
     if (1 == n && lastDigit >= 0 && lastDigit <= 9) {
         return GrGLRenderer::kPowerVR54x;
     }
+    if (strstr(rendererString, "PowerVR B-Series")) {
+        return GrGLRenderer::kPowerVRBSeries;
+    }
     // certain iOS devices also use PowerVR54x GPUs
     static const char kAppleA4Str[] = "Apple A4";
     static const char kAppleA5Str[] = "Apple A5";
@@ -227,9 +234,6 @@ static GrGLRenderer get_renderer(const char* rendererString, const GrGLExtension
             }
         }
     }
-    if (0 == strcmp("Google SwiftShader", rendererString)) {
-        return GrGLRenderer::kGoogleSwiftShader;
-    }
 
     if (const char* intelString = strstr(rendererString, "Intel")) {
         // These generic strings seem to always come from Haswell: Iris 5100 or Iris Pro 5200
@@ -490,24 +494,7 @@ static std::tuple<GrGLDriver, GrGLDriverVersion> get_driver_and_version(GrGLStan
     }
 
     if (driver == GrGLDriver::kUnknown) {
-        if (vendor == GrGLVendor::kGoogle) {
-            // Swiftshader is the only Google vendor at the moment
-            driver = GrGLDriver::kSwiftShader;
-
-            // Swiftshader has a strange version string: w.x.y.z  Going to arbitrarily ignore
-            // y and assume w,x and z are major, minor, point.
-            // As of writing, version is 4.0.0.6
-            int n = sscanf(versionString,
-                           "OpenGL ES %d.%d SwiftShader %d.%d.0.%d",
-                           &major,
-                           &minor,
-                           &driverMajor,
-                           &driverMinor,
-                           &driverPoint);
-            if (n == 5) {
-                driverVersion = GR_GL_DRIVER_VER(driverMajor, driverMinor, driverPoint);
-            }
-        } else if (vendor == GrGLVendor::kIntel) {
+        if (vendor == GrGLVendor::kIntel) {
             // We presume we're on the Intel driver since it hasn't identified itself as Mesa.
             driver = GrGLDriver::kIntel;
 
@@ -607,7 +594,7 @@ static std::tuple<GrGLVendor, GrGLRenderer, GrGLDriver, GrGLDriverVersion>
 get_angle_gl_vendor_and_renderer(
         const char* innerString,
         const GrGLExtensions& extensions) {
-    SkTArray<SkString> parts;
+    TArray<SkString> parts;
     SkStrSplit(innerString, ",", &parts);
     // This would need some fixing if we have substrings that contain commas.
     if (parts.size() != 3) {
