@@ -8,6 +8,7 @@
 #include "src/gpu/ganesh/mtl/GrMtlCaps.h"
 
 #include "include/core/SkRect.h"
+#include "include/core/SkTextureCompressionType.h"
 #include "include/gpu/GrBackendSurface.h"
 #include "src/core/SkCompressedDataUtils.h"
 #include "src/core/SkReadBuffer.h"
@@ -23,6 +24,7 @@
 #include "src/gpu/ganesh/mtl/GrMtlRenderTarget.h"
 #include "src/gpu/ganesh/mtl/GrMtlTexture.h"
 #include "src/gpu/ganesh/mtl/GrMtlUtil.h"
+#include "src/gpu/mtl/MtlUtilsPriv.h"
 
 #if GR_TEST_UTILS
     #include "src/gpu/ganesh/TestFormatColorTypeCombination.h"
@@ -61,6 +63,9 @@ GrMtlCaps::GrMtlCaps(const GrContextOptions& contextOptions, const id<MTLDevice>
 bool GrMtlCaps::getGPUFamilyFromFeatureSet(id<MTLDevice> device,
                                            GPUFamily* gpuFamily,
                                            int* group) {
+// MTLFeatureSet is deprecated for newer versions of the SDK
+#if GR_METAL_SDK_VERSION < 300
+
 #if defined(SK_BUILD_FOR_MAC)
     // Apple Silicon is only available in later OSes
     *gpuFamily = GPUFamily::kMac;
@@ -159,6 +164,8 @@ bool GrMtlCaps::getGPUFamilyFromFeatureSet(id<MTLDevice> device,
     // We don't support earlier OSes
 #endif
 
+#endif // GR_METAL_SDK_VERSION < 300
+
     // No supported GPU families were found
     return false;
 }
@@ -625,7 +632,7 @@ size_t GrMtlCaps::GetFormatIndex(MTLPixelFormat pixelFormat) {
             return i;
         }
     }
-    SK_ABORT("Invalid MTLPixelFormat");
+    SK_ABORT("Invalid MTLPixelFormat: %d", static_cast<int>(pixelFormat));
 }
 
 void GrMtlCaps::initFormatTable() {
@@ -743,7 +750,7 @@ void GrMtlCaps::initFormatTable() {
     // Format: RG8Unorm
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatRG8Unorm)];
-        info->fFlags = FormatInfo::kTexturable_Flag;
+        info->fFlags = FormatInfo::kAllFlags;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -1001,7 +1008,7 @@ GrCaps::SurfaceReadPixelsSupport GrMtlCaps::surfaceSupportsReadPixels(
         const GrSurface* surface) const {
     if (auto tex = static_cast<const GrMtlTexture*>(surface->asTexture())) {
         // We disallow reading back directly from compressed textures.
-        if (GrMtlFormatIsCompressed(tex->attachment()->mtlFormat())) {
+        if (skgpu::MtlFormatIsCompressed(tex->attachment()->mtlFormat())) {
             return SurfaceReadPixelsSupport::kCopyToTexture2D;
         }
     }
@@ -1051,11 +1058,11 @@ GrBackendFormat GrMtlCaps::onGetDefaultBackendFormat(GrColorType ct) const {
 }
 
 GrBackendFormat GrMtlCaps::getBackendFormatFromCompressionType(
-        SkImage::CompressionType compressionType) const {
+        SkTextureCompressionType compressionType) const {
     switch (compressionType) {
-        case SkImage::CompressionType::kNone:
+        case SkTextureCompressionType::kNone:
             return {};
-        case SkImage::CompressionType::kETC2_RGB8_UNORM:
+        case SkTextureCompressionType::kETC2_RGB8_UNORM:
             if (@available(macOS 11.0, *)) {
                 if (this->isApple()) {
                     return GrBackendFormat::MakeMtl(MTLPixelFormatETC2_RGB8);
@@ -1065,10 +1072,10 @@ GrBackendFormat GrMtlCaps::getBackendFormatFromCompressionType(
             } else {
                 return {};
             }
-        case SkImage::CompressionType::kBC1_RGB8_UNORM:
+        case SkTextureCompressionType::kBC1_RGB8_UNORM:
             // Metal only supports the RGBA BC1 variant (see following)
             return {};
-        case SkImage::CompressionType::kBC1_RGBA8_UNORM:
+        case SkTextureCompressionType::kBC1_RGBA8_UNORM:
 #ifdef SK_BUILD_FOR_MAC
             if (this->isMac()) {
                 return GrBackendFormat::MakeMtl(MTLPixelFormatBC1_RGBA);
@@ -1145,13 +1152,13 @@ GrCaps::SupportedWrite GrMtlCaps::supportedWritePixelsColorType(
 GrCaps::SupportedRead GrMtlCaps::onSupportedReadPixelsColorType(
         GrColorType srcColorType, const GrBackendFormat& srcBackendFormat,
         GrColorType dstColorType) const {
-    SkImage::CompressionType compression = GrBackendFormatToCompressionType(srcBackendFormat);
-    if (compression != SkImage::CompressionType::kNone) {
+    SkTextureCompressionType compression = GrBackendFormatToCompressionType(srcBackendFormat);
+    if (compression != SkTextureCompressionType::kNone) {
 #ifdef SK_BUILD_FOR_IOS
         // Reading back to kRGB_888x doesn't work on Metal/iOS (skbug.com/9839)
         return { GrColorType::kUnknown, 0 };
 #else
-        return { SkCompressionTypeIsOpaque(compression) ? GrColorType::kRGB_888x
+        return { SkTextureCompressionTypeIsOpaque(compression) ? GrColorType::kRGB_888x
                                                         : GrColorType::kRGBA_8888, 0 };
 #endif
     }
@@ -1267,7 +1274,7 @@ void GrMtlCaps::onDumpJSON(SkJSONWriter* writer) const {
 
     writer->beginObject("Preferred Stencil Format");
     writer->appendS32("stencil bits", GrMtlFormatStencilBits(fPreferredStencilFormat));
-    writer->appendS32("total bytes", GrMtlFormatBytesPerBlock(fPreferredStencilFormat));
+    writer->appendS32("total bytes", skgpu::MtlFormatBytesPerBlock(fPreferredStencilFormat));
     writer->endObject();
 
     switch (fGPUFamily) {
