@@ -17,6 +17,8 @@
 #include "src/gpu/tessellate/Tessellation.h"
 #include "src/gpu/tessellate/WangsFormula.h"
 
+using namespace skia_private;
+
 namespace {
 
 using namespace skgpu::tess;
@@ -121,7 +123,7 @@ private:
     std::unique_ptr<ProgramImpl> makeProgramImpl(const GrShaderCaps&) const final;
 
     constexpr static int kMaxInstanceAttribCount = 5;
-    SkSTArray<kMaxInstanceAttribCount, Attribute> fInstanceAttribs;
+    STArray<kMaxInstanceAttribCount, Attribute> fInstanceAttribs;
 };
 
 std::unique_ptr<GrGeometryProcessor::ProgramImpl> MiddleOutShader::makeProgramImpl(
