@@ -10,7 +10,7 @@
 namespace skgpu::graphite {
 
 Sampler::Sampler(const SharedContext* sharedContext)
-        : Resource(sharedContext, Ownership::kOwned, skgpu::Budgeted::kYes) {}
+        : Resource(sharedContext, Ownership::kOwned, skgpu::Budgeted::kYes, /*gpuMemorySize=*/0) {}
 
 Sampler::~Sampler() {}
 
