@@ -9,32 +9,50 @@
 
 #include "src/gpu/graphite/Log.h"
 #include "src/gpu/graphite/TextureProxy.h"
+#include "src/gpu/graphite/compute/DispatchGroup.h"
 #include "src/gpu/graphite/dawn/DawnBuffer.h"
 #include "src/gpu/graphite/dawn/DawnCaps.h"
 #include "src/gpu/graphite/dawn/DawnGraphicsPipeline.h"
+#include "src/gpu/graphite/dawn/DawnGraphiteUtilsPriv.h"
 #include "src/gpu/graphite/dawn/DawnQueueManager.h"
 #include "src/gpu/graphite/dawn/DawnResourceProvider.h"
 #include "src/gpu/graphite/dawn/DawnSampler.h"
 #include "src/gpu/graphite/dawn/DawnSharedContext.h"
 #include "src/gpu/graphite/dawn/DawnTexture.h"
-#include "src/gpu/graphite/dawn/DawnUtilsPriv.h"
 
 namespace skgpu::graphite {
 
 namespace {
 using IntrinsicConstant = float[4];
-}
 
-std::unique_ptr<DawnCommandBuffer> DawnCommandBuffer::Make(const DawnSharedContext* sharedContext) {
-    std::unique_ptr<DawnCommandBuffer> cmdBuffer(new DawnCommandBuffer(sharedContext));
+uint64_t clamp_ubo_binding_size(uint64_t offset, uint64_t bufferSize) {
+    // Dawn's limit
+    constexpr uint32_t kMaxUniformBufferBindingSize = 64 * 1024;
+
+    SkASSERT(offset <= bufferSize);
+    auto remainSize = bufferSize - offset;
+    if (remainSize > kMaxUniformBufferBindingSize) {
+        return kMaxUniformBufferBindingSize;
+    }
+
+    return wgpu::kWholeSize;
+}
+}  // namespace
+
+std::unique_ptr<DawnCommandBuffer> DawnCommandBuffer::Make(const DawnSharedContext* sharedContext,
+                                                           DawnResourceProvider* resourceProvider) {
+    std::unique_ptr<DawnCommandBuffer> cmdBuffer(
+            new DawnCommandBuffer(sharedContext, resourceProvider));
     if (!cmdBuffer->setNewCommandBufferResources()) {
         return {};
     }
     return cmdBuffer;
 }
 
-DawnCommandBuffer::DawnCommandBuffer(const DawnSharedContext* sharedContext)
-        : fSharedContext(sharedContext) {}
+DawnCommandBuffer::DawnCommandBuffer(const DawnSharedContext* sharedContext,
+                                     DawnResourceProvider* resourceProvider)
+        : fSharedContext(sharedContext)
+        , fResourceProvider(resourceProvider) {}
 
 DawnCommandBuffer::~DawnCommandBuffer() {}
 
@@ -71,7 +89,7 @@ bool DawnCommandBuffer::onAddRenderPass(const RenderPassDesc& renderPassDesc,
                                         const Texture* resolveTexture,
                                         const Texture* depthStencilTexture,
                                         SkRect viewport,
-                                        const std::vector<std::unique_ptr<DrawPass>>& drawPasses) {
+                                        const DrawPassList& drawPasses) {
     // Update viewport's constant buffer before starting a render pass.
     this->preprocessViewport(viewport);
 
@@ -81,24 +99,34 @@ bool DawnCommandBuffer::onAddRenderPass(const RenderPassDesc& renderPassDesc,
 
     this->setViewport(viewport);
 
-    for (size_t i = 0; i < drawPasses.size(); ++i) {
-        this->addDrawPass(drawPasses[i].get());
+    for (const auto& drawPass : drawPasses) {
+        this->addDrawPass(drawPass.get());
     }
 
     this->endRenderPass();
     return true;
 }
 
-bool DawnCommandBuffer::onAddComputePass(const ComputePassDesc& computePassDesc,
-                                         const ComputePipeline* pipeline,
-                                         const std::vector<ResourceBinding>& bindings) {
+bool DawnCommandBuffer::onAddComputePass(const DispatchGroupList& groups) {
     this->beginComputePass();
-    this->bindComputePipeline(pipeline);
-    for (const ResourceBinding& binding : bindings) {
-        this->bindBuffer(binding.fBuffer.fBuffer, binding.fBuffer.fOffset, binding.fIndex);
+    for (const auto& group : groups) {
+        group->addResourceRefs(this);
+        for (const auto& dispatch : group->dispatches()) {
+            this->bindComputePipeline(group->getPipeline(dispatch.fPipelineIndex));
+            for (const ResourceBinding& binding : dispatch.fBindings) {
+                if (const BindBufferInfo* buffer =
+                            std::get_if<BindBufferInfo>(&binding.fResource)) {
+                    this->bindBuffer(buffer->fBuffer, buffer->fOffset, binding.fIndex);
+                } else {
+                    const TextureIndex* texIdx = std::get_if<TextureIndex>(&binding.fResource);
+                    SkASSERT(texIdx);
+                    this->bindTexture(group->getTexture(*texIdx), binding.fIndex);
+                }
+            }
+            this->dispatchThreadgroups(dispatch.fParams.fGlobalDispatchSize,
+                                       dispatch.fParams.fLocalDispatchSize);
+        }
     }
-    this->dispatchThreadgroups(computePassDesc.fGlobalDispatchSize,
-                               computePassDesc.fLocalDispatchSize);
     this->endComputePass();
     return true;
 }
@@ -204,15 +232,118 @@ bool DawnCommandBuffer::beginRenderPass(const RenderPassDesc& renderPassDesc,
         // Manually load the contents of the resolve texture into the MSAA attachment as a draw,
         // so the actual load op for the MSAA attachment had better have been discard.
 
-        // TODO: https://b.corp.google.com/issues/258652999
-        SkASSERT(false);
-    } else {
+        if (!this->loadMSAAFromResolveAndBeginRenderPassEncoder(
+                    renderPassDesc,
+                    wgpuRenderPass,
+                    static_cast<const DawnTexture*>(colorTexture))) {
+            return false;
+        }
+    }
+    else {
         fActiveRenderPassEncoder = fCommandEncoder.BeginRenderPass(&wgpuRenderPass);
     }
 
     return true;
 }
 
+bool DawnCommandBuffer::loadMSAAFromResolveAndBeginRenderPassEncoder(
+        const RenderPassDesc& frontendRenderPassDesc,
+        const wgpu::RenderPassDescriptor& wgpuRenderPassDesc,
+        const DawnTexture* msaaTexture) {
+    SkASSERT(!fActiveRenderPassEncoder);
+
+    // Copy from resolve texture to an intermediate texture. Using blit with draw
+    // pipeline because the resolveTexture might be created from a swapchain, and it
+    // is possible that only its texture view is available. So onCopyTextureToTexture()
+    // which operates on wgpu::Texture instead of wgpu::TextureView cannot be used in that case.
+    auto msaaLoadTexture = fResourceProvider->findOrCreateDiscardableMSAALoadTexture(
+            msaaTexture->dimensions(), msaaTexture->textureInfo());
+    if (!msaaLoadTexture) {
+        SKGPU_LOG_E("DawnCommandBuffer::loadMSAAFromResolveAndBeginRenderPassEncoder: "
+                    "Can't create MSAA Load Texture.");
+        return false;
+    }
+
+    // Creating intermediate render pass (copy from resolve texture -> MSAA load texture)
+    RenderPassDesc intermediateRenderPassDesc = {};
+    intermediateRenderPassDesc.fColorAttachment.fLoadOp = LoadOp::kDiscard;
+    intermediateRenderPassDesc.fColorAttachment.fStoreOp = StoreOp::kStore;
+    intermediateRenderPassDesc.fColorAttachment.fTextureInfo =
+            frontendRenderPassDesc.fColorResolveAttachment.fTextureInfo;
+
+    wgpu::RenderPassColorAttachment wgpuIntermediateColorAttachment;
+     // Dawn doesn't support actual DontCare so use LoadOp::Clear.
+    wgpuIntermediateColorAttachment.loadOp = wgpu::LoadOp::Clear;
+    wgpuIntermediateColorAttachment.clearValue = {1, 1, 1, 1};
+    wgpuIntermediateColorAttachment.storeOp = wgpu::StoreOp::Store;
+    wgpuIntermediateColorAttachment.view = msaaLoadTexture->dawnTextureView();
+
+    wgpu::RenderPassDescriptor wgpuIntermediateRenderPassDesc;
+    wgpuIntermediateRenderPassDesc.colorAttachmentCount = 1;
+    wgpuIntermediateRenderPassDesc.colorAttachments = &wgpuIntermediateColorAttachment;
+
+    fActiveRenderPassEncoder = fCommandEncoder.BeginRenderPass(&wgpuIntermediateRenderPassDesc);
+
+    if (!this->doBlitWithDraw(
+                intermediateRenderPassDesc,
+                /*sourceTextureView=*/wgpuRenderPassDesc.colorAttachments[0].resolveTarget,
+                msaaTexture->dimensions().width(),
+                msaaTexture->dimensions().height())) {
+        return false;
+    }
+
+    fActiveRenderPassEncoder.End();
+
+    // Start actual render pass (blit from MSAA load texture -> MSAA texture)
+    fActiveRenderPassEncoder = fCommandEncoder.BeginRenderPass(&wgpuRenderPassDesc);
+
+    return this->doBlitWithDraw(frontendRenderPassDesc,
+                                /*sourceTextureView=*/msaaLoadTexture->dawnTextureView(),
+                                msaaTexture->dimensions().width(),
+                                msaaTexture->dimensions().height());
+}
+
+bool DawnCommandBuffer::doBlitWithDraw(const RenderPassDesc& frontendRenderPassDesc,
+                                       const wgpu::TextureView& sourceTextureView,
+                                       int width,
+                                       int height) {
+    auto loadPipeline = fResourceProvider->findOrCreateBlitWithDrawPipeline(frontendRenderPassDesc);
+    if (!loadPipeline) {
+        SKGPU_LOG_E("Unable to create pipeline to blit with draw");
+        return false;
+    }
+
+    SkASSERT(fActiveRenderPassEncoder);
+
+    fActiveRenderPassEncoder.SetPipeline(loadPipeline);
+
+    // The load msaa pipeline takes no uniforms, no vertex/instance attributes and only uses
+    // one texture that does not require a sampler.
+
+    // TODO: b/260368758
+    // cache single texture's bind group creation.
+    wgpu::BindGroupEntry entry;
+    entry.binding = 0;
+    entry.textureView = sourceTextureView;
+
+    wgpu::BindGroupDescriptor desc;
+    desc.layout = loadPipeline.GetBindGroupLayout(0);
+    desc.entryCount = 1;
+    desc.entries = &entry;
+
+    auto bindGroup = fSharedContext->device().CreateBindGroup(&desc);
+
+    fActiveRenderPassEncoder.SetBindGroup(0, bindGroup);
+
+    fActiveRenderPassEncoder.SetScissorRect(0, 0, width, height);
+    fActiveRenderPassEncoder.SetViewport(0, 0, width, height, 0, 1);
+
+    // Fullscreen triangle
+    fActiveRenderPassEncoder.Draw(3);
+
+    return true;
+}
+
 void DawnCommandBuffer::endRenderPass() {
     SkASSERT(fActiveRenderPassEncoder);
     fActiveRenderPassEncoder.End();
@@ -240,7 +371,8 @@ void DawnCommandBuffer::addDrawPass(const DrawPass* drawPass) {
             }
             case DrawPassCommands::Type::kBindDrawBuffers: {
                 auto bdb = static_cast<DrawPassCommands::BindDrawBuffers*>(cmdPtr);
-                this->bindDrawBuffers(bdb->fVertices, bdb->fInstances, bdb->fIndices);
+                this->bindDrawBuffers(
+                        bdb->fVertices, bdb->fInstances, bdb->fIndices, bdb->fIndirect);
                 break;
             }
             case DrawPassCommands::Type::kBindTexturesAndSamplers: {
@@ -284,6 +416,16 @@ void DawnCommandBuffer::addDrawPass(const DrawPass* drawPass) {
                                            draw->fInstanceCount);
                 break;
             }
+            case DrawPassCommands::Type::kDrawIndirect: {
+                auto draw = static_cast<DrawPassCommands::DrawIndirect*>(cmdPtr);
+                this->drawIndirect(draw->fType);
+                break;
+            }
+            case DrawPassCommands::Type::kDrawIndexedIndirect: {
+                auto draw = static_cast<DrawPassCommands::DrawIndexedIndirect*>(cmdPtr);
+                this->drawIndexedIndirect(draw->fType);
+                break;
+            }
         }
     }
 }
@@ -319,7 +461,8 @@ void DawnCommandBuffer::bindUniformBuffer(const BindBufferInfo& info, UniformSlo
 
 void DawnCommandBuffer::bindDrawBuffers(const BindBufferInfo& vertices,
                                         const BindBufferInfo& instances,
-                                        const BindBufferInfo& indices) {
+                                        const BindBufferInfo& indices,
+                                        const BindBufferInfo& indirect) {
     SkASSERT(fActiveRenderPassEncoder);
 
     if (vertices.fBuffer) {
@@ -337,6 +480,13 @@ void DawnCommandBuffer::bindDrawBuffers(const BindBufferInfo& vertices,
         fActiveRenderPassEncoder.SetIndexBuffer(
                 dawnBuffer, wgpu::IndexFormat::Uint16, indices.fOffset);
     }
+    if (indirect.fBuffer) {
+        fCurrentIndirectBuffer = static_cast<const DawnBuffer*>(indirect.fBuffer)->dawnBuffer();
+        fCurrentIndirectBufferOffset = indirect.fOffset;
+    } else {
+        fCurrentIndirectBuffer = nullptr;
+        fCurrentIndirectBufferOffset = 0;
+    }
 }
 
 void DawnCommandBuffer::bindTextureAndSamplers(
@@ -385,32 +535,41 @@ void DawnCommandBuffer::syncUniformBuffers() {
         uint32_t numBuffers = 0;
 
         entries[numBuffers].binding = DawnGraphicsPipeline::kIntrinsicUniformBufferIndex;
-        entries[numBuffers].buffer = fInstrinsicConstantBuffer;
+        entries[numBuffers].buffer = fIntrinsicConstantBuffer;
         entries[numBuffers].offset = 0;
         entries[numBuffers].size = sizeof(IntrinsicConstant);
         ++numBuffers;
 
         if (fActiveGraphicsPipeline->hasStepUniforms() &&
             fBoundUniformBuffers[DawnGraphicsPipeline::kRenderStepUniformBufferIndex]) {
+            auto boundBuffer =
+                    fBoundUniformBuffers[DawnGraphicsPipeline::kRenderStepUniformBufferIndex];
+
             entries[numBuffers].binding = DawnGraphicsPipeline::kRenderStepUniformBufferIndex;
-            entries[numBuffers].buffer =
-                    fBoundUniformBuffers[DawnGraphicsPipeline::kRenderStepUniformBufferIndex]
-                            ->dawnBuffer();
+            entries[numBuffers].buffer = boundBuffer->dawnBuffer();
+
             entries[numBuffers].offset =
                     fBoundUniformBufferOffsets[DawnGraphicsPipeline::kRenderStepUniformBufferIndex];
-            entries[numBuffers].size = wgpu::kWholeSize;
+
+            entries[numBuffers].size =
+                    clamp_ubo_binding_size(entries[numBuffers].offset, boundBuffer->size());
+
             ++numBuffers;
         }
 
         if (fActiveGraphicsPipeline->hasFragment() &&
             fBoundUniformBuffers[DawnGraphicsPipeline::kPaintUniformBufferIndex]) {
+            auto boundBuffer = fBoundUniformBuffers[DawnGraphicsPipeline::kPaintUniformBufferIndex];
+
             entries[numBuffers].binding = DawnGraphicsPipeline::kPaintUniformBufferIndex;
-            entries[numBuffers].buffer =
-                    fBoundUniformBuffers[DawnGraphicsPipeline::kPaintUniformBufferIndex]
-                            ->dawnBuffer();
+            entries[numBuffers].buffer = boundBuffer->dawnBuffer();
+
             entries[numBuffers].offset =
                     fBoundUniformBufferOffsets[DawnGraphicsPipeline::kPaintUniformBufferIndex];
-            entries[numBuffers].size = wgpu::kWholeSize;
+
+            entries[numBuffers].size =
+                    clamp_ubo_binding_size(entries[numBuffers].offset, boundBuffer->size());
+
             ++numBuffers;
         }
 
@@ -432,40 +591,46 @@ void DawnCommandBuffer::setScissor(unsigned int left,
                                    unsigned int width,
                                    unsigned int height) {
     SkASSERT(fActiveRenderPassEncoder);
-    fActiveRenderPassEncoder.SetScissorRect(left, top, width, height);
+    SkIRect scissor = SkIRect::MakeXYWH(
+            left + fReplayTranslation.x(), top + fReplayTranslation.y(), width, height);
+    if (!scissor.intersect(SkIRect::MakeSize(fRenderPassSize))) {
+        scissor.setEmpty();
+    }
+    fActiveRenderPassEncoder.SetScissorRect(
+            scissor.x(), scissor.y(), scissor.width(), scissor.height());
 }
 
 void DawnCommandBuffer::preprocessViewport(const SkRect& viewport) {
     // Dawn's framebuffer space has (0, 0) at the top left. This agrees with Skia's device coords.
     // However, in NDC (-1, -1) is the bottom left. So we flip the origin here (assuming all
     // surfaces we have are TopLeft origin).
-    const float x = viewport.x();
-    const float y = viewport.y();
+    const float x = viewport.x() - fReplayTranslation.x();
+    const float y = viewport.y() - fReplayTranslation.y();
     const float invTwoW = 2.f / viewport.width();
     const float invTwoH = 2.f / viewport.height();
     const IntrinsicConstant rtAdjust = {invTwoW, -invTwoH, -1.f - x * invTwoW, 1.f + y * invTwoH};
 
-    if (!fInstrinsicConstantBuffer) {
+    if (!fIntrinsicConstantBuffer) {
         wgpu::BufferDescriptor desc;
 #if defined(SK_DEBUG)
-        desc.label = "CommandBufferInstrinsicConstant";
+        desc.label = "CommandBufferIntrinsicConstant";
 #endif
         desc.usage = wgpu::BufferUsage::CopyDst | wgpu::BufferUsage::Uniform;
         desc.size = sizeof(IntrinsicConstant);
         desc.mappedAtCreation = false;
-        fInstrinsicConstantBuffer = fSharedContext->device().CreateBuffer(&desc);
-        SkASSERT(fInstrinsicConstantBuffer);
+        fIntrinsicConstantBuffer = fSharedContext->device().CreateBuffer(&desc);
+        SkASSERT(fIntrinsicConstantBuffer);
     }
 
     // TODO: https://b.corp.google.com/issues/259267703
-    // Make updating instrinsic constants faster. Metal has setVertexBytes method
-    // to quickly sending instrinsic constants to vertex shader without any buffer. But Dawn doesn't
+    // Make updating intrinsic constants faster. Metal has setVertexBytes method
+    // to quickly sending intrinsic constants to vertex shader without any buffer. But Dawn doesn't
     // have similar capability. So we have to use WriteBuffer(), and this method is not allowed to
     // be called when there is an active render pass.
     SkASSERT(!fActiveRenderPassEncoder);
     SkASSERT(!fActiveComputePassEncoder);
 
-    fCommandEncoder.WriteBuffer(fInstrinsicConstantBuffer,
+    fCommandEncoder.WriteBuffer(fIntrinsicConstantBuffer,
                                 0,
                                 reinterpret_cast<const uint8_t*>(rtAdjust),
                                 sizeof(rtAdjust));
@@ -535,6 +700,27 @@ void DawnCommandBuffer::drawIndexedInstanced(PrimitiveType type,
             indexCount, instanceCount, baseIndex, baseVertex, baseInstance);
 }
 
+void DawnCommandBuffer::drawIndirect(PrimitiveType type) {
+    SkASSERT(fActiveRenderPassEncoder);
+    SkASSERT(fActiveGraphicsPipeline->primitiveType() == type);
+    SkASSERT(fCurrentIndirectBuffer);
+
+    this->syncUniformBuffers();
+
+    fActiveRenderPassEncoder.DrawIndirect(fCurrentIndirectBuffer, fCurrentIndirectBufferOffset);
+}
+
+void DawnCommandBuffer::drawIndexedIndirect(PrimitiveType type) {
+    SkASSERT(fActiveRenderPassEncoder);
+    SkASSERT(fActiveGraphicsPipeline->primitiveType() == type);
+    SkASSERT(fCurrentIndirectBuffer);
+
+    this->syncUniformBuffers();
+
+    fActiveRenderPassEncoder.DrawIndexedIndirect(fCurrentIndirectBuffer,
+                                                 fCurrentIndirectBufferOffset);
+}
+
 void DawnCommandBuffer::beginComputePass() { SkASSERT(false); }
 
 void DawnCommandBuffer::bindComputePipeline(const ComputePipeline* computePipeline) {
@@ -547,6 +733,11 @@ void DawnCommandBuffer::bindBuffer(const Buffer* buffer, unsigned int offset, un
     SkASSERT(false);
 }
 
+void DawnCommandBuffer::bindTexture(const Texture* texture, unsigned int index) {
+    // TODO: https://b.corp.google.com/issues/260341543
+    SkASSERT(false);
+}
+
 void DawnCommandBuffer::dispatchThreadgroups(const WorkgroupSize& globalSize,
                                              const WorkgroupSize& localSize) {
     // TODO: https://b.corp.google.com/issues/260341543
@@ -629,6 +820,7 @@ bool DawnCommandBuffer::onCopyBufferToTexture(const Buffer* buffer,
 
         dst.origin.x = copyData[i].fRect.x();
         dst.origin.y = copyData[i].fRect.y();
+        dst.mipLevel = copyData[i].fMipLevel;
 
         wgpu::Extent3D copySize = {static_cast<uint32_t>(copyData[i].fRect.width()),
                                    static_cast<uint32_t>(copyData[i].fRect.height()),
