@@ -22,9 +22,11 @@ class MtlSharedContext;
 
 class MtlComputePipeline final : public ComputePipeline {
 public:
-    static sk_sp<MtlComputePipeline> Make(ResourceProvider*,
-                                          const MtlSharedContext*,
-                                          const ComputePipelineDesc&);
+    using MSLFunction = std::pair<id<MTLLibrary>, std::string>;
+
+    static sk_sp<MtlComputePipeline> Make(const MtlSharedContext*,
+                                          std::string label,
+                                          MSLFunction computeMain);
     ~MtlComputePipeline() override = default;
 
     id<MTLComputePipelineState> mtlPipelineState() const { return fPipelineState.get(); }
