@@ -689,7 +689,7 @@ bool BeagleWksSSParser::readChart()
     long actPos=input->tell();
     auto dSz=static_cast<int>(input->readULong(2));
     auto sSz=static_cast<int>(input->readULong(1));
-    if (sSz+1!=dSz && sSz+2!=dSz) {
+    if (!input->checkPosition(actPos+2+dSz) || (sSz+1!=dSz && sSz+2!=dSz)) {
       input->seek(pos, librevenge::RVNG_SEEK_SET);
       MWAW_DEBUG_MSG(("BeagleWksSSParser::readChart: can not read the chart\n"));
       f << "###";
@@ -740,6 +740,10 @@ bool BeagleWksSSParser::readChart()
   ascii().addNote(f.str().c_str());
 
   pos=input->tell();
+  if (!input->checkPosition(pos+31)) {
+    MWAW_DEBUG_MSG(("BeagleWksSSParser::readChart: the zone B seems too short\n"));
+    return false;
+  }
   f.str("");
   f << "Chart-B:";
   val=static_cast<int>(input->readLong(1));
@@ -770,6 +774,11 @@ bool BeagleWksSSParser::readChart()
   ascii().addNote(f.str().c_str());
 
   pos=input->tell();
+  if (!input->checkPosition(pos+46)) {
+    MWAW_DEBUG_MSG(("BeagleWksSSParser::readChart: the zone C seems too short\n"));
+    return false;
+  }
+
   f.str("");
   f << "Chart-C:";
   for (int i=0; i<3; ++i) {
@@ -878,6 +887,11 @@ bool BeagleWksSSParser::readChart()
 
   pos=input->tell();
   long endPos = pos+67;
+  if (!input->checkPosition(endPos)) {
+    MWAW_DEBUG_MSG(("BeagleWksSSParser::readChart: the zone D seems too short\n"));
+    return false;
+  }
+
   f.str("");
   f << "Chart-D:";
   auto sSz=static_cast<int>(input->readULong(1));
@@ -1085,7 +1099,7 @@ bool BeagleWksSSParser::readCellSheet(BeagleWksSSParserInternal::Cell &cell)
           break;
         case 5:
           format.m_format=MWAWCell::F_DATE;
-          format.m_DTFormat="%m/%d/%y:";
+          format.m_DTFormat="%m/%d/%y";
           break;
         case 6:
           switch (form & 0x7) {
