@@ -333,7 +333,7 @@ std::shared_ptr<ClarisWksStruct::DSET> ClarisWksPresentation::readPresentationZo
 
   long dataEnd = entry.end()-N*data0Length;
   input->seek(dataEnd, librevenge::RVNG_SEEK_SET);
-  for (int i = 0; i < N; i++) {
+  for (long i = 0; i < N; i++) {
     pos = input->tell();
 
     f.str("");
@@ -382,7 +382,7 @@ bool ClarisWksPresentation::readZone1(ClarisWksPresentationInternal::Presentatio
     ascFile.addPos(pos);
     ascFile.addNote(f.str().c_str());
 
-    for (int i = 0; i < N; i++) {
+    for (long i = 0; i < N; i++) {
       f.str("");
       f << "PresentationStr" << st << "-" << i << ":";
       pos = input->tell();
@@ -404,7 +404,7 @@ bool ClarisWksPresentation::readZone1(ClarisWksPresentationInternal::Presentatio
       long endStringPos=pos+12+sSz;
       if (sSz < 0 || !input->checkPosition(endStringPos+4)) {
         input->seek(pos, librevenge::RVNG_SEEK_SET);
-        MWAW_DEBUG_MSG(("ClarisWksPresentation::readZone1: can not read string %d\n", i));
+        MWAW_DEBUG_MSG(("ClarisWksPresentation::readZone1: can not read string %ld\n", i));
         return false;
       }
       librevenge::RVNGString title("");
