@@ -609,7 +609,7 @@ std::shared_ptr<FullWrtStruct::ZoneHeader> FullWrtGraph::readGraphicData(FullWrt
     }
     else if (sz) {   // a serie of doc id ( normally 1e )
       f << "docId[type1e?]=[";
-      for (int i = 0; i < sz/2; i++) {
+      for (long i = 0; i < sz/2; i++) {
         auto id = int(input->readLong(2));
         std::string type=m_mainParser->getDocumentTypeName(id);
         if (type.empty())
@@ -648,8 +648,8 @@ bool FullWrtGraph::readGraphic(FullWrtStruct::EntryPtr zone)
   input->seek(sz, librevenge::RVNG_SEEK_CUR);
   f << "Entries(Graphic)";
   f << "|" << *zone << ":";
-  if (zone->m_type >= 0)
-    f << "type=" << std::hex << zone->m_type << std::dec << ",";
+  if (zone->m_fileType >= 0)
+    f << "type=" << std::hex << zone->m_fileType << std::dec << ",";
   asciiFile.addPos(pos);
   asciiFile.addNote(f.str().c_str());
 
