@@ -18,6 +18,8 @@ namespace IWAObjectType
 
 enum
 {
+  AuthorStorage = 212,
+  CustomDateTimeFormat = 222,
   // 401: StyleRoot, ie. list of different styles id
   Text = 2001,
   ShapeField = 2003,
@@ -36,6 +38,7 @@ enum
   DateTimeField = 2034,
   PageField = 2043,
   Image = 3005,
+  Mask = 3006,
   Group = 3008,
   ConnectionLine = 3009,
   MediaStyle = 3016,
@@ -56,7 +59,11 @@ enum
 
   PageMaster = 10011,
   PageMasterData = 10016,
-  HeadersAndFooters = 10143
+  HeadersAndFooters = 10143,
+
+  MetaData = 11006,
+  FileColorMap = 11014,
+  FileColor = 11015
 };
 
 }
