@@ -2124,7 +2124,7 @@ void LotusGraph::sendTextBoxWK4(std::shared_ptr<WPSStream> stream, WPSEntry cons
 	}
 }
 
-void LotusGraph::sendZone(LotusGraphInternal::ZonePcList const &zoneList, size_t id, WPSTransformation &transf)
+void LotusGraph::sendZone(LotusGraphInternal::ZonePcList const &zoneList, size_t id, WPSTransformation const &transf)
 {
 	if (!m_listener)
 	{
@@ -2332,7 +2332,7 @@ bool LotusGraph::readGraphZone(std::shared_ptr<WPSStream> stream, int zId)
 		}
 		val &=0xFFFC;
 		if (val) f << "f0=" << val << ",";
-		for (int i=1; i<sz/2; ++i)   // always f0=1, other 0
+		for (long i=1; i<sz/2; ++i)   // always f0=1, other 0
 		{
 			val=int(libwps::read16(input));
 			if (val) f << "f" << i << "=" << val << ",";
