@@ -79,6 +79,7 @@ struct GraphicState {
   {
   }
   GraphicState &operator=(GraphicState const &)=default;
+  GraphicState &operator=(GraphicState &&)=default;
   //! destructor
   ~GraphicState()
   {
@@ -119,7 +120,7 @@ struct State {
   //! returns true if we are in a text zone, ie. either in a textbox or a table cell
   bool isInTextZone() const
   {
-    return m_inNote || m_isTextBoxOpened || m_isTableCellOpened;
+    return m_inNote || m_inLink || m_isTextBoxOpened || m_isTableCellOpened;
   }
   //! the origin position
   MWAWVec2f m_origin;
@@ -449,7 +450,7 @@ void MWAWGraphicListener::openLink(MWAWLink const &link)
   m_documentInterface->openLink(propList);
   _pushParsingState();
   m_ps->m_inLink=true;
-// we do not want any close open paragraph in a link
+  // we do not want any close open paragraph in a link
   m_ps->m_isParagraphOpened=true;
 }
 
@@ -459,6 +460,7 @@ void MWAWGraphicListener::closeLink()
     MWAW_DEBUG_MSG(("MWAWGraphicListener::closeLink: closed outside a link\n"));
     return;
   }
+  _flushText();
   m_documentInterface->closeLink();
   _popParsingState();
 }
@@ -820,6 +822,8 @@ void MWAWGraphicListener::_openSpan()
 {
   if (m_ps->m_isTableOpened && !m_ps->m_isTableCellOpened)
     return;
+  if (m_ps->m_inLink)
+    return;
   if (!m_ps->isInTextZone()) {
     MWAW_DEBUG_MSG(("MWAWGraphicListener::_openSpan: called outsize a text zone\n"));
     return;
@@ -1125,6 +1129,13 @@ void MWAWGraphicListener::insertTextBox
     return;
   librevenge::RVNGPropertyList propList;
   _handleFrameParameters(propList, pos, style);
+  if (!style.m_frameName.empty() || !style.m_frameNextName.empty()) {
+    static bool first=true;
+    if (first) {
+      MWAW_DEBUG_MSG(("MWAWGraphicListener::insertTextBox: setting frame name is not implemented\n"));
+      first=false;
+    }
+  }
   float rotate = style.m_rotate;
   // flip does not works on text, so we ignore it...
   if (style.m_flip[0]&&style.m_flip[1]) rotate += 180.f;
