@@ -713,7 +713,7 @@ bool MWAWOLEParser::readCompObj(MWAWInputStreamPtr ip, std::string const &oleNam
       st = f.str();
     }
     else {
-      for (int i = 0; i < sz; i++)
+      for (long i = 0; i < sz; i++)
         st += char(ip->readULong(1));
     }
 
@@ -770,24 +770,32 @@ bool MWAWOLEParser::readCompObj(MWAWInputStreamPtr ip, std::string const &oleNam
 // summary and doc summary
 //////////////////////////////////////////////////
 bool MWAWOLEParser::readSummaryInformation(MWAWInputStreamPtr input, std::string const &oleName,
-    int &encoding, librevenge::RVNGPropertyList &pList, libmwaw::DebugFile &ascii) const
+    int &encoding, librevenge::RVNGPropertyList &pList, libmwaw::DebugFile &ascii,
+    long endPos) const
 {
   if (oleName!="SummaryInformation" && oleName!="DocumentSummaryInformation") return false;
-  input->seek(0, librevenge::RVNG_SEEK_SET);
+  if (endPos<0) {
+    input->seek(0, librevenge::RVNG_SEEK_SET);
+    endPos=input->size();
+  }
+  long pos=input->tell();
   libmwaw::DebugStream f;
   f << "Entries(SumInfo):";
   bool isDoc=oleName=="DocumentSummaryInformation";
   if (isDoc) f << "doc,";
   auto val=int(input->readULong(2));
+  bool invertOLE=false;
   if (val==0xfeff) {
-    input->setReadInverted(false);
+    invertOLE=true;
+    input->setReadInverted(!input->readInverted());
     val=0xfffe;
   }
-  if (input->size()<48 || val!=0xfffe) {
+  if (pos+48>endPos || val!=0xfffe) {
     MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: header seems bad\n"));
     f << "###";
-    ascii.addPos(0);
+    ascii.addPos(pos);
     ascii.addNote(f.str().c_str());
+    if (invertOLE) input->setReadInverted(!input->readInverted());
     return true;
   }
   for (int i=0; i<11; ++i) { // f1=1, f2=0-2
@@ -802,8 +810,9 @@ bool MWAWOLEParser::readSummaryInformation(MWAWInputStreamPtr input, std::string
   if (lVal==0 || lVal>15) { // find 1 or 2 sections, unsure about the maximum numbers
     MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: summary info is bad\n"));
     f << "###sumInfo=" << std::hex << lVal << std::dec << ",";
-    ascii.addPos(0);
+    ascii.addPos(pos);
     ascii.addNote(f.str().c_str());
+    if (invertOLE) input->setReadInverted(!input->readInverted());
     return true;
   }
   auto numSection=int(lVal);
@@ -822,35 +831,37 @@ bool MWAWOLEParser::readSummaryInformation(MWAWInputStreamPtr input, std::string
     }
   }
   auto decal=int(input->readULong(4));
-  if (decal<0x30 || input->size()<decal) {
+  if (decal<0x30 || pos+decal>endPos) {
     MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: decal is bad\n"));
     f << "decal=" << val << ",";
-    ascii.addPos(0);
+    ascii.addPos(pos);
     ascii.addNote(f.str().c_str());
+    if (invertOLE) input->setReadInverted(!input->readInverted());
     return true;
 
   }
-  ascii.addPos(0);
+  ascii.addPos(pos);
   ascii.addNote(f.str().c_str());
   if (decal!=0x30) {
     ascii.addPos(0x30);
     ascii.addNote("_");
-    input->seek(decal, librevenge::RVNG_SEEK_SET);
+    input->seek(pos+decal, librevenge::RVNG_SEEK_SET);
   }
 
   for (int sect=0; sect<numSection; ++sect) {
-    long pos=input->tell();
+    pos=input->tell();
     f.str("");
     f << "SumInfo-A:";
     auto pSectSize=long(input->readULong(4));
     long endSect=pos+pSectSize;
     auto N=int(input->readULong(4));
     f << "N=" << N << ",";
-    if (pSectSize<0 || input->size()-pos<pSectSize || (pSectSize-8)/8<N) {
+    if (pSectSize<0 || endPos-pos<pSectSize || (pSectSize-8)/8<N) {
       MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: psetstruct is bad\n"));
       f << "###";
       ascii.addPos(pos);
       ascii.addNote(f.str().c_str());
+      if (invertOLE) input->setReadInverted(!input->readInverted());
       return true;
     }
     f << "[";
@@ -870,41 +881,41 @@ bool MWAWOLEParser::readSummaryInformation(MWAWInputStreamPtr input, std::string
     ascii.addPos(pos);
     ascii.addNote(f.str().c_str());
 
-    long endPos=input->size();
     for (auto it=posToTypeMap.begin(); it!=posToTypeMap.end(); ++it) {
       pos=it->first;
       auto nextIt=it;
-      if (++nextIt!=posToTypeMap.end()) endPos=nextIt->first;
+      long sEndPos= (++nextIt!=posToTypeMap.end()) ? nextIt->first : endSect;
       input->seek(pos, librevenge::RVNG_SEEK_SET);
       f.str("");
       f << "SumInfo-B" << it->second << ":";
       auto type=int(input->readULong(4));
       if (sect==0 && it->second==1 && !isDoc && type==2) {
         long value=-1;
-        if (readSummaryPropertyLong(input,endPos,type,value,f) && value>=0 && value<10000) // 10000 is mac
+        if (readSummaryPropertyLong(input,sEndPos,type,value,f) && value>=0 && value<10000) // 10000 is mac
           encoding=int(value);
       }
       else if (sect==0 && type==0x1e && !isDoc && ((it->second>=2 && it->second<=6) || it->second==8)) {
         librevenge::RVNGString text;
-        if (readSummaryPropertyString(input, endPos, type, text, f) && !text.empty()) {
-          static char const *(attribNames[]) = {
+        if (readSummaryPropertyString(input, sEndPos, type, text, f) && !text.empty()) {
+          static char const *attribNames[] = {
             "", "", "dc:title", "dc:subject", "meta:initial-creator",
             "meta:keywords", "dc:description"/*comment*/, "", "dc:creator"
           };
           pList.insert(attribNames[it->second], text);
         }
       }
-      else if (!readSummaryProperty(input, endPos, type, ascii, f)) {
+      else if (!readSummaryProperty(input, sEndPos, type, ascii, f)) {
         MWAW_DEBUG_MSG(("MWAWOLEParser::readSummaryInformation: find unknown type\n"));
         f << "##type=" << std::hex << type << std::dec << ",";
       }
-      if (input->tell()!=endPos && input->tell()!=pos)
+      if (input->tell()!=sEndPos && input->tell()!=pos)
         ascii.addDelimiter(input->tell(),'|');
       ascii.addPos(pos);
       ascii.addNote(f.str().c_str());
     }
     input->seek(endSect, librevenge::RVNG_SEEK_SET);
   }
+  if (invertOLE) input->setReadInverted(!input->readInverted());
   return true;
 }
 
