@@ -157,6 +157,10 @@ MWAWParagraph::MWAWParagraph()
   , m_listLevel()
   , m_backgroundColor(MWAWColor::white())
   , m_borders()
+
+  , m_dropNumCharacters(0)
+  , m_dropNumLines(0)
+
   , m_styleName("")
   , m_extra("")
 {
@@ -185,37 +189,37 @@ int MWAWParagraph::cmp(MWAWParagraph const &para) const
     if (*(m_spacings[i]) > *(para.m_spacings[i])) return 1;
   }
   if (*m_justify < *para.m_justify) return -1;
-  if (*m_justify > *para.m_justify) return -1;
+  if (*m_justify > *para.m_justify) return 1;
   if (*m_marginsUnit < *para.m_marginsUnit) return -1;
-  if (*m_marginsUnit > *para.m_marginsUnit) return -1;
+  if (*m_marginsUnit > *para.m_marginsUnit) return 1;
   if (*m_spacingsInterlineUnit < *para.m_spacingsInterlineUnit) return -1;
-  if (*m_spacingsInterlineUnit > *para.m_spacingsInterlineUnit) return -1;
+  if (*m_spacingsInterlineUnit > *para.m_spacingsInterlineUnit) return 1;
   if (*m_spacingsInterlineType < *para.m_spacingsInterlineType) return -1;
-  if (*m_spacingsInterlineType > *para.m_spacingsInterlineType) return -1;
+  if (*m_spacingsInterlineType > *para.m_spacingsInterlineType) return 1;
   if (*m_tabsRelativeToLeftMargin < *para.m_tabsRelativeToLeftMargin) return -1;
-  if (*m_tabsRelativeToLeftMargin > *para.m_tabsRelativeToLeftMargin) return -1;
+  if (*m_tabsRelativeToLeftMargin > *para.m_tabsRelativeToLeftMargin) return 1;
 
   if (m_tabs->size() < para.m_tabs->size()) return -1;
-  if (m_tabs->size() > para.m_tabs->size()) return -1;
+  if (m_tabs->size() > para.m_tabs->size()) return 1;
 
   for (size_t i=0; i < m_tabs->size(); i++) {
     int diff=(*m_tabs)[i].cmp((*para.m_tabs)[i]);
     if (diff) return diff;
   }
   if (*m_breakStatus < *para.m_breakStatus) return -1;
-  if (*m_breakStatus > *para.m_breakStatus) return -1;
+  if (*m_breakStatus > *para.m_breakStatus) return 1;
   if (*m_writingMode < *para.m_writingMode) return -1;
-  if (*m_writingMode > *para.m_writingMode) return -1;
+  if (*m_writingMode > *para.m_writingMode) return 1;
   if (*m_listLevelIndex < *para.m_listLevelIndex) return -1;
-  if (*m_listLevelIndex > *para.m_listLevelIndex) return -1;
+  if (*m_listLevelIndex > *para.m_listLevelIndex) return 1;
   if (*m_listId < *para.m_listId) return -1;
-  if (*m_listId > *para.m_listId) return -1;
+  if (*m_listId > *para.m_listId) return 1;
   if (*m_listStartValue < *para.m_listStartValue) return -1;
-  if (*m_listStartValue > *para.m_listStartValue) return -1;
+  if (*m_listStartValue > *para.m_listStartValue) return 1;
   int diff=m_listLevel->cmp(*para.m_listLevel);
   if (diff) return diff;
   if (*m_backgroundColor < *para.m_backgroundColor) return -1;
-  if (*m_backgroundColor > *para.m_backgroundColor) return -1;
+  if (*m_backgroundColor > *para.m_backgroundColor) return 1;
 
   if (m_borders.size() < para.m_borders.size()) return -1;
   if (m_borders.size() > para.m_borders.size()) return 1;
@@ -225,6 +229,11 @@ int MWAWParagraph::cmp(MWAWParagraph const &para) const
     diff = m_borders[i]->compare(*(para.m_borders[i]));
     if (diff) return diff;
   }
+
+  if (*m_dropNumCharacters < *para.m_dropNumCharacters) return -1;
+  if (*m_dropNumCharacters > *para.m_dropNumCharacters) return 1;
+  if (*m_dropNumLines < *para.m_dropNumLines) return -1;
+  if (*m_dropNumLines > *para.m_dropNumLines) return 1;
   if (m_styleName<para.m_styleName) return -1;
   if (m_styleName>para.m_styleName) return 1;
   return 0;
@@ -258,13 +267,15 @@ void MWAWParagraph::insert(MWAWParagraph const &para)
   m_writingMode.insert(para.m_writingMode);
   m_listLevelIndex.insert(para.m_listLevelIndex);
   m_listId.insert(para.m_listId);
-  m_listStartValue.insert(m_listStartValue);
+  m_listStartValue.insert(para.m_listStartValue);
   m_listLevel.insert(para.m_listLevel);
   m_backgroundColor.insert(para.m_backgroundColor);
   if (m_borders.size() < para.m_borders.size())
     m_borders.resize(para.m_borders.size());
   for (size_t i = 0; i < para.m_borders.size(); i++)
     m_borders[i].insert(para.m_borders[i]);
+  m_dropNumCharacters.insert(para.m_dropNumCharacters);
+  m_dropNumLines.insert(para.m_dropNumLines);
   m_styleName=para.m_styleName; // checkme
   m_extra += para.m_extra;
 }
@@ -421,6 +432,16 @@ void MWAWParagraph::addTo(librevenge::RVNGPropertyList &propList, bool inTable)
       tab.addTo(tabs, decalX);
     propList.insert("style:tab-stops", tabs);
   }
+
+  if (m_dropNumCharacters.get()>0 && m_dropNumLines.get()>1) {
+    librevenge::RVNGPropertyList cap;
+    //addme cap.insert("style:distance", )
+    cap.insert("style:length", *m_dropNumCharacters);
+    cap.insert("style:lines", *m_dropNumLines);
+    librevenge::RVNGPropertyListVector capVector;
+    capVector.append(cap);
+    propList.insert("style:drop-cap", capVector);
+  }
 }
 
 std::ostream &operator<<(std::ostream &o, MWAWParagraph const &pp)
@@ -507,6 +528,8 @@ std::ostream &operator<<(std::ostream &o, MWAWParagraph const &pp)
     o << "=" << border << ",";
   }
 
+  if (pp.m_dropNumCharacters.get()>0 && pp.m_dropNumLines.get()>1)
+    o << "drop=" << *pp.m_dropNumCharacters << "[" << *pp.m_dropNumLines << "l],";
   if (!pp.m_extra.empty()) o << "extras=(" << pp.m_extra << ")";
   return o;
 }
