@@ -79,6 +79,7 @@ struct GraphicState {
   {
   }
   GraphicState &operator=(GraphicState const &)=default;
+  GraphicState &operator=(GraphicState &&)=default;
   //! destructor
   ~GraphicState()
   {
@@ -117,7 +118,7 @@ struct State {
 //! returns true if we are in a text zone, ie. either in a textbox or a table cell
   bool isInTextZone() const
   {
-    return m_inNote || m_isTextBoxOpened || m_isTableCellOpened;
+    return m_inNote || m_inLink || m_isTextBoxOpened || m_isTableCellOpened;
   }
   //! the origin position
   MWAWVec2f m_origin;
@@ -440,6 +441,7 @@ void MWAWPresentationListener::closeLink()
     MWAW_DEBUG_MSG(("MWAWPresentationListener::closeLink: closed outside a link\n"));
     return;
   }
+  _flushText();
   m_documentInterface->closeLink();
   _popParsingState();
 }
@@ -801,6 +803,8 @@ void MWAWPresentationListener::_openSpan()
 {
   if (m_ps->m_isTableOpened && !m_ps->m_isTableCellOpened)
     return;
+  if (m_ps->m_inLink)
+    return;
   if (!m_ps->isInTextZone()) {
     MWAW_DEBUG_MSG(("MWAWPresentationListener::_openSpan: called outsize a text zone\n"));
     return;
