@@ -141,7 +141,7 @@ struct Pixmap {
       }
       m_colors.resize(size_t(H*W));
     }
-    long maxValues = m_pixelSize<=8 ? (1 << m_pixelSize)-1 : 0;
+    long maxValues = m_pixelSize<=8 ? long(1 << m_pixelSize)-1 : 0;
     std::vector<unsigned char> values;
     values.resize(size_t(m_rowBytes+24), static_cast<unsigned char>(0));
 
@@ -1508,7 +1508,7 @@ bool MacDraft5StyleManager::readPixPat(MWAWEntry const &entry, bool inRsrc)
 bool MacDraft5StyleManager::readVersion(MWAWEntry &entry)
 {
   MWAWInputStreamPtr input = m_parserState->m_input;
-  if (!input || !entry.valid() || entry.length()<8) {
+  if (!input || !entry.valid() || entry.length()<8 || !input->checkPosition(entry.end())) {
     MWAW_DEBUG_MSG(("MWAWMacDraft5StyleManager::readVersion: entry is invalid\n"));
     return false;
   }
