@@ -31,13 +31,17 @@
 * instead of those above.
 */
 
+#include <cctype>
 #include <iomanip>
+#include <iostream>
 #include <sstream>
 
 #include "MWAWDebug.hxx"
 #include "MWAWPrinter.hxx"
 
 #include "RagTime5ClusterManager.hxx"
+#include "RagTime5Document.hxx"
+#include "RagTime5Formula.hxx"
 #include "RagTime5Text.hxx"
 
 #include "RagTime5StructManager.hxx"
@@ -45,7 +49,8 @@
 ////////////////////////////////////////////////////////////
 // constructor/destructor, ...
 ////////////////////////////////////////////////////////////
-RagTime5StructManager::RagTime5StructManager()
+RagTime5StructManager::RagTime5StructManager(RagTime5Document &doc)
+  : m_document(doc)
 {
 }
 
@@ -53,6 +58,13 @@ RagTime5StructManager::~RagTime5StructManager()
 {
 }
 
+RagTime5StructManager::DataParser::DataParser(std::string const &zoneName)
+  : m_name(zoneName)
+{
+  if (!m_name.empty())
+    m_name[0]=char(std::toupper((unsigned char)(m_name[0])));
+}
+
 RagTime5StructManager::DataParser::~DataParser()
 {
 }
@@ -79,6 +91,45 @@ bool RagTime5StructManager::readCompressedLong(MWAWInputStreamPtr &input, long e
   return input->tell()<=endPos;
 }
 
+std::string RagTime5StructManager::printType(unsigned long fileType)
+{
+  static std::map<unsigned long, char const *> const s_typeToName = {
+    {0x145e042, "fillStyle[container]"},
+    {0x1460042, "lineStyle[container]"},
+    {0x146902a, "unit[base,from]"},
+    {0x146903a, "unit[base,to]"},
+    {0x146904a, "unit[base,id]"},
+    {0x146905a, "unit[name]"},
+    {0x146907a, "unit[second,id]"}, // rare
+    {0x146908a, "unit[digits,place]"},
+    {0x146a042, "unit[container]"},
+    {0x146e02a, "ruler[unit,id]"},
+    {0x146e03a, "ruler[step,major]"},
+    {0x146e04a, "ruler[step,minor]"},
+    {0x146e05a, "ruler[grid/major]"}, // a fraction or a fixed decimal?
+    {0x146e06a, "ruler[grid,line/gridPoint]"},
+    {0x146f042, "ruler[container]"},
+    {0x17d5042, "color[container]"},
+
+    // functions collections
+    {0x14c2042, "functions[layout]"},
+    {0x1559842, "functions[standart]"},
+    {0x1663842, "functions[spreadsheet]"},
+    {0x1be5042, "functions[fax]"},
+    {0x1d50842, "functions[button]"},
+    {0x1e16842, "functions[slide]"},
+    {0x23aa042, "functions[calendar]"},
+    {0x23af042, "functions[serialNumber]"},
+    {0x23b4042, "functions[euro]"},
+  };
+  auto it=s_typeToName.find(fileType);
+  if (it!=s_typeToName.end())
+    return it->second;
+  std::stringstream s;
+  s << (fileType>>11) << "-" << std::hex << (fileType&0x7ff) << std::dec;
+  return s.str();
+}
+
 bool RagTime5StructManager::readTypeDefinitions(RagTime5Zone &zone)
 {
   if (zone.m_entry.length()<26) return false;
@@ -90,7 +141,7 @@ bool RagTime5StructManager::readTypeDefinitions(RagTime5Zone &zone)
   libmwaw::DebugStream f;
   f << "Entries(TypeDef):[" << zone << "]";
   long N;
-  if (!RagTime5StructManager::readCompressedLong(input, endPos, N) || N<20 || 12+14*N>zone.m_entry.length()) {
+  if (!input->checkPosition(endPos) || !RagTime5StructManager::readCompressedLong(input, endPos, N) || N<20 || 12+14*N>zone.m_entry.length()) {
     MWAW_DEBUG_MSG(("RagTime5StructManager::readTypeDefinitions: can not read the list type zone\n"));
     input->setReadInverted(false);
     return false;
@@ -104,6 +155,10 @@ bool RagTime5StructManager::readTypeDefinitions(RagTime5Zone &zone)
     if (val) f << "f" << i << "=" << val << ",";
   }
   auto sz=static_cast<int>(input->readULong(1));
+  if (input->tell()+sz>endPos) {
+    MWAW_DEBUG_MSG(("RagTime5StructManager::readTypeDefinitions: can not find the data1 sz\n"));
+    return false;
+  }
   if (sz) {
     f << "data1=[" << std::hex;
     for (int i=0; i<sz+1; ++i) {
@@ -151,6 +206,18 @@ bool RagTime5StructManager::readTypeDefinitions(RagTime5Zone &zone)
     return false;
   }
 
+  // first 10 main types are the same:
+  //   0: 1451042, 1:6d042, 2:cf042, 3:1454042, 4:74040, 5:df842, 6:ce042, 7:3c042, 8:ce842, 9:67842, 10:6a842
+  // Component correspondance
+  // 14b5842: layout, 14b7842: master layout,
+  // 14e6842: drawing
+  // 15e0842: Text
+  // 1645042: spreadsheet
+  // 16a8842: Graph
+  // 170c842: Picture
+  // 1d4d042: Button
+  // 1d7f842: Sound
+  // 1db0842: Movie
   for (size_t i=0; i+1<listPtrs.size(); ++i) {
     if (listPtrs[i]<0 || listPtrs[i]==listPtrs[i+1]) continue;
     if (listPtrs[i]==remain) break;
@@ -165,8 +232,8 @@ bool RagTime5StructManager::readTypeDefinitions(RagTime5Zone &zone)
       ascFile.addNote(f.str().c_str());
     }
     input->seek(pos, librevenge::RVNG_SEEK_SET);
-    val=static_cast<int>(input->readLong(4)); // a small number
-    if (val!=1) f << "f0=" << val << ",";
+    val=static_cast<int>(input->readLong(4));
+    if (val!=1) f << "num[used]=" << val << ",";
     auto hSz=static_cast<int>(input->readULong(2));
     int nData=(dSz-4-hSz)/12;
     if (4+hSz>dSz || dSz!=4+hSz+12*nData || nData<0 || hSz<20) {
@@ -179,22 +246,22 @@ bool RagTime5StructManager::readTypeDefinitions(RagTime5Zone &zone)
     val=static_cast<int>(input->readULong(2));
     if (val) f << "fl=" << std::hex << val << std::dec << ",";
     val=static_cast<int>(input->readULong(4));
-    if (val) f << "id?=" << std::hex << val << std::dec << ",";
-    auto type=long(input->readULong(4));
-    if (type) f << "type=" << std::hex << type << std::dec << ",";
+    if (val) f << "id?=" << std::hex << val << std::dec << ","; // maybe a date, origin?
+    auto type=input->readULong(4);
+    if (type) f << "type=" << printType(type) << ",";
     for (int j=0; j<2; ++j) { // f1=0..12,
       val=static_cast<int>(input->readLong(2));
       if (val) f << "f" << j+1 << "=" << val << ",";
     }
-    auto type2=long(input->readULong(4));
-    if (type2) f << "type2=" << std::hex << type2 << std::dec << ",";
+    auto type2=input->readULong(4);
+    if (type2) f << "type2=" << printType(type2) << ",";
     if (hSz>20) {
       auto sSz=static_cast<int>(input->readULong(1));
       if ((sSz%2)!=0 || 20+1+sSz>hSz) {
         f << "###sSz=" << sSz << ",";
         MWAW_DEBUG_MSG(("RagTime5StructManager::readTypeDefinitions: the string size seems bad\n"));
       }
-      else {
+      else { // Layout, Text, Picture, .. a componant name
         std::string name("");
         for (int c=0; c<sSz/2; ++c)
           name+=char(input->readULong(2));
@@ -215,8 +282,8 @@ bool RagTime5StructManager::readTypeDefinitions(RagTime5Zone &zone)
       pos=input->tell();
       f.str("");
       f << "TypeDef-" << i << "[" << j << "]:";
-      f << "type=" << std::hex << input->readULong(4) << std::dec << ","; // a big number
-      f << "type2=" << std::hex << input->readULong(4) << std::dec << ","; // a big number
+      f << "type=" << printType(input->readULong(4)) << ","; // a big number, ~type (but a multiple of 5)
+      f << "type2=" << printType(input->readULong(4)) << std::dec << ","; // a big number
       for (int k=0; k<2; ++k) {
         val=static_cast<int>(input->readULong(1));
         if (val) f << "fl" << k << "=" << std::hex << val << std::dec << ",";
@@ -296,7 +363,7 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
     MWAW_DEBUG_MSG(("RagTime5StructManager::readField: the zone seems too short\n"));
     return false;
   }
-  auto type=long(input->readULong(4));
+  auto type=input->readULong(4);
   if ((type>>16)==0) {
     input->seek(debPos, librevenge::RVNG_SEEK_SET);
     return false;
@@ -340,7 +407,6 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
     field.m_longValue[0]=long(input->readLong(1));
     return true;
 
-  case 0x3c057: // small int 3-9: not in typedef
   case 0x3b880:
   case 0x1479080:
   case 0x147b880:
@@ -438,8 +504,8 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
     field.m_type=Field::T_Unicode;
     field.m_name="unicode";
     auto val=static_cast<int>(input->readULong(2));
-    if ((val&0xF0FF)==0) {
-      if ((val>>8)>2) f << "f1=" << (val>>8);
+    if ((val&0x70FF)==0) {
+      if (val) f << "f1=" << ((val&0x7F00)>>8);
       for (long i=2; i<fSz; ++i)
         libmwaw::appendUnicode(static_cast<uint32_t>(input->readULong(1)), field.m_string);
     }
@@ -462,12 +528,14 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
     }
     field.m_type=Field::T_Unicode;
     field.m_name="int+unicode";
-    auto val=static_cast<int>(input->readULong(2));
-    if (val==0xFF00 || val==1) {
+    int val=int(input->readULong(2));
+    if ((val&0xff) || val==0xff00) { // ff00 means list of {length+str} for normal string
       f << "multistring,";
+      f << "val=" << std::hex << val << std::dec << ",";
       val=static_cast<int>(input->readULong(2));
     }
-    if ((val&0xF0FF)==0) {
+    //std::cout << "ICI1:" << std::hex << val << std::dec << "\n";
+    if ((val&0x8000)==0) {
       if ((val>>8)>2) f << "f1=" << (val>>8);
       fSz=int(endDataPos-input->tell());
       for (long i=0; i<fSz; ++i) {
@@ -478,11 +546,8 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
           field.m_string.append('#');
       }
     }
-    else {
-      input->seek(debDataPos, librevenge::RVNG_SEEK_SET);
-      if (!readUnicodeString(input, endDataPos, field.m_string)) {
-        field.m_string="###";
-      }
+    else if (!readUnicodeString(input, endDataPos, field.m_string)) {
+      field.m_string="###";
     }
     input->seek(endDataPos, librevenge::RVNG_SEEK_SET);
     field.m_extra=f.str();
@@ -620,7 +685,7 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
     field.m_color=MWAWColor(col[0],col[1],col[2],col[3]);
     return true;
   }
-  case 0x8d000:
+  case 0x8d000: {
     if (fSz!=4) {
       MWAW_DEBUG_MSG(("RagTime5StructManager::readField: unexpected data fSz for rsrcName\n"));
       f << "###rsrcName";
@@ -628,9 +693,10 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
     }
     field.m_type=Field::T_Code;
     field.m_name="rsrcName";
-    for (long i=0; i<4; ++i)
-      field.m_string.append(char(input->readULong(1)));
+    auto cod=input->readULong(4);
+    for (int i=0; i<4; ++i) field.m_string.append(char(cod>>(24-8*i)));
     return true;
+  }
   case 0x31e040:
     // <!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
     if (fSz<30) {
@@ -866,6 +932,7 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
       field.m_longList.push_back(long(input->readLong(4)));
     return true;
   }
+  case 0x3c057:
   case 0x80045080: // child of 14741fa
   case 0xcf042: { // list of small int: header fl=2000 with f2=9
     if ((fSz%2)!=0) {
@@ -912,6 +979,7 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
 
   case 0x33000: // maybe one 2xint
   case 0x1671817:
+  case 0x16b491a: // in chart,
   case 0x16b492a: // in chart, dim[4byte], id, 0
   case 0x16b5aea: // chart preference, always 0,0,id,0
   case 0x80033000:
@@ -1033,7 +1101,11 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
   case 0x14c2042: /* function container*/
   case 0x1559842:
   case 0x1663842:
+  case 0x1be5042:
+  case 0x1d50842:
   case 0x1e16842:
+  case 0x23aa042:
+  case 0x23af042:
   case 0x23b4042:
 
   case 0x7a09a: // with a4000 or a4840
@@ -1166,97 +1238,6 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
     field.m_extra=f.str();
     return true;
   }
-  case 0x14c2017: // function name
-  case 0x14c2037:
-  case 0x1559937:
-  case 0x1559f47:
-  case 0x1559857:
-  case 0x1559987:
-  case 0x15598a7:
-  case 0x15598b7:
-  case 0x15598d7:
-  case 0x15598e7:
-  case 0x1559907:
-  case 0x1559a27:
-  case 0x1559bc7:
-  case 0x1559df7:
-  case 0x1559e07:
-  case 0x1559e37:
-  case 0x1559e57:
-  case 0x1559e67:
-  case 0x1559f17:
-  case 0x1559f37:
-  case 0x1559f57:
-  case 0x1559f67:
-  case 0x1559fe7:
-  case 0x1559ff7:
-  case 0x155a1c7:
-  case 0x155a017:
-  case 0x155a027:
-  case 0x155a047:
-  case 0x155a077:
-  case 0x155a087:
-  case 0x155a1f7:
-  case 0x155a207:
-  case 0x1663817:
-  case 0x1663827:
-  case 0x16638a7:
-  case 0x16638c7:
-  case 0x1663907:
-  case 0x1663917:
-  case 0x1663947:
-  case 0x1663967:
-  case 0x1663977:
-  case 0x1663987:
-  case 0x16639e7:
-  case 0x16639f7:
-  case 0x1663a07:
-  case 0x1663a17:
-  case 0x1e16827:
-  case 0x1e16837:
-  case 0x1e16847:
-  case 0x1e16857:
-  case 0x1e16867:
-  case 0x1e16877:
-  case 0x1e16887:
-  case 0x1e16897:
-  case 0x1e168a7:
-  case 0x1e168b7:
-  case 0x1e168c7:
-  case 0x1e168d7:
-  case 0x1e168e7:
-  case 0x23b4027: {
-    if (fSz<14) {
-      MWAW_DEBUG_MSG(("RagTime5StructManager::readField: unexpected field size for functions name list\n"));
-      f << "###func[name],";
-      break;
-    }
-    for (int i=0; i<3; ++i) { // f1=0|-1, f2=small number, other 0
-      auto val=static_cast<int>(input->readLong(2));
-      if (val) f << "f" << i+1 << "=" << val << ",";
-    }
-    field.m_type=Field::T_FieldList;
-    field.m_name="func[name]";
-    bool ok=true;
-    for (int i=0; i<2; ++i) {
-      Field child;
-      long pos=input->tell();
-      if (!readField(input, endDataPos, ascFile, child)) {
-        ok=false;
-        input->seek(pos, librevenge::RVNG_SEEK_SET);
-        break;
-      }
-      field.m_fieldList.push_back(child);
-    }
-    if (!ok || input->tell() != endDataPos) {
-      MWAW_DEBUG_MSG(("RagTime5StructManager::readField: can not read some 2fields list data\n"));
-      f.str("");
-      f << "###pos=" << input->tell()-debPos;
-      input->seek(endDataPos, librevenge::RVNG_SEEK_SET);
-    }
-    field.m_extra+=f.str();
-    return true;
-  }
   case 0x42040: {
     if (fSz<10) {
       MWAW_DEBUG_MSG(("RagTime5StructManager::readField: unexpected field size for day/month list\n"));
@@ -1309,9 +1290,43 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
       field.m_longList.push_back(long(input->readLong(2)));
     return true;
   }
-  default:
+  default: {
+    auto const &functIds=m_document.getFormulaParser()->getFunctionsId();
+    if (functIds.find(static_cast<unsigned long>(type))!=functIds.end()) {
+      if (fSz<14) {
+        MWAW_DEBUG_MSG(("RagTime5StructManager::readField: unexpected field size for functions name list\n"));
+        f << "###func[name],";
+        break;
+      }
+      for (int i=0; i<3; ++i) { // f1=0|-1, f2=small number, other 0
+        auto val=static_cast<int>(input->readLong(2));
+        if (val) f << "f" << i+1 << "=" << val << ",";
+      }
+      field.m_type=Field::T_FieldList;
+      field.m_name="func[name]";
+      bool ok=true;
+      for (int i=0; i<2; ++i) {
+        Field child;
+        long pos=input->tell();
+        if (!readField(input, endDataPos, ascFile, child)) {
+          ok=false;
+          input->seek(pos, librevenge::RVNG_SEEK_SET);
+          break;
+        }
+        field.m_fieldList.push_back(child);
+      }
+      if (!ok || input->tell() != endDataPos) {
+        MWAW_DEBUG_MSG(("RagTime5StructManager::readField: can not read some 2fields list data\n"));
+        f.str("");
+        f << "###pos=" << input->tell()-debPos;
+        input->seek(endDataPos, librevenge::RVNG_SEEK_SET);
+      }
+      field.m_extra+=f.str();
+      return true;
+    }
     break;
   }
+  }
 
   input->seek(debDataPos, librevenge::RVNG_SEEK_SET);
   if (!complex) {
@@ -1337,11 +1352,13 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
   case 0x15f4815: // maybe related to list
   case 0x160f815:
   case 0x1671845:
+  case 0x17db015: // in group cluster
     field.m_name="longList";
     break;
   case 0x1451025:
   case 0x146c015: // with ce017
   case 0x14e6875:
+  case 0x15f4015: // with ce017
   case 0x15f6815:
   case 0x15f9015: // sometimes a list of 15f6815
     field.m_name="unstructList";
@@ -1388,9 +1405,9 @@ bool RagTime5StructManager::readField(MWAWInputStreamPtr input, long endPos, lib
 std::ostream &operator<<(std::ostream &o, RagTime5StructManager::Field const &field)
 {
   if (!field.m_name.empty())
-    o << field.m_name << ":" << std::hex << field.m_fileType << std::dec;
+    o << field.m_name << ":" << RagTime5StructManager::printType(field.m_fileType);
   else
-    o << "T" << std::hex << field.m_fileType << std::dec;
+    o << "T:" << RagTime5StructManager::printType(field.m_fileType);
   switch (field.m_type) {
   case RagTime5StructManager::Field::T_Double:
     o << "=" << field.m_doubleValue << ",";
@@ -1633,6 +1650,8 @@ RagTime5Zone::~RagTime5Zone()
 
 void RagTime5Zone::createAsciiFile()
 {
+  if (!m_input)
+    return;
   if (m_asciiName.empty()) {
     MWAW_DEBUG_MSG(("RagTime5Zone::createAsciiFile: can not find the ascii name\n"));
     return;
