@@ -48,29 +48,27 @@
 namespace StarParagraphAttribute
 {
 //! a character bool attribute
-class StarPAttributeBool : public StarAttributeBool
+class StarPAttributeBool final : public StarAttributeBool
 {
 public:
   //! constructor
-  StarPAttributeBool(Type type, std::string const &debugName, bool value) :
-    StarAttributeBool(type, debugName, value)
+  StarPAttributeBool(Type type, std::string const &debugName, bool value)
+    : StarAttributeBool(type, debugName, value)
   {
   }
   //! destructor
-  ~StarPAttributeBool();
+  ~StarPAttributeBool() final;
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeBool(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeBool(*this));
   }
   //! add to a para
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
 
 protected:
   //! copy constructor
-  StarPAttributeBool(StarPAttributeBool const &orig) : StarAttributeBool(orig)
-  {
-  }
+  StarPAttributeBool(StarPAttributeBool const &) = default;
 };
 
 StarPAttributeBool::~StarPAttributeBool()
@@ -78,55 +76,52 @@ StarPAttributeBool::~StarPAttributeBool()
 }
 
 //! a character color attribute
-class StarPAttributeColor : public StarAttributeColor
+class StarPAttributeColor final : public StarAttributeColor
 {
 public:
   //! constructor
-  StarPAttributeColor(Type type, std::string const &debugName, STOFFColor const &value) : StarAttributeColor(type, debugName, value)
+  StarPAttributeColor(Type type, std::string const &debugName, STOFFColor const &value)
+    : StarAttributeColor(type, debugName, value)
   {
   }
   //! destructor
-  ~StarPAttributeColor();
+  ~StarPAttributeColor() final;
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeColor(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeColor(*this));
   }
   //! add to a para
-  // virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  // void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
 protected:
   //! copy constructor
-  StarPAttributeColor(StarPAttributeColor const &orig) : StarAttributeColor(orig)
-  {
-  }
+  StarPAttributeColor(StarPAttributeColor const &) = default;
 };
 
 StarPAttributeColor::~StarPAttributeColor()
 {
 }
 //! a character integer attribute
-class StarPAttributeInt : public StarAttributeInt
+class StarPAttributeInt final : public StarAttributeInt
 {
 public:
   //! constructor
-  StarPAttributeInt(Type type, std::string const &debugName, int intSize, int value) :
-    StarAttributeInt(type, debugName, intSize, value)
+  StarPAttributeInt(Type type, std::string const &debugName, int intSize, int value)
+    : StarAttributeInt(type, debugName, intSize, value)
   {
   }
   //! destructor
-  ~StarPAttributeInt();
+  ~StarPAttributeInt() final;
   //! add to a para
-  // virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  // void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeInt(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeInt(*this));
   }
 protected:
   //! copy constructor
-  StarPAttributeInt(StarPAttributeInt const &orig) : StarAttributeInt(orig)
-  {
-  }
+  StarPAttributeInt(StarPAttributeInt const &) = default;
 };
 
 StarPAttributeInt::~StarPAttributeInt()
@@ -134,23 +129,23 @@ StarPAttributeInt::~StarPAttributeInt()
 }
 
 //! a character unsigned integer attribute
-class StarPAttributeUInt : public StarAttributeUInt
+class StarPAttributeUInt final : public StarAttributeUInt
 {
 public:
   //! constructor
-  StarPAttributeUInt(Type type, std::string const &debugName, int intSize, unsigned int value) :
-    StarAttributeUInt(type, debugName, intSize, value)
+  StarPAttributeUInt(Type type, std::string const &debugName, int intSize, unsigned int value)
+    : StarAttributeUInt(type, debugName, intSize, value)
   {
   }
   //! destructor
-  ~StarPAttributeUInt();
+  ~StarPAttributeUInt() final;
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeUInt(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeUInt(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object)
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final
   {
     STOFFInputStreamPtr input=zone.input();
     long pos=input->tell();
@@ -160,7 +155,7 @@ public:
       libstoff::DebugStream f;
       m_value=static_cast<unsigned int>(input->readULong(1));
       f << "Entries(StarAttribute)[" << zone.getRecordLevel() << "]:" << m_debugName << "=" << m_value << ",";
-      int tmp=int(input->readULong(1));
+      auto tmp=int(input->readULong(1));
       if (tmp) f << "#unkn=" << tmp << ",";
       ascFile.addPos(pos);
       ascFile.addNote(f.str().c_str());
@@ -169,39 +164,36 @@ public:
     return StarAttributeUInt::read(zone, vers, endPos, object);
   }
   //! add to a para
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
 protected:
   //! copy constructor
-  StarPAttributeUInt(StarPAttributeUInt const &orig) : StarAttributeUInt(orig)
-  {
-  }
+  StarPAttributeUInt(StarPAttributeUInt const &) = default;
 };
 
 StarPAttributeUInt::~StarPAttributeUInt()
 {
 }
 //! a void attribute
-class StarPAttributeVoid : public StarAttributeVoid
+class StarPAttributeVoid final : public StarAttributeVoid
 {
 public:
   //! constructor
-  StarPAttributeVoid(Type type, std::string const &debugName) : StarAttributeVoid(type, debugName)
+  StarPAttributeVoid(Type type, std::string const &debugName)
+    : StarAttributeVoid(type, debugName)
   {
   }
   //! destructor
-  ~StarPAttributeVoid();
+  ~StarPAttributeVoid() final;
   //! add to a para
-  // virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  // void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeVoid(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeVoid(*this));
   }
 protected:
   //! copy constructor
-  StarPAttributeVoid(StarPAttributeVoid const &orig) : StarAttributeVoid(orig)
-  {
-  }
+  StarPAttributeVoid(StarPAttributeVoid const &) = default;
 };
 
 StarPAttributeVoid::~StarPAttributeVoid()
@@ -230,7 +222,7 @@ void StarPAttributeUInt::addTo(StarState &state, std::set<StarAttribute const *>
     state.m_paragraph.m_propertyList.insert("fo:widows", int(m_value));
   else if (m_type==ATTR_PARA_VERTALIGN) {
     if (m_value<=4) {
-      char const *(wh[])= {"auto", "baseline", "top", "middle", "bottom"};
+      char const *wh[]= {"auto", "baseline", "top", "middle", "bottom"};
       state.m_paragraph.m_propertyList.insert("style:vertical-align", wh[m_value]);
     }
     else {
@@ -244,29 +236,29 @@ void StarPAttributeUInt::addTo(StarState &state, std::set<StarAttribute const *>
 }
 
 //! add a bool attribute
-inline void addAttributeBool(std::map<int, shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, bool defValue)
+inline void addAttributeBool(std::map<int, std::shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, bool defValue)
 {
-  map[type]=shared_ptr<StarAttribute>(new StarPAttributeBool(type,debugName, defValue));
+  map[type]=std::shared_ptr<StarAttribute>(new StarPAttributeBool(type,debugName, defValue));
 }
 //! add a color attribute
-inline void addAttributeColor(std::map<int, shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, STOFFColor const &defValue)
+inline void addAttributeColor(std::map<int, std::shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, STOFFColor const &defValue)
 {
-  map[type]=shared_ptr<StarAttribute>(new StarPAttributeColor(type,debugName, defValue));
+  map[type]=std::shared_ptr<StarAttribute>(new StarPAttributeColor(type,debugName, defValue));
 }
 //! add a int attribute
-inline void addAttributeInt(std::map<int, shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, int numBytes, int defValue)
+inline void addAttributeInt(std::map<int, std::shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, int numBytes, int defValue)
 {
-  map[type]=shared_ptr<StarAttribute>(new StarPAttributeInt(type,debugName, numBytes, defValue));
+  map[type]=std::shared_ptr<StarAttribute>(new StarPAttributeInt(type,debugName, numBytes, defValue));
 }
 //! add a unsigned int attribute
-inline void addAttributeUInt(std::map<int, shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, int numBytes, unsigned int defValue)
+inline void addAttributeUInt(std::map<int, std::shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName, int numBytes, unsigned int defValue)
 {
-  map[type]=shared_ptr<StarAttribute>(new StarPAttributeUInt(type,debugName, numBytes, defValue));
+  map[type]=std::shared_ptr<StarAttribute>(new StarPAttributeUInt(type,debugName, numBytes, defValue));
 }
 //! add a void attribute
-inline void addAttributeVoid(std::map<int, shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName)
+inline void addAttributeVoid(std::map<int, std::shared_ptr<StarAttribute> > &map, StarAttribute::Type type, std::string const &debugName)
 {
-  map[type]=shared_ptr<StarAttribute>(new StarPAttributeVoid(type,debugName));
+  map[type]=std::shared_ptr<StarAttribute>(new StarPAttributeVoid(type,debugName));
 }
 
 }
@@ -275,24 +267,27 @@ namespace StarParagraphAttribute
 {
 // ------------------------------------------------------------
 //! a adjust attribute
-class StarPAttributeAdjust : public StarAttribute
+class StarPAttributeAdjust final : public StarAttribute
 {
 public:
   //! constructor
-  StarPAttributeAdjust(Type type, std::string const &debugName) : StarAttribute(type, debugName), m_adjust(0), m_flags(0)
+  StarPAttributeAdjust(Type type, std::string const &debugName)
+    : StarAttribute(type, debugName)
+    , m_adjust(0)
+    , m_flags(0)
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeAdjust(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeAdjust(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a para
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const final
   {
     o << m_debugName << "=[";
     if (m_adjust) o << "adjust=" << m_adjust << ",";
@@ -301,9 +296,7 @@ public:
   }
 protected:
   //! copy constructor
-  StarPAttributeAdjust(StarPAttributeAdjust const &orig) : StarAttribute(orig), m_adjust(orig.m_adjust), m_flags(orig.m_flags)
-  {
-  }
+  StarPAttributeAdjust(StarPAttributeAdjust const &) = default;
   //! the adjust value
   int m_adjust;
   //! the flags
@@ -312,24 +305,29 @@ protected:
 
 // ------------------------------------------------------------
 //! a numeric bullet attribute
-class StarPAttributeBulletNumeric : public StarAttribute
+class StarPAttributeBulletNumeric final : public StarAttribute
 {
 public:
   //! constructor
-  StarPAttributeBulletNumeric(Type type, std::string const &debugName) : StarAttribute(type, debugName), m_numType(0), m_numLevels(0), m_flags(0), m_continuous(true)
+  StarPAttributeBulletNumeric(Type type, std::string const &debugName)
+    : StarAttribute(type, debugName)
+    , m_numType(0)
+    , m_numLevels(0)
+    , m_flags(0)
+    , m_continuous(true)
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeBulletNumeric(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeBulletNumeric(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a para
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const final
   {
     o << m_debugName << "=[";
     if (m_numType) o << "type=" << m_numType << ",";
@@ -340,9 +338,7 @@ public:
   }
 protected:
   //! copy constructor
-  StarPAttributeBulletNumeric(StarPAttributeBulletNumeric const &orig) : StarAttribute(orig), m_numType(orig.m_numType), m_numLevels(orig.m_numLevels), m_flags(orig.m_flags), m_continuous(orig.m_continuous)
-  {
-  }
+  StarPAttributeBulletNumeric(StarPAttributeBulletNumeric const &) = default;
   //! the type
   int m_numType;
   //! the numLevels
@@ -355,57 +351,63 @@ protected:
 
 // ------------------------------------------------------------
 //! a simple bullet attribute
-class StarPAttributeBulletSimple : public StarAttribute
+class StarPAttributeBulletSimple final : public StarAttribute
 {
 public:
   //! constructor
-  StarPAttributeBulletSimple(Type type, std::string const &debugName) :
-    StarAttribute(type, debugName), m_level()
+  StarPAttributeBulletSimple(Type type, std::string const &debugName)
+    : StarAttribute(type, debugName)
+    , m_level()
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeBulletSimple(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeBulletSimple(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a para
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const final
   {
     o << m_debugName;
   }
 protected:
   //! copy constructor
-  StarPAttributeBulletSimple(StarPAttributeBulletSimple const &orig) :
-    StarAttribute(orig), m_level(orig.m_level)
-  {
-  }
+  StarPAttributeBulletSimple(StarPAttributeBulletSimple const &) = default;
   //! the level
   STOFFListLevel m_level;
 };
 
 //! a drop attribute
-class StarPAttributeDrop : public StarAttribute
+class StarPAttributeDrop final : public StarAttribute
 {
 public:
   //! constructor
-  StarPAttributeDrop(Type type, std::string const &debugName) : StarAttribute(type, debugName), m_numFormats(0), m_numLines(0), m_numChars(0), m_numDistances(0), m_whole(false), m_numX(0), m_numY(0)
+  StarPAttributeDrop(Type type, std::string const &debugName)
+    : StarAttribute(type, debugName)
+    , m_numFormats(0)
+    , m_numLines(0)
+    , m_numChars(0)
+    , m_numDistances(0)
+    , m_whole(false)
+    , m_numX(0)
+    , m_numY(0)
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeDrop(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeDrop(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a para
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const final
   {
     o << m_debugName << "=[";
     if (m_numFormats!=0xFFFF) o << "num[formats]=" << m_numFormats << ",";
@@ -419,9 +421,7 @@ public:
   }
 protected:
   //! copy constructor
-  StarPAttributeDrop(StarPAttributeDrop const &orig) : StarAttribute(orig), m_numFormats(orig.m_numFormats), m_numLines(orig.m_numLines), m_numChars(orig.m_numChars), m_numDistances(orig.m_numDistances), m_whole(orig.m_whole), m_numX(orig.m_numX), m_numY(orig.m_numY)
-  {
-  }
+  StarPAttributeDrop(StarPAttributeDrop const &) = default;
   //! the number of format
   int m_numFormats;
   //! the number of lines
@@ -439,24 +439,30 @@ protected:
 };
 
 //! a hyphen attribute
-class StarPAttributeHyphen : public StarAttribute
+class StarPAttributeHyphen final : public StarAttribute
 {
 public:
   //! constructor
-  StarPAttributeHyphen(Type type, std::string const &debugName) : StarAttribute(type, debugName), m_hyphenZone(0), m_pageEnd(0), m_minLead(0), m_minTail(0), m_maxHyphen(0)
+  StarPAttributeHyphen(Type type, std::string const &debugName)
+    : StarAttribute(type, debugName)
+    , m_hyphenZone(0)
+    , m_pageEnd(0)
+    , m_minLead(0)
+    , m_minTail(0)
+    , m_maxHyphen(0)
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeHyphen(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeHyphen(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a para
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const final
   {
     o << m_debugName << "=[";
     if (m_hyphenZone) o << "hyphenZone=" << m_hyphenZone << ",";
@@ -468,9 +474,7 @@ public:
   }
 protected:
   //! copy constructor
-  StarPAttributeHyphen(StarPAttributeHyphen const &orig) : StarAttribute(orig), m_hyphenZone(orig.m_hyphenZone), m_pageEnd(orig.m_pageEnd), m_minLead(orig.m_minLead), m_minTail(orig.m_minTail), m_maxHyphen(orig.m_maxHyphen)
-  {
-  }
+  StarPAttributeHyphen(StarPAttributeHyphen const &) = default;
   //! the hyphen value
   int m_hyphenZone;
   //! the page end flag
@@ -484,24 +488,30 @@ protected:
 };
 
 //! a line spacing attribute
-class StarPAttributeLineSpacing : public StarAttribute
+class StarPAttributeLineSpacing final : public StarAttribute
 {
 public:
   //! constructor
-  StarPAttributeLineSpacing(Type type, std::string const &debugName) : StarAttribute(type, debugName), m_propLineSpace(100), m_interLineSpace(0), m_lineHeight(200), m_lineSpaceRule(0), m_interLineSpaceRule(0)
+  StarPAttributeLineSpacing(Type type, std::string const &debugName)
+    : StarAttribute(type, debugName)
+    , m_propLineSpace(100)
+    , m_interLineSpace(0)
+    , m_lineHeight(200)
+    , m_lineSpaceRule(0)
+    , m_interLineSpaceRule(0)
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeLineSpacing(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeLineSpacing(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a para
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const final
   {
     o << m_debugName << "=[";
     if (m_interLineSpace) o << "lineSpace=" << m_interLineSpace;
@@ -513,9 +523,7 @@ public:
   }
 protected:
   //! copy constructor
-  StarPAttributeLineSpacing(StarPAttributeLineSpacing const &orig) : StarAttribute(orig), m_propLineSpace(orig.m_propLineSpace), m_interLineSpace(orig.m_interLineSpace), m_lineHeight(orig.m_lineHeight), m_lineSpaceRule(orig.m_lineSpaceRule), m_interLineSpaceRule(orig.m_interLineSpaceRule)
-  {
-  }
+  StarPAttributeLineSpacing(StarPAttributeLineSpacing const &) = default;
   //! the prop lineSpacing
   int m_propLineSpace;
   //! the line spacing
@@ -529,24 +537,27 @@ protected:
 };
 
 //! a numRule attribute
-class StarPAttributeNumericRuler : public StarAttribute
+class StarPAttributeNumericRuler final : public StarAttribute
 {
 public:
   //! constructor
-  StarPAttributeNumericRuler(Type type, std::string const &debugName) : StarAttribute(type, debugName), m_name(""), m_poolId(0)
+  StarPAttributeNumericRuler(Type type, std::string const &debugName)
+    : StarAttribute(type, debugName)
+    , m_name("")
+    , m_poolId(0)
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeNumericRuler(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeNumericRuler(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a para
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const final
   {
     o << m_debugName << "=[";
     if (!m_name.empty()) o << m_name.cstr() << ",";
@@ -555,9 +566,7 @@ public:
   }
 protected:
   //! copy constructor
-  StarPAttributeNumericRuler(StarPAttributeNumericRuler const &orig) : StarAttribute(orig), m_name(orig.m_name), m_poolId(orig.m_poolId)
-  {
-  }
+  StarPAttributeNumericRuler(StarPAttributeNumericRuler const &) = default;
   //! the name value
   librevenge::RVNGString m_name;
   //! the poolId
@@ -565,13 +574,17 @@ protected:
 };
 
 //! a tabStop attribute
-class StarPAttributeTabStop : public StarAttribute
+class StarPAttributeTabStop final : public StarAttribute
 {
 public:
   //! a tabs structure
   struct TabStop {
     //! constructor
-    TabStop() : m_pos(0), m_type(0), m_decimal(44), m_fill(32)
+    TabStop()
+      : m_pos(0)
+      , m_type(0)
+      , m_decimal(44)
+      , m_fill(32)
     {
     }
     //! debug function to print the data
@@ -579,7 +592,7 @@ public:
     {
       o << "pos=" << m_pos;
       if (m_type>=0 && m_type<5) {
-        char const *(wh[])= {"L", "R", "D", "C", "Def"};
+        char const *wh[]= {"L", "R", "D", "C", "Def"};
         o << wh[m_type];
       }
       else
@@ -598,31 +611,31 @@ public:
     int m_fill;
   };
   //! constructor
-  StarPAttributeTabStop(Type type, std::string const &debugName) : StarAttribute(type, debugName), m_tabList()
+  StarPAttributeTabStop(Type type, std::string const &debugName)
+    : StarAttribute(type, debugName)
+    , m_tabList()
   {
   }
   //! create a new attribute
-  virtual shared_ptr<StarAttribute> create() const
+  std::shared_ptr<StarAttribute> create() const final
   {
-    return shared_ptr<StarAttribute>(new StarPAttributeTabStop(*this));
+    return std::shared_ptr<StarAttribute>(new StarPAttributeTabStop(*this));
   }
   //! read a zone
-  virtual bool read(StarZone &zone, int vers, long endPos, StarObject &object);
+  bool read(StarZone &zone, int vers, long endPos, StarObject &object) final;
   //! add to a para
-  virtual void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const;
+  void addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const final;
   //! debug function to print the data
-  virtual void printData(libstoff::DebugStream &o) const
+  void printData(libstoff::DebugStream &o) const override
   {
     o << m_debugName << "=[";
-    for (size_t t=0; t<m_tabList.size(); ++t)
-      m_tabList[t].printData(o);
+    for (auto const &t : m_tabList)
+      t.printData(o);
     o << "],";
   }
 protected:
   //! copy constructor
-  StarPAttributeTabStop(StarPAttributeTabStop const &orig) : StarAttribute(orig), m_tabList(orig.m_tabList)
-  {
-  }
+  StarPAttributeTabStop(StarPAttributeTabStop const &) = default;
   //! the tabStop list
   std::vector<TabStop> m_tabList;
 };
@@ -661,7 +674,7 @@ void StarPAttributeBulletNumeric::addTo(StarState &state, std::set<StarAttribute
   if (m_type==ATTR_EE_PARA_NUMBULLET) {
     STOFFListLevel level;
     if (m_numType<=4) {
-      char const *(wh[])= {"A", "a", "I", "i", "1"};
+      char const *wh[]= {"A", "a", "I", "i", "1"};
       level.m_propertyList.insert("style:num-format", wh[m_numType]);
       level.m_type=STOFFListLevel::NUMBER;
     }
@@ -743,8 +756,7 @@ void StarPAttributeNumericRuler::addTo(StarState &state, std::set<StarAttribute
 void StarPAttributeTabStop::addTo(StarState &state, std::set<StarAttribute const *> &/*done*/) const
 {
   librevenge::RVNGPropertyListVector tabs;
-  for (size_t t=0; t<m_tabList.size(); ++t) {
-    StarPAttributeTabStop::TabStop const &tabStop=m_tabList[t];
+  for (auto const &tabStop : m_tabList) {
     librevenge::RVNGPropertyList tab;
     switch (tabStop.m_type) {
     case 0:
@@ -932,7 +944,7 @@ bool StarPAttributeTabStop::read(StarZone &zone, int /*vers*/, long endPos, Star
   libstoff::DebugFile &ascFile=zone.ascii();
   libstoff::DebugStream f;
   f << "Entries(StarAttribute)[" << zone.getRecordLevel() << "]:";
-  int N=int(input->readULong(1));
+  auto N=int(input->readULong(1));
   if (input->tell()+7*N>endPos) {
     STOFF_DEBUG_MSG(("StarPAttributeTabStop::read: N is too big\n"));
     f << "###N=" << N << ",";
@@ -959,7 +971,7 @@ bool StarPAttributeTabStop::read(StarZone &zone, int /*vers*/, long endPos, Star
 
 namespace StarParagraphAttribute
 {
-void addInitTo(std::map<int, shared_ptr<StarAttribute> > &map)
+void addInitTo(std::map<int, std::shared_ptr<StarAttribute> > &map)
 {
   addAttributeBool(map,StarAttribute::ATTR_PARA_SPLIT,"para[split]",true);
   addAttributeUInt(map,StarAttribute::ATTR_PARA_WIDOWS,"para[widows]",1,0); // numlines
@@ -972,12 +984,12 @@ void addInitTo(std::map<int, shared_ptr<StarAttribute> > &map)
   addAttributeUInt(map, StarAttribute::ATTR_EE_PARA_OUTLLEVEL,"para[outlevel]",2,0);
   addAttributeBool(map, StarAttribute::ATTR_EE_PARA_ASIANCJKSPACING,"para[asianCJKSpacing]",false);
 
-  map[StarAttribute::ATTR_PARA_ADJUST]=shared_ptr<StarAttribute>(new StarPAttributeAdjust(StarAttribute::ATTR_PARA_ADJUST,"parAtrAdjust"));
-  map[StarAttribute::ATTR_PARA_LINESPACING]=shared_ptr<StarAttribute>(new StarPAttributeLineSpacing(StarAttribute::ATTR_PARA_LINESPACING,"parAtrLinespacing"));
-  map[StarAttribute::ATTR_PARA_TABSTOP]=shared_ptr<StarAttribute>(new StarPAttributeTabStop(StarAttribute::ATTR_PARA_TABSTOP,"parAtrTabStop"));
-  map[StarAttribute::ATTR_EE_PARA_NUMBULLET]=shared_ptr<StarAttribute>(new StarPAttributeBulletNumeric(StarAttribute::ATTR_EE_PARA_NUMBULLET,"eeParaNumBullet"));
-  map[StarAttribute::ATTR_EE_PARA_BULLET]=shared_ptr<StarAttribute>(new StarPAttributeBulletSimple(StarAttribute::ATTR_EE_PARA_BULLET,"paraBullet"));
-  map[StarAttribute::ATTR_PARA_DROP]=shared_ptr<StarAttribute>(new StarPAttributeDrop(StarAttribute::ATTR_PARA_DROP,"parAtrDrop"));
+  map[StarAttribute::ATTR_PARA_ADJUST]=std::shared_ptr<StarAttribute>(new StarPAttributeAdjust(StarAttribute::ATTR_PARA_ADJUST,"parAtrAdjust"));
+  map[StarAttribute::ATTR_PARA_LINESPACING]=std::shared_ptr<StarAttribute>(new StarPAttributeLineSpacing(StarAttribute::ATTR_PARA_LINESPACING,"parAtrLinespacing"));
+  map[StarAttribute::ATTR_PARA_TABSTOP]=std::shared_ptr<StarAttribute>(new StarPAttributeTabStop(StarAttribute::ATTR_PARA_TABSTOP,"parAtrTabStop"));
+  map[StarAttribute::ATTR_EE_PARA_NUMBULLET]=std::shared_ptr<StarAttribute>(new StarPAttributeBulletNumeric(StarAttribute::ATTR_EE_PARA_NUMBULLET,"eeParaNumBullet"));
+  map[StarAttribute::ATTR_EE_PARA_BULLET]=std::shared_ptr<StarAttribute>(new StarPAttributeBulletSimple(StarAttribute::ATTR_EE_PARA_BULLET,"paraBullet"));
+  map[StarAttribute::ATTR_PARA_DROP]=std::shared_ptr<StarAttribute>(new StarPAttributeDrop(StarAttribute::ATTR_PARA_DROP,"parAtrDrop"));
 
   // seems safe to ignore
   addAttributeBool(map,StarAttribute::ATTR_PARA_SCRIPTSPACE,"para[scriptSpace]",false); // script type, use by ui
@@ -992,8 +1004,8 @@ void addInitTo(std::map<int, shared_ptr<StarAttribute> > &map)
   // can we retrieve the following attribute ?
   addAttributeBool(map,StarAttribute::ATTR_PARA_REGISTER,"para[register]",false);
   addAttributeBool(map,StarAttribute::ATTR_PARA_FORBIDDEN_RULES,"para[forbiddenRules]",true); // If the forbidden characters rules are to be applied or not.
-  map[StarAttribute::ATTR_PARA_HYPHENZONE]=shared_ptr<StarAttribute>(new StarPAttributeHyphen(StarAttribute::ATTR_PARA_HYPHENZONE,"parAtrHyphenZone"));
-  map[StarAttribute::ATTR_PARA_NUMRULE]=shared_ptr<StarAttribute>(new StarPAttributeNumericRuler(StarAttribute::ATTR_PARA_NUMRULE,"parAtrNumRule"));
+  map[StarAttribute::ATTR_PARA_HYPHENZONE]=std::shared_ptr<StarAttribute>(new StarPAttributeHyphen(StarAttribute::ATTR_PARA_HYPHENZONE,"parAtrHyphenZone"));
+  map[StarAttribute::ATTR_PARA_NUMRULE]=std::shared_ptr<StarAttribute>(new StarPAttributeNumericRuler(StarAttribute::ATTR_PARA_NUMRULE,"parAtrNumRule"));
 
   // TODO
 }
