@@ -311,12 +311,16 @@ bool SuperPaintParser::createZones()
   readHeader();
   input->seek(512,librevenge::RVNG_SEEK_SET);
   bool ok=true;
-  if (m_state->m_kind==MWAWDocument::MWAW_K_DRAW)
-    ok = readPictures();
-  else
-    ok = readBitmap();
-
   long pos=input->tell();
+  if (m_state->m_kind==MWAWDocument::MWAW_K_DRAW) {
+    ok = readPictures();
+  }
+  else if (readBitmap(true)) {
+    input->seek(pos, librevenge::RVNG_SEEK_SET);
+    ok = readBitmap();
+  }
+
+  pos=input->tell();
   if (input->size()==pos+2 && input->readLong(2)==0) {
     ascii().addPos(pos);
     ascii().addNote("_");
@@ -922,7 +926,6 @@ bool SuperPaintParser::readBitmap(bool onlyCheck)
     colors[1]=MWAWColor::black();
     pict->setColors(colors);
   }
-
   while (!input->isEnd()) {
     pos=input->tell();
     f.str("");
@@ -953,6 +956,7 @@ bool SuperPaintParser::readBitmap(bool onlyCheck)
     if (val!=1) f << "f1=" << val << ",";
     ascii().addPos(pos);
     ascii().addNote(f.str().c_str());
+
     for (int r=dim[0]; r<dim[2]; ++r) {
       pos=input->tell();
       auto len=static_cast<int>(input->readULong(1));
@@ -964,7 +968,7 @@ bool SuperPaintParser::readBitmap(bool onlyCheck)
       f.str("");
       f << "Bitmap-R" << r << ":";
       int col=dim[1];
-      while (input->tell()<rEndPos) {
+      while (input->tell()<rEndPos) { // UnpackBits
         auto wh=static_cast<int>(input->readULong(1));
         if (wh>=0x81) {
           auto color=static_cast<int>(input->readULong(1));
@@ -981,7 +985,7 @@ bool SuperPaintParser::readBitmap(bool onlyCheck)
             }
           }
         }
-        else {
+        else { // checkme normally 0x80 is reserved and almost nobody used it (for ending the compression)
           if (input->tell()+wh+1>rEndPos) {
             MWAW_DEBUG_MSG(("SuperPaintParser::readBitmap: can not read row %d\n", r));
             f << "###";
