@@ -1,27 +1,23 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * Signatures implementation for Microsoft Crypto API.
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2003 Cordys R&D BV, All rights reserved.
  * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2003 Cordys R&D BV, All rights reserved.
  * Copyright (c) 2005-2006 Cryptocom LTD (http://www.cryptocom.ru).
  */
 /**
- * SECTION:signatures
- * @Short_description: Signatures implementation for Microsoft Crypto API.
- * @Stability: Private
- *
+ * SECTION:crypto
  */
 
 #include "globals.h"
 
 #include <string.h>
 
-#include <windows.h>
-#include <wincrypt.h>
 #ifndef XMLSEC_NO_GOST
 #include "csp_calg.h"
 #endif
@@ -767,7 +763,7 @@ xmlSecMSCryptoTransformRsaSha1GetKlass(void) {
 #ifndef XMLSEC_NO_SHA256
 /****************************************************************************
  *
- * RSA-SHA256 signature transform
+ * RSA-SHA2-256 signature transform
  *
  ***************************************************************************/
 static xmlSecTransformKlass xmlSecMSCryptoRsaSha256Klass = {
@@ -800,9 +796,9 @@ static xmlSecTransformKlass xmlSecMSCryptoRsaSha256Klass = {
 /**
  * xmlSecMSCryptoTransformRsaSha256GetKlass:
  *
- * The RSA-SHA256 signature transform klass.
+ * The RSA-SHA2-256 signature transform klass.
  *
- * Returns: RSA-SHA256 signature transform klass.
+ * Returns: RSA-SHA2-256 signature transform klass.
  */
 xmlSecTransformId
 xmlSecMSCryptoTransformRsaSha256GetKlass(void) {
@@ -813,7 +809,7 @@ xmlSecMSCryptoTransformRsaSha256GetKlass(void) {
 #ifndef XMLSEC_NO_SHA384
 /****************************************************************************
  *
- * RSA-SHA384 signature transform
+ * RSA-SHA2-384 signature transform
  *
  ***************************************************************************/
 static xmlSecTransformKlass xmlSecMSCryptoRsaSha384Klass = {
@@ -846,9 +842,9 @@ static xmlSecTransformKlass xmlSecMSCryptoRsaSha384Klass = {
 /**
  * xmlSecMSCryptoTransformRsaSha384GetKlass:
  *
- * The RSA-SHA384 signature transform klass.
+ * The RSA-SHA2-384 signature transform klass.
  *
- * Returns: RSA-SHA384 signature transform klass.
+ * Returns: RSA-SHA2-384 signature transform klass.
  */
 xmlSecTransformId
 xmlSecMSCryptoTransformRsaSha384GetKlass(void) {
@@ -892,9 +888,9 @@ static xmlSecTransformKlass xmlSecMSCryptoRsaSha512Klass = {
 /**
  * xmlSecMSCryptoTransformRsaSha512GetKlass:
  *
- * The RSA-SHA512 signature transform klass.
+ * The RSA-SHA2-512 signature transform klass.
  *
- * Returns: RSA-SHA512 signature transform klass.
+ * Returns: RSA-SHA2-512 signature transform klass.
  */
 xmlSecTransformId
 xmlSecMSCryptoTransformRsaSha512GetKlass(void) {
@@ -1037,7 +1033,7 @@ static xmlSecTransformKlass xmlSecMSCryptoGost2012_256Klass = {
 };
 
 /**
- * xmlSecMSCryptoTransformGost2012GostR3411_94GetKlass:
+ * xmlSecMSCryptoTransformGost2012_256GetKlass:
  *
  * The GOST R 34.10-2012 signature transform klass.
  *
@@ -1082,7 +1078,7 @@ static xmlSecTransformKlass xmlSecMSCryptoGost2012_512Klass = {
 };
 
 /**
- * xmlSecMSCryptoTransformGost2012GostR3411_94GetKlass:
+ * xmlSecMSCryptoTransformGost2012_512GetKlass:
  *
  * The GOST R 34.10-2012 signature transform klass.
  *
@@ -1094,4 +1090,3 @@ xmlSecMSCryptoTransformGost2012_512GetKlass(void) {
 }
 
 #endif /* XMLSEC_NO_GOST2012*/
-
