@@ -1,21 +1,32 @@
 /*
  * XML Security Library (http://www.aleksey.com/xmlsec).
  *
+ * Application support functions for NSS.
+ *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  * Copyright (c) 2003 America Online, Inc.  All rights reserved.
  */
+/**
+ * SECTION:app
+ * @Short_description: Application support functions for NSS.
+ * @Stability: Stable
+ *
+ * Common functions for xmlsec1 command line utility tool for NSS.
+ */
+
 #include "globals.h"
 
 #include <string.h>
 
 #include <nspr.h>
 #include <nss.h>
-#include <pk11func.h>
 #include <cert.h>
+#include <certdb.h>
 #include <keyhi.h>
+#include <pk11func.h>
 #include <pkcs12.h>
 #include <p12plcy.h>
 /*
@@ -26,6 +37,7 @@
 #include <xmlsec/keys.h>
 #include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
+#include <xmlsec/private.h>
 
 #include <xmlsec/nss/app.h>
 #include <xmlsec/nss/crypto.h>
@@ -33,12 +45,9 @@
 #include <xmlsec/nss/pkikeys.h>
 #include <xmlsec/nss/keysstore.h>
 
-/* workaround - NSS exports this but doesn't declare it */
-extern CERTCertificate * __CERT_NewTempCertificate              (CERTCertDBHandle *handle,
-                                                                 SECItem *derCert,
-                                                                 char *nickname,
-                                                                 PRBool isperm,
-                                                                 PRBool copyDER);
+#include "../cast_helpers.h"
+#include "private.h"
+
 static int xmlSecNssAppCreateSECItem                            (SECItem *contents,
                                                                  const xmlSecByte* data,
                                                                  xmlSecSize dataSize);
@@ -51,10 +60,14 @@ static PRBool xmlSecNssAppAscii2UCS2Conv                        (PRBool toUnicod
                                                                  unsigned int   maxOutBufLen,
                                                                  unsigned int  *outBufLen,
                                                                  PRBool         swapBytes);
+static xmlSecKeyPtr     xmlSecNssAppDerKeyLoadSECItem           (SECItem* secItem);
+
+
+#ifndef XMLSEC_NO_X509
 static SECItem *xmlSecNssAppNicknameCollisionCallback           (SECItem *old_nick,
                                                                  PRBool *cancel,
                                                                  void *wincx);
-static xmlSecKeyPtr     xmlSecNssAppDerKeyLoadSECItem           (SECItem* secItem);
+#endif /* XMLSEC_NO_X509 */
 
 /**
  * xmlSecNssAppInit:
@@ -133,18 +146,21 @@ xmlSecNssAppShutdown(void) {
 
 static int
 xmlSecNssAppCreateSECItem(SECItem *contents, const xmlSecByte* data, xmlSecSize dataSize) {
+    unsigned int dataLen;
+
     xmlSecAssert2(contents != NULL, -1);
     xmlSecAssert2(data != NULL, -1);
 
     contents->data = 0;
-    if (!SECITEM_AllocItem(NULL, contents, dataSize)) {
+    XMLSEC_SAFE_CAST_SIZE_TO_UINT(dataSize, dataLen, return(-1), NULL);
+    if (!SECITEM_AllocItem(NULL, contents, dataLen)) {
         xmlSecNssError("SECITEM_AllocItem", NULL);
         return(-1);
     }
 
-    if(dataSize > 0) {
+    if(dataLen > 0) {
         xmlSecAssert2(contents->data != NULL, -1);
-        memcpy(contents->data,  data, dataSize);
+        memcpy(contents->data, data, dataLen);
     }
 
     return (0);
@@ -156,6 +172,7 @@ xmlSecNssAppReadSECItem(SECItem *contents, const char *fn) {
     PRFileDesc *file = NULL;
     PRInt32 numBytes;
     PRStatus prStatus;
+    unsigned int ulen;
     int ret = -1;
 
     xmlSecAssert2(contents != NULL, -1);
@@ -174,9 +191,10 @@ xmlSecNssAppReadSECItem(SECItem *contents, const char *fn) {
                         "filename=%s", xmlSecErrorsSafeString(fn));
         goto done;
     }
+    XMLSEC_SAFE_CAST_INT_TO_UINT(info.size, ulen, goto done, NULL);
 
     contents->data = 0;
-    if (!SECITEM_AllocItem(NULL, contents, info.size)) {
+    if (!SECITEM_AllocItem(NULL, contents, ulen)) {
         xmlSecNssError("SECITEM_AllocItem", NULL);
         goto done;
     }
@@ -219,19 +237,40 @@ xmlSecNssAppAscii2UCS2Conv(PRBool toUnicode,
                                     outBuf, maxOutBufLen, outBufLen));
 }
 
+#ifndef XMLSEC_NO_X509
+/* rename certificate if needed */
 static SECItem *
 xmlSecNssAppNicknameCollisionCallback(SECItem *old_nick ATTRIBUTE_UNUSED,
-                                      PRBool *cancel,
-                                      void *wincx ATTRIBUTE_UNUSED)
-{
-    if (cancel == NULL) {
-        return (NULL);
+    PRBool *cancel, void *wincx ATTRIBUTE_UNUSED
+) {
+    CERTCertificate *cert = (CERTCertificate *)wincx;
+    char *nick = NULL;
+    SECItem *ret_nick = NULL;
+
+    if((cancel  == NULL) || (cert == NULL)) {
+        xmlSecNssError("cert is missing", NULL);
+        return(NULL);
     }
 
-    /* XXX not handled yet  */
-    *cancel = PR_TRUE;
-    return (NULL);
+    nick = CERT_MakeCANickname(cert);
+    if (!nick) {
+        xmlSecNssError("CERT_MakeCANickname", NULL);
+        return(NULL);
+    }
+
+    ret_nick = PORT_ZNew(SECItem);
+    if (ret_nick == NULL) {
+        xmlSecNssError("PORT_ZNew", NULL);
+        PORT_Free(nick);
+        return NULL;
+    }
+
+    /* done */
+    ret_nick->data = (unsigned char *)nick;
+    ret_nick->len = (unsigned int)PORT_Strlen(nick);
+    return ret_nick;
 }
+#endif /* XMLSEC_NO_X509 */
 
 /**
  * xmlSecNssAppKeyLoad:
@@ -241,19 +280,41 @@ xmlSecNssAppNicknameCollisionCallback(SECItem *old_nick ATTRIBUTE_UNUSED,
  * @pwdCallback:        the key password callback.
  * @pwdCallbackCtx:     the user context for password callback.
  *
+ * Deprecated, use @xmlSecNssAppKeyLoadEx instead. Reads key from a file
+ *
+ * Returns: pointer to the key or NULL if an error occurs.
+ */
+xmlSecKeyPtr
+xmlSecNssAppKeyLoad(const char *filename, xmlSecKeyDataFormat format,
+    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
+) {
+    return(xmlSecNssAppKeyLoadEx(filename, xmlSecKeyDataTypeUnknown, format, pwd, pwdCallback, pwdCallbackCtx));
+}
+
+/**
+ * xmlSecNssAppKeyLoadEx:
+ * @filename:           the key filename.
+ * @type:               the key type (public / private).
+ * @format:             the key file format.
+ * @pwd:                the key file password.
+ * @pwdCallback:        the key password callback.
+ * @pwdCallbackCtx:     the user context for password callback.
+ *
  * Reads key from a file
  *
  * Returns: pointer to the key or NULL if an error occurs.
  */
 xmlSecKeyPtr
-xmlSecNssAppKeyLoad(const char *filename, xmlSecKeyDataFormat format,
-                    const char *pwd, void* pwdCallback, void* pwdCallbackCtx) {
+xmlSecNssAppKeyLoadEx(const char *filename, xmlSecKeyDataType type ATTRIBUTE_UNUSED, xmlSecKeyDataFormat format,
+    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
+) {
     SECItem secItem;
     xmlSecKeyPtr res;
     int ret;
 
     xmlSecAssert2(filename != NULL, NULL);
     xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, NULL);
+    UNREFERENCED_PARAMETER(type);
 
     /* read the file contents */
     memset(&secItem, 0, sizeof(secItem));
@@ -329,9 +390,8 @@ xmlSecNssAppKeyLoadMemory(const xmlSecByte* data, xmlSecSize dataSize, xmlSecKey
  */
 xmlSecKeyPtr
 xmlSecNssAppKeyLoadSECItem(SECItem* secItem, xmlSecKeyDataFormat format,
-                    const char *pwd,
-                    void* pwdCallback,
-                    void* pwdCallbackCtx) {
+    const char *pwd, void* pwdCallback, void* pwdCallbackCtx
+) {
     xmlSecKeyPtr key = NULL;
 
     xmlSecAssert2(secItem != NULL, NULL);
@@ -347,6 +407,7 @@ xmlSecNssAppKeyLoadSECItem(SECItem* secItem, xmlSecKeyDataFormat format,
         }
         break;
     case xmlSecKeyDataFormatCertDer:
+    case xmlSecKeyDataFormatCertPem:
         key = xmlSecNssAppKeyFromCertLoadSECItem(secItem, format);
         if(key == NULL) {
             xmlSecInternalError("xmlSecNssAppKeyFromCertLoadSECItem", NULL);
@@ -363,7 +424,7 @@ xmlSecNssAppKeyLoadSECItem(SECItem* secItem, xmlSecKeyDataFormat format,
         break;
     default:
         xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
-                         "format=%d", (int)format);
+            "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
         return(NULL);
     }
 
@@ -400,7 +461,7 @@ xmlSecNssAppDerKeyLoadSECItem(SECItem* secItem) {
 
     /* TRY PRIVATE KEY FIRST
      * Note: This expects the key to be in PrivateKeyInfo format. The
-     * DER files created from PEM via openssl utilities aren't in that
+     * DER files created from PEM via nss utilities aren't in that
      * format
      */
     status = PK11_ImportDERPrivateKeyInfoAndReturnKey(slot, secItem,
@@ -411,6 +472,7 @@ xmlSecNssAppDerKeyLoadSECItem(SECItem* secItem) {
         spki = SECKEY_DecodeDERSubjectPublicKeyInfo(secItem);
         if (spki == NULL) {
             xmlSecNssError("SECKEY_DecodeDERSubjectPublicKeyInfo", NULL);
+            goto done;
         }
 
         pubkey = SECKEY_ExtractPublicKey(spki);
@@ -467,6 +529,73 @@ done:
     return (retval);
 }
 
+/* returns 1 if matches, 0 if not, or a negative value on error */
+static int
+xmlSecNssAppCheckCertMatchesKey(xmlSecKeyPtr key,  CERTCertificate * cert) {
+    xmlSecKeyDataPtr keyData = NULL;
+    SECKEYPublicKey* pubkey = NULL;
+    SECKEYPublicKey* cert_pubkey = NULL;
+    SECItem * der_pubkey = NULL;
+    SECItem * der_cert_pubkey = NULL;
+    int res = -1;
+
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(cert != NULL, -1);
+
+    /* get key's pubkey and its der encoding */
+    keyData = xmlSecKeyGetValue(key);
+    if(keyData == NULL) {
+        res = 0; /* no key -> no match */
+        goto done;
+    }
+    pubkey = xmlSecNssPKIKeyDataGetPubKey(keyData);
+    if(pubkey == NULL) {
+        xmlSecInternalError("xmlSecNssPKIKeyDataGetPubKey", NULL);
+        goto done;
+    }
+    der_pubkey = SECKEY_EncodeDERSubjectPublicKeyInfo(pubkey);
+    if (der_pubkey == NULL) {
+        xmlSecNssError("SECKEY_EncodeDERSubjectPublicKeyInfo", NULL);
+        goto done;
+    }
+
+    /* get certs's pubkey and its der encoding */
+    cert_pubkey = CERT_ExtractPublicKey(cert);
+    if (cert_pubkey == NULL) {
+        xmlSecNssError("CERT_ExtractPublicKey", NULL);
+        goto done;
+    }
+    der_cert_pubkey = SECKEY_EncodeDERSubjectPublicKeyInfo(cert_pubkey);
+    if (der_cert_pubkey == NULL) {
+        xmlSecNssError("SECKEY_EncodeDERSubjectPublicKeyInfo", NULL);
+        goto done;
+    }
+
+    /* compare */
+    if(SECEqual == SECITEM_CompareItem(der_pubkey, der_cert_pubkey)) {
+        /* match */
+        res = 1;
+    } else {
+        /* no match */
+        res = 0;
+    }
+
+done:
+    if(pubkey != NULL) {
+        SECKEY_DestroyPublicKey(pubkey);
+    }
+    if (cert_pubkey) {
+        SECKEY_DestroyPublicKey(cert_pubkey);
+    }
+    if(der_pubkey != NULL) {
+        SECITEM_FreeItem(der_pubkey, PR_TRUE);
+    }
+    if(der_cert_pubkey != NULL) {
+        SECITEM_FreeItem(der_cert_pubkey, PR_TRUE);
+    }
+    return(res);
+}
+
 #ifndef XMLSEC_NO_X509
 /**
  * xmlSecNssAppKeyCertLoad:
@@ -557,47 +686,86 @@ xmlSecNssAppKeyCertLoadMemory(xmlSecKeyPtr key, const xmlSecByte* data, xmlSecSi
  */
 int
 xmlSecNssAppKeyCertLoadSECItem(xmlSecKeyPtr key, SECItem* secItem, xmlSecKeyDataFormat format) {
-    CERTCertificate *cert=NULL;
-    xmlSecKeyDataPtr data;
+    CERTCertificate *cert = NULL;
+    xmlSecKeyDataPtr x509Data;
+    int isKeyCert = 0;
     int ret;
+    int res = -1;
 
     xmlSecAssert2(key != NULL, -1);
     xmlSecAssert2(secItem != NULL, -1);
+    xmlSecAssert2(secItem->type == siBuffer, -1);
+    xmlSecAssert2(secItem->data != NULL, -1);
+    xmlSecAssert2(secItem->len > 0, -1);
     xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, -1);
 
-    data = xmlSecKeyEnsureData(key, xmlSecNssKeyDataX509Id);
-    if(data == NULL) {
-        xmlSecInternalError("xmlSecKeyEnsureData(xmlSecNssKeyDataX509Id)", NULL);
-        return(-1);
-    }
-
+    /* read cert */
     switch(format) {
     case xmlSecKeyDataFormatPkcs8Der:
     case xmlSecKeyDataFormatDer:
-        cert = __CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
-                                         secItem, NULL, PR_FALSE, PR_TRUE);
+        cert = xmlSecNssX509CertDerRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
         if(cert == NULL) {
-            xmlSecNssError2("__CERT_NewTempCertificate", NULL,
-                            "format=%d", (int)format);
-            return(-1);
+            xmlSecInternalError2("xmlSecNssX509CertDerRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            goto done;
+        }
+        break;
+    case xmlSecKeyDataFormatCertPem:
+        cert = xmlSecNssX509CertPemRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
+        if(cert == NULL) {
+            xmlSecInternalError2("xmlSecNssX509CertPemRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            goto done;
         }
         break;
     default:
         xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
-                         "format=%d", (int)format);
-        return(-1);
+            "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+        goto done;
     }
-
     xmlSecAssert2(cert != NULL, -1);
-    ret = xmlSecNssKeyDataX509AdoptCert(data, cert);
-    if(ret < 0) {
-        xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert",
-                            xmlSecKeyDataGetName(data));
-        CERT_DestroyCertificate(cert);
-        return(-1);
+
+    /* add cert to the key */
+    x509Data = xmlSecKeyEnsureData(key, xmlSecNssKeyDataX509Id);
+    if(x509Data == NULL) {
+        xmlSecInternalError("xmlSecKeyEnsureData(xmlSecNssKeyDataX509Id)", NULL);
+        goto done;
     }
 
-    return(0);
+    /* do we want to add this cert as a key cert? */
+    if(xmlSecNssKeyDataX509GetKeyCert(x509Data) == NULL) {
+        ret = xmlSecNssAppCheckCertMatchesKey(key, cert);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecNssAppCheckCertMatchesKey", NULL);
+            goto done;
+        }
+        if(ret == 1) {
+            isKeyCert = 1;
+        }
+    }
+    if(isKeyCert != 0) {
+        ret = xmlSecNssKeyDataX509AdoptKeyCert(x509Data, cert);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert", NULL);
+            goto done;
+        }
+    } else {
+        ret = xmlSecNssKeyDataX509AdoptCert(x509Data, cert);
+        if(ret < 0) {
+            xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert", NULL);
+            goto done;
+        }
+    }
+    cert = NULL; /* owned by x509Data now */
+
+    /* success */
+    res = 0;
+
+done:
+    if(cert != NULL) {
+        CERT_DestroyCertificate(cert);
+    }
+    return(res);
 }
 
 /**
@@ -608,7 +776,7 @@ xmlSecNssAppKeyCertLoadSECItem(xmlSecKeyPtr key, SECItem* secItem, xmlSecKeyData
  * @pwdCallbackCtx:     the user context for password callback.
  *
  * Reads key and all associated certificates from the PKCS12 file.
- * For uniformity, call xmlSecNssAppKeyLoad instead of this function. Pass
+ * For uniformity, call @xmlSecNssAppKeyLoadEx instead of this function. Pass
  * in format=xmlSecKeyDataFormatPkcs12.
  *
  * Returns: pointer to the key or NULL if an error occurs.
@@ -651,7 +819,7 @@ xmlSecNssAppPkcs12Load(const char *filename, const char *pwd,
  * @pwdCallbackCtx:     the user context for password callback.
  *
  * Reads key and all associated certificates from the PKCS12 binary data.
- * For uniformity, call xmlSecNssAppKeyLoad instead of this function. Pass
+ * For uniformity, call @xmlSecNssAppKeyLoadEx instead of this function. Pass
  * in format=xmlSecKeyDataFormatPkcs12.
  *
  * Returns: pointer to the key or NULL if an error occurs.
@@ -693,7 +861,7 @@ xmlSecNssAppPkcs12LoadMemory(const xmlSecByte* data, xmlSecSize dataSize, const
  * @pwdCallbackCtx:     the user context for password callback.
  *
  * Reads key and all associated certificates from the PKCS12 SECItem.
- * For uniformity, call xmlSecNssAppKeyLoad instead of this function. Pass
+ * For uniformity, call @xmlSecNssAppKeyLoadEx instead of this function. Pass
  * in format=xmlSecKeyDataFormatPkcs12.
  *
  * Returns: pointer to the key or NULL if an error occurs.
@@ -703,7 +871,7 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
                        void *pwdCallback ATTRIBUTE_UNUSED,
                        void* pwdCallbackCtx ATTRIBUTE_UNUSED) {
     xmlSecKeyPtr key = NULL;
-    xmlSecKeyDataPtr data = NULL;
+    xmlSecKeyDataPtr keyValueData = NULL;
     xmlSecKeyDataPtr x509Data = NULL;
     int ret;
     PK11SlotInfo *slot = NULL;
@@ -717,7 +885,9 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
     CERTCertificate     *cert = NULL;
     CERTCertificate     *tmpcert = NULL;
     SEC_PKCS12DecoderContext *p12ctx = NULL;
-
+    const SEC_PKCS12DecoderItem *dip;
+    size_t pwdSize;
+    xmlSecKeyPtr res = NULL;
 
     xmlSecAssert2((secItem != NULL), NULL);
 
@@ -736,7 +906,9 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
     }
 
     pwditem.data = (unsigned char *)pwd;
-    pwditem.len = strlen(pwd)+1;
+    pwdSize = strlen(pwd) + 1;
+    XMLSEC_SAFE_CAST_SIZE_T_TO_UINT(pwdSize, pwditem.len, goto done, NULL);
+
     if (!SECITEM_AllocItem(NULL, &uc2_pwditem, 2*pwditem.len)) {
         xmlSecNssError("SECITEM_AllocItem", NULL);
         goto done;
@@ -788,8 +960,7 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
 
     x509Data = xmlSecKeyDataCreate(xmlSecNssKeyDataX509Id);
     if(x509Data == NULL) {
-        xmlSecInternalError("xmlSecKeyDataCreate",
-                            xmlSecTransformKlassGetName(xmlSecNssKeyDataX509Id));
+        xmlSecInternalError("xmlSecKeyDataCreate(xmlSecNssKeyDataX509Id)", NULL);
         goto done;
     }
 
@@ -798,96 +969,112 @@ xmlSecNssAppPkcs12LoadSECItem(SECItem* secItem, const char *pwd,
         privkey = PK11_FindKeyByAnyCert(cert, NULL);
 
         if (privkey != NULL) {
-            if (data != NULL) {
+            if (keyValueData != NULL) {
                 /* we already found a private key.
                  * assume the first private key we find is THE ONE
                  */
                 SECKEY_DestroyPrivateKey(privkey);
                 privkey = NULL;
-            } else {
-                pubkey = CERT_ExtractPublicKey(cert);
-                if (pubkey == NULL) {
-                    xmlSecNssError("CERT_ExtractPublicKey",
-                                   xmlSecKeyDataGetName(x509Data));
-                    goto done;
-                }
-                data = xmlSecNssPKIAdoptKey(privkey, pubkey);
-                if(data == NULL) {
-                    xmlSecInternalError("xmlSecNssPKIAdoptKey",
-                                        xmlSecKeyDataGetName(x509Data));
-                    goto done;
-                }
-
-                pubkey = NULL;
-                privkey = NULL;
-
-                tmpcert = CERT_DupCertificate(cert);
-                if(tmpcert == NULL) {
-                    xmlSecNssError("CERT_DupCertificate",
-                                   xmlSecKeyDataGetName(x509Data));
-                    goto done;
-                }
-
-                ret = xmlSecNssKeyDataX509AdoptKeyCert(x509Data, tmpcert);
-                if(ret < 0) {
-                    xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert",
-                                        xmlSecKeyDataGetName(x509Data));
-                    CERT_DestroyCertificate(tmpcert);
-                    goto done;
-                }
-
+                continue;
             }
-        }
 
-        tmpcert = CERT_DupCertificate(cert);
-        if(tmpcert == NULL) {
-            xmlSecNssError("CERT_DupCertificate",
-                           xmlSecKeyDataGetName(x509Data));
-            goto done;
-        }
-        ret = xmlSecNssKeyDataX509AdoptCert(x509Data, tmpcert);
-        if(ret < 0) {
-            xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert",
-                                xmlSecKeyDataGetName(x509Data));
-            CERT_DestroyCertificate(tmpcert);
-            goto done;
-        }
+            pubkey = CERT_ExtractPublicKey(cert);
+            if (pubkey == NULL) {
+                xmlSecNssError("CERT_ExtractPublicKey", NULL);
+                goto done;
+            }
+            keyValueData = xmlSecNssPKIAdoptKey(privkey, pubkey);
+            if(keyValueData == NULL) {
+                xmlSecInternalError("xmlSecNssPKIAdoptKey", NULL);
+                goto done;
+            }
 
+            pubkey = NULL;
+            privkey = NULL;
+
+            tmpcert = CERT_DupCertificate(cert);
+            if(tmpcert == NULL) {
+                xmlSecNssError("CERT_DupCertificate", NULL);
+                goto done;
+            }
+
+            ret = xmlSecNssKeyDataX509AdoptKeyCert(x509Data, tmpcert);
+            if(ret < 0) {
+                xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert", NULL);
+                CERT_DestroyCertificate(tmpcert);
+                goto done;
+            }
+            tmpcert = NULL; /* owned by x509Data now */
+        } else {
+            tmpcert = CERT_DupCertificate(cert);
+            if(tmpcert == NULL) {
+                xmlSecNssError("CERT_DupCertificate", NULL);
+                goto done;
+            }
+            ret = xmlSecNssKeyDataX509AdoptCert(x509Data, tmpcert);
+            if(ret < 0) {
+                xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert", NULL);
+                CERT_DestroyCertificate(tmpcert);
+                goto done;
+            }
+            tmpcert = NULL; /* owned by x509Data now */
+        }
     } /* end for loop */
 
-    if (data == NULL) {
+    if (keyValueData == NULL) {
         /* private key not found in PKCS12 file */
         xmlSecInternalError("xmlSecNssAppPkcs12Load(private key)", NULL);
         goto done;
      }
 
+    /* create key and set key value and x509 data into it */
     key = xmlSecKeyCreate();
     if(key == NULL) {
         xmlSecInternalError("xmlSecKeyCreate", NULL);
         goto done;
     }
-
-    ret = xmlSecKeySetValue(key, data);
+    ret = xmlSecKeySetValue(key, keyValueData);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecKeySetValue",
-                            xmlSecKeyDataGetName(x509Data));
-        xmlSecKeyDestroy(key);
-        key = NULL;
+        xmlSecInternalError("xmlSecKeySetValue", NULL);
         goto done;
     }
-    data = NULL;
+    keyValueData = NULL; /* owned by key now */
 
     ret = xmlSecKeyAdoptData(key, x509Data);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecKeyAdoptData",
-                            xmlSecKeyDataGetName(x509Data));
-        xmlSecKeyDestroy(key);
-        key = NULL;
+        xmlSecInternalError("xmlSecKeyAdoptData", NULL);
         goto done;
     }
-    x509Data = NULL;
+    x509Data = NULL; /* owned by key now */
+
+    /* try to find key name */
+    rv = SEC_PKCS12DecoderIterateInit(p12ctx);
+    if (rv != SECSuccess) {
+        xmlSecNssError("SEC_PKCS12DecoderIterateInit", NULL);
+        goto done;
+    }
+    /* read pkcs12 bags */
+    while (SEC_PKCS12DecoderIterateNext(p12ctx, &dip) == SECSuccess) {
+         if((dip->friendlyName != NULL) && (dip->friendlyName->data != NULL) && (dip->friendlyName->len > 0) ) {
+            ret = xmlSecKeySetNameEx(key, dip->friendlyName->data, dip->friendlyName->len);
+            if(ret < 0) {
+                xmlSecInternalError("xmlSecKeySetNameEx", NULL);
+                goto done;
+            }
+            /* use the first one */
+            break;
+         }
+    }
+
+
+    /* success */
+    res = key;
+    key = NULL;
 
 done:
+    if(key != NULL) {
+        xmlSecKeyDestroy(key);
+    }
     if (p12ctx) {
         SEC_PKCS12DecoderFinish(p12ctx);
     }
@@ -901,8 +1088,8 @@ done:
     if(x509Data != NULL) {
         xmlSecKeyDataDestroy(x509Data);
     }
-    if(data != NULL) {
-        xmlSecKeyDataDestroy(data);
+    if(keyValueData != NULL) {
+        xmlSecKeyDataDestroy(keyValueData);
     }
     if (privkey) {
         SECKEY_DestroyPrivateKey(privkey);
@@ -911,7 +1098,7 @@ done:
         SECKEY_DestroyPublicKey(pubkey);
     }
 
-    return(key);
+    return(res);
 }
 
 /**
@@ -925,81 +1112,93 @@ done:
  */
 xmlSecKeyPtr
 xmlSecNssAppKeyFromCertLoadSECItem(SECItem* secItem, xmlSecKeyDataFormat format) {
-    xmlSecKeyPtr key;
-    xmlSecKeyDataPtr keyData;
+    xmlSecKeyPtr key = NULL;
+    xmlSecKeyDataPtr keyData = NULL;
     xmlSecKeyDataPtr certData;
-    CERTCertificate *cert=NULL;
+    CERTCertificate *cert = NULL;
     int ret;
+    xmlSecKeyPtr res = NULL;
 
     xmlSecAssert2(secItem != NULL, NULL);
+    xmlSecAssert2(secItem->type == siBuffer, NULL);
+    xmlSecAssert2(secItem->data != NULL, NULL);
+    xmlSecAssert2(secItem->len > 0, NULL);
     xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, NULL);
 
     /* load cert */
     switch(format) {
     case xmlSecKeyDataFormatCertDer:
-        cert = __CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
-                                         secItem, NULL, PR_FALSE, PR_TRUE);
+        cert = xmlSecNssX509CertDerRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
         if(cert == NULL) {
-            xmlSecNssError2("__CERT_NewTempCertificate", NULL,
-                            "format=%d", (int)format);
-            return(NULL);
+            xmlSecInternalError2("xmlSecNssX509CertDerRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            goto done;
+        }
+        break;
+    case xmlSecKeyDataFormatCertPem:
+        cert = xmlSecNssX509CertPemRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
+        if(cert == NULL) {
+            xmlSecInternalError2("xmlSecNssX509CertPemRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            goto done;
         }
         break;
     default:
         xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
-                         "format=%d", (int)format);
-        return(NULL);
+            "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+        goto done;
     }
 
     /* get key value */
     keyData = xmlSecNssX509CertGetKey(cert);
     if(keyData == NULL) {
         xmlSecInternalError("xmlSecNssX509CertGetKey", NULL);
-        CERT_DestroyCertificate(cert);
-        return(NULL);
+        goto done;
     }
 
-    /* create key */
+    /* create key set key value */
     key = xmlSecKeyCreate();
     if(key == NULL) {
         xmlSecInternalError("xmlSecKeyCreate", NULL);
-        xmlSecKeyDataDestroy(keyData);
-        CERT_DestroyCertificate(cert);
-        return(NULL);
+        goto done;
     }
-
-    /* set key value */
     ret = xmlSecKeySetValue(key, keyData);
     if(ret < 0) {
         xmlSecInternalError("xmlSecKeySetValue", NULL);
-        xmlSecKeyDestroy(key);
-        xmlSecKeyDataDestroy(keyData);
-        CERT_DestroyCertificate(cert);
-        return(NULL);
+        goto done;
     }
+    keyData = NULL; /* owned by key now */
 
-    /* create cert data */
+    /* create cert data put key's cert into it */
     certData = xmlSecKeyEnsureData(key, xmlSecNssKeyDataX509Id);
     if(certData == NULL) {
         xmlSecInternalError("xmlSecKeyEnsureData", NULL);
-        xmlSecKeyDestroy(key);
-        CERT_DestroyCertificate(cert);
-        return(NULL);
+        goto done;
     }
-
-    /* put cert in the cert data */
-    ret = xmlSecNssKeyDataX509AdoptCert(certData, cert);
+    ret = xmlSecNssKeyDataX509AdoptKeyCert(certData, cert);
     if(ret < 0) {
-        xmlSecInternalError("xmlSecNssKeyDataX509AdoptCert", NULL);
-        xmlSecKeyDestroy(key);
-        CERT_DestroyCertificate(cert);
-        return(NULL);
+        xmlSecInternalError("xmlSecNssKeyDataX509AdoptKeyCert", NULL);
+        goto done;
     }
+    cert = NULL; /* owned by data now */
 
-    return(key);
+    /* success */
+    res = key;
+    key = NULL;
+
+done:
+    if(key != NULL) {
+        xmlSecKeyDestroy(key);
+    }
+    if(keyData != NULL) {
+        xmlSecKeyDataDestroy(keyData);
+    }
+    if(cert != NULL) {
+        CERT_DestroyCertificate(cert);
+    }
+    return(res);
 }
 
-
 /**
  * xmlSecNssAppKeysMngrCertLoad:
  * @mngr:               the pointer to keys manager.
@@ -1045,8 +1244,8 @@ xmlSecNssAppKeysMngrCertLoad(xmlSecKeysMngrPtr mngr, const char *filename,
 /**
  * xmlSecNssAppKeysMngrCertLoadMemory:
  * @mngr:               the pointer to keys manager.
- * @data:               the key binary data.
- * @dataSize:           the key binary data size.
+ * @data:               the certificatedata.
+ * @dataSize:           the certificate data size.
  * @format:             the certificate format (PEM or DER).
  * @type:               the certificate type (trusted/untrusted).
  *
@@ -1106,6 +1305,9 @@ xmlSecNssAppKeysMngrCertLoadSECItem(xmlSecKeysMngrPtr mngr, SECItem* secItem,
 
     xmlSecAssert2(mngr != NULL, -1);
     xmlSecAssert2(secItem != NULL, -1);
+    xmlSecAssert2(secItem->type == siBuffer, -1);
+    xmlSecAssert2(secItem->data != NULL, -1);
+    xmlSecAssert2(secItem->len > 0, -1);
     xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, -1);
 
     x509Store = xmlSecKeysMngrGetDataStore(mngr, xmlSecNssX509StoreId);
@@ -1116,17 +1318,24 @@ xmlSecNssAppKeysMngrCertLoadSECItem(xmlSecKeysMngrPtr mngr, SECItem* secItem,
 
     switch(format) {
     case xmlSecKeyDataFormatDer:
-        cert = __CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
-                                         secItem, NULL, PR_FALSE, PR_TRUE);
+        cert = xmlSecNssX509CertDerRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
         if(cert == NULL) {
-            xmlSecNssError2("__CERT_NewTempCertificate", NULL,
-                            "format=%d", (int)format);
+            xmlSecInternalError2("xmlSecNssX509CertDerRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            return(-1);
+        }
+        break;
+    case xmlSecKeyDataFormatCertPem:
+        cert = xmlSecNssX509CertPemRead(CERT_GetDefaultCertDB(), secItem->data, secItem->len);
+        if(cert == NULL) {
+            xmlSecInternalError2("xmlSecNssX509CertPemRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
             return(-1);
         }
         break;
     default:
         xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
-                         "format=%d", (int)format);
+            "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
         return(-1);
     }
 
@@ -1140,6 +1349,141 @@ xmlSecNssAppKeysMngrCertLoadSECItem(xmlSecKeysMngrPtr mngr, SECItem* secItem,
     return(0);
 }
 
+/**
+ * xmlSecNssAppKeysMngrCrlLoad:
+ * @mngr:               the pointer to keys manager.
+ * @filename:           the CRL file.
+ * @format:             the CRL file format (PEM or DER).
+ *
+ * Reads crl from @filename and adds to the list of crls in @store
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
+int
+xmlSecNssAppKeysMngrCrlLoad(xmlSecKeysMngrPtr mngr, const char *filename, xmlSecKeyDataFormat format) {
+    xmlSecKeyDataStorePtr x509Store;
+    CERTSignedCrl* crl;
+    SECItem secItem;
+    int ret;
+
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(filename != NULL, -1);
+    xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, -1);
+
+    x509Store = xmlSecKeysMngrGetDataStore(mngr, xmlSecNssX509StoreId);
+    if(x509Store == NULL) {
+        xmlSecInternalError("xmlSecKeysMngrGetDataStore(xmlSecNssX509StoreId)", NULL);
+        return(-1);
+    }
+
+    /* read the file contents */
+    memset(&secItem, 0, sizeof(secItem));
+    ret = xmlSecNssAppReadSECItem(&secItem, filename);
+    if((ret < 0) || (secItem.type != siBuffer) ||(secItem.data == NULL) || (secItem.len <= 0)) {
+        xmlSecInternalError("xmlSecNssAppReadSECItem", NULL);
+        return(-1);
+    }
+
+    /* read CRL */
+    switch(format) {
+    case xmlSecKeyDataFormatDer:
+        crl = xmlSecNssX509CrlDerRead(secItem.data, secItem.len, XMLSEC_KEYINFO_FLAGS_X509DATA_SKIP_STRICT_CHECKS);
+        if(crl == NULL) {
+            xmlSecInternalError2("xmlSecNssX509CrlDerRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            SECITEM_FreeItem(&secItem, PR_FALSE);
+            return(-1);
+        }
+        break;
+    default:
+        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
+            "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+        SECITEM_FreeItem(&secItem, PR_FALSE);
+        return(-1);
+    }
+    SECITEM_FreeItem(&secItem, PR_FALSE);
+
+    /* Add CRL to the store */
+    ret = xmlSecNssX509StoreAdoptCrl(x509Store, crl);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssX509StoreAdoptCrl", NULL);
+        SEC_DestroyCrl(crl);
+        return(-1);
+    }
+    crl = NULL; /* owned by x509data now */
+
+    /* done */
+    return(0);
+}
+
+/**
+ * xmlSecNssAppKeysMngrCrlLoadMemory:
+ * @mngr:               the pointer to keys manager.
+ * @data:               the CRL data.
+ * @dataSize:           the CRL data size.
+ * @format:             the CRL format (PEM or DER).
+ *
+ * Reads crl from @data and adds to the list of crls in @store
+ *
+ * Returns: 0 on success or a negative value otherwise.
+ */
+int
+xmlSecNssAppKeysMngrCrlLoadMemory(xmlSecKeysMngrPtr mngr, const xmlSecByte* data, xmlSecSize dataSize, xmlSecKeyDataFormat format) {
+    xmlSecKeyDataStorePtr x509Store;
+    CERTSignedCrl* crl;
+    SECItem secItem;
+    int ret;
+
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(data != NULL, -1);
+    xmlSecAssert2(format != xmlSecKeyDataFormatUnknown, -1);
+
+    x509Store = xmlSecKeysMngrGetDataStore(mngr, xmlSecNssX509StoreId);
+    if(x509Store == NULL) {
+        xmlSecInternalError("xmlSecKeysMngrGetDataStore(xmlSecNssX509StoreId)", NULL);
+        return(-1);
+    }
+
+    memset(&secItem, 0, sizeof(secItem));
+    ret = xmlSecNssAppCreateSECItem(&secItem, data, dataSize);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssAppCreateSECItem", NULL);
+        return(-1);
+    }
+
+    /* read CRL */
+    switch(format) {
+    case xmlSecKeyDataFormatDer:
+        crl = xmlSecNssX509CrlDerRead(secItem.data, secItem.len, XMLSEC_KEYINFO_FLAGS_X509DATA_SKIP_STRICT_CHECKS);
+        if(crl == NULL) {
+            xmlSecInternalError2("xmlSecNssX509CrlDerRead", NULL,
+                "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+            SECITEM_FreeItem(&secItem, PR_FALSE);
+            return(-1);
+        }
+        break;
+    default:
+        xmlSecOtherError2(XMLSEC_ERRORS_R_INVALID_FORMAT, NULL,
+            "format=" XMLSEC_ENUM_FMT, XMLSEC_ENUM_CAST(format));
+        SECITEM_FreeItem(&secItem, PR_FALSE);
+        return(-1);
+    }
+    SECITEM_FreeItem(&secItem, PR_FALSE);
+
+    /* Add CRL to the store */
+    ret = xmlSecNssX509StoreAdoptCrl(x509Store, crl);
+    if(ret < 0) {
+        xmlSecInternalError("xmlSecNssX509StoreAdoptCrl", NULL);
+        SEC_DestroyCrl(crl);
+        return(-1);
+    }
+    crl = NULL; /* owned by x509data now */
+
+    /* done */
+    return(0);
+}
+
+
 #endif /* XMLSEC_NO_X509 */
 
 /**
@@ -1218,6 +1562,51 @@ xmlSecNssAppDefaultKeysMngrAdoptKey(xmlSecKeysMngrPtr mngr, xmlSecKeyPtr key) {
     return(0);
 }
 
+/**
+ * xmlSecNssAppDefaultKeysMngrVerifyKey:
+ * @mngr:               the pointer to keys manager.
+ * @key:                the pointer to key.
+ * @keyInfoCtx:         the key info context for verification.
+ *
+ * Verifies @key with the keys manager @mngr created with #xmlSecCryptoAppDefaultKeysMngrInit
+ * function:
+ * - Checks that key certificate is present
+ * - Checks that key certificate is valid
+ *
+ * Adds @key to the keys manager @mngr created with #xmlSecCryptoAppDefaultKeysMngrInit
+ * function.
+ *
+ * Returns: 1 if key is verified, 0 otherwise, or a negative value if an error occurs.
+ */
+int
+xmlSecNssAppDefaultKeysMngrVerifyKey(xmlSecKeysMngrPtr mngr, xmlSecKeyPtr key, xmlSecKeyInfoCtxPtr keyInfoCtx) {
+#ifndef XMLSEC_NO_X509
+    xmlSecKeyDataStorePtr x509Store;
+
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+
+    x509Store = xmlSecKeysMngrGetDataStore(mngr, xmlSecNssX509StoreId);
+    if(x509Store == NULL) {
+        xmlSecInternalError("xmlSecKeysMngrGetDataStore(xmlSecNssX509StoreId)", NULL);
+        return(-1);
+    }
+
+    return(xmlSecNssX509StoreVerifyKey(x509Store, key, keyInfoCtx));
+
+#else  /* XMLSEC_NO_X509 */
+
+    xmlSecAssert2(mngr != NULL, -1);
+    xmlSecAssert2(key != NULL, -1);
+    xmlSecAssert2(keyInfoCtx != NULL, -1);
+
+    xmlSecNotImplementedError("X509 support is disabled");
+    return(-1);
+
+#endif /* XMLSEC_NO_X509 */
+}
+
 /**
  * xmlSecNssAppDefaultKeysMngrLoad:
  * @mngr:               the pointer to keys manager.
@@ -1297,4 +1686,3 @@ void*
 xmlSecNssAppGetDefaultPwdCallback(void) {
     return(NULL);
 }
-
