@@ -12,15 +12,11 @@
 #include "include/core/SkSpan.h"
 #include "include/core/SkTypes.h"
 #include "include/private/SkSLDefines.h"
-#include "include/private/SkSLIRNode.h"
-#include "include/private/SkSLModifiers.h"
-#include "include/private/SkSLProgramElement.h"
-#include "include/private/SkSLStatement.h"
 #include "include/private/base/SkTArray.h"
-#include "include/sksl/SkSLErrorReporter.h"
-#include "include/sksl/SkSLOperator.h"
-#include "include/sksl/SkSLPosition.h"
 #include "src/sksl/SkSLAnalysis.h"
+#include "src/sksl/SkSLErrorReporter.h"
+#include "src/sksl/SkSLOperator.h"
+#include "src/sksl/SkSLPosition.h"
 #include "src/sksl/analysis/SkSLProgramUsage.h"
 #include "src/sksl/ir/SkSLBinaryExpression.h"
 #include "src/sksl/ir/SkSLChildCall.h"
@@ -36,20 +32,23 @@
 #include "src/sksl/ir/SkSLConstructorStruct.h"
 #include "src/sksl/ir/SkSLDoStatement.h"
 #include "src/sksl/ir/SkSLExpressionStatement.h"
-#include "src/sksl/ir/SkSLExternalFunctionCall.h"
 #include "src/sksl/ir/SkSLFieldAccess.h"
 #include "src/sksl/ir/SkSLForStatement.h"
 #include "src/sksl/ir/SkSLFunctionCall.h"
 #include "src/sksl/ir/SkSLFunctionDeclaration.h"
 #include "src/sksl/ir/SkSLFunctionDefinition.h"
+#include "src/sksl/ir/SkSLIRNode.h"
 #include "src/sksl/ir/SkSLIfStatement.h"
 #include "src/sksl/ir/SkSLIndexExpression.h"
 #include "src/sksl/ir/SkSLLiteral.h"
+#include "src/sksl/ir/SkSLModifiers.h"
 #include "src/sksl/ir/SkSLNop.h"
 #include "src/sksl/ir/SkSLPostfixExpression.h"
 #include "src/sksl/ir/SkSLPrefixExpression.h"
+#include "src/sksl/ir/SkSLProgramElement.h"
 #include "src/sksl/ir/SkSLReturnStatement.h"
 #include "src/sksl/ir/SkSLSetting.h"
+#include "src/sksl/ir/SkSLStatement.h"
 #include "src/sksl/ir/SkSLSwitchCase.h"
 #include "src/sksl/ir/SkSLSwitchStatement.h"
 #include "src/sksl/ir/SkSLSwizzle.h"
@@ -69,6 +68,8 @@
 #include <string_view>
 #include <utility>
 
+using namespace skia_private;
+
 namespace SkSL {
 namespace {
 
@@ -170,7 +171,7 @@ std::unique_ptr<Expression> Inliner::inlineExpression(Position pos,
     };
     auto argList = [&](const ExpressionArray& originalArgs) -> ExpressionArray {
         ExpressionArray args;
-        args.reserve_back(originalArgs.size());
+        args.reserve_exact(originalArgs.size());
         for (const std::unique_ptr<Expression>& arg : originalArgs) {
             args.push_back(expr(arg));
         }
@@ -250,13 +251,6 @@ std::unique_ptr<Expression> Inliner::inlineExpression(Position pos,
                                            *ctor.type().clone(symbolTableForExpression),
                                            argList(ctor.arguments()));
         }
-        case Expression::Kind::kExternalFunctionCall: {
-            const ExternalFunctionCall& externalCall = expression.as<ExternalFunctionCall>();
-            return std::make_unique<ExternalFunctionCall>(pos, &externalCall.function(),
-                                                          argList(externalCall.arguments()));
-        }
-        case Expression::Kind::kExternalFunctionReference:
-            return expression.clone();
         case Expression::Kind::kFieldAccess: {
             const FieldAccess& f = expression.as<FieldAccess>();
             return FieldAccess::Make(*fContext, pos, expr(f.base()), f.fieldIndex(), f.ownerKind());
@@ -331,7 +325,7 @@ std::unique_ptr<Statement> Inliner::inlineStatement(Position pos,
     };
     auto blockStmts = [&](const Block& block) {
         StatementArray result;
-        result.reserve_back(block.children().size());
+        result.reserve_exact(block.children().size());
         for (const std::unique_ptr<Statement>& child : block.children()) {
             result.push_back(stmt(child));
         }
@@ -430,7 +424,7 @@ std::unique_ptr<Statement> Inliner::inlineStatement(Position pos,
         case Statement::Kind::kSwitch: {
             const SwitchStatement& ss = statement.as<SwitchStatement>();
             StatementArray cases;
-            cases.reserve_back(ss.cases().size());
+            cases.reserve_exact(ss.cases().size());
             for (const std::unique_ptr<Statement>& switchCaseStmt : ss.cases()) {
                 const SwitchCase& sc = switchCaseStmt->as<SwitchCase>();
                 if (sc.isDefault()) {
@@ -452,20 +446,21 @@ std::unique_ptr<Statement> Inliner::inlineStatement(Position pos,
             // names are important.
             const std::string* name = symbolTableForStatement->takeOwnershipOfString(
                     fMangler.uniqueName(variable->name(), symbolTableForStatement));
-            auto clonedVar = std::make_unique<Variable>(
-                    pos,
-                    variable->modifiersPosition(),
-                    variableModifiers(*variable, initialValue.get()),
-                    name->c_str(),
-                    variable->type().clone(symbolTableForStatement),
-                    isBuiltinCode,
-                    variable->storage());
+            auto clonedVar =
+                    std::make_unique<Variable>(pos,
+                                               variable->modifiersPosition(),
+                                               variableModifiers(*variable, initialValue.get()),
+                                               name->c_str(),
+                                               variable->type().clone(symbolTableForStatement),
+                                               isBuiltinCode,
+                                               variable->storage());
             varMap->set(variable, VariableReference::Make(pos, clonedVar.get()));
-            auto result = VarDeclaration::Make(*fContext,
-                                               clonedVar.get(),
-                                               decl.baseType().clone(symbolTableForStatement),
-                                               decl.arraySize(),
-                                               std::move(initialValue));
+            std::unique_ptr<Statement> result =
+                    VarDeclaration::Make(*fContext,
+                                         clonedVar.get(),
+                                         decl.baseType().clone(symbolTableForStatement),
+                                         decl.arraySize(),
+                                         std::move(initialValue));
             symbolTableForStatement->takeOwnershipOfSymbol(std::move(clonedVar));
             return result;
         }
@@ -504,7 +499,7 @@ Inliner::InlinedCall Inliner::inlineCall(const FunctionCall& call,
                             arguments.size() +       // Function argument temp-vars
                             body.children().size();  // Inlined code
 
-    inlineStatements.reserve_back(expectedStmtCount);
+    inlineStatements.reserve_exact(expectedStmtCount);
 
     std::unique_ptr<Expression> resultExpr;
     if (returnComplexity > Analysis::ReturnComplexity::kSingleSafeReturn &&
@@ -607,10 +602,13 @@ bool Inliner::isSafeToInline(const FunctionDefinition* functionDef, const Progra
         return false;
     }
 
-    // We don't allow inlining a function with out parameters that are written to.
-    // (See skia:11326 for rationale.)
     for (const Variable* param : functionDef->declaration().parameters()) {
-        if (param->modifiers().fFlags & Modifiers::Flag::kOut_Flag) {
+        // We don't allow inlining functions with parameters that are written-to, if they...
+        // - are `out` parameters (see skia:11326 for rationale.)
+        // - are arrays or structures (introducing temporary copies is non-trivial)
+        if ((param->modifiers().fFlags & Modifiers::Flag::kOut_Flag) ||
+            param->type().isArray() ||
+            param->type().isStruct()) {
             ProgramUsage::VariableCounts counts = usage.get(*param);
             if (counts.fWrite > 0) {
                 return false;
@@ -771,7 +769,7 @@ public:
             }
             case Statement::Kind::kVarDeclaration: {
                 VarDeclaration& varDeclStmt = (*stmt)->as<VarDeclaration>();
-                // Don't need to scan the declaration's sizes; those are always IntLiterals.
+                // Don't need to scan the declaration's sizes; those are always literals.
                 this->visitExpression(&varDeclStmt.value());
                 break;
             }
@@ -789,7 +787,6 @@ public:
         }
 
         switch ((*expr)->kind()) {
-            case Expression::Kind::kExternalFunctionReference:
             case Expression::Kind::kFieldAccess:
             case Expression::Kind::kFunctionReference:
             case Expression::Kind::kLiteral:
@@ -842,13 +839,6 @@ public:
                 }
                 break;
             }
-            case Expression::Kind::kExternalFunctionCall: {
-                ExternalFunctionCall& funcCallExpr = (*expr)->as<ExternalFunctionCall>();
-                for (std::unique_ptr<Expression>& arg : funcCallExpr.arguments()) {
-                    this->visitExpression(&arg);
-                }
-                break;
-            }
             case Expression::Kind::kFunctionCall: {
                 FunctionCall& funcCallExpr = (*expr)->as<FunctionCall>();
                 for (std::unique_ptr<Expression>& arg : funcCallExpr.arguments()) {
@@ -1007,8 +997,8 @@ bool Inliner::analyze(const std::vector<std::unique_ptr<ProgramElement>>& elemen
     this->buildCandidateList(elements, symbols, usage, &candidateList);
 
     // Inline the candidates where we've determined that it's safe to do so.
-    using StatementRemappingTable = SkTHashMap<std::unique_ptr<Statement>*,
-                                               std::unique_ptr<Statement>*>;
+    using StatementRemappingTable = THashMap<std::unique_ptr<Statement>*,
+                                             std::unique_ptr<Statement>*>;
     StatementRemappingTable statementRemappingTable;
 
     bool madeChanges = false;
