@@ -5,14 +5,9 @@
  * found in the LICENSE file.
  */
 
-#include "src/sksl/SkSLPool.h"
-
 #include "include/core/SkTypes.h"
-
-#if SK_SUPPORT_GPU
-// With GPU support, SkSL::MemoryPool is really GrMemoryPool
-#include "src/gpu/ganesh/GrMemoryPool.h"
-#endif
+#include "src/sksl/SkSLMemoryPool.h"
+#include "src/sksl/SkSLPool.h"
 
 #define VLOG(...) // printf(__VA_ARGS__)
 
@@ -28,21 +23,20 @@ static void set_thread_local_memory_pool(MemoryPool* memPool) {
     sMemPool = memPool;
 }
 
+Pool::Pool() = default;
+
 Pool::~Pool() {
     if (get_thread_local_memory_pool() == fMemPool.get()) {
         SkDEBUGFAIL("SkSL pool is being destroyed while it is still attached to the thread");
         set_thread_local_memory_pool(nullptr);
     }
 
-    fMemPool->reportLeaks();
-    SkASSERT(fMemPool->isEmpty());
-
     VLOG("DELETE Pool:0x%016llX\n", (uint64_t)fMemPool.get());
 }
 
 std::unique_ptr<Pool> Pool::Create() {
     auto pool = std::unique_ptr<Pool>(new Pool);
-    pool->fMemPool = MemoryPool::Make(/*preallocSize=*/65536, /*minAllocSize=*/32768);
+    pool->fMemPool = MemoryPool::Make();
     VLOG("CREATE Pool:0x%016llX\n", (uint64_t)pool->fMemPool.get());
     return pool;
 }
@@ -58,10 +52,8 @@ void Pool::attachToThread() {
 }
 
 void Pool::detachFromThread() {
-    MemoryPool* memPool = get_thread_local_memory_pool();
     VLOG("DETACH Pool:0x%016llX\n", (uint64_t)memPool);
-    SkASSERT(memPool == fMemPool.get());
-    memPool->resetScratchSpace();
+    SkASSERT(get_thread_local_memory_pool() == fMemPool.get());
     set_thread_local_memory_pool(nullptr);
 }
 
