@@ -7,9 +7,9 @@
 
 #include "src/sksl/codegen/SkSLSPIRVtoHLSL.h"
 
-#if defined(SK_DIRECT3D)
+#if defined(SK_ENABLE_SPIRV_CROSS)
 
-#include "third_party/externals/spirv-cross/spirv_hlsl.hpp"
+#include <spirv_hlsl.hpp>
 
 /*
  * This translation unit serves as a bridge between Skia/SkSL and SPIRV-Cross.
@@ -19,7 +19,7 @@
 
 namespace SkSL {
 
-bool SPIRVtoHLSL(const String& spirv, String* hlsl) {
+bool SPIRVtoHLSL(const std::string& spirv, std::string* hlsl) {
     spirv_cross::CompilerHLSL hlslCompiler((const uint32_t*)spirv.c_str(),
                                            spirv.size() / sizeof(uint32_t));
 
@@ -44,6 +44,6 @@ bool SPIRVtoHLSL(const String& spirv, String* hlsl) {
 
 #else
 
-namespace SkSL { bool SPIRVtoHLSL(const String&, String*) { return false; } }
+namespace SkSL { bool SPIRVtoHLSL(const std::string&, std::string*) { return false; } }
 
 #endif
