@@ -8,20 +8,30 @@
 #ifndef SKSL_TYPE
 #define SKSL_TYPE
 
-#include "include/core/SkStringView.h"
-#include "include/private/SkSLModifiers.h"
-#include "include/private/SkSLSymbol.h"
-#include "src/sksl/SkSLUtil.h"
+#include "include/core/SkSpan.h"
+#include "include/core/SkTypes.h"
+#include "include/private/SkSLDefines.h"
+#include "include/private/base/SkTArray.h"
+#include "src/sksl/SkSLPosition.h"
+#include "src/sksl/ir/SkSLIRNode.h"
+#include "src/sksl/ir/SkSLModifiers.h"
+#include "src/sksl/ir/SkSLSymbol.h"
 #include "src/sksl/spirv.h"
-#include <algorithm>
-#include <climits>
-#include <vector>
+
+#include <cmath>
+#include <cstddef>
+#include <cstdint>
 #include <memory>
+#include <string>
+#include <string_view>
+#include <tuple>
 
 namespace SkSL {
 
 class Context;
+class Expression;
 class SymbolTable;
+class Type;
 
 struct CoercionCost {
     static CoercionCost Free()              { return {    0,    0, false }; }
@@ -46,36 +56,47 @@ struct CoercionCost {
                std::tie(rhs.fImpossible, rhs.fNarrowingCost, rhs.fNormalCost);
     }
 
+    bool operator<=(CoercionCost rhs) const {
+        return std::tie(    fImpossible,     fNarrowingCost,     fNormalCost) <=
+               std::tie(rhs.fImpossible, rhs.fNarrowingCost, rhs.fNormalCost);
+    }
+
     int  fNormalCost;
     int  fNarrowingCost;
     bool fImpossible;
 };
 
+/**
+ * Represents a single field in a struct type.
+ */
+struct Field {
+    Field(Position pos, Modifiers modifiers, std::string_view name, const Type* type)
+            : fPosition(pos)
+            , fModifiers(modifiers)
+            , fName(name)
+            , fType(type) {}
+
+    std::string description() const;
+
+    Position fPosition;
+    Modifiers fModifiers;
+    std::string_view fName;
+    const Type* fType;
+};
+
 /**
  * Represents a type, such as int or float4.
  */
 class Type : public Symbol {
 public:
-    inline static constexpr Kind kSymbolKind = Kind::kType;
+    inline static constexpr Kind kIRNodeKind = Kind::kType;
     inline static constexpr int kMaxAbbrevLength = 3;
-
-    struct Field {
-        Field(Modifiers modifiers, skstd::string_view name, const Type* type)
-        : fModifiers(modifiers)
-        , fName(name)
-        , fType(std::move(type)) {}
-
-        String description() const {
-            return fType->displayName() + " " + fName + ";";
-        }
-
-        Modifiers fModifiers;
-        skstd::string_view fName;
-        const Type* fType;
-    };
+    // Represents unspecified array dimensions, as in `int[]`.
+    inline static constexpr int kUnsizedArray = -1;
 
     enum class TypeKind : int8_t {
         kArray,
+        kAtomic,
         kGeneric,
         kLiteral,
         kMatrix,
@@ -102,27 +123,41 @@ public:
         kNonnumeric
     };
 
+    enum class TextureAccess : int8_t {
+        kSample,  // `kSample` access level allows both sampling and reading
+        kRead,
+        kWrite,
+        kReadWrite,
+    };
+
     Type(const Type& other) = delete;
 
-    /** Creates an array type. */
-    static std::unique_ptr<Type> MakeArrayType(skstd::string_view name, const Type& componentType,
+    /** Creates an array type. `columns` may be kUnsizedArray. */
+    static std::unique_ptr<Type> MakeArrayType(std::string_view name, const Type& componentType,
                                                int columns);
 
     /** Converts a component type and a size (float, 10) into an array name ("float[10]"). */
-    String getArrayName(int arraySize) const;
+    std::string getArrayName(int arraySize) const;
+
+    /**
+     * Creates an alias which maps to another type.
+     */
+    static std::unique_ptr<Type> MakeAliasType(std::string_view name, const Type& targetType);
 
     /**
      * Create a generic type which maps to the listed types--e.g. $genType is a generic type which
      * can match float, float2, float3 or float4.
      */
-    static std::unique_ptr<Type> MakeGenericType(const char* name, std::vector<const Type*> types);
+    static std::unique_ptr<Type> MakeGenericType(const char* name,
+                                                 SkSpan<const Type* const> types,
+                                                 const Type* slotType);
 
     /** Create a type for literal scalars. */
     static std::unique_ptr<Type> MakeLiteralType(const char* name, const Type& scalarType,
                                                  int8_t priority);
 
     /** Create a matrix type. */
-    static std::unique_ptr<Type> MakeMatrixType(skstd::string_view name, const char* abbrev,
+    static std::unique_ptr<Type> MakeMatrixType(std::string_view name, const char* abbrev,
                                                 const Type& componentType, int columns,
                                                 int8_t rows);
 
@@ -130,7 +165,7 @@ public:
     static std::unique_ptr<Type> MakeSamplerType(const char* name, const Type& textureType);
 
     /** Create a scalar type. */
-    static std::unique_ptr<Type> MakeScalarType(skstd::string_view name, const char* abbrev,
+    static std::unique_ptr<Type> MakeScalarType(std::string_view name, const char* abbrev,
                                                 Type::NumberKind numberKind, int8_t priority,
                                                 int8_t bitWidth);
 
@@ -140,19 +175,28 @@ public:
     static std::unique_ptr<Type> MakeSpecialType(const char* name, const char* abbrev,
                                                  Type::TypeKind typeKind);
 
-    /** Creates a struct type with the given fields. */
-    static std::unique_ptr<Type> MakeStructType(int line, skstd::string_view name,
-                                                std::vector<Field> fields);
+    /**
+     * Creates a struct type with the given fields. Reports an error if the struct is not
+     * well-formed.
+     */
+    static std::unique_ptr<Type> MakeStructType(const Context& context,
+                                                Position pos,
+                                                std::string_view name,
+                                                skia_private::TArray<Field> fields,
+                                                bool interfaceBlock = false);
 
     /** Create a texture type. */
-    static std::unique_ptr<Type> MakeTextureType(const char* name, SpvDim_ dimensions,
-                                                 bool isDepth, bool isArrayedTexture,
-                                                 bool isMultisampled, bool isSampled);
+    static std::unique_ptr<Type> MakeTextureType(const char* name, SpvDim_ dimensions, bool isDepth,
+                                                 bool isArrayedTexture, bool isMultisampled,
+                                                 TextureAccess textureAccess);
 
     /** Create a vector type. */
-    static std::unique_ptr<Type> MakeVectorType(skstd::string_view name, const char* abbrev,
+    static std::unique_ptr<Type> MakeVectorType(std::string_view name, const char* abbrev,
                                                 const Type& componentType, int columns);
 
+    /** Create an atomic type. */
+    static std::unique_ptr<Type> MakeAtomicType(std::string_view name, const char* abbrev);
+
     template <typename T>
     bool is() const {
         return this->typeKind() == T::kTypeKind;
@@ -183,11 +227,11 @@ public:
         return !(this->isArray() || this->isStruct());
     }
 
-    String displayName() const {
-        return String(this->scalarTypeForLiteral().name());
+    std::string displayName() const {
+        return std::string(this->scalarTypeForLiteral().name());
     }
 
-    String description() const override {
+    std::string description() const override {
         return this->displayName();
     }
 
@@ -199,17 +243,14 @@ public:
         return true;
     }
 
-    /** Returns true if this type is either private, or contains a private field (recursively). */
-    virtual bool isPrivate() const {
-        return this->name().starts_with("$");
+    /** If this is an alias, returns the underlying type, otherwise returns this. */
+    virtual const Type& resolve() const {
+        return *this;
     }
 
-    bool operator==(const Type& other) const {
-        return this->name() == other.name();
-    }
-
-    bool operator!=(const Type& other) const {
-        return this->name() != other.name();
+    /** Returns true if these types are equal after alias resolution. */
+    bool matches(const Type& other) const {
+        return this->resolve().name() == other.resolve().name();
     }
 
     /**
@@ -290,18 +331,17 @@ public:
 
     /**
      * Returns true if this is an "opaque type" (an external object which the shader references in
-     * some fashion), or void. https://www.khronos.org/opengl/wiki/Data_Type_(GLSL)#Opaque_types
+     * some fashion). https://www.khronos.org/opengl/wiki/Data_Type_(GLSL)#Opaque_types
      */
     bool isOpaque() const {
         switch (fTypeKind) {
+            case TypeKind::kAtomic:
             case TypeKind::kBlender:
             case TypeKind::kColorFilter:
-            case TypeKind::kOther:
             case TypeKind::kSampler:
             case TypeKind::kSeparateSampler:
             case TypeKind::kShader:
             case TypeKind::kTexture:
-            case TypeKind::kVoid:
                 return true;
             default:
                 return false;
@@ -342,11 +382,11 @@ public:
     }
 
     /**
-     * For texturesamplers, returns the type of texture it samples (e.g., sampler2D has
+     * For texture samplers, returns the type of texture it samples (e.g., sampler2D has
      * a texture type of texture2D).
      */
     virtual const Type& textureType() const {
-        SkDEBUGFAIL("not a texture type");
+        SkDEBUGFAIL("not a sampler type");
         return *this;
     }
 
@@ -369,19 +409,15 @@ public:
         return -1;
     }
 
-    /** For integer types, returns the minimum value that can fit in the type. */
-    int64_t minimumValue() const {
-        SkASSERT(this->isInteger());
-        constexpr int64_t k1 = 1;  // ensures that `1 << n` is evaluated as 64-bit
-        return this->isUnsigned() ? 0 : -(k1 << (this->bitWidth() - 1));
+    /** Returns the minimum value that can fit in the type. */
+    virtual double minimumValue() const {
+        SkDEBUGFAIL("type does not have a minimum value");
+        return -INFINITY;
     }
 
-    /** For integer types, returns the maximum value that can fit in the type. */
-    int64_t maximumValue() const {
-        SkASSERT(this->isInteger());
-        constexpr int64_t k1 = 1;  // ensures that `1 << n` is evaluated as 64-bit
-        return (this->isUnsigned() ? (k1 << this->bitWidth())
-                                   : (k1 << (this->bitWidth() - 1))) - 1;
+    virtual double maximumValue() const {
+        SkDEBUGFAIL("type does not have a maximum value");
+        return +INFINITY;
     }
 
     /**
@@ -391,15 +427,24 @@ public:
         return 0;
     }
 
-    virtual const std::vector<Field>& fields() const {
+    /**
+     * Returns the type of the value in the nth slot. For scalar, vector and matrix types, should
+     * always match `componentType()`.
+     */
+    virtual const Type& slotType(size_t) const {
+        return *this;
+    }
+
+    virtual SkSpan<const Field> fields() const {
         SK_ABORT("Internal error: not a struct");
     }
 
     /**
      * For generic types, returns the types that this generic type can substitute for.
      */
-    virtual const std::vector<const Type*>& coercibleTypes() const {
-        SK_ABORT("Internal error: not a generic type");
+    virtual SkSpan<const Type* const> coercibleTypes() const {
+        SkDEBUGFAIL("Internal error: not a generic type");
+        return {};
     }
 
     virtual SpvDim_ dimensions() const {
@@ -421,6 +466,12 @@ public:
         return fTypeKind == TypeKind::kVoid;
     }
 
+    bool isGeneric() const {
+        return fTypeKind == TypeKind::kGeneric;
+    }
+
+    bool isAtomic() const { return this->typeKind() == TypeKind::kAtomic; }
+
     virtual bool isScalar() const {
         return false;
     }
@@ -445,10 +496,18 @@ public:
         return false;
     }
 
+    virtual bool isUnsizedArray() const {
+        return false;
+    }
+
     virtual bool isStruct() const {
         return false;
     }
 
+    virtual bool isInterfaceBlock() const {
+        return false;
+    }
+
     // Is this type something that can be bound & sampled from an SkRuntimeEffect?
     // Includes types that represent stages of the Skia pipeline (colorFilter, shader, blender).
     bool isEffectChild() const {
@@ -458,13 +517,13 @@ public:
     }
 
     virtual bool isMultisampled() const {
-        SkASSERT(false);
+        SkDEBUGFAIL("not a texture type");
         return false;
     }
 
-    virtual bool isSampled() const {
-        SkASSERT(false);
-        return false;
+    virtual TextureAccess textureAccess() const {
+        SkDEBUGFAIL("not a texture type");
+        return TextureAccess::kSample;
     }
 
     bool hasPrecision() const {
@@ -480,11 +539,8 @@ public:
     }
 
     bool isOrContainsArray() const;
-
-    /**
-     * Returns true if this type is a struct that is too deeply nested.
-     */
-    bool isTooDeeplyNested() const;
+    bool isOrContainsUnsizedArray() const;
+    bool isOrContainsAtomic() const;
 
     /**
      * Returns the corresponding vector or matrix type with the specified number of columns and
@@ -493,14 +549,12 @@ public:
     const Type& toCompound(const Context& context, int columns, int rows) const;
 
     /**
-     * Returns a type which honors the precision qualifiers set in Modifiers. e.g., kMediump_Flag
-     * when applied to `float2` will return `half2`. Generates an error if the precision qualifiers
-     * don't make sense, e.g. `highp bool` or `mediump MyStruct`.
+     * Returns a type which honors the precision and access-level qualifiers set in Modifiers. e.g.:
+     *  - Modifier `mediump` + Type `float2`:     Type `half2`
+     *  - Modifier `readonly` + Type `texture2D`: Type `readonlyTexture2D`
+     * Generates an error if the qualifiers don't make sense (`highp bool`, `writeonly MyStruct`)
      */
-    const Type* applyPrecisionQualifiers(const Context& context,
-                                         Modifiers* modifiers,
-                                         SymbolTable* symbols,
-                                         int line) const;
+    const Type* applyQualifiers(const Context& context, Modifiers* modifiers, Position pos) const;
 
     /**
      * Coerces the passed-in expression to this type. If the types are incompatible, reports an
@@ -513,25 +567,43 @@ public:
     bool checkForOutOfRangeLiteral(const Context& context, const Expression& expr) const;
 
     /** Checks if `value` can fit in this type. The type must be scalar. */
-    bool checkForOutOfRangeLiteral(const Context& context, double value, int line) const;
+    bool checkForOutOfRangeLiteral(const Context& context, double value, Position pos) const;
+
+    /**
+     * Reports errors and returns false if this type cannot be used as the base type for an array.
+     */
+    bool checkIfUsableInArray(const Context& context, Position arrayPos) const;
 
     /**
      * Verifies that the expression is a valid constant array size for this type. Returns the array
-     * size, or zero if the expression isn't a valid literal value.
+     * size, or reports errors and returns zero if the expression isn't a valid literal value.
      */
-    SKSL_INT convertArraySize(const Context& context, std::unique_ptr<Expression> size) const;
+    SKSL_INT convertArraySize(const Context& context,
+                              Position arrayPos,
+                              std::unique_ptr<Expression> size) const;
+
+    SKSL_INT convertArraySize(const Context& context,
+                              Position arrayPos,
+                              Position sizePos,
+                              SKSL_INT size) const;
 
 protected:
-    Type(skstd::string_view name, const char* abbrev, TypeKind kind, int line = -1)
-        : INHERITED(line, kSymbolKind, name)
-        , fTypeKind(kind) {
+    Type(std::string_view name, const char* abbrev, TypeKind kind, Position pos = Position())
+            : INHERITED(pos, kIRNodeKind, name)
+            , fTypeKind(kind) {
         SkASSERT(strlen(abbrev) <= kMaxAbbrevLength);
         strcpy(fAbbreviatedName, abbrev);
     }
 
-private:
-    bool isTooDeeplyNested(int limit) const;
+    const Type* applyPrecisionQualifiers(const Context& context,
+                                         Modifiers* modifiers,
+                                         Position pos) const;
 
+    const Type* applyAccessQualifiers(const Context& context,
+                                      Modifiers* modifiers,
+                                      Position pos) const;
+
+private:
     using INHERITED = Symbol;
 
     char fAbbreviatedName[kMaxAbbrevLength + 1] = {};
