@@ -9,7 +9,6 @@
 #include "orcus/spreadsheet/sheet.hpp"
 #include "orcus/spreadsheet/document.hpp"
 #include "orcus/spreadsheet/view.hpp"
-#include "orcus/global.hpp"
 #include "orcus/measurement.hpp"
 #include "orcus/string_pool.hpp"
 
@@ -22,26 +21,49 @@
 namespace orcus { namespace spreadsheet {
 
 import_sheet_named_exp::import_sheet_named_exp(document& doc, sheet_t sheet_index) :
-    m_doc(doc), m_sheet_index(sheet_index) {}
+    m_doc(doc), m_sheet_index(sheet_index), m_base(sheet_index, 0, 0) {}
 
 import_sheet_named_exp::~import_sheet_named_exp() {}
 
-void import_sheet_named_exp::define_name(
-    const char* p_name, size_t n_name, const char* p_exp, size_t n_exp)
+void import_sheet_named_exp::define(
+    std::string_view name, std::string_view expression, formula_ref_context_t ref_cxt)
 {
-    const ixion::formula_name_resolver* resolver = m_doc.get_formula_name_resolver();
+    string_pool& sp = m_doc.get_string_pool();
+    m_name = sp.intern(name).first;
+
+    const ixion::formula_name_resolver* resolver = m_doc.get_formula_name_resolver(ref_cxt);
     assert(resolver);
 
     ixion::model_context& cxt = m_doc.get_model_context();
+    m_tokens = ixion::parse_formula_string(cxt, m_base, *resolver, expression);
+}
 
-    ixion::formula_tokens_t tokens =
-        ixion::parse_formula_string(
-            cxt, ixion::abs_address_t(0,0,0), *resolver, p_exp, n_exp);
+void import_sheet_named_exp::set_base_position(const src_address_t& pos)
+{
+    m_base.sheet = pos.sheet;
+    m_base.row = pos.row;
+    m_base.column = pos.column;
+}
 
-    std::unique_ptr<ixion::formula_tokens_t> tokens_p =
-        orcus::make_unique<ixion::formula_tokens_t>(std::move(tokens));
+void import_sheet_named_exp::set_named_expression(std::string_view name, std::string_view expression)
+{
+    define(name, expression, formula_ref_context_t::global);
+}
 
-    cxt.set_named_expression(m_sheet_index, p_name, n_name, std::move(tokens_p));
+void import_sheet_named_exp::set_named_range(std::string_view name, std::string_view range)
+{
+    define(name, range, formula_ref_context_t::named_range);
+}
+
+void import_sheet_named_exp::commit()
+{
+    ixion::model_context& cxt = m_doc.get_model_context();
+    cxt.set_named_expression(m_sheet_index, std::string{m_name}, m_base, std::move(m_tokens));
+
+    m_name = std::string_view{};
+    m_base.sheet = 0;
+    m_base.row = 0;
+    m_base.column = 0;
 }
 
 import_data_table::import_data_table(sheet& sh) : m_sheet(sh) {}
@@ -51,19 +73,19 @@ void import_data_table::reset()
 {
 }
 
-void import_data_table::set_type(data_table_type_t type)
+void import_data_table::set_type(data_table_type_t /*type*/)
 {
 }
 
-void import_data_table::set_range(const range_t& range)
+void import_data_table::set_range(const range_t& /*range*/)
 {
 }
 
-void import_data_table::set_first_reference(const char* p_ref, size_t n_ref, bool deleted)
+void import_data_table::set_first_reference(std::string_view /*ref*/, bool /*deleted*/)
 {
 }
 
-void import_data_table::set_second_reference(const char* p_ref, size_t n_ref, bool deleted)
+void import_data_table::set_second_reference(std::string_view /*ref*/, bool /*deleted*/)
 {
 }
 
@@ -93,11 +115,11 @@ void import_auto_filter::set_column(col_t col)
     m_cur_col = col;
 }
 
-void import_auto_filter::append_column_match_value(const char* p, size_t n)
+void import_auto_filter::append_column_match_value(std::string_view value)
 {
     // The string pool belongs to the document.
-    pstring s = m_string_pool.intern(p, n).first;
-    m_cur_col_data.match_values.insert(s);
+    value = m_string_pool.intern(value).first;
+    m_cur_col_data.match_values.insert(value);
 }
 
 void import_auto_filter::commit_column()
@@ -115,7 +137,7 @@ void import_auto_filter::commit()
 }
 
 import_array_formula::import_array_formula(document& doc, sheet& sheet) :
-    m_doc(doc), m_sheet(sheet)
+    m_doc(doc), m_sheet(sheet), m_missing_formula_result(), m_error_policy(formula_error_policy_t::fail)
 {
     m_range.first.column = -1;
     m_range.first.row = -1;
@@ -129,11 +151,51 @@ import_array_formula::~import_array_formula()
 void import_array_formula::set_range(const range_t& range)
 {
     m_range = range;
+
+    // Initialize the result matrix with the missing result value.
+    switch (m_missing_formula_result.get_type())
+    {
+        case ixion::formula_result::result_type::value:
+        {
+            ixion::matrix _mtx(
+                m_range.last.row - m_range.first.row + 1,
+                m_range.last.column - m_range.first.column + 1,
+                m_missing_formula_result.get_value());
+            m_result_mtx.swap(_mtx);
+            break;
+        }
+        case ixion::formula_result::result_type::error:
+        {
+            ixion::matrix _mtx(
+                m_range.last.row - m_range.first.row + 1,
+                m_range.last.column - m_range.first.column + 1,
+                m_missing_formula_result.get_error());
+            m_result_mtx.swap(_mtx);
+            break;
+        }
+        case ixion::formula_result::result_type::string:
+        {
+            ixion::matrix _mtx(
+                m_range.last.row - m_range.first.row + 1,
+                m_range.last.column - m_range.first.column + 1,
+                m_missing_formula_result.get_string());
+            m_result_mtx.swap(_mtx);
+            break;
+        }
+        default:
+        {
+            ixion::matrix _mtx(
+                m_range.last.row - m_range.first.row + 1,
+                m_range.last.column - m_range.first.column + 1);
+            m_result_mtx.swap(_mtx);
+        }
+    }
 }
 
-void import_array_formula::set_formula(formula_grammar_t grammar, const char* p, size_t n)
+void import_array_formula::set_formula(formula_grammar_t /*grammar*/, std::string_view formula)
 {
-    const ixion::formula_name_resolver* resolver = m_doc.get_formula_name_resolver();
+    const ixion::formula_name_resolver* resolver =
+        m_doc.get_formula_name_resolver(spreadsheet::formula_ref_context_t::global);
     if (!resolver)
         return;
 
@@ -141,33 +203,60 @@ void import_array_formula::set_formula(formula_grammar_t grammar, const char* p,
     ixion::model_context& cxt = m_doc.get_model_context();
     ixion::abs_address_t pos(m_sheet.get_index(), m_range.first.row, m_range.first.column);
 
-    m_tokens = ixion::parse_formula_string(cxt, pos, *resolver, p, n);
+    try
+    {
+        m_tokens = ixion::parse_formula_string(cxt, pos, *resolver, formula);
+    }
+    catch (const std::exception& e)
+    {
+        if (m_error_policy == formula_error_policy_t::fail)
+            throw;
+
+        std::string_view error_s = e.what();
+        m_tokens = ixion::create_formula_error_tokens(cxt, formula, error_s);
+    }
 }
 
 void import_array_formula::set_result_value(row_t row, col_t col, double value)
 {
+    m_result_mtx.set(row, col, value);
 }
 
-void import_array_formula::set_result_string(row_t row, col_t col, size_t sindex)
+void import_array_formula::set_result_string(row_t /*row*/, col_t /*col*/, std::string_view /*value*/)
 {
+    // TODO : handle this
 }
 
-void import_array_formula::set_result_empty(row_t row, col_t col)
+void import_array_formula::set_result_empty(row_t /*row*/, col_t /*col*/)
 {
+    // TODO : handle this
 }
 
 void import_array_formula::set_result_bool(row_t row, col_t col, bool value)
 {
+    m_result_mtx.set(row, col, value);
 }
 
 void import_array_formula::commit()
 {
-    m_sheet.set_grouped_formula(m_range, std::move(m_tokens));
+    ixion::formula_result cached_results(std::move(m_result_mtx));
+    m_sheet.set_grouped_formula(m_range, std::move(m_tokens), std::move(cached_results));
+}
+
+void import_array_formula::set_missing_formula_result(ixion::formula_result result)
+{
+    m_missing_formula_result = std::move(result);
+}
+
+void import_array_formula::set_formula_error_policy(formula_error_policy_t policy)
+{
+    m_error_policy = policy;
 }
 
 void import_array_formula::reset()
 {
     m_tokens.clear();
+    m_result_mtx = ixion::matrix();
     m_range.first.row = -1;
     m_range.first.column = -1;
     m_range.last.row = -1;
@@ -181,7 +270,8 @@ import_formula::import_formula(document& doc, sheet& sheet, shared_formula_pool&
     m_row(-1),
     m_col(-1),
     m_shared_index(0),
-    m_shared(false) {}
+    m_shared(false),
+    m_error_policy(formula_error_policy_t::fail) {}
 
 import_formula::~import_formula() {}
 
@@ -191,12 +281,13 @@ void import_formula::set_position(row_t row, col_t col)
     m_col = col;
 }
 
-void import_formula::set_formula(formula_grammar_t grammar, const char* p, size_t n)
+void import_formula::set_formula(formula_grammar_t /*grammar*/, std::string_view formula)
 {
     if (m_row < 0 || m_col < 0)
         return;
 
-    const ixion::formula_name_resolver* resolver = m_doc.get_formula_name_resolver();
+    const ixion::formula_name_resolver* resolver =
+        m_doc.get_formula_name_resolver(spreadsheet::formula_ref_context_t::global);
     if (!resolver)
         return;
 
@@ -204,7 +295,19 @@ void import_formula::set_formula(formula_grammar_t grammar, const char* p, size_
     ixion::model_context& cxt = m_doc.get_model_context();
     ixion::abs_address_t pos(m_sheet.get_index(), m_row, m_col);
 
-    ixion::formula_tokens_t tokens = ixion::parse_formula_string(cxt, pos, *resolver, p, n);
+    ixion::formula_tokens_t tokens;
+    try
+    {
+        tokens = ixion::parse_formula_string(cxt, pos, *resolver, formula);
+    }
+    catch (const std::exception& e)
+    {
+        if (m_error_policy == formula_error_policy_t::fail)
+            throw;
+
+        std::string_view error_s = e.what();
+        tokens = ixion::create_formula_error_tokens(cxt, formula, error_s);
+    }
 
     m_tokens_store = ixion::formula_tokens_store::create();
     m_tokens_store->get() = std::move(tokens);
@@ -216,10 +319,18 @@ void import_formula::set_shared_formula_index(size_t index)
     m_shared_index = index;
 }
 
-void import_formula::set_result_value(double value) {}
-void import_formula::set_result_string(size_t sindex) {}
+void import_formula::set_result_value(double value)
+{
+    m_result = ixion::formula_result(value);
+}
+
+void import_formula::set_result_string(std::string_view value)
+{
+    m_result = ixion::formula_result(std::string{value});
+}
+
 void import_formula::set_result_empty() {}
-void import_formula::set_result_bool(bool value) {}
+void import_formula::set_result_bool(bool /*value*/) {}
 
 void import_formula::commit()
 {
@@ -230,7 +341,11 @@ void import_formula::commit()
     {
         if (m_tokens_store)
         {
-            m_sheet.set_formula(m_row, m_col, m_tokens_store);
+            if (m_result)
+                m_sheet.set_formula(m_row, m_col, m_tokens_store, *m_result);
+            else
+                m_sheet.set_formula(m_row, m_col, m_tokens_store);
+
             m_shared_formula_pool.add(m_shared_index, m_tokens_store);
         }
         else
@@ -239,17 +354,34 @@ void import_formula::commit()
             if (!ts)
                 return;
 
-            m_sheet.set_formula(m_row, m_col, ts);
+            if (m_result)
+                m_sheet.set_formula(m_row, m_col, ts, *m_result);
+            else
+                m_sheet.set_formula(m_row, m_col, ts);
         }
         return;
     }
 
-    m_sheet.set_formula(m_row, m_col, m_tokens_store);
+    if (m_result)
+        m_sheet.set_formula(m_row, m_col, m_tokens_store, *m_result);
+    else
+        m_sheet.set_formula(m_row, m_col, m_tokens_store);
+}
+
+void import_formula::set_missing_formula_result(ixion::formula_result result)
+{
+    m_result = std::move(result);
+}
+
+void import_formula::set_formula_error_policy(formula_error_policy_t policy)
+{
+    m_error_policy = policy;
 }
 
 void import_formula::reset()
 {
     m_tokens_store.reset();
+    m_result.reset();
     m_row = -1;
     m_col = -1;
     m_shared_index = 0;
@@ -266,10 +398,11 @@ import_sheet::import_sheet(document& doc, sheet& sh, sheet_view* view) :
     m_data_table(sh),
     m_auto_filter(sh, doc.get_string_pool()),
     m_table(doc, sh),
-    m_charset(character_set_t::unspecified)
+    m_charset(character_set_t::unspecified),
+    m_fill_missing_formula_results(false)
 {
     if (view)
-        m_sheet_view = orcus::make_unique<import_sheet_view>(*view, sh.get_index());
+        m_sheet_view = std::make_unique<import_sheet_view>(*view, sh.get_index());
 }
 
 import_sheet::~import_sheet() {}
@@ -314,18 +447,32 @@ iface::import_table* import_sheet::get_table()
 iface::import_formula* import_sheet::get_formula()
 {
     m_formula.reset();
+
+    if (m_fill_missing_formula_results)
+    {
+        m_formula.set_missing_formula_result(
+            ixion::formula_result(ixion::formula_error_t::no_result_error));
+    }
+
     return &m_formula;
 }
 
 iface::import_array_formula* import_sheet::get_array_formula()
 {
     m_array_formula.reset();
+
+    if (m_fill_missing_formula_results)
+    {
+        m_array_formula.set_missing_formula_result(
+            ixion::formula_result(ixion::formula_error_t::no_result_error));
+    }
+
     return &m_array_formula;
 }
 
-void import_sheet::set_auto(row_t row, col_t col, const char* p, size_t n)
+void import_sheet::set_auto(row_t row, col_t col, std::string_view s)
 {
-    m_sheet.set_auto(row, col, p, n);
+    m_sheet.set_auto(row, col, s);
 }
 
 void import_sheet::set_bool(row_t row, col_t col, bool value)
@@ -349,7 +496,17 @@ void import_sheet::set_format(
     m_sheet.set_format(row_start, col_start, row_end, col_end, xf_index);
 }
 
-void import_sheet::set_string(row_t row, col_t col, size_t sindex)
+void import_sheet::set_column_format(col_t col, col_t col_span, std::size_t xf_index)
+{
+    m_sheet.set_column_format(col, col_span, xf_index);
+}
+
+void import_sheet::set_row_format(row_t row, std::size_t xf_index)
+{
+    m_sheet.set_row_format(row, xf_index);
+}
+
+void import_sheet::set_string(row_t row, col_t col, string_id_t sindex)
 {
     m_sheet.set_string(row, col, sindex);
 }
@@ -366,10 +523,7 @@ void import_sheet::fill_down_cells(row_t src_row, col_t src_col, row_t range_siz
 
 range_size_t import_sheet::get_sheet_size() const
 {
-    range_size_t ret;
-    ret.rows = m_sheet.row_size();
-    ret.columns = m_sheet.col_size();
-    return ret;
+    return m_doc.get_sheet_size();
 }
 
 void import_sheet::set_character_set(character_set_t charset)
@@ -377,6 +531,17 @@ void import_sheet::set_character_set(character_set_t charset)
     m_charset = charset;
 }
 
+void import_sheet::set_fill_missing_formula_results(bool b)
+{
+    m_fill_missing_formula_results = b;
+}
+
+void import_sheet::set_formula_error_policy(formula_error_policy_t policy)
+{
+    m_formula.set_formula_error_policy(policy);
+    m_array_formula.set_formula_error_policy(policy);
+}
+
 import_sheet_view::import_sheet_view(sheet_view& view, sheet_t si) :
     m_view(view), m_sheet_index(si) {}
 
@@ -416,15 +581,15 @@ import_sheet_properties::import_sheet_properties(document& doc, sheet& sh) :
 
 import_sheet_properties::~import_sheet_properties() {}
 
-void import_sheet_properties::set_column_width(col_t col, double width, orcus::length_unit_t unit)
+void import_sheet_properties::set_column_width(col_t col, col_t col_span, double width, orcus::length_unit_t unit)
 {
     col_width_t w = orcus::convert(width, unit, length_unit_t::twip);
-    m_sheet.set_col_width(col, w);
+    m_sheet.set_col_width(col, col_span, w);
 }
 
-void import_sheet_properties::set_column_hidden(col_t col, bool hidden)
+void import_sheet_properties::set_column_hidden(col_t col, col_t col_span, bool hidden)
 {
-    m_sheet.set_col_hidden(col, hidden);
+    m_sheet.set_col_hidden(col, col_span, hidden);
 }
 
 void import_sheet_properties::set_row_height(row_t row, double height, orcus::length_unit_t unit)
