@@ -1,11 +1,15 @@
+/*
+ * Copyright 2021 Axel Waggershauser
+ */
+// SPDX-License-Identifier: Apache-2.0
+
 #include <stdint.h>
 #include <stddef.h>
 
 #include "BitArray.h"
-#include "oned/rss/ODRSSExpandedBinaryDecoder.h"
+#include "oned/ODDataBarExpandedBitDecoder.h"
 
 using namespace ZXing;
-using namespace ZXing::OneD::RSS;
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size)
 {
@@ -16,7 +20,7 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t* data, size_t size)
 	for (size_t i = 0; i < size; ++i)
 		bits.appendBits(data[i], 8);
 
-	ExpandedBinaryDecoder::Decode(bits);
+	OneD::DataBar::DecodeExpandedBits(bits);
 
 	return 0;
 }
