@@ -1,23 +1,16 @@
 /*
 * Copyright 2017 Huy Cuong Nguyen
 * Copyright 2013 ZXing authors
-*
-* Licensed under the Apache License, Version 2.0 (the "License");
-* you may not use this file except in compliance with the License.
-* You may obtain a copy of the License at
-*
-*      http://www.apache.org/licenses/LICENSE-2.0
-*
-* Unless required by applicable law or agreed to in writing, software
-* distributed under the License is distributed on an "AS IS" BASIS,
-* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-* See the License for the specific language governing permissions and
-* limitations under the License.
 */
+// SPDX-License-Identifier: Apache-2.0
 
 #include "aztec/AZDetector.h"
+
 #include "BitMatrixIO.h"
+#include "DecoderResult.h"
 #include "PseudoRandom.h"
+#include "Utf.h"
+#include "aztec/AZDecoder.h"
 #include "aztec/AZDetectorResult.h"
 
 #include "gtest/gtest.h"
@@ -48,53 +41,43 @@ namespace {
 		return result;
 	}
 
-	// Zooms a bit matrix so that each bit is factor x factor
-	BitMatrix MakeLarger(const BitMatrix& input, int factor) {
-		return Inflate(input.copy(), factor * input.width(), factor * input.height(), 0);
-	}
-
 	// Test that we can tolerate errors in the parameter locator bits
-	void TestErrorInParameterLocator(std::string_view data, int nbLayers, bool isCompact, const BitMatrix &matrix_)
+	void TestErrorInOrientationBits(std::string_view data, int nbLayers, bool isCompact, const BitMatrix &matrix_)
 	{
 		PseudoRandom random(std::hash<std::string_view>()(data));
 		auto orientationPoints = GetOrientationPoints(matrix_, isCompact);
-		for (bool isMirror : { false, true }) {
+		for (bool mirror : { false, true }) {
 			BitMatrix matrix = matrix_.copy();
+			if (mirror)
+				matrix.mirror();
 			for (int i = 0; i < 4; ++i) {
+				Aztec::DetectorResult r = Aztec::Detect(matrix, true, false);
+				EXPECT_EQ(r.isValid(), true);
+				EXPECT_EQ(r.nbLayers(), nbLayers);
+				EXPECT_EQ(r.isCompact(), isCompact);
+				EXPECT_EQ(r.isMirrored(), mirror);
+				EXPECT_EQ(data, ToUtf8(Aztec::Decode(r).text()));
+
 				// Systematically try every possible 1- and 2-bit error.
 				for (int error1 = 0; error1 < Size(orientationPoints); error1++) {
 					for (int error2 = error1; error2 < Size(orientationPoints); error2++) {
 						BitMatrix copy = matrix.copy();
-						if (isMirror) {
-							copy.mirror();
-						}
 						copy.flip(orientationPoints[error1].x, orientationPoints[error1].y);
-						if (error2 > error1) {
+						if (error2 > error1 && (nbLayers > 1 || !mirror)) { // in ErrorInModeMessageZero and mirror==true test only 1-bit errors
 							// if error2 == error1, we only test a single error
 							copy.flip(orientationPoints[error2].x, orientationPoints[error2].y);
 						}
-						// The detector doesn't seem to work when matrix bits are only 1x1.  So magnify.
-						Aztec::DetectorResult r = Aztec::Detector::Detect(MakeLarger(copy, 3), isMirror, true);
+						Aztec::DetectorResult r = Aztec::Detect(copy, true, false);
 						EXPECT_EQ(r.isValid(), true);
 						EXPECT_EQ(r.nbLayers(), nbLayers);
 						EXPECT_EQ(r.isCompact(), isCompact);
-						//DecoderResult res = new Decoder().decode(r);
-						//assertEquals(data, res.getText());
+						EXPECT_EQ(r.isMirrored(), mirror);
+						EXPECT_EQ(data, ToUtf8(Aztec::Decode(r).text()));
 					}
 				}
-				// Try a few random three-bit errors;
-				for (int i = 0; i < 5; i++) {
-					BitMatrix copy = matrix.copy();
-					std::set<size_t> errors;
-					while (errors.size() < 3) {
-						errors.insert(random.next(size_t(0), orientationPoints.size() - 1));
-					}
-					for (auto error : errors) {
-						copy.flip(orientationPoints[error].x, orientationPoints[error].y);
-					}
-					Aztec::DetectorResult r = Aztec::Detector::Detect(MakeLarger(copy, 3), false, true);
-					EXPECT_EQ(r.isValid(), false);
-				}
+
+				// here used to be a test for 3-bit errors but since we determine both the rotation and the mirrored info
+				// from the orentation bits (as suggested in the specification) this does not work anymore.
 
 				matrix.rotate90();
 			}
@@ -102,11 +85,10 @@ namespace {
 	}
 } // anonymous
 
-TEST(AZDetectorTest, ErrorInParameterLocatorZeroZero)
+TEST(AZDetectorTest, ErrorInModeMessageZero)
 {
-	// Layers=1, CodeWords=1.  So the parameter info and its Reed-Solomon info
-	// will be completely zero!
-	TestErrorInParameterLocator("X", 1, true, ParseBitMatrix(
+	// Layers=1, CodeWords=1. So the ModeMessage and its Reed-Solomon bits will be completely zero!
+	TestErrorInOrientationBits("X", 1, true, ParseBitMatrix(
 		"    X X X X X X X   X X X X X \n"
 		"X X X X   X     X X         X \n"
 		"    X X                 X   X \n"
@@ -126,9 +108,9 @@ TEST(AZDetectorTest, ErrorInParameterLocatorZeroZero)
 	);
 }
 
-TEST(AZDetectorTest, ErrorInParameterLocatorCompact)
+TEST(AZDetectorTest, ErrorInOrientationBitsCompact)
 {
-	TestErrorInParameterLocator("This is an example Aztec symbol for Wikipedia.", 3, true, ParseBitMatrix(
+	TestErrorInOrientationBits("This is an example Aztec symbol for Wikipedia.", 3, true, ParseBitMatrix(
 		"X     X X       X     X X     X     X         \n"
 		"X         X     X X     X   X X   X X       X \n"
 		"X X   X X X X X   X X X                 X     \n"
@@ -156,10 +138,10 @@ TEST(AZDetectorTest, ErrorInParameterLocatorCompact)
 	);
 }
 
-TEST(AZDetectorTest, ErrorInParameterLocatorNotCompact)
+TEST(AZDetectorTest, ErrorInOrientationBitsNotCompact)
 {
 	std::string alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYabcdefghijklmnopqrstuvwxyz";
-	TestErrorInParameterLocator(alphabet + alphabet + alphabet, 6, false, ParseBitMatrix(
+	TestErrorInOrientationBits(alphabet + alphabet + alphabet, 6, false, ParseBitMatrix(
 		"    X   X     X     X     X   X X X X   X   X   X     X X     X X       X X X X   \n"
 		"  X         X   X         X X X X X   X   X X X   X   X X X X X   X X X       X   \n"
 		"    X   X       X X X X X   X X X X   X X   X X X X X   X X X     X   X X X   X   \n"
@@ -209,7 +191,7 @@ TEST(AZDetectorTest, ReaderInitFull2Layers)
 {
 	{
 		// Null (not set)
-		auto r = Aztec::Detector::Detect(ParseBitMatrix(
+		auto r = Aztec::Detect(ParseBitMatrix(
 			"      X X X   X   X X   X X X X   X X X X X   \n"
 			"    X   X X X X   X X X     X X X         X   \n"
 			"  X X   X X   X   X X   X X X   X X X     X X \n"
@@ -233,7 +215,7 @@ TEST(AZDetectorTest, ReaderInitFull2Layers)
 			"    X     X X X         X   X X X     X       \n"
 			"    X X X         X   X   X X   X       X   X \n"
 			"  X   X X X             X   X   X       X     \n"
-		), false /*isMirror*/, true /*isPure*/);
+		), false /*isPure*/, false /*tryHarder*/);
 		EXPECT_TRUE(r.isValid());
 		EXPECT_FALSE(r.readerInit());
 		EXPECT_FALSE(r.isCompact());
@@ -241,7 +223,7 @@ TEST(AZDetectorTest, ReaderInitFull2Layers)
 	}
 	{
 		// Set
-		auto r = Aztec::Detector::Detect(ParseBitMatrix(
+		auto r = Aztec::Detect(ParseBitMatrix(
 			"      X X X   X   X X   X X X X   X X X X X   \n"
 			"    X   X X X X   X X X     X X X         X   \n"
 			"  X X   X X   X   X X   X X X   X X X     X X \n"
@@ -265,7 +247,7 @@ TEST(AZDetectorTest, ReaderInitFull2Layers)
 			"    X     X X X         X   X X X     X       \n"
 			"    X X X         X   X   X X   X       X   X \n"
 			"  X   X X X             X   X   X       X     \n"
-		), false /*isMirror*/, true /*isPure*/);
+		), true /*isPure*/, false /*tryHarder*/);
 		EXPECT_TRUE(r.isValid());
 		EXPECT_TRUE(r.readerInit());
 		EXPECT_FALSE(r.isCompact());
@@ -276,7 +258,7 @@ TEST(AZDetectorTest, ReaderInitFull2Layers)
 TEST(AZDetectorTest, ReaderInitFull22Layers)
 {
 	// Set
-	auto r = Aztec::Detector::Detect(ParseBitMatrix(
+	auto r = Aztec::Detect(ParseBitMatrix(
 		"            X X   X   X X X     X       X   X     X         X   X     X X   X   X       X X   X X X         X       X   X     X   X         X X X X X X   X   X   X   X X   X X     X     X X X X     X   X X X X   X X   \n"
 		"        X X     X X   X   X X X X   X X   X           X     X   X   X X   X   X X X X X X X       X   X         X X X   X X     X X   X             X         X   X X   X X   X   X   X   X X     X X X X     X     X   X \n"
 		"    X   X   X   X       X           X X     X       X       X   X X       X X X     X   X   X X     X X   X X X X         X X X   X     X X X   X   X X     X       X X X   X   X     X X   X X X           X X   X X     \n"
@@ -386,7 +368,7 @@ TEST(AZDetectorTest, ReaderInitFull22Layers)
 		"X X X     X X   X   X   X X X   X X X     X X X X   X   X X X   X   X X   X X       X             X X X     X   X X   X X     X X           X X X             X X     X   X X       X X X X X X         X   X   X       X \n"
 		"  X   X   X   X   X   X X X X     X X X X     X X       X X X   X X X     X     X X X X X X X X       X X     X X   X   X       X         X   X X   X X X X   X   X   X X     X     X X   X X   X X X X   X           X   \n"
 		"X X X X X   X X X X   X X X   X X X         X X       X       X       X X X X X X X     X   X X X         X X       X X X X X   X     X   X X X X X   X   X     X           X X X   X X X X X X       X X X X       X X   \n"
-	), false /*isMirror*/, true /*isPure*/);
+	), true /*isPure*/, false /*tryHarder*/);
 	EXPECT_TRUE(r.isValid());
 	EXPECT_TRUE(r.readerInit());
 	EXPECT_FALSE(r.isCompact());
@@ -397,7 +379,7 @@ TEST(AZDetectorTest, ReaderInitCompact)
 {
 	{
 		// Null (not set)
-		auto r = Aztec::Detector::Detect(ParseBitMatrix(
+		auto r = Aztec::Detect(ParseBitMatrix(
 			"            X X   X         X \n"
 			"    X X X X   X X X   X     X \n"
 			"    X X             X   X     \n"
@@ -413,7 +395,7 @@ TEST(AZDetectorTest, ReaderInitCompact)
 			"X             X X         X X \n"
 			"X         X   X     X X       \n"
 			"X X X X     X X X         X X \n"
-		), false /*isMirror*/, true /*isPure*/);
+		), true /*isPure*/, false /*tryHarder*/);
 		EXPECT_TRUE(r.isValid());
 		EXPECT_FALSE(r.readerInit());
 		EXPECT_TRUE(r.isCompact());
@@ -421,7 +403,7 @@ TEST(AZDetectorTest, ReaderInitCompact)
 	}
 	{
 		// Set
-		auto r = Aztec::Detector::Detect(ParseBitMatrix(
+		auto r = Aztec::Detect(ParseBitMatrix(
 			"            X X   X         X \n"
 			"    X X X X   X X X   X     X \n"
 			"    X X     X       X   X     \n"
@@ -437,7 +419,7 @@ TEST(AZDetectorTest, ReaderInitCompact)
 			"X       X X   X   X X     X X \n"
 			"X         X   X     X X       \n"
 			"X X X X     X X X         X X \n"
-		), false /*isMirror*/, true /*isPure*/);
+		), true /*isPure*/, false /*tryHarder*/);
 		EXPECT_TRUE(r.isValid());
 		EXPECT_TRUE(r.readerInit());
 		EXPECT_TRUE(r.isCompact());
