@@ -1,4 +1,4 @@
-// Copyright 2019 PDFium Authors. All rights reserved.
+// Copyright 2019 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -27,7 +27,7 @@ XFATestEnvironment::~XFATestEnvironment() {
 }
 
 void XFATestEnvironment::SetUp() {
-  CFX_GEModule::Get()->GetFontMgr()->SetSystemFontInfo(
+  CFX_GEModule::Get()->GetFontMgr()->GetBuiltinMapper()->SetSystemFontInfo(
       CFX_GEModule::Get()->GetPlatform()->CreateDefaultSystemFontInfo());
 
   // The font loading that takes place in CFGAS_GEModule::Create() is slow,
