@@ -5,34 +5,59 @@
  * found in the LICENSE file.
  */
 
+#include "include/core/SkAlphaType.h"
 #include "include/core/SkBitmap.h"
 #include "include/core/SkCanvas.h"
 #include "include/core/SkColor.h"
 #include "include/core/SkColorSpace.h"
+#include "include/core/SkColorType.h"
 #include "include/core/SkImageInfo.h"
 #include "include/core/SkPaint.h"
+#include "include/core/SkPixmap.h"
 #include "include/core/SkRect.h"
 #include "include/core/SkRefCnt.h"
 #include "include/core/SkSurface.h"
+#include "include/core/SkSurfaceProps.h"
 #include "include/core/SkTypes.h"
+#include "include/gpu/GpuTypes.h"
 #include "include/gpu/GrContextOptions.h"
 #include "include/gpu/GrDirectContext.h"
-#include "include/private/GrTypesPriv.h"
+#include "include/gpu/ganesh/SkSurfaceGanesh.h"
 #include "include/private/SkColorData.h"
+#include "include/private/gpu/ganesh/GrTypesPriv.h"
 #include "src/core/SkAutoPixmapStorage.h"
-#include "src/gpu/GrColor.h"
-#include "src/gpu/GrDirectContextPriv.h"
-#include "src/gpu/GrImageInfo.h"
-#include "src/gpu/ops/ClearOp.h"
-#include "src/gpu/v1/SurfaceDrawContext_v1.h"
+#include "src/gpu/SkBackingFit.h"
+#include "src/gpu/ganesh/GrCaps.h"
+#include "src/gpu/ganesh/GrColor.h"
+#include "src/gpu/ganesh/GrDirectContextPriv.h"
+#include "src/gpu/ganesh/SurfaceDrawContext.h"
+#include "src/gpu/ganesh/ops/ClearOp.h"
+#include "src/gpu/ganesh/ops/GrOp.h"
+#include "src/gpu/ganesh/ops/OpsTask.h"
+#include "tests/CtsEnforcement.h"
 #include "tests/Test.h"
-#include "tools/gpu/GrContextFactory.h"
 
+#include <array>
 #include <cstdint>
 #include <memory>
 
-using SurfaceDrawContext = skgpu::v1::SurfaceDrawContext;
-using ClearOp = skgpu::v1::ClearOp;
+class GrRecordingContext;
+
+using SurfaceDrawContext = skgpu::ganesh::SurfaceDrawContext;
+using ClearOp = skgpu::ganesh::ClearOp;
+
+static bool pixel_matches(const SkPixmap& pm, int x, int y, uint32_t expectedValue,
+                          uint32_t* actualValue, int* failX, int* failY) {
+    uint32_t pixel = pm.addr32()[y * pm.width() + x];
+    if (pixel != expectedValue) {
+        *actualValue = pixel;
+        *failX = x;
+        *failY = y;
+        return false;
+    }
+
+    return true;
+}
 
 static bool check_rect(GrDirectContext* dContext,
                        SurfaceDrawContext* sdc,
@@ -41,8 +66,8 @@ static bool check_rect(GrDirectContext* dContext,
                        uint32_t* actualValue,
                        int* failX,
                        int* failY) {
-    int w = rect.width();
-    int h = rect.height();
+    int w = sdc->width();
+    int h = sdc->height();
 
     SkImageInfo dstInfo = SkImageInfo::Make(w, h, kRGBA_8888_SkColorType, kPremul_SkAlphaType);
 
@@ -50,17 +75,14 @@ static bool check_rect(GrDirectContext* dContext,
     readback.alloc(dstInfo);
 
     readback.erase(~expectedValue);
-    if (!sdc->readPixels(dContext, readback, {rect.fLeft, rect.fTop})) {
+    if (!sdc->readPixels(dContext, readback, {0, 0})) {
         return false;
     }
 
-    for (int y = 0; y < h; ++y) {
-        for (int x = 0; x < w; ++x) {
-            uint32_t pixel = readback.addr32()[y * w + x];
-            if (pixel != expectedValue) {
-                *actualValue = pixel;
-                *failX = x + rect.fLeft;
-                *failY = y + rect.fTop;
+    SkASSERT(rect.fTop < rect.fBottom && rect.fLeft < rect.fRight);
+    for (int y = rect.fTop; y < rect.fBottom; ++y) {
+        for (int x = rect.fLeft; x < rect.fRight; ++x) {
+            if (!pixel_matches(readback, x, y, expectedValue, actualValue, failX, failY)) {
                 return false;
             }
         }
@@ -68,9 +90,47 @@ static bool check_rect(GrDirectContext* dContext,
     return true;
 }
 
+// Check a 1-pixel wide ring 'inset' from the outer edge
+static bool check_ring(GrDirectContext* dContext,
+                       SurfaceDrawContext* sdc,
+                       int inset,
+                       uint32_t expectedValue,
+                       uint32_t* actualValue,
+                       int* failX,
+                       int* failY) {
+    int w = sdc->width();
+    int h = sdc->height();
+
+    SkImageInfo dstInfo = SkImageInfo::Make(w, h, kRGBA_8888_SkColorType, kPremul_SkAlphaType);
+
+    SkAutoPixmapStorage readback;
+    readback.alloc(dstInfo);
+
+    readback.erase(~expectedValue);
+    if (!sdc->readPixels(dContext, readback, {0, 0})) {
+        return false;
+    }
+
+    for (int y = inset; y < h-inset; ++y) {
+        if (!pixel_matches(readback, inset, y, expectedValue, actualValue, failX, failY) ||
+            !pixel_matches(readback, w-1-inset, y, expectedValue, actualValue, failX, failY)) {
+            return false;
+        }
+    }
+    for (int x = inset+1; x < w-inset-1; ++x) {
+        if (!pixel_matches(readback, x, inset, expectedValue, actualValue, failX, failY) ||
+            !pixel_matches(readback, x, h-1-inset, expectedValue, actualValue, failX, failY)) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
 std::unique_ptr<SurfaceDrawContext> newSDC(GrRecordingContext* rContext, int w, int h) {
     return SurfaceDrawContext::Make(rContext, GrColorType::kRGBA_8888, nullptr,
-                                    SkBackingFit::kExact, {w, h}, SkSurfaceProps());
+                                    SkBackingFit::kExact, {w, h}, SkSurfaceProps(),
+                                    /*label=*/{});
 }
 
 static void clear_op_test(skiatest::Reporter* reporter, GrDirectContext* dContext) {
@@ -83,18 +143,10 @@ static void clear_op_test(skiatest::Reporter* reporter, GrDirectContext* dContex
     // A rectangle that is inset by one on all sides and the 1-pixel wide rectangles that surround
     // it.
     SkIRect mid1Rect = SkIRect::MakeXYWH(1, 1, kW-2, kH-2);
-    SkIRect outerLeftEdge = SkIRect::MakeXYWH(0, 0, 1, kH);
-    SkIRect outerTopEdge = SkIRect::MakeXYWH(0, 0, kW, 1);
-    SkIRect outerRightEdge = SkIRect::MakeXYWH(kW-1, 0, 1, kH);
-    SkIRect outerBottomEdge = SkIRect::MakeXYWH(0, kH-1, kW, 1);
 
     // A rectangle that is inset by two on all sides and the 1-pixel wide rectangles that surround
     // it.
     SkIRect mid2Rect = SkIRect::MakeXYWH(2, 2, kW-4, kH-4);
-    SkIRect innerLeftEdge = SkIRect::MakeXYWH(1, 1, 1, kH-2);
-    SkIRect innerTopEdge = SkIRect::MakeXYWH(1, 1, kW-2, 1);
-    SkIRect innerRightEdge = SkIRect::MakeXYWH(kW-2, 1, 1, kH-2);
-    SkIRect innerBottomEdge = SkIRect::MakeXYWH(1, kH-2, kW-2, 1);
 
     uint32_t actualValue;
     int failX, failY;
@@ -168,10 +220,8 @@ static void clear_op_test(skiatest::Reporter* reporter, GrDirectContext* dContex
         ERRORF(reporter, "Expected 0x%08x but got 0x%08x at (%d, %d).", kColor2, actualValue,
                failX, failY);
     }
-    if (!check_rect(dContext, sdc.get(), outerLeftEdge, kColor1, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), outerTopEdge, kColor1, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), outerRightEdge, kColor1, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), outerBottomEdge, kColor1, &actualValue, &failX, &failY)) {
+
+    if (!check_ring(dContext, sdc.get(), /*inset=*/ 0, kColor1, &actualValue, &failX, &failY)) {
         ERRORF(reporter, "Expected 0x%08x but got 0x%08x at (%d, %d).", kColor1, actualValue,
                failX, failY);
     }
@@ -199,17 +249,13 @@ static void clear_op_test(skiatest::Reporter* reporter, GrDirectContext* dContex
         ERRORF(reporter, "Expected 0x%08x but got 0x%08x at (%d, %d).", kColor1, actualValue,
                failX, failY);
     }
-    if (!check_rect(dContext, sdc.get(), innerLeftEdge, kColor2, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), innerTopEdge, kColor2, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), innerRightEdge, kColor2, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), innerBottomEdge, kColor2, &actualValue, &failX, &failY)) {
+
+    if (!check_ring(dContext, sdc.get(), /*inset=*/ 1, kColor2, &actualValue, &failX, &failY)) {
         ERRORF(reporter, "Expected 0x%08x but got 0x%08x at (%d, %d).", kColor2, actualValue,
                failX, failY);
     }
-    if (!check_rect(dContext, sdc.get(), outerLeftEdge, kColor1, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), outerTopEdge, kColor1, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), outerRightEdge, kColor1, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), outerBottomEdge, kColor1, &actualValue, &failX, &failY)) {
+
+    if (!check_ring(dContext, sdc.get(), /*inset=*/ 0, kColor1, &actualValue, &failX, &failY)) {
         ERRORF(reporter, "Expected 0x%08x but got 0x%08x at (%d, %d).", kColor1, actualValue,
                failX, failY);
     }
@@ -225,10 +271,7 @@ static void clear_op_test(skiatest::Reporter* reporter, GrDirectContext* dContex
         ERRORF(reporter, "Expected 0x%08x but got 0x%08x at (%d, %d).", kColor2, actualValue,
                failX, failY);
     }
-    if (!check_rect(dContext, sdc.get(), outerLeftEdge, kColor1, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), outerTopEdge, kColor1, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), outerRightEdge, kColor1, &actualValue, &failX, &failY) ||
-        !check_rect(dContext, sdc.get(), outerBottomEdge, kColor1, &actualValue, &failX, &failY)) {
+    if (!check_ring(dContext, sdc.get(), /*inset=*/ 0, kColor1, &actualValue, &failX, &failY)) {
         ERRORF(reporter, "Expected 0x%08x but got 0x%08x at (%d, %d).", kColor1, actualValue,
                failX, failY);
     }
@@ -292,7 +335,7 @@ static void clear_op_test(skiatest::Reporter* reporter, GrDirectContext* dContex
     }
 }
 
-DEF_GPUTEST_FOR_RENDERING_CONTEXTS(ClearOp, reporter, ctxInfo) {
+DEF_GANESH_TEST_FOR_RENDERING_CONTEXTS(ClearOp, reporter, ctxInfo, CtsEnforcement::kApiLevel_T) {
     // Regular clear
     clear_op_test(reporter, ctxInfo.directContext());
 
@@ -306,7 +349,7 @@ DEF_GPUTEST_FOR_RENDERING_CONTEXTS(ClearOp, reporter, ctxInfo) {
 void fullscreen_clear_with_layer_test(skiatest::Reporter* reporter, GrRecordingContext* rContext) {
     const SkImageInfo ii = SkImageInfo::Make(400, 77, kRGBA_8888_SkColorType, kPremul_SkAlphaType);
 
-    sk_sp<SkSurface> surf = SkSurface::MakeRenderTarget(rContext, SkBudgeted::kYes, ii);
+    sk_sp<SkSurface> surf = SkSurfaces::RenderTarget(rContext, skgpu::Budgeted::kYes, ii);
     SkCanvas* canvas = surf->getCanvas();
 
     SkPaint paints[2];
@@ -358,7 +401,10 @@ void fullscreen_clear_with_layer_test(skiatest::Reporter* reporter, GrRecordingC
     REPORTER_ASSERT(reporter, isCorrect);
 }
 // From crbug.com/768134
-DEF_GPUTEST_FOR_RENDERING_CONTEXTS(FullScreenClearWithLayers, reporter, ctxInfo) {
+DEF_GANESH_TEST_FOR_RENDERING_CONTEXTS(FullScreenClearWithLayers,
+                                       reporter,
+                                       ctxInfo,
+                                       CtsEnforcement::kApiLevel_T) {
     // Regular clear
     fullscreen_clear_with_layer_test(reporter, ctxInfo.directContext());
 
