@@ -22,19 +22,20 @@
 #include "include/core/SkTileMode.h"
 #include "include/core/SkTypes.h"
 #include "include/effects/SkGradientShader.h"
+#include "include/gpu/GpuTypes.h"
 #include "include/gpu/GrDirectContext.h"
 #include "include/gpu/GrTypes.h"
 #include "include/private/base/SkFloatBits.h"
 #include "include/private/base/SkTemplates.h"
 #include "include/private/gpu/ganesh/GrTypesPriv.h"
-#include "include/utils/SkRandom.h"
 #include "src/base/SkArenaAlloc.h"
-#include "src/core/SkMatrixProvider.h"
+#include "src/base/SkRandom.h"
 #include "src/core/SkPathPriv.h"
 #include "src/gpu/SkBackingFit.h"
 #include "src/gpu/ganesh/GrColorInfo.h"
 #include "src/gpu/ganesh/GrEagerVertexAllocator.h"
 #include "src/gpu/ganesh/GrFragmentProcessor.h"
+#include "src/gpu/ganesh/GrFragmentProcessors.h"
 #include "src/gpu/ganesh/GrPaint.h"
 #include "src/gpu/ganesh/GrStyle.h"
 #include "src/gpu/ganesh/GrUserStencilSettings.h"
@@ -46,7 +47,6 @@
 #include "src/gpu/ganesh/geometry/GrStyledShape.h"
 #include "src/gpu/ganesh/geometry/GrTriangulator.h"
 #include "src/gpu/ganesh/ops/TriangulatingPathRenderer.h"
-#include "src/shaders/SkShaderBase.h"
 #include "tests/CtsEnforcement.h"
 #include "tests/Test.h"
 #include "tools/ToolUtils.h"
@@ -529,7 +529,7 @@ CreatePathFn kNonEdgeAAPaths[] = {
     },
 };
 
-#if SK_GPU_V1
+#if defined(SK_GANESH)
 
 // A simple concave path. Test this with a non-invertible matrix.
 static SkPath create_path_17() {
@@ -820,26 +820,24 @@ static SkPath create_path_47() {
     return path;
 }
 
-static std::unique_ptr<GrFragmentProcessor> create_linear_gradient_processor(
-            GrRecordingContext* rContext) {
-
+static std::unique_ptr<GrFragmentProcessor>
+create_linear_gradient_processor(GrRecordingContext* rContext, const SkMatrix& ctm) {
     SkPoint pts[2] = { {0, 0}, {1, 1} };
     SkColor colors[2] = { SK_ColorGREEN, SK_ColorBLUE };
     sk_sp<SkShader> shader = SkGradientShader::MakeLinear(
         pts, colors, nullptr, std::size(colors), SkTileMode::kClamp);
     GrColorInfo colorInfo(GrColorType::kRGBA_8888, kPremul_SkAlphaType, nullptr);
-    SkMatrixProvider matrixProvider(SkMatrix::I());
     SkSurfaceProps props; // default props for testing
-    return as_SB(shader)->asFragmentProcessor({rContext, matrixProvider, &colorInfo, props});
+    return GrFragmentProcessors::Make(shader.get(), {rContext, &colorInfo, props}, ctm);
 }
 
 static void test_path(GrRecordingContext* rContext,
-                      skgpu::v1::SurfaceDrawContext* sdc,
+                      skgpu::ganesh::SurfaceDrawContext* sdc,
                       const SkPath& path,
                       const SkMatrix& matrix = SkMatrix::I(),
                       GrAAType aaType = GrAAType::kNone,
                       std::unique_ptr<GrFragmentProcessor> fp = nullptr) {
-    skgpu::v1::TriangulatingPathRenderer pr;
+    skgpu::ganesh::TriangulatingPathRenderer pr;
     pr.setMaxVerbCount(100);
 
     GrPaint paint;
@@ -851,16 +849,16 @@ static void test_path(GrRecordingContext* rContext,
     SkIRect clipConservativeBounds = SkIRect::MakeWH(sdc->width(), sdc->height());
     GrStyle style(SkStrokeRec::kFill_InitStyle);
     GrStyledShape shape(path, style);
-    skgpu::v1::PathRenderer::DrawPathArgs args{rContext,
-                                               std::move(paint),
-                                               &GrUserStencilSettings::kUnused,
-                                               sdc,
-                                               nullptr,
-                                               &clipConservativeBounds,
-                                               &matrix,
-                                               &shape,
-                                               aaType,
-                                               false};
+    skgpu::ganesh::PathRenderer::DrawPathArgs args{rContext,
+                                                   std::move(paint),
+                                                   &GrUserStencilSettings::kUnused,
+                                                   sdc,
+                                                   nullptr,
+                                                   &clipConservativeBounds,
+                                                   &matrix,
+                                                   &shape,
+                                                   aaType,
+                                                   false};
     pr.drawPath(args);
 }
 
@@ -869,10 +867,17 @@ DEF_GANESH_TEST_FOR_ALL_CONTEXTS(TriangulatingPathRendererTests,
                                  ctxInfo,
                                  CtsEnforcement::kNever) {
     auto ctx = ctxInfo.directContext();
-    auto sdc = skgpu::v1::SurfaceDrawContext::Make(
-            ctx, GrColorType::kRGBA_8888, nullptr, SkBackingFit::kApprox, {800, 800},
-            SkSurfaceProps(),/*label=*/{}, 1, GrMipmapped::kNo, GrProtected::kNo,
-            kTopLeft_GrSurfaceOrigin);
+    auto sdc = skgpu::ganesh::SurfaceDrawContext::Make(ctx,
+                                                       GrColorType::kRGBA_8888,
+                                                       nullptr,
+                                                       SkBackingFit::kApprox,
+                                                       {800, 800},
+                                                       SkSurfaceProps(),
+                                                       /*label=*/{},
+                                                       1,
+                                                       GrMipmapped::kNo,
+                                                       GrProtected::kNo,
+                                                       kTopLeft_GrSurfaceOrigin);
     if (!sdc) {
         return;
     }
@@ -885,7 +890,7 @@ DEF_GANESH_TEST_FOR_ALL_CONTEXTS(TriangulatingPathRendererTests,
         test_path(ctx, sdc.get(), createPath());
     }
     SkMatrix nonInvertibleMatrix = SkMatrix::Scale(0, 0);
-    std::unique_ptr<GrFragmentProcessor> fp(create_linear_gradient_processor(ctx));
+    std::unique_ptr<GrFragmentProcessor> fp(create_linear_gradient_processor(ctx, SkMatrix()));
     test_path(ctx, sdc.get(), create_path_17(), nonInvertibleMatrix, GrAAType::kCoverage,
               std::move(fp));
     test_path(ctx, sdc.get(), create_path_20(), SkMatrix(), GrAAType::kCoverage);
@@ -904,7 +909,7 @@ DEF_GANESH_TEST_FOR_ALL_CONTEXTS(TriangulatingPathRendererTests,
     test_path(ctx, sdc.get(), create_path_47(), SkMatrix(), GrAAType::kCoverage);
 }
 
-#endif // SK_GPU_V1
+#endif // defined(SK_GANESH)
 
 namespace {
 
