@@ -18,10 +18,10 @@
 #include "include/core/SkString.h"
 #include "include/core/SkTypeface.h"
 #include "include/core/SkTypes.h"
-#include "include/private/SkFixed.h"
+#include "include/private/base/SkFixed.h"
 #include "include/private/base/SkTemplates.h"
+#include "src/base/SkEndian.h"
 #include "src/base/SkUTF.h"
-#include "src/core/SkEndian.h"
 #include "src/core/SkFontDescriptor.h"
 #include "src/core/SkFontPriv.h"
 #include "src/core/SkTypefaceCache.h"
@@ -144,8 +144,7 @@ DEF_TEST(TypefaceRoundTrip, reporter) {
     int fontIndex;
     std::unique_ptr<SkStreamAsset> stream = typeface->openStream(&fontIndex);
 
-    sk_sp<SkFontMgr> fm = SkFontMgr::RefDefault();
-    sk_sp<SkTypeface> typeface2 = fm->makeFromStream(std::move(stream), fontIndex);
+    sk_sp<SkTypeface> typeface2 = SkTypeface::MakeFromStream(std::move(stream), fontIndex);
     REPORTER_ASSERT(reporter, typeface2);
 }
 
@@ -525,18 +524,22 @@ DEF_TEST(TypefaceCache, reporter) {
     REPORTER_ASSERT(reporter, t1->unique());
 }
 
-static void check_serialize_behaviors(sk_sp<SkTypeface> tf, bool isLocalData,
-                                      skiatest::Reporter* reporter) {
+static void check_serialize_behaviors(sk_sp<SkTypeface> tf, skiatest::Reporter* reporter) {
     if (!tf) {
         return;
     }
+
+    SkFontDescriptor desc;
+    bool serialize;
+    tf->getFontDescriptor(&desc, &serialize);
+
     auto data0 = tf->serialize(SkTypeface::SerializeBehavior::kDoIncludeData);
     auto data1 = tf->serialize(SkTypeface::SerializeBehavior::kDontIncludeData);
     auto data2 = tf->serialize(SkTypeface::SerializeBehavior::kIncludeDataIfLocal);
 
     REPORTER_ASSERT(reporter, data0->size() >= data1->size());
 
-    if (isLocalData) {
+    if (serialize) {
         REPORTER_ASSERT(reporter, data0->equals(data2.get()));
     } else {
         REPORTER_ASSERT(reporter, data1->equals(data2.get()));
@@ -544,10 +547,9 @@ static void check_serialize_behaviors(sk_sp<SkTypeface> tf, bool isLocalData,
 }
 
 DEF_TEST(Typeface_serialize, reporter) {
-    check_serialize_behaviors(SkTypeface::MakeDefault(), false, reporter);
-    check_serialize_behaviors(SkTypeface::MakeFromStream(
-                                         GetResourceAsStream("fonts/Distortable.ttf")),
-                              true, reporter);
+    check_serialize_behaviors(SkTypeface::MakeDefault(), reporter);
+    check_serialize_behaviors(
+        SkTypeface::MakeFromStream(GetResourceAsStream("fonts/Distortable.ttf")), reporter);
 
 }
 
