@@ -7,18 +7,26 @@
 
 #include "include/core/SkRefCnt.h"
 #include "include/core/SkSpan.h"
-#include "include/utils/SkRandom.h"
+#include "include/private/base/SkTemplates.h"
+#include "src/base/SkRandom.h"
+#include "src/base/SkTSearch.h"
+#include "src/base/SkTSort.h"
+#include "src/base/SkZip.h"
 #include "src/core/SkEnumerate.h"
-#include "src/core/SkTSearch.h"
-#include "src/core/SkTSort.h"
-#include "src/core/SkZip.h"
 #include "tests/Test.h"
 
 #include <array>
+#include <cstddef>
+#include <cstdint>
 #include <initializer_list>
+#include <memory>
+#include <new>
 #include <tuple>
+#include <utility>
 #include <vector>
 
+using namespace skia_private;
+
 class RefClass : public SkRefCnt {
 public:
     RefClass(int n) : fN(n) {}
@@ -58,7 +66,7 @@ static void test_autostarray(skiatest::Reporter* reporter) {
     REPORTER_ASSERT(reporter, obj1.unique());
 
     {
-        SkAutoSTArray<2, sk_sp<RefClass> > tmp;
+        AutoSTArray<2, sk_sp<RefClass> > tmp;
         REPORTER_ASSERT(reporter, 0 == tmp.count());
 
         tmp.reset(0);   // test out reset(0) when already at 0
@@ -87,7 +95,7 @@ static void test_autostarray(skiatest::Reporter* reporter) {
 
     {
         // test out allocating ctor (this should allocate new memory)
-        SkAutoSTArray<2, sk_sp<RefClass> > tmp(4);
+        AutoSTArray<2, sk_sp<RefClass> > tmp(4);
         REPORTER_ASSERT(reporter, 4 == tmp.count());
 
         tmp[0].reset(SkRef(&obj0));
@@ -175,57 +183,6 @@ DEF_TEST(Utils, reporter) {
     test_autostarray(reporter);
 }
 
-DEF_TEST(SkSpan, reporter) {
-    // Test constness preservation for SkMakeSpan.
-    {
-        std::vector<int> v = {{1, 2, 3, 4, 5}};
-        auto s = SkMakeSpan(v);
-        REPORTER_ASSERT(reporter, s[3] == 4);
-        s[3] = 100;
-        REPORTER_ASSERT(reporter, s[3] == 100);
-    }
-
-    {
-        std::vector<int> t = {{1, 2, 3, 4, 5}};
-        const std::vector<int>& v = t;
-        auto s = SkMakeSpan(v);
-        //s[3] = 100; // Should fail to compile
-        REPORTER_ASSERT(reporter, s[3] == 4);
-        REPORTER_ASSERT(reporter, t[3] == 4);
-        t[3] = 100;
-        REPORTER_ASSERT(reporter, s[3] == 100);
-    }
-
-    {
-        std::array<int, 5> v = {{1, 2, 3, 4, 5}};
-        auto s = SkMakeSpan(v);
-        REPORTER_ASSERT(reporter, s[3] == 4);
-        s[3] = 100;
-        REPORTER_ASSERT(reporter, s[3] == 100);
-        auto s1 = s.subspan(1,3);
-        REPORTER_ASSERT(reporter, s1.size() == 3);
-        REPORTER_ASSERT(reporter, s1.front() == 2);
-        REPORTER_ASSERT(reporter, s1.back() == 100);
-    }
-
-    {
-        std::array<int, 5> t = {{1, 2, 3, 4, 5}};
-        const std::array<int, 5>& v = t;
-        auto s = SkMakeSpan(v);
-        //s[3] = 100; // Should fail to compile
-        REPORTER_ASSERT(reporter, s[3] == 4);
-        REPORTER_ASSERT(reporter, t[3] == 4);
-        t[3] = 100;
-        REPORTER_ASSERT(reporter, s[3] == 100);
-    }
-
-    {
-        std::vector<int> v;
-        auto s = SkMakeSpan(v);
-        REPORTER_ASSERT(reporter, s.empty());
-    }
-}
-
 DEF_TEST(SkEnumerate, reporter) {
 
     int A[] = {1, 2, 3, 4};
@@ -257,14 +214,14 @@ DEF_TEST(SkEnumerate, reporter) {
     REPORTER_ASSERT(reporter, check == 4);
 
     check = 0;
-    for (auto [i, v] : SkMakeEnumerate(SkMakeSpan(vec))) {
+    for (auto [i, v] : SkMakeEnumerate(SkSpan(vec))) {
         REPORTER_ASSERT(reporter, i == check);
         REPORTER_ASSERT(reporter, v == (int)check+1);
         check++;
     }
 
     {
-        auto e = SkMakeEnumerate(SkMakeSpan(vec)).first(2);
+        auto e = SkMakeEnumerate(SkSpan(vec)).first(2);
         for (auto[i, v] : e) {
             REPORTER_ASSERT(reporter, v == (int) i + 1);
         }
@@ -272,7 +229,7 @@ DEF_TEST(SkEnumerate, reporter) {
     }
 
     {
-        auto e = SkMakeEnumerate(SkMakeSpan(vec)).last(2);
+        auto e = SkMakeEnumerate(SkSpan(vec)).last(2);
         for (auto[i, v] : e) {
             REPORTER_ASSERT(reporter, v == (int) i + 1);
         }
@@ -280,7 +237,7 @@ DEF_TEST(SkEnumerate, reporter) {
     }
 
     {
-        auto e = SkMakeEnumerate(SkMakeSpan(vec)).subspan(1, 2);
+        auto e = SkMakeEnumerate(SkSpan(vec)).subspan(1, 2);
         for (auto[i, v] : e) {
             REPORTER_ASSERT(reporter, v == (int) i + 1);
         }
@@ -297,7 +254,7 @@ DEF_TEST(SkEnumerate, reporter) {
         };
 
         I is[10];
-        auto s = SkMakeSpan(is);
+        auto s = SkSpan(is);
         for (auto [i, v] : SkMakeEnumerate(s)) {
             new (&v) I(i);
         }
@@ -311,12 +268,12 @@ DEF_TEST(SkEnumerate, reporter) {
     {
         std::unique_ptr<int> is[10];
         std::unique_ptr<int> os[10];
-        auto s = SkMakeSpan(is);
+        auto s = SkSpan(is);
         for (auto [i, v] : SkMakeEnumerate(s)) {
             v = std::make_unique<int>(i);
         }
 
-        for (auto [i, v] : SkMakeEnumerate(SkMakeSpan(os))) {
+        for (auto [i, v] : SkMakeEnumerate(SkSpan(os))) {
             v = std::move(s[i]);
         }
 
@@ -329,7 +286,7 @@ DEF_TEST(SkEnumerate, reporter) {
     {
         std::unique_ptr<int> is[10];
         std::unique_ptr<int> os[10];
-        auto s = SkMakeSpan(is);
+        auto s = SkSpan(is);
         for (auto [i, v] : SkMakeEnumerate(s)) {
             v = std::make_unique<int>(i);
         }
@@ -350,7 +307,7 @@ DEF_TEST(SkZip, reporter) {
     const float B[] = {10.f, 20.f, 30.f, 40.f};
     std::vector<int> C = {{20, 30, 40, 50}};
     std::array<int, 4> D = {{100, 200, 300, 400}};
-    SkSpan<int> S = SkMakeSpan(C);
+    SkSpan<int> S = SkSpan(C);
 
     // Check SkZip calls
     SkZip<uint16_t, const float, int, int, int>
@@ -523,7 +480,7 @@ DEF_TEST(SkMakeZip, reporter) {
     const float B[] = {10.f, 20.f, 30.f, 40.f};
     const std::vector<int> C = {{20, 30, 40, 50}};
     std::array<int, 4> D = {{100, 200, 300, 400}};
-    SkSpan<const int> S = SkMakeSpan(C);
+    SkSpan<const int> S = SkSpan(C);
     uint16_t* P = &A[0];
     {
         // Check make zip
