@@ -15,19 +15,21 @@
 #include "include/core/SkColorType.h"
 #include "include/core/SkImage.h"
 #include "include/core/SkRefCnt.h"
+#include "include/gpu/GpuTypes.h"
 #include "include/gpu/GrBackendSurface.h"
 #include "include/gpu/GrDirectContext.h"
 #include "include/gpu/GrTypes.h"
+#include "include/gpu/ganesh/SkImageGanesh.h"
 #include "include/gpu/vk/GrVkTypes.h"
 #include "include/private/gpu/ganesh/GrTypesPriv.h"
 #include "src/gpu/ganesh/GrDirectContextPriv.h"
 #include "src/gpu/ganesh/GrSurface.h"
 #include "src/gpu/ganesh/GrSurfaceProxy.h"
 #include "src/gpu/ganesh/GrTextureProxy.h"
+#include "src/gpu/ganesh/image/GrImageUtils.h"
 #include "src/gpu/ganesh/vk/GrVkCaps.h"
 #include "src/gpu/ganesh/vk/GrVkImage.h"
 #include "src/gpu/ganesh/vk/GrVkTexture.h"
-#include "src/image/SkImage_Base.h"
 #include "tests/CtsEnforcement.h"
 #include "tests/Test.h"
 #include "tools/gpu/ManagedBackendTexture.h"
@@ -68,18 +70,20 @@ DEF_GANESH_TEST_FOR_VULKAN_CONTEXT(VkDRMModifierTest, reporter, ctxInfo, CtsEnfo
     REPORTER_ASSERT(reporter, drmBETex.textureType() == GrTextureType::kExternal);
 
     // Now wrap the texture in an SkImage and make sure we have the required read only properties
-    sk_sp<SkImage> drmImage = SkImage::MakeFromTexture(dContext,
-                                                       drmBETex,
-                                                       kTopLeft_GrSurfaceOrigin,
-                                                       kRGBA_8888_SkColorType,
-                                                       kPremul_SkAlphaType,
-                                                       nullptr);
+    sk_sp<SkImage> drmImage = SkImages::BorrowTextureFrom(dContext,
+                                                          drmBETex,
+                                                          kTopLeft_GrSurfaceOrigin,
+                                                          kRGBA_8888_SkColorType,
+                                                          kPremul_SkAlphaType,
+                                                          nullptr);
     REPORTER_ASSERT(reporter, drmImage);
 
-    REPORTER_ASSERT(reporter,
-            GrBackendTexture::TestingOnly_Equals(drmImage->getBackendTexture(false), drmBETex));
+    GrBackendTexture actual;
+    bool ok = SkImages::GetBackendTextureFromImage(drmImage, &actual, false);
+    REPORTER_ASSERT(reporter, ok);
+    REPORTER_ASSERT(reporter, GrBackendTexture::TestingOnly_Equals(actual, drmBETex));
 
-    auto[view, _] = as_IB(drmImage.get()) -> asView(dContext, GrMipmapped::kNo);
+    auto [view, _] = skgpu::ganesh::AsView(dContext, drmImage, GrMipmapped::kNo);
     REPORTER_ASSERT(reporter, view);
     const GrSurfaceProxy* proxy = view.proxy();
     REPORTER_ASSERT(reporter, proxy);
@@ -125,15 +129,15 @@ DEF_GANESH_TEST_FOR_VULKAN_CONTEXT(VkImageLayoutTest, reporter, ctxInfo, CtsEnfo
     // Setting back the layout since we didn't actually change it
     backendTex1.setVkImageLayout(initLayout);
 
-    sk_sp<SkImage> wrappedImage = SkImage::MakeFromTexture(
-            dContext,
-            backendTex1,
-            kTopLeft_GrSurfaceOrigin,
-            kRGBA_8888_SkColorType,
-            kPremul_SkAlphaType,
-            /*color space*/ nullptr,
-            sk_gpu_test::ManagedBackendTexture::ReleaseProc,
-            mbet->releaseContext());
+    sk_sp<SkImage> wrappedImage =
+            SkImages::BorrowTextureFrom(dContext,
+                                        backendTex1,
+                                        kTopLeft_GrSurfaceOrigin,
+                                        kRGBA_8888_SkColorType,
+                                        kPremul_SkAlphaType,
+                                        /*color space*/ nullptr,
+                                        sk_gpu_test::ManagedBackendTexture::ReleaseProc,
+                                        mbet->releaseContext());
     REPORTER_ASSERT(reporter, wrappedImage.get());
 
     GrSurfaceProxy* proxy = sk_gpu_test::GetTextureImageProxy(wrappedImage.get(), dContext);
@@ -150,7 +154,9 @@ DEF_GANESH_TEST_FOR_VULKAN_CONTEXT(VkImageLayoutTest, reporter, ctxInfo, CtsEnfo
     REPORTER_ASSERT(reporter, backendTex1.getVkImageInfo(&info));
     REPORTER_ASSERT(reporter, VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL == info.fImageLayout);
 
-    GrBackendTexture backendTexImage = wrappedImage->getBackendTexture(false);
+    GrBackendTexture backendTexImage;
+    bool ok = SkImages::GetBackendTextureFromImage(wrappedImage, &backendTexImage, false);
+    REPORTER_ASSERT(reporter, ok);
     REPORTER_ASSERT(reporter, backendTexImage.getVkImageInfo(&info));
     REPORTER_ASSERT(reporter, VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL == info.fImageLayout);
 
@@ -220,7 +226,7 @@ DEF_GANESH_TEST_FOR_VULKAN_CONTEXT(VkTransitionExternalQueueTest, reporter, ctxI
 
     GrBackendTexture vkExtTex(1, 1, vkInfo);
     REPORTER_ASSERT(reporter, vkExtTex.isValid());
-    image = SkImage::MakeFromTexture(dContext, vkExtTex, kTopLeft_GrSurfaceOrigin,
+    image = SkImages::BorrowTextureFrom(dContext, vkExtTex, kTopLeft_GrSurfaceOrigin,
                                      kRGBA_8888_SkColorType, kPremul_SkAlphaType, nullptr, nullptr,
                                      nullptr);
 
