@@ -15,11 +15,13 @@
 #include "include/gpu/GpuTypes.h"
 #include "include/gpu/graphite/BackendTexture.h"
 #include "include/gpu/graphite/Context.h"
+#include "include/gpu/graphite/Image.h"
 #include "include/gpu/graphite/Recorder.h"
 #include "include/gpu/graphite/Recording.h"
+#include "include/gpu/graphite/Surface.h"
 #include "include/gpu/graphite/TextureInfo.h"
+#include "src/base/SkRectMemcpy.h"
 #include "src/core/SkAutoPixmapStorage.h"
-#include "src/core/SkConvertPixels.h"
 #include "src/core/SkImageInfoPriv.h"
 #include "src/gpu/graphite/Caps.h"
 #include "src/gpu/graphite/ContextPriv.h"
@@ -29,6 +31,8 @@
 #include "tests/TestUtils.h"
 #include "tools/ToolUtils.h"
 
+using Mipmapped = skgpu::Mipmapped;
+
 static constexpr int min_rgb_channel_bits(SkColorType ct) {
     switch (ct) {
         case kUnknown_SkColorType:            return 0;
@@ -48,6 +52,7 @@ static constexpr int min_rgb_channel_bits(SkColorType ct) {
         case kRGB_101010x_SkColorType:        return 10;
         case kBGRA_1010102_SkColorType:       return 10;
         case kBGR_101010x_SkColorType:        return 10;
+        case kBGR_101010x_XR_SkColorType:     return 10;
         case kGray_8_SkColorType:             return 8;   // counting gray as "rgb"
         case kRGBA_F16Norm_SkColorType:       return 10;  // just counting the mantissa
         case kRGBA_F16_SkColorType:           return 10;  // just counting the mantissa
@@ -77,6 +82,7 @@ static constexpr int alpha_channel_bits(SkColorType ct) {
         case kRGB_101010x_SkColorType:        return 0;
         case kBGRA_1010102_SkColorType:       return 2;
         case kBGR_101010x_SkColorType:        return 0;
+        case kBGR_101010x_XR_SkColorType:     return 0;
         case kGray_8_SkColorType:             return 0;
         case kRGBA_F16Norm_SkColorType:       return 10;  // just counting the mantissa
         case kRGBA_F16_SkColorType:           return 10;  // just counting the mantissa
@@ -158,7 +164,7 @@ struct GraphiteReadPixelTestRules {
 
 // Makes a src populated with the pixmap. The src should get its image info (or equivalent) from
 // the pixmap.
-template <typename T> using GraphiteSrcFactory = T(SkPixmap&);
+template <typename T> using GraphiteSrcFactory = T(skgpu::graphite::Recorder*, SkPixmap&);
 
 enum class Result {
     kFail,
@@ -180,7 +186,7 @@ SkPixmap make_pixmap_have_valid_alpha_type(SkPixmap pm) {
 static SkAutoPixmapStorage make_ref_data(const SkImageInfo& info, bool forceOpaque) {
     SkAutoPixmapStorage result;
     result.alloc(info);
-    auto surface = SkSurface::MakeRasterDirect(make_pixmap_have_valid_alpha_type(result));
+    auto surface = SkSurfaces::WrapPixels(make_pixmap_have_valid_alpha_type(result));
     if (!surface) {
         return result;
     }
@@ -217,6 +223,7 @@ static SkAutoPixmapStorage make_ref_data(const SkImageInfo& info, bool forceOpaq
 
 template <typename T>
 static void graphite_read_pixels_test_driver(skiatest::Reporter* reporter,
+                                             skgpu::graphite::Context* context,
                                              const GraphiteReadPixelTestRules& rules,
                                              const std::function<GraphiteSrcFactory<T>>& srcFactory,
                                              const std::function<GraphiteReadSrcFn<T>>& read,
@@ -427,7 +434,9 @@ static void graphite_read_pixels_test_driver(skiatest::Reporter* reporter,
                 refPixels.readPixels(readPixmap, 0, 0);
             }
 
-            auto src = srcFactory(srcPixels);
+            std::unique_ptr<skgpu::graphite::Recorder> recorder = context->makeRecorder();
+
+            auto src = srcFactory(recorder.get(), srcPixels);
             if (!src) {
                 continue;
             }
@@ -446,6 +455,7 @@ static void graphite_read_pixels_test_driver(skiatest::Reporter* reporter,
                 // ComparePixels will end up converting these types to kUnknown
                 // because there's no corresponding GrColorType, and hence it will fail
                 if (readCT == kRGB_101010x_SkColorType ||
+                    readCT == kBGR_101010x_XR_SkColorType ||
                     readCT == kBGR_101010x_SkColorType) {
                     continue;
                 }
@@ -508,8 +518,7 @@ DEF_GRAPHITE_TEST_FOR_RENDERING_CONTEXTS(ImageAsyncReadPixelsGraphite,
                                          reporter,
                                          context) {
     using Image = sk_sp<SkImage>;
-    using Recorder = skgpu::graphite::Recorder;
-    using Renderable = skgpu::graphite::Renderable;
+    using Renderable = skgpu::Renderable;
     using TextureInfo = skgpu::graphite::TextureInfo;
 
     auto reader = std::function<GraphiteReadSrcFn<Image>>([context](const Image& image,
@@ -521,7 +530,7 @@ DEF_GRAPHITE_TEST_FOR_RENDERING_CONTEXTS(ImageAsyncReadPixelsGraphite,
         // types.
         TextureInfo texInfo = context->priv().caps()->getDefaultSampledTextureInfo(
                 image->colorType(),
-                skgpu::graphite::Mipmapped::kNo,
+                Mipmapped::kNo,
                 skgpu::Protected::kNo,
                 Renderable::kYes);
         if (!context->priv().caps()->isRenderable(texInfo)) {
@@ -549,14 +558,14 @@ DEF_GRAPHITE_TEST_FOR_RENDERING_CONTEXTS(ImageAsyncReadPixelsGraphite,
     rules.fAllowUnpremulSrc = true;
     rules.fUncontainedRectSucceeds = false;
 
-    std::unique_ptr<Recorder> recorder = context->makeRecorder();
-
     for (auto renderable : {Renderable::kNo, Renderable::kYes}) {
-        auto factory = std::function<GraphiteSrcFactory<Image>>([&](const SkPixmap& src) {
+        auto factory = std::function<GraphiteSrcFactory<Image>>([&](
+                skgpu::graphite::Recorder* recorder,
+                const SkPixmap& src) {
             // TODO: put this in the equivalent of sk_gpu_test::MakeBackendTextureImage
             TextureInfo info = recorder->priv().caps()->getDefaultSampledTextureInfo(
                     src.colorType(),
-                    skgpu::graphite::Mipmapped::kNo,
+                    Mipmapped::kNo,
                     skgpu::Protected::kNo,
                     renderable);
             auto texture = recorder->createBackendTexture(src.dimensions(), info);
@@ -564,11 +573,11 @@ DEF_GRAPHITE_TEST_FOR_RENDERING_CONTEXTS(ImageAsyncReadPixelsGraphite,
                 return (Image)(nullptr);
             }
 
-            Image image = SkImage::MakeGraphiteFromBackendTexture(recorder.get(),
-                                                                  texture,
-                                                                  src.colorType(),
-                                                                  src.alphaType(),
-                                                                  /*colorSpace=*/nullptr);
+            Image image = SkImages::AdoptTextureFrom(recorder,
+                                                     texture,
+                                                     src.colorType(),
+                                                     src.alphaType(),
+                                                     /*colorSpace=*/nullptr);
 
             std::unique_ptr<skgpu::graphite::Recording> recording = recorder->snap();
             skgpu::graphite::InsertRecordingInfo recordingInfo;
@@ -578,7 +587,7 @@ DEF_GRAPHITE_TEST_FOR_RENDERING_CONTEXTS(ImageAsyncReadPixelsGraphite,
             return image;
         });
         auto label = SkStringPrintf("Renderable: %d", (int)renderable);
-        graphite_read_pixels_test_driver(reporter, rules, factory, reader, label);
+        graphite_read_pixels_test_driver(reporter, context, rules, factory, reader, label);
     }
 
     // It's possible that we've created an Image using the factory, but then don't try to do
@@ -589,7 +598,6 @@ DEF_GRAPHITE_TEST_FOR_RENDERING_CONTEXTS(ImageAsyncReadPixelsGraphite,
 DEF_GRAPHITE_TEST_FOR_RENDERING_CONTEXTS(SurfaceAsyncReadPixelsGraphite,
                                          reporter,
                                          context) {
-    using Recorder = skgpu::graphite::Recorder;
     using Surface = sk_sp<SkSurface>;
 
     auto reader = std::function<GraphiteReadSrcFn<Surface>>([context](const Surface& surface,
@@ -619,24 +627,24 @@ DEF_GRAPHITE_TEST_FOR_RENDERING_CONTEXTS(SurfaceAsyncReadPixelsGraphite,
     rules.fAllowUnpremulSrc = true;
     rules.fUncontainedRectSucceeds = false;
 
-    std::unique_ptr<Recorder> recorder = context->makeRecorder();
-    auto factory = std::function<GraphiteSrcFactory<Surface>>([&](const SkPixmap& src) {
-        Surface surface = SkSurface::MakeGraphite(recorder.get(),
-                                                  src.info(),
-                                                  skgpu::graphite::Mipmapped::kNo,
-                                                  /*surfaceProps=*/nullptr);
-        if (surface) {
-            surface->writePixels(src, 0, 0);
+    auto factory = std::function<GraphiteSrcFactory<Surface>>(
+            [&](skgpu::graphite::Recorder* recorder, const SkPixmap& src) {
+                Surface surface = SkSurfaces::RenderTarget(recorder,
+                                                           src.info(),
+                                                           Mipmapped::kNo,
+                                                           /*surfaceProps=*/nullptr);
+                if (surface) {
+                    surface->writePixels(src, 0, 0);
 
-            std::unique_ptr<skgpu::graphite::Recording> recording = recorder->snap();
-            skgpu::graphite::InsertRecordingInfo recordingInfo;
-            recordingInfo.fRecording = recording.get();
-            context->insertRecording(recordingInfo);
-        }
+                    std::unique_ptr<skgpu::graphite::Recording> recording = recorder->snap();
+                    skgpu::graphite::InsertRecordingInfo recordingInfo;
+                    recordingInfo.fRecording = recording.get();
+                    context->insertRecording(recordingInfo);
+                }
 
-        return surface;
-    });
-    graphite_read_pixels_test_driver(reporter, rules, factory, reader, {});
+                return surface;
+            });
+    graphite_read_pixels_test_driver(reporter, context, rules, factory, reader, {});
 
     // It's possible that we've created an Image using the factory, but then don't try to do
     // readPixels on it, leaving a hanging command buffer. So we submit here to clean up.
