@@ -5,7 +5,7 @@
  * found in the LICENSE file.
  */
 
-#include "include/private/SkHalf.h"
+#include "src/base/SkHalf.h"
 #include "src/core/SkSLTypeShared.h"
 #include "src/gpu/graphite/PipelineData.h"
 #include "src/gpu/graphite/Uniform.h"
@@ -51,12 +51,12 @@ static constexpr int32_t kInts[16] = { 1,  -2,  3,  -4,
                                       13, -14, 15, -16 };
 
 static size_t element_size(Layout layout, SkSLType type) {
-    // Metal should encode half-precision uniforms in 16 bits.
-    // Other layouts should always encode uniforms in 32 bit.
+    // Metal encodes half-precision uniforms in 16 bits.
+    // Other layouts are expected to encode uniforms in 32 bits.
     return (layout == Layout::kMetal && !SkSLTypeIsFullPrecisionNumericType(type)) ? 2 : 4;
 }
 
-DEF_TEST(UniformManagerCheckSingleUniform, r) {
+DEF_GRAPHITE_TEST(UniformManagerCheckSingleUniform, r) {
     // Verify that the uniform manager can hold all the basic uniform types, in every layout.
     for (Layout layout : kLayouts) {
         UniformManager mgr(layout);
@@ -72,7 +72,7 @@ DEF_TEST(UniformManagerCheckSingleUniform, r) {
     }
 }
 
-DEF_TEST(UniformManagerCheckFloatEncoding, r) {
+DEF_GRAPHITE_TEST(UniformManagerCheckFloatEncoding, r) {
     // Verify that the uniform manager encodes float data properly.
     for (Layout layout : kLayouts) {
         UniformManager mgr(layout);
@@ -96,13 +96,14 @@ DEF_TEST(UniformManagerCheckFloatEncoding, r) {
             const void* validData = (elementSize == 4) ? (const void*)kFloats : (const void*)kHalfs;
             REPORTER_ASSERT(r, uniformData.size() >= vecLength * elementSize);
             REPORTER_ASSERT(r, 0 == memcmp(validData, uniformData.data(), vecLength * elementSize),
-                            "Layout:%d Type:%d float encoding failed", (int)layout, (int)type);
+                            "Layout: %d - Type: %s float encoding failed",
+                            (int)layout, SkSLTypeString(type));
             mgr.reset();
         }
     }
 }
 
-DEF_TEST(UniformManagerCheckIntEncoding, r) {
+DEF_GRAPHITE_TEST(UniformManagerCheckIntEncoding, r) {
     // Verify that the uniform manager encodes int data properly.
     for (Layout layout : kLayouts) {
         UniformManager mgr(layout);
@@ -125,13 +126,14 @@ DEF_TEST(UniformManagerCheckIntEncoding, r) {
             const void* validData = (elementSize == 4) ? (const void*)kInts : (const void*)kShorts;
             REPORTER_ASSERT(r, uniformData.size() >= vecLength * elementSize);
             REPORTER_ASSERT(r, 0 == memcmp(validData, uniformData.data(), vecLength * elementSize),
-                            "Layout:%d Type:%d int encoding failed", (int)layout, (int)type);
+                            "Layout: %d - Type: %s int encoding failed",
+                            (int)layout, SkSLTypeString(type));
             mgr.reset();
         }
     }
 }
 
-DEF_TEST(UniformManagerCheckScalarVectorPacking, r) {
+DEF_GRAPHITE_TEST(UniformManagerCheckScalarVectorPacking, r) {
     // Verify that the uniform manager can pack scalars and vectors of identical type correctly.
     for (Layout layout : kLayouts) {
         UniformManager mgr(layout);
@@ -150,19 +152,20 @@ DEF_TEST(UniformManagerCheckScalarVectorPacking, r) {
             mgr.write(type, kFloats);
             mgr.doneWithExpectedUniforms();
 
-            // Verify that the uniform data was packed as tight as it should be.
+            // Verify the uniform data packing.
             UniformDataBlock uniformData = mgr.finishUniformDataBlock();
             size_t elementSize = element_size(layout, type);
-            // Vec3s should be packed as if they were vec4s.
+            // Vec3s must be laid out as if they were vec4s.
             size_t effectiveVecLength = (vecLength == 3) ? 4 : vecLength;
             REPORTER_ASSERT(r, uniformData.size() == elementSize * effectiveVecLength * 3,
-                            "Layout:%d Type:%d tight packing failed", (int)layout, (int)type);
+                            "Layout: %d - Type: %s tight packing failed",
+                            (int)layout, SkSLTypeString(type));
             mgr.reset();
         }
     }
 }
 
-DEF_TEST(UniformManagerCheckMatrixPacking, r) {
+DEF_GRAPHITE_TEST(UniformManagerCheckMatrixPacking, r) {
     // Verify that the uniform manager can pack matrices correctly.
     for (Layout layout : kLayouts) {
         UniformManager mgr(layout);
@@ -181,19 +184,28 @@ DEF_TEST(UniformManagerCheckMatrixPacking, r) {
             mgr.write(type, kFloats);
             mgr.doneWithExpectedUniforms();
 
-            // Verify that the uniform data was packed as tight as it should be.
+            // Verify the uniform data packing.
             UniformDataBlock uniformData = mgr.finishUniformDataBlock();
             size_t elementSize = element_size(layout, type);
-            // In all layouts, mat3s should burn 12 elements, not 9.
-            size_t numElements = (matrixSize == 3) ? 12 : (matrixSize * matrixSize);
+            // In all layouts, mat3s burn 12 elements, not 9. In std140, mat2s burn 8 elements
+            // instead of 4.
+            size_t numElements;
+            if (matrixSize == 3) {
+                numElements = 12;
+            } else if (matrixSize == 2 && layout == Layout::kStd140) {
+                numElements = 8;
+            } else {
+                numElements = matrixSize * matrixSize;
+            }
             REPORTER_ASSERT(r, uniformData.size() == elementSize * numElements * 3,
-                            "Layout:%d Type:%d matrix packing failed", (int)layout, (int)type);
+                            "Layout: %d - Type: %s matrix packing failed",
+                            (int)layout, SkSLTypeString(type));
             mgr.reset();
         }
     }
 }
 
-DEF_TEST(UniformManagerCheckPaddingScalarVector, r) {
+DEF_GRAPHITE_TEST(UniformManagerCheckPaddingScalarVector, r) {
     // Verify that the uniform manager properly adds padding between pairs of scalar/vector.
     for (Layout layout : kLayouts) {
         UniformManager mgr(layout);
@@ -237,8 +249,8 @@ DEF_TEST(UniformManagerCheckPaddingScalarVector, r) {
                                         elementSize1;
                     UniformDataBlock uniformData = mgr.finishUniformDataBlock();
                     REPORTER_ASSERT(r, uniformData.size() == size,
-                                    "Layout:%d Types:%d %d padding test failed",
-                                    (int)layout, (int)type1, (int)type2);
+                                    "Layout: %d - Types: %s, %s padding test failed",
+                                    (int)layout, SkSLTypeString(type1), SkSLTypeString(type2));
                 } else if (elementSize1 == 2 && elementSize2 == 4) {
                     // Elements in the array below correspond to 16 bits apiece.
                     // The expected uniform layout is listed as strings below.
@@ -255,8 +267,8 @@ DEF_TEST(UniformManagerCheckPaddingScalarVector, r) {
                     const size_t size = strlen(kExpectedLayout[vecLength1][vecLength2]) * 2;
                     UniformDataBlock uniformData = mgr.finishUniformDataBlock();
                     REPORTER_ASSERT(r, uniformData.size() == size,
-                                    "Layout:%d Types:%d %d padding test failed",
-                                    (int)layout, (int)type1, (int)type2);
+                                    "Layout: %d - Types: %s, %s padding test failed",
+                                    (int)layout, SkSLTypeString(type1), SkSLTypeString(type2));
                 } else if (elementSize1 == 4 && elementSize2 == 2) {
                     // Elements in the array below correspond to 16 bits apiece.
                     // The expected uniform layout is listed as strings below.
@@ -281,8 +293,8 @@ DEF_TEST(UniformManagerCheckPaddingScalarVector, r) {
                     const size_t size = strlen(kExpectedLayout[vecLength1][vecLength2]) * 2;
                     UniformDataBlock uniformData = mgr.finishUniformDataBlock();
                     REPORTER_ASSERT(r, uniformData.size() == size,
-                                    "Layout:%d Types:%d %d padding test failed",
-                                    (int)layout, (int)type1, (int)type2);
+                                    "Layout: %d - Types: %s, %s padding test failed",
+                                    (int)layout, SkSLTypeString(type1), SkSLTypeString(type2));
                 } else {
                     ERRORF(r, "Unexpected element sizes: %zu %zu", elementSize1, elementSize2);
                 }
@@ -292,7 +304,7 @@ DEF_TEST(UniformManagerCheckPaddingScalarVector, r) {
     }
 }
 
-DEF_TEST(UniformManagerCheckPaddingVectorMatrix, r) {
+DEF_GRAPHITE_TEST(UniformManagerCheckPaddingVectorMatrix, r) {
     // Verify that the uniform manager properly adds padding between vectors and matrices.
     for (Layout layout : kLayouts) {
         UniformManager mgr(layout);
@@ -325,19 +337,31 @@ DEF_TEST(UniformManagerCheckPaddingVectorMatrix, r) {
                     // A/B: uniform values.
                     // a/b: padding as part of the uniform type (vec3 takes 4 slots)
                     // _  : padding between uniforms for alignment
-                    static constexpr const char* kExpectedLayout[5][5] = {
-                        { "", "", "",         "",                     "" },
-                        { "", "", "A_BBBB",   "A___BBBbBBBbBBBb", "A___BBBBBBBBBBBBBBBB" },
-                        { "", "", "AABBBB",   "AA__BBBbBBBbBBBb", "AA__BBBBBBBBBBBBBBBB" },
-                        { "", "", "AAAaBBBB", "AAAaBBBbBBBbBBBb", "AAAaBBBBBBBBBBBBBBBB" },
-                        { "", "", "AAAABBBB", "AAAABBBbBBBbBBBb", "AAAABBBBBBBBBBBBBBBB" },
+                    static constexpr const char* kExpectedLayout[2][5][5] = {
+                        // std140 layout
+                        {
+                            { "", "", "",             "",                 "" },
+                            { "", "", "A___BBbbBBbb", "A___BBBbBBBbBBBb", "A___BBBBBBBBBBBBBBBB" },
+                            { "", "", "AA__BBbbBBbb", "AA__BBBbBBBbBBBb", "AA__BBBBBBBBBBBBBBBB" },
+                            { "", "", "AAAaBBbbBBbb", "AAAaBBBbBBBbBBBb", "AAAaBBBBBBBBBBBBBBBB" },
+                            { "", "", "AAAABBbbBBbb", "AAAABBBbBBBbBBBb", "AAAABBBBBBBBBBBBBBBB" },
+                        },
+                        // All other layouts
+                        {
+                            { "", "", "",         "",                 "" },
+                            { "", "", "A_BBBB",   "A___BBBbBBBbBBBb", "A___BBBBBBBBBBBBBBBB" },
+                            { "", "", "AABBBB",   "AA__BBBbBBBbBBBb", "AA__BBBBBBBBBBBBBBBB" },
+                            { "", "", "AAAaBBBB", "AAAaBBBbBBBbBBBb", "AAAaBBBBBBBBBBBBBBBB" },
+                            { "", "", "AAAABBBB", "AAAABBBbBBBbBBBb", "AAAABBBBBBBBBBBBBBBB" },
+                        },
                     };
-                    const size_t size = strlen(kExpectedLayout[vecLength1][matSize2]) *
+                    int layoutIdx = static_cast<int>(layout != Layout::kStd140);
+                    const size_t size = strlen(kExpectedLayout[layoutIdx][vecLength1][matSize2]) *
                                         elementSize1;
                     UniformDataBlock uniformData = mgr.finishUniformDataBlock();
                     REPORTER_ASSERT(r, uniformData.size() == size,
-                                    "Types:%d %d vector-matrix padding test failed",
-                                    (int)type1, (int)type2);
+                                    "Types: %s, %s vector-matrix padding test failed",
+                                    SkSLTypeString(type1), SkSLTypeString(type2));
                 } else if (elementSize1 == 2 && elementSize2 == 4) {
                     // Elements in the array below correspond to 16 bits apiece.
                     // The expected uniform layout is listed as strings below.
@@ -366,8 +390,8 @@ DEF_TEST(UniformManagerCheckPaddingVectorMatrix, r) {
                     const size_t size = strlen(kExpectedLayout[vecLength1][matSize2]) * 2;
                     UniformDataBlock uniformData = mgr.finishUniformDataBlock();
                     REPORTER_ASSERT(r, uniformData.size() == size,
-                                    "Types:%d %d vector-matrix padding test failed",
-                                    (int)type1, (int)type2);
+                                    "Types: %s, %s vector-matrix padding test failed",
+                                    SkSLTypeString(type1), SkSLTypeString(type2));
                 } else if (elementSize1 == 4 && elementSize2 == 2) {
                     // Elements in the array below correspond to 16 bits apiece.
                     // The expected uniform layout is listed as strings below.
@@ -390,8 +414,8 @@ DEF_TEST(UniformManagerCheckPaddingVectorMatrix, r) {
                     const size_t size = strlen(kExpectedLayout[vecLength1][matSize2]) * 2;
                     UniformDataBlock uniformData = mgr.finishUniformDataBlock();
                     REPORTER_ASSERT(r, uniformData.size() == size,
-                                    "Types:%d %d vector-matrix padding test failed",
-                                    (int)type1, (int)type2);
+                                    "Types: %s, %s vector-matrix padding test failed",
+                                    SkSLTypeString(type1), SkSLTypeString(type2));
                 }
                 mgr.reset();
             }
@@ -399,7 +423,7 @@ DEF_TEST(UniformManagerCheckPaddingVectorMatrix, r) {
     }
 }
 
-DEF_TEST(UniformManagerCheckPaddingMatrixVector, r) {
+DEF_GRAPHITE_TEST(UniformManagerCheckPaddingMatrixVector, r) {
     // Verify that the uniform manager properly adds padding between matrices and vectors.
     for (Layout layout : kLayouts) {
         UniformManager mgr(layout);
@@ -432,27 +456,47 @@ DEF_TEST(UniformManagerCheckPaddingMatrixVector, r) {
                     // A/B: uniform values.
                     // a/b: padding as part of the uniform type (vec3 takes 4 slots)
                     // _  : padding between uniforms for alignment
-                    static constexpr const char* kExpectedLayout[5][5] = {
-                        { "", "", "", "", "" },
-                        { "", "", "", "", "" },
-                        { "", "AAAAB_", "AAAABB", "AAAABBBb", "AAAABBBB" },
-                        { "",
-                          "AAAaAAAaAAAaB___",
-                          "AAAaAAAaAAAaBB__",
-                          "AAAaAAAaAAAaBBBb",
-                          "AAAaAAAaAAAaBBBB" },
-                        { "",
-                          "AAAAAAAAAAAAAAAAB___",
-                          "AAAAAAAAAAAAAAAABB__",
-                          "AAAAAAAAAAAAAAAABBBb",
-                          "AAAAAAAAAAAAAAAABBBB" },
+                    static constexpr const char* kExpectedLayout[2][5][5] = {
+                        // std140 layout
+                        {
+                            { "", "", "", "", "" },
+                            { "", "", "", "", "" },
+                            { "", "AAaaAAaaB_", "AAaaAAaaBB", "AAaaAAaaBBBb", "AAaaAAaaBBBB" },
+                            { "",
+                              "AAAaAAAaAAAaB___",
+                              "AAAaAAAaAAAaBB__",
+                              "AAAaAAAaAAAaBBBb",
+                              "AAAaAAAaAAAaBBBB" },
+                            { "",
+                              "AAAAAAAAAAAAAAAAB___",
+                              "AAAAAAAAAAAAAAAABB__",
+                              "AAAAAAAAAAAAAAAABBBb",
+                              "AAAAAAAAAAAAAAAABBBB" },
+                        },
+                        // All other layouts
+                        {
+                            { "", "", "", "", "" },
+                            { "", "", "", "", "" },
+                            { "", "AAAAB_", "AAAABB", "AAAABBBb", "AAAABBBB" },
+                            { "",
+                              "AAAaAAAaAAAaB___",
+                              "AAAaAAAaAAAaBB__",
+                              "AAAaAAAaAAAaBBBb",
+                              "AAAaAAAaAAAaBBBB" },
+                            { "",
+                              "AAAAAAAAAAAAAAAAB___",
+                              "AAAAAAAAAAAAAAAABB__",
+                              "AAAAAAAAAAAAAAAABBBb",
+                              "AAAAAAAAAAAAAAAABBBB" },
+                        },
                     };
-                    const size_t size = strlen(kExpectedLayout[matSize1][vecLength2]) *
+                    int layoutIdx = static_cast<int>(layout != Layout::kStd140);
+                    const size_t size = strlen(kExpectedLayout[layoutIdx][matSize1][vecLength2]) *
                                         elementSize1;
                     UniformDataBlock uniformData = mgr.finishUniformDataBlock();
                     REPORTER_ASSERT(r, uniformData.size() == size,
-                                    "Types:%d %d matrix-vector padding test failed",
-                                    (int)type1, (int)type2);
+                                    "Types: %s, %s matrix-vector padding test failed",
+                                    SkSLTypeString(type1), SkSLTypeString(type2));
                 } else if (elementSize1 == 2 && elementSize2 == 4) {
                     // Elements in the array below correspond to 16 bits apiece.
                     // The expected uniform layout is listed as strings below.
@@ -477,8 +521,8 @@ DEF_TEST(UniformManagerCheckPaddingMatrixVector, r) {
                     const size_t size = strlen(kExpectedLayout[matSize1][vecLength2]) * 2;
                     UniformDataBlock uniformData = mgr.finishUniformDataBlock();
                     REPORTER_ASSERT(r, uniformData.size() == size,
-                                    "Types:%d %d matrix-vector padding test failed",
-                                    (int)type1, (int)type2);
+                                    "Types: %s, %s matrix-vector padding test failed",
+                                    SkSLTypeString(type1), SkSLTypeString(type2));
                 } else if (elementSize1 == 4 && elementSize2 == 2) {
                     // Elements in the array below correspond to 16 bits apiece.
                     // The expected uniform layout is listed as strings below.
@@ -503,8 +547,8 @@ DEF_TEST(UniformManagerCheckPaddingMatrixVector, r) {
                     const size_t size = strlen(kExpectedLayout[matSize1][vecLength2]) * 2;
                     UniformDataBlock uniformData = mgr.finishUniformDataBlock();
                     REPORTER_ASSERT(r, uniformData.size() == size,
-                                    "Types:%d %d matrix-vector padding test failed",
-                                    (int)type1, (int)type2);
+                                    "Types: %s, %s matrix-vector padding test failed",
+                                    SkSLTypeString(type1), SkSLTypeString(type2));
                 }
                 mgr.reset();
             }
@@ -512,7 +556,7 @@ DEF_TEST(UniformManagerCheckPaddingMatrixVector, r) {
     }
 }
 
-DEF_TEST(UniformManagerMetalArrayLayout, r) {
+DEF_GRAPHITE_TEST(UniformManagerMetalArrayLayout, r) {
     UniformManager mgr(Layout::kMetal);
 
     // Tests set up a uniform block with a single half (to force alignment) and an array of 3
@@ -522,10 +566,10 @@ DEF_TEST(UniformManagerMetalArrayLayout, r) {
     // Buffer large enough to hold a float4x4[3] array.
     static constexpr uint8_t kBuffer[192] = {};
     static const char* kExpectedLayout[] = {
-        // Elements in the array below correspond to 16 bits apiece.
+        // Each letter (A/B/a/b) corresponds to a single byte.
         // The expected uniform layout is listed as strings below.
         // A/B: uniform values.
-        // a/b: padding as part of the uniform type (vec3 takes 4 slots)
+        // a/b: padding as part of the uniform type.
         // _  : padding between uniforms for alignment.
 
         /* {half, short[3]}  */ "AABBBBBB",
@@ -584,7 +628,7 @@ DEF_TEST(UniformManagerMetalArrayLayout, r) {
     }
 }
 
-DEF_TEST(UniformManagerStd430ArrayLayout, r) {
+DEF_GRAPHITE_TEST(UniformManagerStd430ArrayLayout, r) {
     UniformManager mgr(Layout::kStd430);
 
     // Tests set up a uniform block with a single half (to force alignment) and an array of 3
@@ -594,10 +638,10 @@ DEF_TEST(UniformManagerStd430ArrayLayout, r) {
     // Buffer large enough to hold a float4x4[3] array.
     static constexpr uint8_t kBuffer[192] = {};
     static const char* kExpectedLayout[] = {
-        // Elements in the array below correspond to 16 bits apiece.
+        // Each letter (A/B/a/b) corresponds to a single byte.
         // The expected uniform layout is listed as strings below.
         // A/B: uniform values.
-        // a/b: padding as part of the uniform type (vec3 takes 4 slots)
+        // a/b: padding as part of the uniform type.
         // _  : padding between uniforms for alignment.
 
         /* {half, short[3]}  */ "AA__BBBBBBBBBBBB",
@@ -656,7 +700,7 @@ DEF_TEST(UniformManagerStd430ArrayLayout, r) {
     }
 }
 
-DEF_TEST(UniformManagerStd140ArrayLayout, r) {
+DEF_GRAPHITE_TEST(UniformManagerStd140ArrayLayout, r) {
     UniformManager mgr(Layout::kStd140);
 
     // Tests set up a uniform block with a single half (to force alignment) and an array of 3
@@ -666,10 +710,10 @@ DEF_TEST(UniformManagerStd140ArrayLayout, r) {
     // Buffer large enough to hold a float4x4[3] array.
     static constexpr uint8_t kBuffer[192] = {};
     static const char* kExpectedLayout[] = {
-        // Elements in the array below correspond to 16 bits apiece.
+        // Each letter (A/B/a/b) corresponds to a single byte.
         // The expected uniform layout is listed as strings below.
         // A/B: uniform values.
-        // a/b: padding as part of the uniform type (vec3 takes 4 slots)
+        // a/b: padding as part of the uniform type.
         // _  : padding between uniforms for alignment.
 
         /* {half, short[3]}  */ "AA______________BBbbbbbbbbbbbbbbBBbbbbbbbbbbbbbbBBbbbbbbbbbbbbbb",
@@ -689,7 +733,10 @@ DEF_TEST(UniformManagerStd140ArrayLayout, r) {
         /* {half, int3[3]}   */ "AA______________BBBBBBBBBBBBbbbbBBBBBBBBBBBBbbbbBBBBBBBBBBBBbbbb",
         /* {half, int4[3]}   */ "AA______________BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",
 
-        /* {half, float2x2[3] */ "AA______________BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",
+        /* {half, float2x2[3] */ "AA______________"
+                                 "BBBBBBBBbbbbbbbbBBBBBBBBbbbbbbbb"
+                                 "BBBBBBBBbbbbbbbbBBBBBBBBbbbbbbbb"
+                                 "BBBBBBBBbbbbbbbbBBBBBBBBbbbbbbbb",
         /* {half, float3x3[3] */ "AA______________"
                                  "BBBBBBBBBBBBbbbbBBBBBBBBBBBBbbbbBBBBBBBBBBBBbbbb"
                                  "BBBBBBBBBBBBbbbbBBBBBBBBBBBBbbbbBBBBBBBBBBBBbbbb"
@@ -699,7 +746,10 @@ DEF_TEST(UniformManagerStd140ArrayLayout, r) {
                                  "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB"
                                  "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",
 
-        /* {half, half2x2[3]  */ "AA______________BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",
+        /* {half, half2x2[3]  */ "AA______________"
+                                 "BBBBBBBBbbbbbbbbBBBBBBBBbbbbbbbb"
+                                 "BBBBBBBBbbbbbbbbBBBBBBBBbbbbbbbb"
+                                 "BBBBBBBBbbbbbbbbBBBBBBBBbbbbbbbb",
         /* {half, half3x3[3]  */ "AA______________"
                                  "BBBBBBBBBBBBbbbbBBBBBBBBBBBBbbbbBBBBBBBBBBBBbbbb"
                                  "BBBBBBBBBBBBbbbbBBBBBBBBBBBBbbbbBBBBBBBBBBBBbbbb"
@@ -727,3 +777,112 @@ DEF_TEST(UniformManagerStd140ArrayLayout, r) {
         mgr.reset();
     }
 }
+
+// This test validates that the uniform data for matrix types get written out according to the
+// layout expectations.
+DEF_GRAPHITE_TEST(UniformManagerStd140MatrixLayoutContents, r) {
+    UniformManager mgr(Layout::kStd140);
+
+    // float2x2, half2x2
+    for (SkSLType type : {SkSLType::kFloat2x2, SkSLType::kHalf2x2}) {
+        const Uniform expectations[] = {{"m", type}};
+        mgr.setExpectedUniforms(SkSpan(expectations));
+        mgr.write(type, kFloats);
+        mgr.doneWithExpectedUniforms();
+        const UniformDataBlock uniformData = mgr.finishUniformDataBlock();
+        REPORTER_ASSERT(r, uniformData.size() == 32,
+                        "%s layout size expected 32, got %zu",
+                        SkSLTypeString(type), uniformData.size());
+
+        // The expected offsets of the 4 matrix elements.
+        const int kOffsets[4] = {0, 1, 4, 5};
+        const float* elements = reinterpret_cast<const float*>(uniformData.data());
+        for (size_t i = 0u; i < std::size(kOffsets); ++i) {
+            float expected = kFloats[i];
+            float el = elements[kOffsets[i]];
+            REPORTER_ASSERT(r, el == expected,
+                            "Incorrect %s element %zu - expected %f, got %f",
+                            SkSLTypeString(type), i, expected, el);
+        }
+        mgr.reset();
+    }
+
+    // float3x3, half3x3
+    for (SkSLType type : {SkSLType::kFloat3x3, SkSLType::kHalf3x3}) {
+        const Uniform expectations[] = {{"m", type}};
+        mgr.setExpectedUniforms(SkSpan(expectations));
+        mgr.write(type, kFloats);
+        mgr.doneWithExpectedUniforms();
+        const UniformDataBlock uniformData = mgr.finishUniformDataBlock();
+        REPORTER_ASSERT(r, uniformData.size() == 48,
+                        "%s layout size expected 48, got %zu",
+                        SkSLTypeString(type), uniformData.size());
+
+        // The expected offsets of the 9 matrix elements.
+        const int kOffsets[9] = {0, 1, 2, 4, 5, 6, 8, 9, 10};
+        const float* elements = reinterpret_cast<const float*>(uniformData.data());
+        for (size_t i = 0u; i < std::size(kOffsets); ++i) {
+            float expected = kFloats[i];
+            float el = elements[kOffsets[i]];
+            REPORTER_ASSERT(r, el == expected,
+                            "Incorrect %s element %zu - expected %f, got %f",
+                            SkSLTypeString(type), i, expected, el);
+        }
+        mgr.reset();
+    }
+}
+
+// This test validates that the uniform data for matrix types get written out according to the
+// layout expectations.
+DEF_GRAPHITE_TEST(UniformManagerStd430MatrixLayoutContents, r) {
+    UniformManager mgr(Layout::kStd430);
+
+    // float2x2, half2x2
+    for (SkSLType type : {SkSLType::kFloat2x2, SkSLType::kHalf2x2}) {
+        const Uniform expectations[] = {{"m", type}};
+        mgr.setExpectedUniforms(SkSpan(expectations));
+        mgr.write(type, kFloats);
+        mgr.doneWithExpectedUniforms();
+        const UniformDataBlock uniformData = mgr.finishUniformDataBlock();
+        REPORTER_ASSERT(r, uniformData.size() == 16,
+                        "%s layout size expected 16, got %zu",
+                        SkSLTypeString(type), uniformData.size());
+
+        // The expected offsets of the 4 matrix elements. This uses a tighter packing than std140
+        // layout.
+        const int kOffsets[4] = {0, 1, 2, 3};
+        const float* elements = reinterpret_cast<const float*>(uniformData.data());
+        for (size_t i = 0u; i < std::size(kOffsets); ++i) {
+            float expected = kFloats[i];
+            float el = elements[kOffsets[i]];
+            REPORTER_ASSERT(r, el == expected,
+                            "Incorrect %s element %zu - expected %f, got %f",
+                            SkSLTypeString(type), i, expected, el);
+        }
+        mgr.reset();
+    }
+
+    // float3x3, half3x3
+    for (SkSLType type : {SkSLType::kFloat3x3, SkSLType::kHalf3x3}) {
+        const Uniform expectations[] = {{"m", type}};
+        mgr.setExpectedUniforms(SkSpan(expectations));
+        mgr.write(type, kFloats);
+        mgr.doneWithExpectedUniforms();
+        const UniformDataBlock uniformData = mgr.finishUniformDataBlock();
+        REPORTER_ASSERT(r, uniformData.size() == 48,
+                        "%s layout size expected 48, got %zu",
+                        SkSLTypeString(type), uniformData.size());
+
+        // The expected offsets of the 9 matrix elements. This is the same as std140 layout.
+        const int kOffsets[9] = {0, 1, 2, 4, 5, 6, 8, 9, 10};
+        const float* elements = reinterpret_cast<const float*>(uniformData.data());
+        for (size_t i = 0u; i < std::size(kOffsets); ++i) {
+            float expected = kFloats[i];
+            float el = elements[kOffsets[i]];
+            REPORTER_ASSERT(r, el == expected,
+                            "Incorrect %s element %zu - expected %f, got %f",
+                            SkSLTypeString(type), i, expected, el);
+        }
+        mgr.reset();
+    }
+}
