@@ -31,11 +31,6 @@ struct chunk_data {
   int print_content;
 };
 
-static
-long chunk_bgn(const struct curl_fileinfo *finfo, void *ptr, int remains);
-static
-long chunk_end(void *ptr);
-
 static
 long chunk_bgn(const struct curl_fileinfo *finfo, void *ptr, int remains)
 {
@@ -76,8 +71,8 @@ long chunk_bgn(const struct curl_fileinfo *finfo, void *ptr, int remains)
   }
   if(finfo->filetype == CURLFILETYPE_FILE) {
     ch_d->print_content = 1;
-    printf("Content:\n-----------------------"
-           "--------------------------------------\n");
+    printf("Content:\n"
+      "-------------------------------------------------------------\n");
   }
   if(strcmp(finfo->filename, "someothertext.txt") == 0) {
     printf("# THIS CONTENT WAS SKIPPED IN CHUNK_BGN CALLBACK #\n");
@@ -99,7 +94,7 @@ long chunk_end(void *ptr)
   return CURL_CHUNK_END_FUNC_OK;
 }
 
-int test(char *URL)
+CURLcode test(char *URL)
 {
   CURL *handle = NULL;
   CURLcode res = CURLE_OK;
