@@ -17,9 +17,12 @@
 #include "include/utils/SkNoDrawCanvas.h"
 #include "src/base/SkTLazy.h"
 #include "src/core/SkCanvasPriv.h"
+#include "src/core/SkStringUtils.h"
 #include "src/utils/SkMultiPictureDocument.h"
 #include "tools/SkSharingProc.h"
 
+using namespace skia_private;
+
 ///////////////////////////////////////////////////////////////////////////////
 
 // Base Cmd struct.
@@ -291,7 +294,7 @@ protected:
     void onDrawAnnotation(const SkRect& rect, const char key[], SkData* value) override {
         static constexpr char kOffscreenLayerDraw[] = "OffscreenLayerDraw";
         static constexpr char kSurfaceID[] = "SurfaceID";
-        SkTArray<SkString> tokens;
+        TArray<SkString> tokens;
         SkStrSplit(key, "|", kStrict_SkStrSplitMode, &tokens);
         if (tokens.size() == 2) {
             if (tokens[0].equals(kOffscreenLayerDraw)) {
