@@ -8,22 +8,47 @@
 #ifndef SKDRAWCOMMAND_H_
 #define SKDRAWCOMMAND_H_
 
-#include "include/core/SkBitmap.h"
 #include "include/core/SkCanvas.h"
-#include "include/core/SkFlattenable.h"
+#include "include/core/SkColor.h"
+#include "include/core/SkData.h"
+#include "include/core/SkDrawable.h"
+#include "include/core/SkImage.h"
+#include "include/core/SkImageFilter.h"
+#include "include/core/SkM44.h"
+#include "include/core/SkMatrix.h"
+#include "include/core/SkPaint.h"
 #include "include/core/SkPath.h"
+#include "include/core/SkPicture.h"
+#include "include/core/SkPoint.h"
 #include "include/core/SkRRect.h"
-#include "include/core/SkRSXform.h"
+#include "include/core/SkRect.h"
+#include "include/core/SkRefCnt.h"
 #include "include/core/SkRegion.h"
+#include "include/core/SkSamplingOptions.h"
+#include "include/core/SkScalar.h"
+#include "include/core/SkShader.h"
 #include "include/core/SkString.h"
+#include "include/core/SkTextBlob.h"
+#include "include/core/SkTypes.h"
 #include "include/core/SkVertices.h"
-#include "include/private/SkTDArray.h"
+#include "include/private/base/SkTDArray.h"
+#include "include/private/base/SkTemplates.h"
+#include "src/base/SkTLazy.h"
 #include "src/core/SkDrawShadowInfo.h"
-#include "src/core/SkTLazy.h"
-#include "src/utils/SkJSONWriter.h"
-#include "tools/UrlDataManager.h"
+
+#include <cstddef>
+#include <cstdint>
 
 class DebugLayerManager;
+class SkBitmap;
+class SkFlattenable;
+class SkJSONWriter;
+class SkWStream;
+class UrlDataManager;
+enum class SkBlendMode;
+enum class SkClipOp;
+struct SkPoint3;
+struct SkRSXform;
 
 class DrawCommand {
 public:
@@ -104,6 +129,7 @@ public:
     static void MakeJsonMatrix44(SkJSONWriter&, const SkM44&);
     static void MakeJsonPath(SkJSONWriter&, const SkPath& path);
     static void MakeJsonRegion(SkJSONWriter&, const SkRegion& region);
+    static void MakeJsonSampling(SkJSONWriter&, const SkSamplingOptions& sampling);
     static void MakeJsonPaint(SkJSONWriter&, const SkPaint& paint, UrlDataManager& urlDataManager);
     static void MakeJsonLattice(SkJSONWriter&, const SkCanvas::Lattice& lattice);
 
@@ -718,13 +744,13 @@ public:
     void execute(SkCanvas* canvas) const override;
 
 private:
-    SkAutoTArray<SkCanvas::ImageSetEntry> fSet;
-    int                                   fCount;
-    SkAutoTArray<SkPoint>                 fDstClips;
-    SkAutoTArray<SkMatrix>                fPreViewMatrices;
-    SkSamplingOptions                     fSampling;
-    SkTLazy<SkPaint>                      fPaint;
-    SkCanvas::SrcRectConstraint           fConstraint;
+    skia_private::AutoTArray<SkCanvas::ImageSetEntry> fSet;
+    int                                               fCount;
+    skia_private::AutoTArray<SkPoint>                 fDstClips;
+    skia_private::AutoTArray<SkMatrix>                fPreViewMatrices;
+    SkSamplingOptions                                 fSampling;
+    SkTLazy<SkPaint>                                  fPaint;
+    SkCanvas::SrcRectConstraint                       fConstraint;
 
     using INHERITED = DrawCommand;
 };
