@@ -11,7 +11,7 @@
 #include "include/core/SkStream.h"
 #include "include/core/SkSurface.h"
 #include "include/core/SkTime.h"
-#include "include/private/SkTPin.h"
+#include "include/private/base/SkTPin.h"
 #include "modules/skottie/include/Skottie.h"
 #include "modules/skresources/include/SkResources.h"
 #include "src/utils/SkOSPath.h"
@@ -20,6 +20,7 @@
 #include "tools/gpu/GrContextFactory.h"
 
 #include "include/gpu/GrContextOptions.h"
+#include "include/gpu/ganesh/SkSurfaceGanesh.h"
 
 static DEFINE_string2(input, i, "", "skottie animation to render");
 static DEFINE_string2(output, o, "", "mp4 file to create");
@@ -47,7 +48,7 @@ int main(int argc, char** argv) {
     CommandLineFlags::SetUsage("Converts skottie to a mp4");
     CommandLineFlags::Parse(argc, argv);
 
-    if (FLAGS_input.count() == 0) {
+    if (FLAGS_input.size() == 0) {
         SkDebugf("-i input_file.json argument required\n");
         return -1;
     }
@@ -57,7 +58,7 @@ int main(int argc, char** argv) {
     sk_gpu_test::GrContextFactory factory(grCtxOptions);
 
     SkString assetPath;
-    if (FLAGS_assetPath.count() > 0) {
+    if (FLAGS_assetPath.size() > 0) {
         assetPath.set(FLAGS_assetPath[0]);
     } else {
         assetPath = SkOSPath::Dirname(FLAGS_input[0]);
@@ -110,18 +111,18 @@ int main(int argc, char** argv) {
         if (!surf) {
             if (FLAGS_gpu) {
                 grctx = factory.getContextInfo(contextType).directContext();
-                surf = SkSurface::MakeRenderTarget(grctx,
-                                                   SkBudgeted::kNo,
-                                                   info,
-                                                   0,
-                                                   GrSurfaceOrigin::kTopLeft_GrSurfaceOrigin,
-                                                   nullptr);
+                surf = SkSurfaces::RenderTarget(grctx,
+                                                skgpu::Budgeted::kNo,
+                                                info,
+                                                0,
+                                                GrSurfaceOrigin::kTopLeft_GrSurfaceOrigin,
+                                                nullptr);
                 if (!surf) {
                     grctx = nullptr;
                 }
             }
             if (!surf) {
-                surf = SkSurface::MakeRaster(info);
+                surf = SkSurfaces::Raster(info);
             }
             surf->getCanvas()->scale(scale, scale);
         }
@@ -168,7 +169,7 @@ int main(int argc, char** argv) {
         }
     } while (FLAGS_loop);
 
-    if (FLAGS_output.count() == 0) {
+    if (FLAGS_output.size() == 0) {
         SkDebugf("missing -o output_file.mp4 argument\n");
         return 0;
     }
