@@ -47,6 +47,9 @@ int bug_8378(MYSQL *mysql) {
   MYSQL_RES *res;
   MYSQL_ROW row;
 
+  /* MXS-4898: MaxScale sends utf8mb4 in handshake OK packet */
+  SKIP_MAXSCALE;
+
   len= mysql_real_escape_string(mysql, out, TEST_BUG8378_IN, 4);
   FAIL_IF(memcmp(out, TEST_BUG8378_OUT, len), "wrong result");
 
@@ -71,14 +74,16 @@ int bug_8378(MYSQL *mysql) {
 int test_client_character_set(MYSQL *mysql)
 {
   MY_CHARSET_INFO cs;
-  char *csname= (char*) "utf8";
+  char *csname= (char*) "latin2";
   char *csdefault= (char*)mysql_character_set_name(mysql);
 
+
   FAIL_IF(mysql_set_character_set(mysql, csname), mysql_error(mysql));
 
   mysql_get_character_set_info(mysql, &cs);
 
-  FAIL_IF(strcmp(cs.csname, "utf8") || strcmp(cs.name, "utf8_general_ci"), "Character set != UTF8");
+  FAIL_IF(strcmp(cs.csname, "latin2") || strcmp(cs.name, "latin2_general_ci"),
+          "Character set != latin2");
   FAIL_IF(mysql_set_character_set(mysql, csdefault), mysql_error(mysql));
 
   return OK;
@@ -100,7 +105,7 @@ int bug_10214(MYSQL *mysql)
   check_mysql_rc(rc, mysql);
 
   len= mysql_real_escape_string(mysql, out, "a'b\\c", 5);
-  FAIL_IF(memcmp(out, "a\\'b\\\\c", len), NULL);
+  FAIL_IF(memcmp(out, "a\\'b\\\\c", len), "wrong result");
 
   rc= mysql_query(mysql, "set sql_mode='NO_BACKSLASH_ESCAPES'");
   check_mysql_rc(rc, mysql);
@@ -535,6 +540,8 @@ static int test_bug30472(MYSQL *mysql)
   char character_set_results_4[MY_CS_NAME_SIZE];
   char collation_connnection_4[MY_CS_NAME_SIZE];
 
+  SKIP_MAXSCALE;
+
   if (mysql_get_server_version(mysql) < 50100 || !is_mariadb) 
   {
     diag("Test requires MySQL Server version 5.1 or above");
@@ -551,7 +558,8 @@ static int test_bug30472(MYSQL *mysql)
 
   /* Switch client character set. */
 
-  FAIL_IF(mysql_set_character_set(mysql, "utf8"), "Setting cs to utf8 failed");
+  FAIL_IF(mysql_set_character_set(mysql, "ascii"),
+          "Setting cs to ascii failed");
 
   /* Retrieve character set information. */
 
@@ -567,10 +575,11 @@ static int test_bug30472(MYSQL *mysql)
       2) new character set is different from the original one.
   */
 
-  FAIL_UNLESS(strcmp(character_set_name_2, "utf8") == 0, "cs_name != utf8");
-  FAIL_UNLESS(strcmp(character_set_client_2, "utf8") == 0, "cs_client != utf8");
-  FAIL_UNLESS(strcmp(character_set_results_2, "utf8") == 0, "cs_result != ut8");
-  FAIL_UNLESS(strcmp(collation_connnection_2, "utf8_general_ci") == 0, "collation != utf8_general_ci");
+  FAIL_UNLESS(strcmp(character_set_name_2, "ascii") == 0, "cs_name != ascii");
+  FAIL_UNLESS(strcmp(character_set_client_2, "ascii") == 0, "cs_client != ascii");
+  FAIL_UNLESS(strcmp(character_set_results_2, "ascii") == 0, "cs_result != ascii");
+  FAIL_UNLESS(strcmp(collation_connnection_2, "ascii_general_ci") == 0,
+              "collation != ascii_general_ci");
 
   diag("%s %s", character_set_name_1, character_set_name_2);
   FAIL_UNLESS(strcmp(character_set_name_1, character_set_name_2) != 0, "cs_name1 = cs_name2");
@@ -601,7 +610,7 @@ static int test_bug30472(MYSQL *mysql)
 
   /* Change connection-default character set in the client. */
 
-  mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8");
+  mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "latin2");
 
   /*
     Call mysql_change_user() in order to check that new connection will
@@ -621,10 +630,11 @@ static int test_bug30472(MYSQL *mysql)
 
   /* Check that we have UTF8 on the server and on the client. */
 
-  FAIL_UNLESS(strcmp(character_set_name_4, "utf8") == 0, "cs_name != utf8");
-  FAIL_UNLESS(strcmp(character_set_client_4, "utf8") == 0, "cs_client != utf8");
-  FAIL_UNLESS(strcmp(character_set_results_4, "utf8") == 0, "cs_result != utf8");
-  FAIL_UNLESS(strcmp(collation_connnection_4, "utf8_general_ci") == 0, "collation_connection != utf8_general_ci");
+  FAIL_UNLESS(strcmp(character_set_name_4, "latin2") == 0, "cs_name != latin2");
+  FAIL_UNLESS(strcmp(character_set_client_4, "latin2") == 0, "cs_client != latin2");
+  FAIL_UNLESS(strcmp(character_set_results_4, "latin2") == 0, "cs_result != latin2");
+  FAIL_UNLESS(strcmp(collation_connnection_4, "latin2_general_ci") == 0,
+              "collation_connection != latin2_general_ci");
 
   /* That's it. Cleanup. */
 
@@ -645,8 +655,12 @@ static int test_bug_54100(MYSQL *mysql)
   while ((row= mysql_fetch_row(result)))
   {
     /* ignore ucs2 */
-    if (strcmp(row[0], "ucs2") && strcmp(row[0], "utf16le") && strcmp(row[0], "utf8mb4") && 
-        strcmp(row[0], "utf16") && strcmp(row[0], "utf32")) {
+    if (strcmp(row[0], "ucs2")
+        && strcmp(row[0], "utf16le")
+        && (strcmp(row[0], "utf8mb4") && mariadb_connection(mysql) && mysql_get_server_version(mysql) < 100600)
+        && (strcmp(row[0], "utf8") && mariadb_connection(mysql) && mysql_get_server_version(mysql) >= 100600)
+        && strcmp(row[0], "utf16")
+        && strcmp(row[0], "utf32")) {
       rc= mysql_set_character_set(mysql, row[0]);
       check_mysql_rc(rc, mysql);
     }
@@ -661,6 +675,10 @@ static int test_bug_54100(MYSQL *mysql)
 
 static int test_utf16_utf32_noboms(MYSQL *mysql __attribute__((unused)))
 {
+#ifndef HAVE_ICONV
+  diag("MariaDB Connector/C was built without iconv support");
+  return SKIP;
+#else
   const char *csname[]= {"utf16", "utf16le", "utf32", "utf8"};
   MARIADB_CHARSET_INFO  *csinfo[sizeof(csname)/sizeof(char*)];
 
@@ -724,6 +742,7 @@ static int test_utf16_utf32_noboms(MYSQL *mysql __attribute__((unused)))
   }
 
   return OK;
+#endif
 }
 
 static int charset_auto(MYSQL *my __attribute__((unused)))
@@ -744,8 +763,9 @@ static int charset_auto(MYSQL *my __attribute__((unused)))
   csname1= mysql_character_set_name(mysql);
   diag("Character set: %s os charset: %s", csname1, osname);
 
-  FAIL_IF(strcmp(osname, csname1), "character set is not os character set");
-
+  FAIL_IF(!strcasecmp(osname,"utf8") ? strncmp(osname, csname1, 4) :
+                                       strcmp(osname, csname1),
+          "character set is not os character set");
   if (strcmp(osname, "utf8"))
   {
     rc= mysql_set_character_set(mysql, "utf8");
@@ -753,7 +773,6 @@ static int charset_auto(MYSQL *my __attribute__((unused)))
 
     csname2= mysql_character_set_name(mysql);
     diag("Character set: %s", csname2);
-
     FAIL_IF(!strcmp(csname2, csname1), "Wrong charset: expected utf8");
 
     rc= mysql_set_character_set(mysql, "auto");
@@ -774,18 +793,62 @@ static int test_conc223(MYSQL *mysql)
   MYSQL_RES *res;
   MYSQL_ROW row;
   int found= 0;
+  int mdev27266= 0;
+  int unsupported[]= {
+                      579, /* utf8mb3_general1400_as_ci added in 11.5 */
+                      611, /* utf8mb4_general1400_as_ci added in 11.5 */
+                      0};
 
-  rc= mysql_query(mysql, "SELECT ID, CHARACTER_SET_NAME, COLLATION_NAME FROM INFORMATION_SCHEMA.COLLATIONS");
+  SKIP_MYSQL(mysql);
+
+  /*
+    Test if we're running against an MDEV-27266 server.
+    It can be detected by the presense of the FULL_COLLATION_NAME
+    column in I_S.COLLATION_CHARACTER_SET_APPLICABILITY.
+  */
+  rc= mysql_query(mysql,
+        "SELECT COUNT(*) "
+        "FROM INFORMATION_SCHEMA.COLUMNS "
+        "WHERE COLUMN_NAME='FULL_COLLATION_NAME' "
+        "  AND TABLE_NAME='COLLATION_CHARACTER_SET_APPLICABILITY'");
+  check_mysql_rc(rc, mysql);
+  res= mysql_store_result(mysql);
+  if ((row= mysql_fetch_row(res)))
+    mdev27266= atoi(row[0]);
+  mysql_free_result(res);
+  diag("MDEV-27266 aware server: %d", mdev27266);
+
+  /*
+    Now get the list of collations either from I_S.COLLATIONS
+    or I_S.COLLATION_CHARACTER_SET_APPLICABILITY,
+    depending on the MDEV-27266 server awareness.
+  */
+  if (mdev27266)
+    rc= mysql_query(mysql,
+          "SELECT ID, CHARACTER_SET_NAME, FULL_COLLATION_NAME "
+          "FROM INFORMATION_SCHEMA.COLLATION_CHARACTER_SET_APPLICABILITY");
+  else
+    rc= mysql_query(mysql,
+         "SELECT ID, CHARACTER_SET_NAME, COLLATION_NAME "
+         "FROM INFORMATION_SCHEMA.COLLATIONS");
   check_mysql_rc(rc, mysql);
 
   res= mysql_store_result(mysql);
   while ((row = mysql_fetch_row(res)))
   {
-    int id= atoi(row[0]);
-    if (!mariadb_get_charset_by_nr(id))
+    int id;
+
+    if (row[0])
     {
-      diag("%04d %s %s", id, row[1], row[2]);
-      found++;
+      id= atoi(row[0]);
+      if (!mariadb_get_charset_by_nr(id))
+      {
+        int j=0;
+        found++;
+        for (j=0; unsupported[j]; j++)
+          if (unsupported[j] == id)
+            found--;
+      }
     }
   }
   mysql_free_result(res);
