@@ -12,13 +12,13 @@
 // Copyright 2017, 2019 Adrian Johnson <ajohnson@redneon.com>
 // Copyright 2018 Chinmoy Ranjan Pradhan <chinmoyrp65@protonmail.com>
 // Copyright 2019 Alexey Pavlov <alexpux@gmail.com>
-// Copyright 2019 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright 2019. 2023, 2024 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright 2019 Nelson Efrain A. Cruz <neac03@gmail.com>
 // Copyright 2021 Georgiy Sgibnev <georgiy@sgibnev.com>. Work sponsored by lab50.net.
 // Copyright 2021 Theofilos Intzoglou <int.teo@gmail.com>
 // Copyright 2022 Felix Jung <fxjung@posteo.de>
-// Copyright 2022 Erich E. Hoover <erich.e.hoover@gmail.com>
-// Copyright 2023 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright 2022, 2024 Erich E. Hoover <erich.e.hoover@gmail.com>
+// Copyright 2023, 2024 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 //========================================================================
 
@@ -38,6 +38,7 @@
 #include "Page.h"
 #include "PDFDoc.h"
 #include "PDFDocFactory.h"
+#include "DateInfo.h"
 #include "Error.h"
 #include "GlobalParams.h"
 #ifdef ENABLE_NSS3
@@ -52,6 +53,14 @@
 #    include <libgen.h>
 #endif
 
+#ifdef HAVE_GETTEXT
+#    include <libintl.h>
+#    include <clocale>
+#    define _(STRING) gettext(STRING)
+#else
+#    define _(STRING) STRING
+#endif
+
 static const char *getReadableSigState(SignatureValidationStatus sig_vs)
 {
     switch (sig_vs) {
@@ -120,10 +129,10 @@ static bool dumpSignature(int sig_num, int sigCount, FormFieldSignature *s, cons
     // We want format to be {0:s}.sig{1:Xd} where X is sigCountLength
     // since { is the magic character to replace things we need to put it twice where
     // we don't want it to be replaced
-    const std::unique_ptr<GooString> format = GooString::format("{{0:s}}.sig{{1:{0:d}d}}", sigCountLength);
-    const std::unique_ptr<GooString> path = GooString::format(format->c_str(), gbasename(filename).c_str(), sig_num);
-    printf("Signature #%d (%u bytes) => %s\n", sig_num, signature->getLength(), path->c_str());
-    std::ofstream outfile(path->c_str(), std::ofstream::binary);
+    const std::string format = GooString::format("{{0:s}}.sig{{1:{0:d}d}}", sigCountLength);
+    const std::string path = GooString::format(format.c_str(), gbasename(filename).c_str(), sig_num);
+    printf("Signature #%d (%u bytes) => %s\n", sig_num, signature->getLength(), path.c_str());
+    std::ofstream outfile(path.c_str(), std::ofstream::binary);
     outfile.write(signature->c_str(), signature->getLength());
     outfile.close();
 
@@ -139,6 +148,7 @@ static bool printHelp = false;
 static bool printCryptoSignBackends = false;
 static bool dontVerifyCert = false;
 static bool noOCSPRevocationCheck = false;
+static bool noAppearance = false;
 static bool dumpSignatures = false;
 static bool etsiCAdESdetached = false;
 static char backendString[256] = "";
@@ -156,6 +166,7 @@ static const ArgDesc argDesc[] = { { "-nssdir", argGooString, &nssDir, 0, "path
                                    { "-nss-pwd", argGooString, &nssPassword, 0, "password to access the NSS database (if any)" },
                                    { "-nocert", argFlag, &dontVerifyCert, 0, "don't perform certificate validation" },
                                    { "-no-ocsp", argFlag, &noOCSPRevocationCheck, 0, "don't perform online OCSP certificate revocation check" },
+                                   { "-no-appearance", argFlag, &noAppearance, 0, "don't add appearance information when signing existing fields" },
                                    { "-aia", argFlag, &useAIACertFetch, 0, "use Authority Information Access (AIA) extension for certificate fetching" },
                                    { "-dump", argFlag, &dumpSignatures, 0, "dump all signatures into current directory" },
                                    { "-add-signature", argFlag, &addNewSignature, 0, "adds a new signature to the document" },
@@ -275,16 +286,14 @@ static std::string TextStringToUTF8(const std::string &str)
 {
     const UnicodeMap *utf8Map = globalParams->getUtf8Map();
 
-    Unicode *u;
-    const int len = TextStringToUCS4(str, &u);
+    std::vector<Unicode> u = TextStringToUCS4(str);
 
     std::string convertedStr;
-    for (int i = 0; i < len; ++i) {
+    for (auto &c : u) {
         char buf[8];
-        const int n = utf8Map->mapUnicode(u[i], buf, sizeof(buf));
+        const int n = utf8Map->mapUnicode(c, buf, sizeof(buf));
         convertedStr.append(buf, n);
     }
-    gfree(u);
 
     return convertedStr;
 }
@@ -425,7 +434,7 @@ int main(int argc, char *argv[])
             return 2;
         }
 
-        const auto rs = std::unique_ptr<GooString>(reason.toStr().empty() ? nullptr : utf8ToUtf16WithBom(reason.toStr()));
+        const auto rs = std::unique_ptr<GooString>(reason.toStr().empty() ? nullptr : std::make_unique<GooString>(utf8ToUtf16WithBom(reason.toStr())));
 
         if (newSignatureFieldName.getLength() == 0) {
             // Create a random field name, it could be anything but 32 hex numbers should
@@ -509,13 +518,33 @@ int main(int argc, char *argv[])
         if (etsiCAdESdetached) {
             ffs->setSignatureType(ETSI_CAdES_detached);
         }
-        const auto rs = std::unique_ptr<GooString>(reason.toStr().empty() ? nullptr : utf8ToUtf16WithBom(reason.toStr()));
+        const auto rs = std::unique_ptr<GooString>(reason.toStr().empty() ? nullptr : std::make_unique<GooString>(utf8ToUtf16WithBom(reason.toStr())));
         if (ffs->getNumWidgets() != 1) {
             printf("Unexpected number of widgets for the signature: %d\n", ffs->getNumWidgets());
             return 2;
         }
+#ifdef HAVE_GETTEXT
+        if (!noAppearance) {
+            setlocale(LC_ALL, "");
+            bindtextdomain("pdfsig", CMAKE_INSTALL_LOCALEDIR);
+            textdomain("pdfsig");
+        }
+#endif
         FormWidgetSignature *fws = static_cast<FormWidgetSignature *>(ffs->getWidget(0));
-        const bool success = fws->signDocument(std::string { argv[2] }, std::string { certNickname }, std::string { password }, rs.get());
+        auto backend = CryptoSign::Factory::createActive();
+        auto sigHandler = backend->createSigningHandler(certNickname, HashAlgorithm::Sha256);
+        std::unique_ptr<X509CertificateInfo> certInfo = sigHandler->getCertificateInfo();
+        if (!certInfo) {
+            fprintf(stderr, "signDocument: error getting signature info\n");
+            return 2;
+        }
+        const std::string signerName = certInfo->getSubjectInfo().commonName;
+        const std::string timestamp = timeToStringWithFormat(nullptr, "%Y.%m.%d %H:%M:%S %z");
+        const AnnotColor blackColor(0, 0, 0);
+        const std::string signatureText(GooString::format(_("Digitally signed by {0:s}"), signerName.c_str()) + "\n" + GooString::format(_("Date: {0:s}"), timestamp.c_str()));
+        const auto gSignatureText = std::make_unique<GooString>((signatureText.empty() || noAppearance) ? "" : utf8ToUtf16WithBom(signatureText));
+        const auto gSignatureLeftText = std::make_unique<GooString>((signerName.empty() || noAppearance) ? "" : utf8ToUtf16WithBom(signerName));
+        const bool success = fws->signDocumentWithAppearance(argv[2], std::string { certNickname }, std::string { password }, rs.get(), nullptr, {}, {}, *gSignatureText, *gSignatureLeftText, 0, 0, std::make_unique<AnnotColor>(blackColor));
         return success ? 0 : 3;
     }
 
@@ -545,6 +574,16 @@ int main(int argc, char *argv[])
         printf("File '%s' does not contain any signatures\n", fileName->c_str());
         return 2;
     }
+    std::unordered_map<int, SignatureInfo *> signatureInfos;
+    for (unsigned int i = 0; i < sigCount; i++) {
+        // Let's start the signature check first for signatures.
+        // we can always wait for completion later
+        FormFieldSignature *ffs = signatures.at(i);
+        if (ffs->getSignatureType() == unsigned_signature_field) {
+            continue;
+        }
+        signatureInfos[i] = ffs->validateSignatureAsync(!dontVerifyCert, false, -1 /* now */, !noOCSPRevocationCheck, useAIACertFetch, {});
+    }
 
     for (unsigned int i = 0; i < sigCount; i++) {
         FormFieldSignature *ffs = signatures.at(i);
@@ -560,7 +599,8 @@ int main(int argc, char *argv[])
             continue;
         }
 
-        const SignatureInfo *sig_info = ffs->validateSignature(!dontVerifyCert, false, -1 /* now */, !noOCSPRevocationCheck, useAIACertFetch);
+        const SignatureInfo *sig_info = signatureInfos[i];
+        CertificateValidationStatus certificateStatus = ffs->validateSignatureResult();
         printf("  - Signer Certificate Common Name: %s\n", sig_info->getSignerName().c_str());
         printf("  - Signer full Distinguished Name: %s\n", sig_info->getSubjectDN().c_str());
         printf("  - Signing Time: %s\n", time_str = getReadableTime(sig_info->getSigningTime()));
@@ -620,7 +660,7 @@ int main(int argc, char *argv[])
         if (sig_info->getSignatureValStatus() != SIGNATURE_VALID || dontVerifyCert) {
             continue;
         }
-        printf("  - Certificate Validation: %s\n", getReadableCertState(sig_info->getCertificateValStatus()));
+        printf("  - Certificate Validation: %s\n", getReadableCertState(certificateStatus));
     }
 
     return 0;
