@@ -1,4 +1,4 @@
-// Copyright 2017 PDFium Authors. All rights reserved.
+// Copyright 2017 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -13,9 +13,11 @@ CFGAS_BreakLine::CFGAS_BreakLine() = default;
 
 CFGAS_BreakLine::~CFGAS_BreakLine() = default;
 
-CFGAS_Char* CFGAS_BreakLine::GetChar(int32_t index) {
-  DCHECK(fxcrt::IndexInBounds(m_LineChars, index));
-  return &m_LineChars[index];
+CFGAS_Char* CFGAS_BreakLine::LastChar() {
+  if (m_LineChars.empty())
+    return nullptr;
+
+  return &m_LineChars.back();
 }
 
 int32_t CFGAS_BreakLine::GetLineEnd() const {
