@@ -1,4 +1,4 @@
-// Copyright 2016 PDFium Authors. All rights reserved.
+// Copyright 2016 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,19 +6,14 @@
 
 #include "xfa/fwl/cfwl_comboedit.h"
 
-#include <memory>
-#include <utility>
-
 #include "xfa/fde/cfde_texteditengine.h"
 #include "xfa/fwl/cfwl_combobox.h"
 #include "xfa/fwl/cfwl_messagemouse.h"
 
-CFWL_ComboEdit::CFWL_ComboEdit(
-    const CFWL_App* app,
-    std::unique_ptr<CFWL_WidgetProperties> properties,
-    CFWL_Widget* pOuter)
-    : CFWL_Edit(app, std::move(properties), pOuter) {
-}
+CFWL_ComboEdit::CFWL_ComboEdit(CFWL_App* app,
+                               const Properties& properties,
+                               CFWL_Widget* pOuter)
+    : CFWL_Edit(app, properties, pOuter) {}
 
 CFWL_ComboEdit::~CFWL_ComboEdit() = default;
 
@@ -28,40 +23,27 @@ void CFWL_ComboEdit::ClearSelected() {
 }
 
 void CFWL_ComboEdit::SetSelected() {
-  FlagFocus(true);
+  m_Properties.m_dwStates |= FWL_STATE_WGT_Focused;
   SelectAll();
 }
 
-void CFWL_ComboEdit::FlagFocus(bool bSet) {
-  if (bSet) {
-    m_pProperties->m_dwStates |= FWL_WGTSTATE_Focused;
-    return;
-  }
-
-  m_pProperties->m_dwStates &= ~FWL_WGTSTATE_Focused;
-  HideCaret(nullptr);
-}
-
 void CFWL_ComboEdit::OnProcessMessage(CFWL_Message* pMessage) {
-  if (!pMessage)
-    return;
-
   bool backDefault = true;
   switch (pMessage->GetType()) {
-    case CFWL_Message::Type::SetFocus: {
-      m_pProperties->m_dwStates |= FWL_WGTSTATE_Focused;
+    case CFWL_Message::Type::kSetFocus: {
+      m_Properties.m_dwStates |= FWL_STATE_WGT_Focused;
       backDefault = false;
       break;
     }
-    case CFWL_Message::Type::KillFocus: {
-      m_pProperties->m_dwStates &= ~FWL_WGTSTATE_Focused;
+    case CFWL_Message::Type::kKillFocus: {
+      m_Properties.m_dwStates &= ~FWL_STATE_WGT_Focused;
       backDefault = false;
       break;
     }
-    case CFWL_Message::Type::Mouse: {
+    case CFWL_Message::Type::kMouse: {
       CFWL_MessageMouse* pMsg = static_cast<CFWL_MessageMouse*>(pMessage);
-      if ((pMsg->m_dwCmd == FWL_MouseCommand::LeftButtonDown) &&
-          ((m_pProperties->m_dwStates & FWL_WGTSTATE_Focused) == 0)) {
+      if ((pMsg->m_dwCmd == CFWL_MessageMouse::MouseCommand::kLeftButtonDown) &&
+          ((m_Properties.m_dwStates & FWL_STATE_WGT_Focused) == 0)) {
         SetSelected();
       }
       break;
