@@ -1,4 +1,4 @@
-// Copyright 2016 PDFium Authors. All rights reserved.
+// Copyright 2016 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -12,11 +12,15 @@
 
 class CFWL_EventMouse final : public CFWL_Event {
  public:
-  explicit CFWL_EventMouse(CFWL_Widget* pSrcTarget);
-  CFWL_EventMouse(CFWL_Widget* pSrcTarget, CFWL_Widget* pDstTarget);
+  CFWL_EventMouse(CFWL_Widget* pSrcTarget,
+                  CFWL_Widget* pDstTarget,
+                  CFWL_MessageMouse::MouseCommand cmd);
   ~CFWL_EventMouse() override;
 
-  FWL_MouseCommand m_dwCmd;
+  CFWL_MessageMouse::MouseCommand GetCommand() const { return m_dwCmd; }
+
+ private:
+  const CFWL_MessageMouse::MouseCommand m_dwCmd;
 };
 
 #endif  // XFA_FWL_CFWL_EVENTMOUSE_H_
