@@ -1,4 +1,4 @@
-// Copyright 2016 PDFium Authors. All rights reserved.
+// Copyright 2016 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,7 @@
 #ifndef XFA_FWL_CFWL_THEMEBACKGROUND_H_
 #define XFA_FWL_CFWL_THEMEBACKGROUND_H_
 
+#include "core/fxcrt/fx_memory.h"
 #include "core/fxcrt/unowned_ptr.h"
 #include "xfa/fwl/cfwl_themepart.h"
 
@@ -15,15 +16,20 @@ class CFGAS_GEPath;
 
 class CFWL_ThemeBackground final : public CFWL_ThemePart {
  public:
-  CFWL_ThemeBackground();
+  FX_STACK_ALLOCATED();
+
+  CFWL_ThemeBackground(Part iPart,
+                       CFWL_Widget* pWidget,
+                       CFGAS_GEGraphics* pGraphics);
   ~CFWL_ThemeBackground();
 
-  UnownedPtr<CFGAS_GEGraphics> m_pGraphics;
+  CFGAS_GEGraphics* GetGraphics() const { return m_pGraphics; }
+  const CFGAS_GEPath* GetPath() const { return m_pPath; }
+  void SetPath(const CFGAS_GEPath* pPath) { m_pPath = pPath; }
+
+ private:
   UnownedPtr<const CFGAS_GEPath> m_pPath;
+  UnownedPtr<CFGAS_GEGraphics> const m_pGraphics;
 };
 
-inline CFWL_ThemeBackground::CFWL_ThemeBackground() = default;
-
-inline CFWL_ThemeBackground::~CFWL_ThemeBackground() = default;
-
 #endif  // XFA_FWL_CFWL_THEMEBACKGROUND_H_
