@@ -1,4 +1,4 @@
-// Copyright 2016 PDFium Authors. All rights reserved.
+// Copyright 2016 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -13,6 +13,7 @@
 
 #include "fxjs/xfa/cjx_object.h"
 #include "third_party/base/notreached.h"
+#include "third_party/base/numerics/safe_conversions.h"
 #include "xfa/fgas/graphics/cfgas_gegraphics.h"
 #include "xfa/fgas/graphics/cfgas_gepath.h"
 #include "xfa/fgas/graphics/cfgas_gepattern.h"
@@ -89,15 +90,17 @@ XFA_AttributeValue CXFA_Box::GetPresence() {
       .value_or(XFA_AttributeValue::Visible);
 }
 
-int32_t CXFA_Box::CountEdges() {
+size_t CXFA_Box::CountEdges() {
   return CountChildren(XFA_Element::Edge, false);
 }
 
-CXFA_Edge* CXFA_Box::GetEdgeIfExists(int32_t nIndex) {
-  if (nIndex == 0)
-    return JSObject()->GetOrCreateProperty<CXFA_Edge>(nIndex,
-                                                      XFA_Element::Edge);
-  return JSObject()->GetProperty<CXFA_Edge>(nIndex, XFA_Element::Edge);
+CXFA_Edge* CXFA_Box::GetEdgeIfExists(size_t nIndex) {
+  if (nIndex == 0) {
+    return JSObject()->GetOrCreateProperty<CXFA_Edge>(
+        pdfium::base::checked_cast<int32_t>(nIndex), XFA_Element::Edge);
+  }
+  return JSObject()->GetProperty<CXFA_Edge>(
+      pdfium::base::checked_cast<int32_t>(nIndex), XFA_Element::Edge);
 }
 
 std::vector<CXFA_Stroke*> CXFA_Box::GetStrokes() {
@@ -204,7 +207,7 @@ void CXFA_Box::Draw(CFGAS_GEGraphics* pGS,
   } else if (type == XFA_Element::Rectangle || type == XFA_Element::Border) {
     ToRectangle(this)->Draw(strokes, pGS, rtWidget, matrix);
   } else {
-    NOTREACHED();
+    NOTREACHED_NORETURN();
   }
 }
 
@@ -217,10 +220,9 @@ void CXFA_Box::DrawFill(const std::vector<CXFA_Stroke*>& strokes,
   if (!fill || !fill->IsVisible())
     return;
 
-  pGS->SaveGraphState();
-
   CFGAS_GEPath fillPath;
   XFA_Element type = GetElementType();
+  CFGAS_GEGraphics::StateRestorer restorer(pGS);
   if (type == XFA_Element::Arc || forceRound) {
     CXFA_Edge* edge = GetEdgeIfExists(0);
     float fThickness = fmax(0.0, edge ? edge->GetThickness() : 0);
@@ -235,12 +237,10 @@ void CXFA_Box::DrawFill(const std::vector<CXFA_Stroke*>& strokes,
   } else if (type == XFA_Element::Rectangle || type == XFA_Element::Border) {
     ToRectangle(this)->GetFillPath(strokes, rtWidget, &fillPath);
   } else {
-    NOTREACHED();
+    NOTREACHED_NORETURN();
   }
   fillPath.Close();
-
   fill->Draw(pGS, fillPath, rtWidget, matrix);
-  pGS->RestoreGraphState();
 }
 
 void CXFA_Box::GetPathArcOrRounded(CFX_RectF rtDraw,
@@ -307,7 +307,7 @@ void CXFA_Box::StrokeArcOrRounded(CFGAS_GEGraphics* pGS,
       edge->Stroke(pGS, arcPath, matrix);
     return;
   }
-  pGS->SaveGraphState();
+  CFGAS_GEGraphics::StateRestorer restorer(pGS);
   pGS->SetLineWidth(fHalf);
 
   float a = rtWidget.width / 2.0f;
@@ -348,5 +348,4 @@ void CXFA_Box::StrokeArcOrRounded(CFGAS_GEGraphics* pGS,
 
   pGS->SetStrokeColor(CFGAS_GEColor(0xFFC0C0C0));
   pGS->StrokePath(arcPath, matrix);
-  pGS->RestoreGraphState();
 }
